@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -24,23 +24,25 @@
 #include <mysql/group_replication.h>
 
 #include "modules/adminapi/cluster/dissolve.h"
+#include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/metadata_storage.h"
 #include "modules/adminapi/common/provision.h"
 #include "modules/adminapi/common/sql.h"
 #include "mysqlshdk/include/shellcore/console.h"
+#include "mysqlshdk/libs/db/mysql/session.h"
 #include "mysqlshdk/libs/mysql/group_replication.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
 namespace mysqlsh {
 namespace dba {
+namespace cluster {
 
 Dissolve::Dissolve(const bool interactive,
                    mysqlshdk::utils::nullable<bool> force,
                    Cluster_impl *cluster)
     : m_interactive(interactive), m_force(force), m_cluster(cluster) {
   assert(cluster);
-  m_primary_index = SIZE_MAX;
 }
 
 Dissolve::~Dissolve() {}
@@ -50,23 +52,19 @@ void Dissolve::prompt_to_confirm_dissolve() const {
   // dissolving the right cluster.
   auto console = mysqlsh::current_console();
 
-  if (!m_cluster->get_metadata_storage()->is_cluster_empty(
-          m_cluster->get_id())) {
-    // Show cluster description.
-    console->println(
-        "The cluster still has the following registered ReplicaSets:");
-    shcore::Value res = m_cluster->describe();
+  // Show cluster description.
+  console->println("The cluster still has the following registered instances:");
+  shcore::Value res = m_cluster->describe();
 
-    // Pretty print description only if wrap_json is not json/raw.
-    bool use_pretty_print =
-        (current_shell_options()->get().wrap_json.compare("json/raw") != 0);
-    console->println(res.descr(use_pretty_print));
-  }
+  // Pretty print description only if wrap_json is not json/raw.
+  bool use_pretty_print =
+      (current_shell_options()->get().wrap_json.compare("json/raw") != 0);
+  console->println(res.descr(use_pretty_print));
 
   console->print_warning(
       "You are about to dissolve the whole cluster and lose the high "
       "availability features provided by it. This operation cannot be "
-      "reverted. All members will be removed from their ReplicaSet and "
+      "reverted. All members will be removed from the cluster and "
       "replication will be stopped, internal recovery user accounts and "
       "the cluster metadata will be dropped. User data will be maintained "
       "intact in all instances.");
@@ -89,35 +87,18 @@ bool Dissolve::prompt_to_force_dissolve() const {
   return result;
 }
 
-void Dissolve::ensure_instance_reachable(
-    const std::string &instance_address,
-    const Connection_options &cluster_cnx_opt) {
-  // Set login credentials to connect to instance.
-  // NOTE: It is assumed that the same login credentials can be used to connect
-  //       to all cluster instances.
-  Connection_options instance_cnx_opts =
-      shcore::get_connection_options(instance_address, false);
-  instance_cnx_opts.set_login_options_from(cluster_cnx_opt);
-
-  // Try to connect to instance and add it to list of available instances to
-  // remove from the cluster.
-  log_debug("Connecting to instance '%s'", instance_address.c_str());
-  std::shared_ptr<mysqlshdk::db::ISession> session;
+void Dissolve::ensure_instance_reachable(const std::string &instance_address) {
   try {
-    session = mysqlshdk::db::mysql::Session::create();
-    session->connect(instance_cnx_opts);
-    m_available_instances.emplace_back(new mysqlshdk::mysql::Instance(session));
-    log_debug("Successfully connected to instance");
+    m_available_instances.emplace_back(
+        m_cluster->get_session_to_cluster_instance(instance_address));
   } catch (const std::exception &err) {
-    log_debug("Failed to connect to instance: %s", err.what());
-
+    // instance not reachable
     auto console = mysqlsh::current_console();
-
     // Handle use of 'force' option.
     if (m_force.is_null() || *m_force == false) {
       console->print_error(
           "Unable to connect to instance '" + instance_address +
-          "'. Please, verify connection credentials and make sure the "
+          "'. Please verify connection credentials and make sure the "
           "instance is available.");
 
       // In interactive mode and 'force' option not used, ask user to
@@ -151,7 +132,11 @@ void Dissolve::ensure_transactions_sync() {
   // replication errors (until now).
   for (const auto &instance : m_available_instances) {
     try {
-      m_cluster->sync_transactions(*instance);
+      mysqlsh::current_console()->print_info(
+          "* Waiting for instance to synchronize with the primary...");
+      m_cluster->sync_transactions(
+          *instance, mysqlshdk::gr::k_gr_applier_channel,
+          current_shell_options()->get().dba_gtid_wait_timeout);
     } catch (const std::exception &err) {
       std::string instance_address =
           (*instance).get_connection_options().as_uri(
@@ -258,57 +243,53 @@ void Dissolve::prepare() {
 
   // Get cluster session to use the same authentication credentials for all
   // cluster instances.
-  std::shared_ptr<mysqlshdk::db::ISession> cluster_session =
-      m_cluster->get_group_session();
-  Connection_options cluster_cnx_opt =
-      cluster_session->get_connection_options();
+  std::shared_ptr<Instance> cluster_instance = m_cluster->get_cluster_server();
 
   // Determine the primary (if it exists), in order to be the last to be
   // removed from the cluster. Only for single primary mode.
   // NOTE: This is need to avoid a new primary election in the group and GR
   //       BUG#24818604.
 
-  std::string primary =
-      mysqlshdk::gr::get_group_primary_uuid(cluster_session, nullptr);
+  bool single_primary = false;
+  std::vector<mysqlshdk::gr::Member> members = mysqlshdk::gr::get_members(
+      *cluster_instance, &single_primary, nullptr, nullptr);
 
-  if (!primary.empty()) {
-    mysqlshdk::utils::nullable<mysqlshdk::innodbcluster::Instance_info> info =
-        m_cluster->get_metadata_storage()
-            ->get_new_metadata()
-            ->get_instance_info_by_uuid(primary);
-    if (info) m_primary_address = info->classic_endpoint;
-  }
+  auto get_member_state = [&members](const std::string &uuid) {
+    auto it = std::find_if(
+        members.begin(), members.end(),
+        [&](const mysqlshdk::gr::Member &m) { return m.uuid == uuid; });
+    if (it != members.end())
+      return std::make_pair(it->state,
+                            it->role == mysqlshdk::gr::Member_role::PRIMARY);
+    return std::make_pair(mysqlshdk::gr::Member_state::MISSING, false);
+  };
 
   // Get all cluster instances, including state information.
-  std::vector<Instance_definition> instance_defs =
-      m_cluster->get_metadata_storage()->get_replicaset_instances(
-          m_cluster->get_default_replicaset()->get_id(), true);
+  std::vector<Instance_metadata> instance_defs = m_cluster->get_instances();
 
   // Check if instances can be dissolved. More specifically, verify if their
   // state is valid, if they are reachable (i.e., able to connect to them).
   // Also determine if instance removal can be skipped, according to 'force'
   // option and user prompts.
   for (const auto &instance_def : instance_defs) {
-    // Use the GR state hold by instance_def.state (but convert it to a proper
-    // mysqlshdk::gr::Member_state to be handled properly).
-    mysqlshdk::gr::Member_state state =
-        mysqlshdk::gr::to_member_state(instance_def.state);
+    mysqlshdk::gr::Member_state state;
+    bool is_primary;
+    std::tie(state, is_primary) = get_member_state(instance_def.uuid);
+
     if (state == mysqlshdk::gr::Member_state::ONLINE ||
         state == mysqlshdk::gr::Member_state::RECOVERING) {
       // Verify if active instances are reachable.
-      ensure_instance_reachable(instance_def.endpoint, cluster_cnx_opt);
-
-      // If the instance is the primary then set the index in the list of
-      // available instance (to remove it last from the cluster).
-      if (!m_primary_address.empty() &&
-          m_primary_address == instance_def.endpoint) {
-        m_primary_index = m_available_instances.size() - 1;
-      }
+      ensure_instance_reachable(instance_def.endpoint);
     } else {
       // Handle not active instances, determining if they can be skipped.
       handle_unavailable_instances(instance_def.endpoint,
                                    mysqlshdk::gr::to_string(state));
     }
+
+    // check if primary
+    if (single_primary && is_primary) {
+      m_primary_uuid = instance_def.uuid;
+    }
   }
 
   // Verify if there is any replication error that might make the operation
@@ -327,8 +308,8 @@ void Dissolve::remove_instance(const std::string &instance_address,
                                const std::size_t instance_index) {
   try {
     // Stop Group Replication and reset (persist) GR variables.
-    mysqlsh::dba::leave_replicaset(*m_available_instances[instance_index]);
-
+    mysqlsh::dba::leave_cluster(*m_available_instances[instance_index],
+                                m_cluster->is_cluster_set_member());
   } catch (const std::exception &err) {
     auto console = mysqlsh::current_console();
 
@@ -339,7 +320,7 @@ void Dissolve::remove_instance(const std::string &instance_address,
       throw shcore::Exception::runtime_error(err.what());
     } else {
       m_skipped_instances.push_back(instance_address);
-      log_error("Instance '%s' failed to leave the ReplicaSet: %s",
+      log_error("Instance '%s' failed to leave the cluster: %s",
                 instance_address.c_str(), err.what());
       console->print_warning(
           "An error occurred when trying to remove instance '" +
@@ -356,8 +337,19 @@ shcore::Value Dissolve::execute() {
   std::shared_ptr<MetadataStorage> metadata = m_cluster->get_metadata_storage();
   auto console = mysqlsh::current_console();
 
+  // Disable super_read_only mode if it is enabled.
+  bool super_read_only = m_cluster->get_cluster_server()
+                             ->get_sysvar_bool("super_read_only")
+                             .get_safe();
+  if (super_read_only) {
+    log_info(
+        "Disabling super_read_only mode on instance '%s' to run dissolve().",
+        m_cluster->get_cluster_server()->descr().c_str());
+    m_cluster->get_cluster_server()->set_sysvar("super_read_only", false);
+  }
+
   // JOB: Remove replication accounts used for recovery of GR.
-  // Note: This operation MUST be performed before leave-replicaset to ensure
+  // Note: This operation MUST be performed before leave-cluster to ensure
   // that all changed are propagated to the online instances.
   for (auto &instance : m_available_instances) {
     m_cluster->drop_replication_user(instance.get());
@@ -367,24 +359,33 @@ shcore::Value Dissolve::execute() {
   std::string cluster_name = m_cluster->get_name();
   metadata->drop_cluster(cluster_name);
 
+  size_t primary_index = SIZE_MAX;
+
   // We must stop GR on the online instances only, otherwise we'll
   // get connection failures to the (MISSING) instances
   // BUG#26001653.
   for (std::size_t i = 0; i < m_available_instances.size(); i++) {
+    auto &instance = *m_available_instances[i];
+
     // Remove all instances except the primary.
-    if (i != m_primary_index) {
-      std::string instance_address =
-          m_available_instances[i]->get_connection_options().as_uri(
-              mysqlshdk::db::uri::formats::only_transport());
+    if (instance.get_uuid() != m_primary_uuid) {
+      std::string instance_address = instance.get_connection_options().as_uri(
+          mysqlshdk::db::uri::formats::only_transport());
 
       // JOB: Sync transactions with the cluster.
       try {
         // Catch-up with all cluster transaction to ensure cluster metadata is
         // removed on the instance.
-        m_cluster->sync_transactions(*m_available_instances[i]);
+        console->print_info(
+            "* Waiting for instance to synchronize with the primary...");
 
-        // Remove instance from list of instance with sync error in case it was
-        // previously added during initial verification, since it succeeded now.
+        m_cluster->sync_transactions(
+            instance, mysqlshdk::gr::k_gr_applier_channel,
+            current_shell_options()->get().dba_gtid_wait_timeout);
+
+        // Remove instance from list of instance with sync error in case it
+        // was previously added during initial verification, since it
+        // succeeded now.
         m_sync_error_instances.erase(
             std::remove(m_sync_error_instances.begin(),
                         m_sync_error_instances.end(), instance_address),
@@ -419,19 +420,21 @@ shcore::Value Dissolve::execute() {
         }
       }
 
-      // JOB: Remove the instance from the replicaset (Stop GR)
+      // JOB: Remove the instance from the cluster (Stop GR)
       remove_instance(instance_address, i);
+    } else {
+      primary_index = i;
     }
   }
 
   // Remove primary instance at the end (if there is one).
-  if (m_primary_index != SIZE_MAX) {
+  if (primary_index != SIZE_MAX) {
     // No need to catch with transactions, since it is the primary.
-    // JOB: Remove the instance from the replicaset (Stop GR)
+    // JOB: Remove the instance from the cluster (Stop GR)
     std::string instance_address =
-        m_available_instances[m_primary_index]->get_connection_options().as_uri(
+        m_available_instances[primary_index]->get_connection_options().as_uri(
             mysqlshdk::db::uri::formats::only_transport());
-    remove_instance(instance_address, m_primary_index);
+    remove_instance(instance_address, primary_index);
   }
 
   // Disconnect all internal sessions
@@ -508,9 +511,8 @@ void Dissolve::finish() {
   m_available_instances.clear();
   m_skipped_instances.clear();
   m_sync_error_instances.clear();
-  m_primary_address.clear();
-  m_primary_index = SIZE_MAX;
 }
 
+}  // namespace cluster
 }  // namespace dba
 }  // namespace mysqlsh
