@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -28,6 +28,7 @@
 #include <string>
 #include <vector>
 
+#include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/metadata_storage.h"
 #include "modules/adminapi/common/provision.h"
 #include "modules/adminapi/common/sql.h"
@@ -37,6 +38,7 @@
 #include "mysqlshdk/libs/mysql/replication.h"
 #include "mysqlshdk/libs/textui/textui.h"
 #include "mysqlshdk/libs/utils/utils_net.h"
+#include "mysqlshdk/libs/utils/version.h"
 
 // Naming convention for validations:
 //
@@ -67,11 +69,11 @@ namespace checks {
  * - GR compatible storage engines only (InnoDB and MEMORY)
  * - all tables must have a PK or a UNIQUE NOT NULL key
  *
- * @param  session session for the schema. Must be authenticated with an account
- *          with SELECT access to all schemas.
- * @return         true if no issues found.
+ * @param  instance target instance for the validation. Must be authenticated
+ *         with an account with SELECT access to all schemas.
+ * @return true if no issues found.
  */
-bool validate_schemas(std::shared_ptr<mysqlshdk::db::ISession> session) {
+bool validate_schemas(const mysqlshdk::mysql::IInstance &instance) {
   bool ok = true;
   std::string k_gr_compliance_skip_schemas =
       "('mysql', 'sys', 'performance_schema', 'information_schema')";
@@ -80,7 +82,7 @@ bool validate_schemas(std::shared_ptr<mysqlshdk::db::ISession> session) {
   auto console = mysqlsh::current_console();
 
   {
-    std::shared_ptr<mysqlshdk::db::IResult> result = session->query(
+    std::shared_ptr<mysqlshdk::db::IResult> result = instance.query(
         "SELECT table_schema, table_name, engine "
         " FROM information_schema.tables "
         " WHERE engine NOT IN " +
@@ -89,7 +91,7 @@ bool validate_schemas(std::shared_ptr<mysqlshdk::db::ISession> session) {
 
     auto row = result->fetch_one();
     if (row) {
-      console->print_warning(
+      console->print_error(
           "The following tables use a storage engine that are not supported by "
           "Group Replication:");
       ok = false;
@@ -106,7 +108,7 @@ bool validate_schemas(std::shared_ptr<mysqlshdk::db::ISession> session) {
     }
   }
   {
-    std::shared_ptr<mysqlshdk::db::IResult> result = session->query(
+    std::shared_ptr<mysqlshdk::db::IResult> result = instance.query(
         "SELECT t.table_schema, t.table_name "
         "FROM information_schema.tables t "
         "    LEFT JOIN (SELECT table_schema, table_name "
@@ -125,7 +127,7 @@ bool validate_schemas(std::shared_ptr<mysqlshdk::db::ISession> session) {
 
     auto row = result->fetch_one();
     if (row) {
-      console->print_warning(
+      console->print_error(
           "The following tables do not have a Primary Key or equivalent "
           "column: ");
       ok = false;
@@ -152,6 +154,11 @@ bool validate_schemas(std::shared_ptr<mysqlshdk::db::ISession> session) {
         "DELETE) to these tables, ensure they use the InnoDB "
         "storage engine and have a PRIMARY KEY or PRIMARY KEY "
         "Equivalent.");
+    console->print_info(
+        "If you can't change the tables structure to include an extra visible "
+        "key to be used as PRIMARY KEY, you can make use of the INVISIBLE "
+        "COLUMN feature available since 8.0.23: "
+        "https://dev.mysql.com/doc/refman/8.0/en/invisible-columns.html");
   }
   return ok;
 }
@@ -195,10 +202,13 @@ void validate_innodb_page_size(mysqlshdk::mysql::IInstance *instance) {
  * The host address will be printed to the console along with an informational
  * message.
  *
- * If the target host name resolves to a loopback address (127.*) and error
+ * If the target host name resolves to a loopback address (127.*) an error
  * will be printed and the function returns false. Unless the target is a
  * sandbox, detected by checking that the name of the datadir is sandboxdata.
  *
+ * If an IPv6 address is used and the instance version is < 8.0.14
+ * throw an exception.
+ *
  * @param  instance target instance with cached global sysvars
  * @param  verbose  if 1 additional informational messages are shown, if 2
  * further explanations are provided
@@ -211,15 +221,12 @@ void validate_host_address(const mysqlshdk::mysql::IInstance &instance,
   bool report_host_set;
   std::string hostname;
   std::string report_host;
-  int port;
 
-  auto result = instance.query(
-      "SELECT @@hostname, @@report_host, COALESCE(@@report_port, @@port)");
+  auto result = instance.query("SELECT @@hostname, @@report_host");
   auto row = result->fetch_one_or_throw();
   hostname = row->get_string(0);
   report_host = row->get_string(1, "");
   report_host_set = !row->is_null(1);
-  port = row->get_int(2);
 
   if (report_host_set) {
     log_debug("Target has report_host=%s", report_host.c_str());
@@ -232,7 +239,7 @@ void validate_host_address(const mysqlshdk::mysql::IInstance &instance,
 
   if (report_host_set && report_host.empty()) {
     console->print_error("Invalid 'report_host' value for instance '" +
-                         instance.descr() +
+                         instance.get_connection_options().uri_endpoint() +
                          "'. The value cannot be empty if defined.");
     // NOTE: The value for report_host can be set to an empty string which is
     // invalid. If defined the report_host value should not be an empty
@@ -245,7 +252,7 @@ void validate_host_address(const mysqlshdk::mysql::IInstance &instance,
     console->println();
     console->print_info(
         "This instance reports its own address as " +
-        mysqlshdk::textui::bold(hostname + ":" + std::to_string(port)));
+        mysqlshdk::textui::bold(instance.get_canonical_address()));
     if (!report_host_set && verbose > 1) {
       console->print_info(
           "Clients and other cluster members will communicate with it through "
@@ -256,21 +263,51 @@ void validate_host_address(const mysqlshdk::mysql::IInstance &instance,
   }
 
   // Validate the IP of the hostname used for GR.
-  // IP address '127.0.1.1' is not supported by GCS leading to errors.
-  // NOTE: This IP is set by default in Debian platforms.
-  std::string seed_ip = mysqlshdk::utils::Net::resolve_hostname_ipv4(hostname);
-  if (seed_ip == "127.0.1.1") {
-    console->print_error(
-        "Cannot use host '" + hostname + "' for instance '" + instance.descr() +
-        "' because it resolves to an IP address (127.0.1.1) that does not "
-        "match a real network interface, thus it is not supported by the Group "
-        "Replication communication layer. Change your system settings and/or "
-        "set the MySQL server 'report_host' variable to a hostname that "
-        "resolves to a supported IP address.");
+  try {
+    std::string seed_ip =
+        mysqlshdk::utils::Net::resolve_hostname_ipv4(hostname);
+    const bool is_loopback = mysqlshdk::utils::Net::is_loopback(hostname);
+    if (is_loopback && seed_ip != "127.0.0.1") {
+      // Loopback IPv4 addresses except for 127.0.0.1 are not supported by GCS
+      // leading to errors.
+      console->print_error(shcore::str_format(
+          "Cannot use host '%s' for instance '%s' because it resolves to an IP "
+          "address (%s) that does not "
+          "match a real network interface, thus it is not supported. Change "
+          "your system settings and/or "
+          "set the MySQL server 'report_host' variable to a hostname that "
+          "resolves to a supported IP address.",
+          hostname.c_str(), instance.descr().c_str(), seed_ip.c_str()));
 
-    throw std::runtime_error("Invalid host/IP '" + hostname +
-                             "' resolves to '127.0.1.1' which is not "
-                             "supported by Group Replication.");
+      throw std::runtime_error(shcore::str_format(
+          "Invalid host/IP '%s' resolves to '%s' which is not "
+          "supported.",
+          hostname.c_str(), seed_ip.c_str()));
+    }
+  } catch (const mysqlshdk::utils::net_error &error) {
+    // if it is an IPv6 address and the instance version does not support IPv6
+    // throw an error
+    // Do not try to resolve hostnames. Even if we can resolve them, there is
+    // no guarantee that name resolution is the same across cluster instances
+    // and the instance where we are running the ngshell
+    bool is_ipv6 = mysqlshdk::utils::Net::is_ipv6(hostname);
+    bool supports_ipv6 =
+        instance.get_version() >= mysqlshdk::utils::Version(8, 0, 14);
+    if (is_ipv6 && !supports_ipv6) {
+      // Using an IPv6 address and the instance does not support it.
+      console->print_error(shcore::str_format(
+          "Cannot use host '%s' for instance '%s' "
+          "because it is an IPv6 address which is only supported by "
+          "Group Replication from MySQL version >= 8.0.14. Set the MySQL "
+          "server 'report_host' "
+          "variable to an IPv4 address or hostname that resolves an IPv4 "
+          "address.",
+          hostname.c_str(), instance.descr().c_str()));
+      throw std::runtime_error(shcore::str_format(
+          "Unsupported IP address '%s'. IPv6 is only "
+          "supported by Group Replication on MySQL version >= 8.0.14.",
+          hostname.c_str()));
+    }
   }
 }
 
@@ -285,6 +322,7 @@ void validate_host_address(const mysqlshdk::mysql::IInstance &instance,
  * @param  instance             target instance
  * @param  mycnf_path           optional config file path, for local instances
  * @param  config               pointer to config handler
+ * @param  cluster_type         target cluster/replicaset type
  * @param  can_persist          true if instance has support to persist
  *                              variables, false if has but it is disabled and
  *                              null if it is not supported.
@@ -294,9 +332,9 @@ void validate_host_address(const mysqlshdk::mysql::IInstance &instance,
  * @param  ret_val[out]         assigned to check results
  * @return                      [description]
  */
-std::vector<mysqlshdk::gr::Invalid_config> validate_configuration(
+std::vector<mysqlshdk::mysql::Invalid_config> validate_configuration(
     mysqlshdk::mysql::IInstance *instance, const std::string &mycnf_path,
-    mysqlshdk::config::Config *const config,
+    mysqlshdk::config::Config *const config, Cluster_type cluster_type,
     const mysqlshdk::utils::nullable<bool> &can_persist, bool *restart_needed,
     bool *mycnf_change_needed, bool *sysvar_change_needed,
     shcore::Value *ret_val) {
@@ -310,8 +348,8 @@ std::vector<mysqlshdk::gr::Invalid_config> validate_configuration(
   log_info("Validating configuration of %s (mycnf = %s)",
            instance->descr().c_str(), mycnf_path.c_str());
   // Perform check with no update
-  std::vector<mysqlshdk::gr::Invalid_config> invalid_cfs_vec =
-      check_instance_config(*instance, *config);
+  std::vector<mysqlshdk::mysql::Invalid_config> invalid_cfs_vec =
+      check_instance_config(*instance, *config, cluster_type);
 
   // Sort invalid cfs_vec by the name of the variable
   std::sort(invalid_cfs_vec.begin(), invalid_cfs_vec.end());
@@ -330,16 +368,16 @@ std::vector<mysqlshdk::gr::Invalid_config> validate_configuration(
     (*map_val)["status"] = shcore::Value("error");
     shcore::Value::Array_type_ref config_errors(
         new shcore::Value::Array_type());
-    for (mysqlshdk::gr::Invalid_config &cfg : invalid_cfs_vec) {
+    for (mysqlshdk::mysql::Invalid_config &cfg : invalid_cfs_vec) {
       shcore::Value::Map_type_ref error(new shcore::Value::Map_type());
       std::string action;
       if (!log_bin_wrong && cfg.var_name == "log_bin" &&
-          cfg.types.is_set(mysqlshdk::gr::Config_type::CONFIG)) {
+          cfg.types.is_set(mysqlshdk::mysql::Config_type::CONFIG)) {
         log_bin_wrong = true;
       }
 
-      if (cfg.types.is_set(mysqlshdk::gr::Config_type::CONFIG) &&
-          cfg.types.is_set(mysqlshdk::gr::Config_type::SERVER)) {
+      if (cfg.types.is_set(mysqlshdk::mysql::Config_type::CONFIG) &&
+          cfg.types.is_set(mysqlshdk::mysql::Config_type::SERVER)) {
         *mycnf_change_needed = true;
         if (cfg.restart) {
           *restart_needed = true;
@@ -348,18 +386,19 @@ std::vector<mysqlshdk::gr::Invalid_config> validate_configuration(
           action = "server_update+config_update";
           *sysvar_change_needed = true;
         }
-      } else if (cfg.types.is_set(mysqlshdk::gr::Config_type::CONFIG)) {
+      } else if (cfg.types.is_set(mysqlshdk::mysql::Config_type::CONFIG)) {
         *mycnf_change_needed = true;
         action = "config_update";
-      } else if (cfg.types.is_set(mysqlshdk::gr::Config_type::SERVER)) {
+      } else if (cfg.types.is_set(mysqlshdk::mysql::Config_type::SERVER)) {
+        *sysvar_change_needed = true;
         if (cfg.restart) {
           *restart_needed = true;
           action = "server_update+restart";
         } else {
           action = "server_update";
-          *sysvar_change_needed = true;
         }
-      } else if (cfg.types.is_set(mysqlshdk::gr::Config_type::RESTART_ONLY)) {
+      } else if (cfg.types.is_set(
+                     mysqlshdk::mysql::Config_type::RESTART_ONLY)) {
         *restart_needed = true;
         action = "restart";
       } else if (cfg.types.empty()) {
@@ -422,7 +461,6 @@ void validate_performance_schema_enabled(
       "performance_schema", mysqlshdk::mysql::Var_qualifier::GLOBAL);
 
   auto console = mysqlsh::current_console();
-
   if (ps_enabled.is_null() || !*ps_enabled) {
     console->print_error("Instance '" + instance.descr() +
                          "' has the performance_schema "
@@ -434,41 +472,59 @@ void validate_performance_schema_enabled(
   }
 }
 
-void ensure_instance_not_belong_to_cluster(
+TargetType::Type ensure_instance_not_belong_to_cluster(
     const mysqlshdk::mysql::IInstance &instance,
-    const std::shared_ptr<mysqlshdk::db::ISession> &cluster_session) {
-  GRInstanceType type =
-      mysqlsh::dba::get_gr_instance_type(instance.get_session());
+    const std::shared_ptr<Instance> &cluster_instance,
+    bool *out_already_member) {
+  TargetType::Type type = mysqlsh::dba::get_gr_instance_type(instance);
 
-  if (type != GRInstanceType::Standalone &&
-      type != GRInstanceType::StandaloneWithMetadata &&
-      type != GRInstanceType::StandaloneInMetadata) {
+  if (out_already_member) *out_already_member = false;
+
+  if (type != TargetType::Standalone &&
+      type != TargetType::StandaloneWithMetadata &&
+      type != TargetType::StandaloneInMetadata) {
     // Retrieves the new instance UUID
     std::string uuid = instance.get_uuid();
 
     // Verifies if this UUID is part of the current replication group
-    std::vector<mysqlshdk::gr::Member> members(mysqlshdk::gr::get_members(
-        mysqlshdk::mysql::Instance(cluster_session)));
+    std::vector<mysqlshdk::gr::Member> members(
+        mysqlshdk::gr::get_members(*cluster_instance));
 
     if (std::find_if(members.begin(), members.end(),
                      [&uuid](const mysqlshdk::gr::Member &member) {
                        return member.uuid == uuid;
                      }) != members.end()) {
-      if (type == GRInstanceType::InnoDBCluster) {
+      if (type == TargetType::InnoDBCluster ||
+          type == TargetType::InnoDBClusterSet) {
         log_debug("Instance '%s' already managed by InnoDB cluster",
                   instance.descr().c_str());
-        throw shcore::Exception::runtime_error(
-            "The instance '" + instance.descr() +
-            "' is already part of this InnoDB cluster");
+        if (out_already_member) {
+          // don't throw exception if out_already_member is given
+          *out_already_member = true;
+        } else {
+          throw shcore::Exception(
+              "The instance '" + instance.descr() +
+                  "' is already part of this InnoDB cluster",
+              SHERR_DBA_BADARG_INSTANCE_MANAGED_IN_CLUSTER);
+        }
       } else {
-        current_console()->print_error(
-            "Instance '" + instance.descr() +
-            "' is part of the Group Replication group but is not in the "
-            "metadata. Please use <Cluster>.rescan() to update the metadata.");
-        throw shcore::Exception::runtime_error("Metadata inconsistent");
+        if (out_already_member) {
+          *out_already_member = true;
+        } else {
+          current_console()->print_error(
+              "Instance '" + instance.descr() +
+              "' is part of the Group Replication group but is not in the "
+              "metadata. Please use <Cluster>.rescan() to update the "
+              "metadata.");
+          throw shcore::Exception(
+              "Metadata inconsistent: " + instance.descr() +
+                  " is a group member but is not in the metadata.",
+              SHERR_DBA_ASYNC_MEMBER_INCONSISTENT);
+        }
       }
     } else {
-      if (type == GRInstanceType::InnoDBCluster) {
+      if (type == TargetType::InnoDBCluster ||
+          type == TargetType::InnoDBClusterSet) {
         // Check if instance is running auto-rejoin and warn user.
         if (mysqlshdk::gr::is_running_gr_auto_rejoin(instance)) {
           throw shcore::Exception::runtime_error(
@@ -488,36 +544,147 @@ void ensure_instance_not_belong_to_cluster(
       }
     }
   }
+  return type;
 }
 
 void ensure_instance_not_belong_to_metadata(
     const mysqlshdk::mysql::IInstance &instance,
     const std::string &address_in_metadata,
-    const mysqlsh::dba::ReplicaSet &replicaset) {
+    const mysqlsh::dba::Cluster_impl &cluster) {
   auto console = mysqlsh::current_console();
 
-  // Check if the instance exists on the ReplicaSet
-  log_debug("Checking if the instance belongs to the replicaset");
-  bool is_instance_on_md =
-      replicaset.get_cluster()
-          ->get_metadata_storage()
-          ->is_instance_on_replicaset(replicaset.get_id(), address_in_metadata);
+  // Check if the instance exists on the cluster
+  log_debug("Checking if the instance belongs to the cluster");
+  Instance_metadata instance_md;
+  try {
+    instance_md = cluster.get_metadata_storage()->get_instance_by_address(
+        address_in_metadata);
+  } catch (const shcore::Exception &e) {
+    if (e.code() == SHERR_DBA_MEMBER_METADATA_MISSING) {
+      return;
+    }
+    throw;
+  }
 
-  if (is_instance_on_md) {
+  if (instance_md.cluster_id == cluster.get_id()) {
     // Check if instance is running auto-rejoin
     bool is_rejoining = mysqlshdk::gr::is_running_gr_auto_rejoin(instance);
 
     std::string err_msg = "The instance '" + instance.descr() +
-                          "' already belongs to the ReplicaSet: '" +
-                          replicaset.get_name() + "'";
-    if (is_rejoining)
+                          "' already belongs to the cluster: '" +
+                          cluster.get_name() + "'";
+    if (is_rejoining) {
       err_msg += " and is currently trying to auto-rejoin.";
-    else
+    } else {
       err_msg += ".";
+    }
     throw shcore::Exception::runtime_error(err_msg);
   }
 }
 
+size_t check_illegal_async_channels(
+    const mysqlshdk::mysql::IInstance &instance,
+    const std::unordered_set<std::string> &allowed_channels_) {
+  auto console = mysqlsh::current_console();
+  std::unordered_set<std::string> allowed_channels(allowed_channels_);
+
+  allowed_channels.insert("group_replication_applier");
+  allowed_channels.insert("group_replication_recovery");
+
+  auto channels = get_incoming_channels(instance);
+  size_t illegal_channels = channels.size();
+
+  for (const auto &ch : channels) {
+    if (std::find(allowed_channels.begin(), allowed_channels.end(),
+                  ch.channel_name) != allowed_channels.end())
+      illegal_channels--;
+    else
+      console->print_note("Found unexpected replication channel " +
+                          ch.channel_name + "at " + instance.descr());
+  }
+
+  return illegal_channels;
+}
+
+/**
+ * Validate if asynchronous replication is configured on the target instance.
+ *
+ * This function validates if replication is configured on the target
+ * instance and depending on the type of check, prints a warning message,
+ * or an error message and terminates with an exception.
+ *
+ * @param instance The instance to validate.
+ * @param allowed_channels List of channel names that can appear at the instance
+ * @param type     The type of check (Check_type)
+ */
+void validate_async_channels(
+    const mysqlshdk::mysql::IInstance &instance,
+    const std::unordered_set<std::string> &allowed_channels, Check_type type) {
+  log_debug(
+      "Checking if instance '%s' has asynchronous (source-replica) "
+      "replication configured.",
+      instance.descr().c_str());
+
+  auto console = mysqlsh::current_console();
+
+  size_t illegal_channels =
+      check_illegal_async_channels(instance, allowed_channels);
+
+  if (illegal_channels > 0) {
+    std::string error_msg;
+
+    switch (type) {
+      case Check_type::CHECK:
+        error_msg += "The";
+        break;
+      case Check_type::CREATE:
+        error_msg += "Cannot create cluster on";
+        break;
+      case Check_type::BOOTSTRAP:
+        error_msg += "Cannot bootstrap cluster on";
+        break;
+      case Check_type::JOIN:
+        error_msg += "Cannot join";
+        break;
+      case Check_type::REJOIN:
+        error_msg += "Cannot rejoin";
+        break;
+    }
+
+    error_msg +=
+        " instance '" + instance.descr() + "' " +
+        std::string((type == Check_type::JOIN || type == Check_type::REJOIN)
+                        ? "to the cluster "
+                        : "") +
+        std::string(type == Check_type::CHECK
+                        ? "cannot be added to an InnoDB cluster "
+                        : "") +
+        "because it has asynchronous (source-replica) "
+        "replication channel(s) configured. "
+        "MySQL InnoDB Cluster does not support manually configured channels "
+        "as they are not managed using the AdminAPI (e.g. when PRIMARY moves "
+        "to another member) which may cause cause replication to break or "
+        "even create split-brain scenarios (data loss).";
+
+    if (type == Check_type::CREATE) {
+      error_msg +=
+          " Use the 'force' option to skip this validation on a temporary "
+          "scenario (e.g. migrating from a replication topology to InnoDB "
+          "Cluster).";
+    }
+
+    if (type == Check_type::CHECK) {
+      console->print_warning(error_msg);
+    } else {
+      console->print_error(error_msg);
+
+      throw shcore::Exception::runtime_error(
+          "The instance '" + instance.descr() +
+          "' has asynchronous replication configured.");
+    }
+  }
+}
+
 }  // namespace checks
 }  // namespace dba
 }  // namespace mysqlsh
