@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -25,8 +25,10 @@
 #include <algorithm>
 #include <functional>
 #include <memory>
+#include <set>
 #include <utility>
 
+#include "mysqlshdk/include/scripting/type_info/custom.h"
 #include "mysqlshdk/include/shellcore/console.h"
 #include "mysqlshdk/include/shellcore/utils_help.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
@@ -34,46 +36,10 @@
 
 namespace mysqlsh {
 
-REGISTER_HELP_TOPIC(Plugins, TOPIC, SHELL_PLUGINS, Contents, SCRIPTING);
-REGISTER_HELP_TOPIC_TEXT(SHELL_PLUGINS, R"*(
-The MySQL Shell allows extending its base functionality through the creation of
-plugins.
-
-An plugin is a folder containing the code that provides the functionality to be
-made available on the MySQL Shell.
-
-User defined plugins should be located at plugins folder at the following paths:
-
-@li Windows: %AppData%\\MySQL\\mysqlsh\\plugins
-@li Others: ~/.mysqlsh/plugins
-
-A plugin must contain an init file which is the entry point to load the
-extension:
-
-@li init.js for plugins written in JavaScript.
-@li init.py for plugins written in Python.
-
-On startup, the shell traverses the folders inside of the *plugins* folder
-searching for the plugin init file. The init file will be loaded on the
-corresponding context (JavaScript or Python).
-
-<b>Use Cases</b>
-
-The main use cases for MySQL Shell plugins include:
-
-@li Definition of shell reports to be used with the \show and \watch Shell
-Commands.
-@li Definition of new Global Objects with user defined functionality.
-
-For additional information on shell reports execute: \? reports
-
-For additional information on extension objects execute: \? extension objects
-)*");
-
 REGISTER_HELP_TOPIC(Extension Objects, TOPIC, EXTENSION_OBJECTS, Contents,
                     SCRIPTING);
 REGISTER_HELP_TOPIC_TEXT(EXTENSION_OBJECTS, R"*(
-The MySQL Shell allows for an extension of its base functionality by using 
+The MySQL Shell allows for an extension of its base functionality by using
 special objects called <b>Extension Objects</b>.
 
 Once an extension object has been created it is possible to attach different
@@ -83,7 +49,7 @@ type of object members such as:
 @li Properties
 @li Other extension objects
 
-Once an extension object has been fully defined it can be registered as a 
+Once an extension object has been fully defined it can be registered as a
 regular <b>Global Object</b>. This way the object and its members will be
 available in the scripting languages the MySQL Shell supports, just as any other
 global object such as <b>shell</b>, <b>util</b>, etc.
@@ -115,8 +81,8 @@ You can get more details on these functions by executing:
 
 <b>Naming Convention</b>
 
-The core MySQL Shell APIs follow a specific naming convention for object 
-members. Extension objects should follow the same naming convention for 
+The core MySQL Shell APIs follow a specific naming convention for object
+members. Extension objects should follow the same naming convention for
 consistency reasons.
 
 @li JavaScript members use camelCaseNaming
@@ -126,10 +92,10 @@ To simplify this across languages, it is important to use camelCaseNaming when
 specifying the 'name' parameter of the shell.<<<addExtensionObjectMember>>>
 function.
 
-The MySQL Shell will then automatically handle the snake_case_naming for that 
+The MySQL Shell will then automatically handle the snake_case_naming for that
 member when it is switched to Python mode.
 
-NOTE: the naming convention is only applicable for extension object members. 
+NOTE: the naming convention is only applicable for extension object members.
 However, when a global object is registered, the name used to register that
 object will be exactly the same in both JavaScript and Python modes.
 
@@ -202,6 +168,21 @@ shcore::Cpp_function::Raw_signature to_raw_signature(
   return out;
 }
 
+const std::set<shcore::Value_type> kAllowedMemberTypes = {
+    shcore::Function, shcore::Array,   shcore::Map,      shcore::Null,
+    shcore::Bool,     shcore::Integer, shcore::UInteger, shcore::Float,
+    shcore::String,   shcore::Object};
+
+const std::set<std::string> kAllowedParamTypes = {
+    "string", "integer", "float", "bool", "object", "array", "dictionary"};
+
+const std::map<std::string, shcore::Value_type> kTypeMapping = {
+    {"string", shcore::String},  {"integer", shcore::Integer},
+    {"float", shcore::Float},    {"bool", shcore::Bool},
+    {"object", shcore::Object},  {"array", shcore::Array},
+    {"dictionary", shcore::Map}, {"function", shcore::Function},
+};
+
 }  // namespace
 
 Parameter_definition::Parameter_definition()
@@ -236,21 +217,6 @@ void Parameter_definition::validate_options() const {
   }
 }
 
-std::set<shcore::Value_type> Extensible_object::s_allowed_member_types = {
-    shcore::Function, shcore::Array,   shcore::Map,      shcore::Null,
-    shcore::Bool,     shcore::Integer, shcore::UInteger, shcore::Float,
-    shcore::String,   shcore::Object};
-
-std::set<std::string> Extensible_object::s_allowed_param_types = {
-    "string", "integer", "float", "bool", "object", "array", "dictionary"};
-
-std::map<std::string, shcore::Value_type> Extensible_object::s_type_mapping = {
-    {"string", shcore::String},  {"integer", shcore::Integer},
-    {"float", shcore::Float},    {"bool", shcore::Bool},
-    {"object", shcore::Object},  {"array", shcore::Array},
-    {"dictionary", shcore::Map}, {"function", shcore::Function},
-};
-
 Extensible_object::Extensible_object(const std::string &name,
                                      const std::string &qualified_name)
     : m_name(name),
@@ -324,13 +290,14 @@ shcore::Value Extensible_object::call(const std::string &name,
 }
 
 shcore::Value Extensible_object::call_advanced(
-    const std::string &name, const shcore::Argument_list &args) {
+    const std::string &name, const shcore::Argument_list &args,
+    const shcore::Dictionary_t &kwargs) {
   if (!is_registered()) {
     throw shcore::Exception::runtime_error(
         "Unable to call functions in an unregistered extension object.");
   }
 
-  return Cpp_object_bridge::call_advanced(name, args);
+  return Cpp_object_bridge::call_advanced(name, args, kwargs);
 }
 
 void Extensible_object::register_member(
@@ -348,7 +315,7 @@ void Extensible_object::register_member(
     target_object =
         shcore::str_format("the '%s' extension object", m_name.c_str());
 
-  if (s_allowed_member_types.find(value.type) == s_allowed_member_types.end()) {
+  if (kAllowedMemberTypes.find(value.type) == kAllowedMemberTypes.end()) {
     throw shcore::Exception::argument_error("Unsupported member type.");
   } else if (value.type == shcore::Object) {
     auto object = value.as_object<Extensible_object>();
@@ -402,7 +369,9 @@ void Extensible_object::set_registered(const std::string &name) {
 
     for (const auto &fdef : m_function_definition) register_function_help(fdef);
 
-    for (auto &child : m_children) child.second->set_registered();
+    for (auto &child : m_children) {
+      child.second->set_registered();
+    }
   }
 }
 
@@ -448,6 +417,8 @@ void Extensible_object::register_object(
 
   object->m_parent = shared_from_this();
 
+  if (object->cli_enabled()) enable_cli();
+
   m_children.emplace(object->m_definition->name, object);
 
   add_property(object->m_definition->name);
@@ -507,13 +478,14 @@ Extensible_object::parse_function_definition(
     unpacker.optional("parameters", &params);
     unpacker.optional("brief", &def->brief);
     unpacker.optional("details", &def->details);
+    unpacker.optional("cli", &def->cli_enabled);
     unpacker.end("at function definition");
   }
 
   // Builds the parameter list
   shcore::Parameter_context context{"", {{"parameter", {}}}};
   def->parameters =
-      parse_parameters(params, &context, s_allowed_param_types, true);
+      parse_parameters(params, &context, kAllowedParamTypes, true);
 
   return def;
 }
@@ -540,7 +512,10 @@ void Extensible_object::register_function(
 
   validate_function(definition);
 
-  expose(definition->name, function, to_raw_signature(definition->parameters));
+  expose(definition->name, function, to_raw_signature(definition->parameters))
+      ->cli(definition->cli_enabled);
+
+  if (definition->cli_enabled) enable_cli();
 
   if (is_registered())
     register_function_help(definition);
@@ -549,9 +524,9 @@ void Extensible_object::register_function(
 }
 
 shcore::Value_type Extensible_object::map_type(
-    const std::string &type, const std::set<std::string> &allowed_types) {
+    const std::string &type, const std::set<std::string> &allowed_types) const {
   if (allowed_types.find(type) != allowed_types.end())
-    return s_type_mapping[type];
+    return kTypeMapping.at(type);
   else
     return shcore::Value_type::Undefined;
 }
@@ -624,7 +599,7 @@ Extensible_object::start_parsing_parameter(const shcore::Dictionary_t &,
 std::shared_ptr<Parameter_definition> Extensible_object::parse_parameter(
     const shcore::Dictionary_t &definition, shcore::Parameter_context *context,
     const std::set<std::string> &allowed_types, bool as_parameters) {
-  std::string type;
+  mysqlshdk::utils::nullable<std::string> type;
 
   // By default, parameters are required, unless required=false is specified
   // By default, options are not required, unless required=true is specified
@@ -635,12 +610,18 @@ std::shared_ptr<Parameter_definition> Extensible_object::parse_parameter(
   const auto &param = param_definition->parameter;
 
   unpacker.required("name", &param->name);
-  unpacker.required("type", &type);
+  unpacker.optional("type", &type);
   unpacker.optional("required", &required);
+  unpacker.optional("default", &param->def_value);
   unpacker.optional("brief", &param_definition->brief);
   unpacker.optional("details", &param_definition->details);
 
-  param->set_type(map_type(type, allowed_types));
+  if (!type.is_null()) {
+    param->set_type(map_type(*type, allowed_types));
+  } else {
+    param->set_type(shcore::Value_type::Undefined);
+  }
+  param->cmd_line_enabled = true;
 
   if (param->type() == shcore::Value_type::String) {
     std::vector<std::string> allowed;
@@ -649,6 +630,13 @@ std::shared_ptr<Parameter_definition> Extensible_object::parse_parameter(
         std::move(allowed));
   }
 
+  if (param->type() == shcore::Value_type::Array) {
+    std::string itemtype;
+    unpacker.optional("itemtype", &itemtype);
+    param->validator<shcore::List_validator>()->set_element_type(
+        map_type(itemtype, kAllowedParamTypes));
+  }
+
   if (param->type() == shcore::Value_type::Object) {
     std::string error;
     mysqlshdk::utils::nullable<std::vector<std::string>> allowed_classes;
@@ -670,15 +658,16 @@ std::shared_ptr<Parameter_definition> Extensible_object::parse_parameter(
         error = "An empty array is not valid for the classes option.";
       } else {
         callowed = *allowed_classes;
-        auto help = shcore::Help_registry::get();
-        std::set<std::string> classes;
-        std::vector<shcore::Topic_type> allowed_types = {
-            shcore::Topic_type::CLASS, shcore::Topic_type::GLOBAL_OBJECT,
-            shcore::Topic_type::OBJECT};
 
-        for (const auto type : allowed_types) {
-          auto topics = help->get_help_topics(type);
-          for (auto topic : topics) {
+        std::set<std::string> classes;
+
+        for (const auto allowed_type :
+             {shcore::Topic_type::CLASS, shcore::Topic_type::GLOBAL_OBJECT,
+              shcore::Topic_type::OBJECT}) {
+          auto topics =
+              shcore::Help_registry::get()->get_help_topics(allowed_type);
+
+          for (const auto &topic : topics) {
             classes.insert(topic->get_base_name());
           }
         }
@@ -715,14 +704,14 @@ std::shared_ptr<Parameter_definition> Extensible_object::parse_parameter(
 
   shcore::Array_t options;
   if (param->type() == shcore::Value_type::Map)
-    unpacker.required("options", &options);
+    unpacker.optional("options", &options);
 
   std::string current_ctx;
   auto ctx_name_backup = context->levels.back().name;
   auto ctx_position_backup = context->levels.back().position;
 
   if (shcore::is_valid_identifier(param->name)) {
-    context->levels.back().name = (type.empty() ? "" : type + " ") +
+    context->levels.back().name = (type.get_safe().empty() ? "" : *type + " ") +
                                   ctx_name_backup + " '" + param->name + "'";
     context->levels.back().position = nullptr;
   }
@@ -732,7 +721,8 @@ std::shared_ptr<Parameter_definition> Extensible_object::parse_parameter(
 
   context->levels.back().name = ctx_name_backup + " '" + param->name + "'";
 
-  if (param->type() == shcore::Value_type::Undefined) {
+  // If type was specified, it can't be Undefined
+  if (!type.is_null() && param->type() == shcore::Value_type::Undefined) {
     const auto error =
         "Unsupported type used at " + context->str() +
         ". Allowed types: " + shcore::str_join(allowed_types, ", ");
@@ -790,15 +780,20 @@ void Extensible_object::register_help(const std::string &brief,
       // Creates the help topic for the object
       auto type = is_global ? shcore::Topic_type::GLOBAL_OBJECT
                             : shcore::Topic_type::OBJECT;
-      help->add_help_topic(m_name, type, m_qualified_name, parent, mask);
+      help->add_help_topic(m_name, type, m_qualified_name, parent, mask,
+                           shcore::Keyword_location::LOCAL_CTX);
 
       // Now registers the object brief, parameters and details
       auto prefix = shcore::str_upper(m_qualified_name);
 
-      help->add_help(prefix, "BRIEF", brief);
-      help->add_help(prefix, "DETAIL", &m_detail_sequence, details);
+      help->add_help(prefix, "BRIEF", brief,
+                     shcore::Keyword_location::LOCAL_CTX);
+      help->add_help(prefix, "DETAIL", &m_detail_sequence, details,
+                     shcore::Keyword_location::LOCAL_CTX);
 
-      if (is_global) help->add_help(prefix, "GLOBAL_BRIEF", brief);
+      if (is_global)
+        help->add_help(prefix, "GLOBAL_BRIEF", brief,
+                       shcore::Keyword_location::LOCAL_CTX);
     }
   } else {
     m_definition.reset(new Member_definition("", brief, details));
@@ -814,26 +809,30 @@ void Extensible_object::register_property_help(
 
   // Creates the help topic for the object
   help->add_help_topic(def->name, shcore::Topic_type::PROPERTY, def->name,
-                       m_qualified_name, mask);
+                       m_qualified_name, mask,
+                       shcore::Keyword_location::LOCAL_CTX);
 
   // Now registers the object brief, parameters and details
   auto object = shcore::str_upper(m_name);
   auto prefix = object + "_" + shcore::str_upper(def->name);
   if (!def->brief.empty()) {
-    help->add_help(prefix, "BRIEF", def->brief);
+    help->add_help(prefix, "BRIEF", def->brief,
+                   shcore::Keyword_location::LOCAL_CTX);
   }
 
   size_t member_sequence = 0;
 
   if (!def->details.empty()) {
-    help->add_help(prefix, "DETAIL", &member_sequence, def->details);
+    help->add_help(prefix, "DETAIL", &member_sequence, def->details,
+                   shcore::Keyword_location::LOCAL_CTX);
   }
 }
 
 void Extensible_object::register_function_help(
     const std::string &name, const std::string &brief,
     const std::vector<std::string> &params,
-    const std::vector<std::string> &details) {
+    const std::vector<std::string> &details,
+    const Function_definition::Examples &examples) {
   auto help = shcore::Help_registry::get();
 
   auto names = shcore::str_split(name, "|");
@@ -847,13 +846,17 @@ void Extensible_object::register_function_help(
 
     // Creates the help topic for the function
     help->add_help_topic(name, shcore::Topic_type::FUNCTION, names[0],
-                         m_qualified_name, mask);
+                         m_qualified_name, mask,
+                         shcore::Keyword_location::LOCAL_CTX);
 
     // Now registers the function brief, parameters and details
     auto prefix = shcore::str_upper(m_name + "_" + names[0]);
     if (!brief.empty()) help->add_help(prefix, "BRIEF", brief);
-    help->add_help(prefix, "PARAM", params);
-    help->add_help(prefix, "DETAIL", details);
+    help->add_help(prefix, "PARAM", params,
+                   shcore::Keyword_location::LOCAL_CTX);
+    help->add_help(prefix, "DETAIL", details,
+                   shcore::Keyword_location::LOCAL_CTX);
+    help->add_help(prefix, examples, shcore::Keyword_location::LOCAL_CTX);
   } else {
     topic->set_enabled(true);
   }
@@ -891,7 +894,8 @@ void Extensible_object::register_function_help(
     }
   }
 
-  register_function_help(definition->name, definition->brief, params, details);
+  register_function_help(definition->name, definition->brief, params, details,
+                         definition->examples);
 }
 
 void Extensible_object::get_param_help_brief(
@@ -902,15 +906,25 @@ void Extensible_object::get_param_help_brief(
 
   param_help += param->name;
 
-  if (as_parameter && !param_definition.is_required())
-    param_help += " Optional";
-  else if (!as_parameter && param_definition.is_required())
+  if (!as_parameter) {
+    param_help += ":";
+  } else {
+    if (!param_definition.is_required()) param_help += " Optional";
+  }
+
+  size_t help_size = param_help.size();
+  if (param->type() != shcore::Value_type::Undefined)
+    param_help += " " + to_string(param->type());
+
+  if (!as_parameter && param_definition.is_required())
     param_help += " (required)";
 
-  param_help += " " + to_string(param->type()) + ".";
-
-  if (!param_definition.brief.empty())
+  if (!param_definition.brief.empty()) {
+    if (param_help.size() > help_size) param_help.append(" -");
     param_help.append(" " + param_definition.brief);
+  }
+
+  if (!shcore::str_endswith(param_help, ".")) param_help.append(".");
 
   target->emplace_back(std::move(param_help));
 }
@@ -947,8 +961,9 @@ void Extensible_object::get_param_help_detail(
           param->validator<shcore::String_validator>()->allowed();
 
       if (!values.empty()) {
-        help_entry += " accepts the following values: " +
-                      shcore::str_join(values, ", ") + ".";
+        help_entry +=
+            " accepts the following values: " + shcore::str_join(values, ", ") +
+            ".";
         details->emplace_back(std::move(help_entry));
       }
 
@@ -956,17 +971,20 @@ void Extensible_object::get_param_help_detail(
     }
 
     case shcore::Value_type::Map: {
-      help_entry += " accepts the following options:";
-      details->emplace_back(std::move(help_entry));
-
       const auto &options = param_definition.options();
+      if (!options.empty()) {
+        help_entry += " accepts the following options:";
+      } else {
+        help_entry += " accepts any key/value pair.";
+      }
+
+      details->emplace_back(std::move(help_entry));
 
       for (const auto &option : options)
         get_param_help_brief(*option, false, details);
 
       for (const auto &option : options)
         get_param_help_detail(*option, false, details);
-
       break;
     }
 
@@ -1031,21 +1049,11 @@ void Extensible_object::validate_parameter(const shcore::Parameter &param,
     throw shcore::Exception::argument_error(error);
   }
 
-  if (param.type() == shcore::Value_type::Undefined) {
-    const auto error =
-        shcore::str_format("Unsupported data type at %s.", current_ctx.c_str());
-    throw shcore::Exception::argument_error(error);
-  }
-
   if (param.type() == shcore::Value_type::Map) {
     const auto &options =
         param.validator<shcore::Option_validator>()->allowed();
     // Options must be defined
-    if (options.empty()) {
-      auto error = shcore::str_format("Missing option definitions at %s.",
-                                      current_ctx.c_str());
-      throw shcore::Exception::argument_error(error);
-    } else {
+    if (!options.empty()) {
       int index = 0;
 
       for (const auto &option : options) {
@@ -1123,3 +1131,26 @@ std::string Extensible_object::get_help_id() const {
 }
 
 }  // namespace mysqlsh
+
+namespace shcore {
+namespace detail {
+
+std::shared_ptr<mysqlsh::Extensible_object>
+Type_info<std::shared_ptr<mysqlsh::Extensible_object>>::to_native(
+    const shcore::Value &in) {
+  std::shared_ptr<mysqlsh::Extensible_object> object;
+
+  if (in.type == shcore::Object)
+    object = in.as_object<mysqlsh::Extensible_object>();
+
+  if (!object) {
+    throw shcore::Exception::type_error(
+        "is expected to be " +
+        Type_info<std::shared_ptr<mysqlsh::Extensible_object>>::desc());
+  }
+
+  return object;
+}
+
+}  // namespace detail
+}  // namespace shcore
