@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -28,16 +28,16 @@
 #include <thread>
 #include <vector>
 
-#include "scripting/lang_base.h"
-#include "scripting/object_factory.h"
-#include "shellcore/shell_core.h"
-#include "shellcore/shell_notifications.h"
-
-#include "modules/mysqlxtest_utils.h"
-#include "scripting/proxy_object.h"
-
 #include "modules/mod_mysql_resultset.h"
 #include "modules/mod_utils.h"
+#include "modules/mysqlxtest_utils.h"
+#include "mysqlshdk/include/scripting/type_info/custom.h"
+#include "mysqlshdk/include/scripting/type_info/generic.h"
+#include "scripting/lang_base.h"
+#include "scripting/object_factory.h"
+#include "scripting/proxy_object.h"
+#include "shellcore/shell_core.h"
+#include "shellcore/shell_notifications.h"
 #include "shellcore/utils_help.h"
 #include "utils/utils_general.h"
 #include "utils/utils_path.h"
@@ -85,18 +85,15 @@ void ClassicSession::init() {
 
   add_property("uri", "getUri");
 
-  add_method("close", std::bind(&ClassicSession::_close, this, _1), "data");
-  add_method("runSql", std::bind(&ClassicSession::run_sql, this, _1), "stmt",
-             shcore::String);
-  add_method("query", std::bind(&ClassicSession::query, this, _1), "stmt",
-             shcore::String);
+  expose("close", &ClassicSession::close);
+  expose("runSql", &ClassicSession::run_sql, "query", "?args");
+  expose("query", &ClassicSession::query, "query", "?args");
+  expose("isOpen", &ClassicSession::is_open);
+  expose("startTransaction", &ClassicSession::_start_transaction);
+  expose("commit", &ClassicSession::_commit);
+  expose("rollback", &ClassicSession::_rollback);
 
-  add_method("isOpen", std::bind(&ClassicSession::_is_open, this, _1));
-  add_method("startTransaction",
-             std::bind(&ClassicSession::_start_transaction, this, _1), "data");
-  add_method("commit", std::bind(&ClassicSession::_commit, this, _1), "data");
-  add_method("rollback", std::bind(&ClassicSession::_rollback, this, _1),
-             "data");
+  expose("_getSocketFd", &ClassicSession::_get_socket_fd);
 }
 
 void ClassicSession::connect(
@@ -139,57 +136,6 @@ void ClassicSession::close() {
   _session.reset();
 }
 
-Value ClassicSession::_close(const shcore::Argument_list &args) {
-  args.ensure_count(0, get_function_name("close").c_str());
-
-  close();
-
-  return shcore::Value();
-}
-
-REGISTER_HELP_FUNCTION(isOpen, ClassicSession);
-REGISTER_HELP(CLASSICSESSION_ISOPEN_BRIEF,
-              "Returns true if session is "
-              "known to be open.");
-REGISTER_HELP(CLASSICSESSION_ISOPEN_RETURNS,
-              "@returns A boolean value "
-              "indicating if the session is still open.");
-REGISTER_HELP(
-    CLASSICSESSION_ISOPEN_DETAIL,
-    "Returns true if the session is "
-    "still open and false otherwise. Note: may return true if connection "
-    "is lost.");
-
-shcore::Value ClassicSession::_is_open(const shcore::Argument_list &args) {
-  args.ensure_count(0, get_function_name("isOpen").c_str());
-
-  return shcore::Value(is_open());
-}
-
-shcore::Value ClassicSession::_run_sql(const std::string &function,
-                                       const shcore::Argument_list &args) {
-  args.ensure_count(1, 2, get_function_name(function).c_str());
-  Value ret_val;
-
-  try {
-    auto query = args.string_at(0);
-    shcore::Array_t values;
-    if (args.size() > 1) values = args.array_at(1);
-
-    // Will return the result of the SQL execution
-    // In case of error will be Undefined
-    if (!_session || !_session->is_open()) {
-      throw Exception::logic_error("Not connected.");
-    } else {
-      Interruptible intr(this);
-      ret_val = execute_sql(query, values);
-    }
-  }
-  CATCH_AND_TRANSLATE_FUNCTION_EXCEPTION(get_function_name(function));
-
-  return ret_val;
-}
-
 REGISTER_HELP_FUNCTION(runSql, ClassicSession);
 REGISTER_HELP_FUNCTION_TEXT(CLASSICSESSION_RUNSQL, R"*(
 Executes a query and returns the corresponding ClassicResult object.
@@ -202,7 +148,6 @@ the query string.
 
 @throw LogicError if there's no open session.
 @throw ArgumentError if the parameters are invalid.
-
 )*");
 /**
  * $(CLASSICSESSION_RUNSQL_BRIEF)
@@ -214,71 +159,63 @@ ClassicResult ClassicSession::runSql(String query, Array args) {}
 #elif DOXYGEN_PY
 ClassicResult ClassicSession::run_sql(str query, list args) {}
 #endif
-Value ClassicSession::run_sql(const shcore::Argument_list &args) {
-  return _run_sql("runSql", args);
+std::shared_ptr<ClassicResult> ClassicSession::run_sql(
+    const std::string &query, const shcore::Array_t &args) {
+  std::shared_ptr<ClassicResult> ret_val;
+
+  // Will return the result of the SQL execution
+  // In case of error will be Undefined
+  if (!_session || !_session->is_open()) {
+    throw Exception::logic_error("Not connected.");
+  } else {
+    Interruptible intr(this);
+
+    auto cresult = std::dynamic_pointer_cast<mysqlshdk::db::mysql::Result>(
+        execute_sql(query, args));
+
+    ret_val = std::make_shared<ClassicResult>(cresult);
+  }
+
+  return ret_val;
 }
 
 REGISTER_HELP_FUNCTION(query, ClassicSession);
-REGISTER_HELP(CLASSICSESSION_QUERY_BRIEF,
-              "Executes a query and returns the "
-              "corresponding ClassicResult object.");
+REGISTER_HELP_FUNCTION_TEXT(CLASSICSESSION_QUERY, R"*(
+Executes a query and returns the corresponding ClassicResult object.
 
-REGISTER_HELP(CLASSICSESSION_QUERY_PARAM,
-              "@param query the SQL query string "
-              "to execute, with optional ? placeholders");
-REGISTER_HELP(
-    CLASSICSESSION_QUERY_PARAM1,
-    "@param args Optional list of "
-    "literals to use when replacing ? placeholders in the query string.");
-REGISTER_HELP(CLASSICSESSION_QUERY_RETURNS,
-              "@returns A ClassicResult "
-              "object.");
-REGISTER_HELP(CLASSICSESSION_QUERY_EXCEPTION,
-              "@exception An exception is "
-              "thrown if an error occurs on the SQL execution.");
+@param query the SQL query string to execute, with optional ? placeholders.
+@param args Optional list of literals to use when replacing ? placeholders in
+the query string.
 
-REGISTER_HELP(CLASSICSESSION_QUERY_DETAIL,
-              "${CLASSICSESSION_QUERY_DEPRECATED}");
+@returns A ClassicResult object.
 
-REGISTER_HELP(CLASSICSESSION_QUERY_DEPRECATED,
-              "@attention This function will be removed in a future release, "
-              "use the <b><<<runSql>>></b> function instead.");
+@attention This function will be removed in a future release, use the
+<b><<<runSql>>></b> function instead.
 
-//! $(CLASSICSESSION_QUERY_BRIEF)
-#if DOXYGEN_CPP
-//! \param args should contain the SQL query to execute against the database.
-#else
-//! $(CLASSICSESSION_QUERY_PARAM)
-//! $(CLASSICSESSION_QUERY_PARAM1)
-#endif
+@throw An exception is thrown if an error occurs on the SQL execution.
+)*");
 /**
- * $(CLASSICSESSION_QUERY_RETURNS)
+ * $(CLASSICSESSION_QUERY_BRIEF)
  *
- * $(CLASSICSESSION_QUERY_EXCEPTION)
- *
- * $(CLASSICSESSION_QUERY_DEPRECATED)
+ * $(CLASSICSESSION_QUERY)
  */
 #if DOXYGEN_JS
 ClassicResult ClassicSession::query(String query, Array args = []) {}
 #elif DOXYGEN_PY
 ClassicResult ClassicSession::query(str query, list args = []) {}
 #endif
-Value ClassicSession::query(const shcore::Argument_list &args) {
+std::shared_ptr<ClassicResult> ClassicSession::query(
+    const std::string &query, const shcore::Array_t &args) {
   auto console = mysqlsh::current_console();
   console->print_warning("'query' is deprecated, use " +
                          get_function_name("runSql") + " instead.");
 
-  return _run_sql("query", args);
+  return run_sql(query, args);
 }
 
-shcore::Object_bridge_ref ClassicSession::raw_execute_sql(
-    const std::string &query) {
-  return execute_sql(query, {}).as_object();
-}
-
-shcore::Value ClassicSession::execute_sql(const std::string &query,
-                                          const shcore::Array_t &args) {
-  Value ret_val;
+std::shared_ptr<mysqlshdk::db::IResult> ClassicSession::execute_sql(
+    const std::string &query, const shcore::Array_t &args) {
+  std::shared_ptr<mysqlshdk::db::IResult> result;
   if (!_session || !_session->is_open()) {
     throw Exception::logic_error("Not connected.");
   } else {
@@ -287,38 +224,16 @@ shcore::Value ClassicSession::execute_sql(const std::string &query,
     } else {
       Interruptible intr(this);
       try {
-        ClassicResult *result;
-        ret_val = Value::wrap(
-            result = new ClassicResult(
-                std::dynamic_pointer_cast<mysqlshdk::db::mysql::Result>(
-                    _session->query(sub_query_placeholders(query, args)))));
+        result = _session->query(sub_query_placeholders(query, args));
       } catch (const mysqlshdk::db::Error &error) {
         throw shcore::Exception::mysql_error_with_code_and_state(
             error.what(), error.code(), error.sqlstate());
       }
     }
   }
-  return ret_val;
-}
 
-// We need to hide this from doxygen to avoif warnings
-#if !defined DOXYGEN_JS && !defined DOXYGEN_PY
-std::shared_ptr<ClassicResult> ClassicSession::execute_sql(
-    const std::string &query) {
-  if (!_session || !_session->is_open()) {
-    throw Exception::logic_error("Not connected.");
-  } else {
-    if (query.empty()) {
-      throw Exception::argument_error("No query specified.");
-    } else {
-      auto result = std::shared_ptr<ClassicResult>(new ClassicResult(
-          std::dynamic_pointer_cast<mysqlshdk::db::mysql::Result>(
-              _session->query(query))));
-      return result;
-    }
-  }
+  return result;
 }
-#endif
 
 void ClassicSession::create_schema(const std::string &name) {
   if (!_session || !_session->is_open()) {
@@ -338,21 +253,17 @@ void ClassicSession::create_schema(const std::string &name) {
 
 // Documentation of getCurrentSchema function
 REGISTER_HELP_PROPERTY(uri, ClassicSession);
-REGISTER_HELP(CLASSICSESSION_URI_BRIEF,
-              "Retrieves the URI for the current "
-              "session.");
+REGISTER_HELP(CLASSICSESSION_URI_BRIEF, "${CLASSICSESSION_GETURI_BRIEF}");
 REGISTER_HELP_FUNCTION(getUri, ClassicSession);
-REGISTER_HELP(CLASSICSESSION_GETURI_BRIEF,
-              "Retrieves the URI for the current "
-              "session.");
-REGISTER_HELP(CLASSICSESSION_GETURI_RETURNS,
-              "@return A string representing "
-              "the connection data.");
+REGISTER_HELP_FUNCTION_TEXT(CLASSICSESSION_GETURI, R"*(
+Retrieves the URI for the current session.
 
+@return A string representing the connection data.
+)*");
 /**
- * $(SHELLBASESESSION_GETURI_BRIEF)
+ * $(CLASSICSESSION_GETURI_BRIEF)
  *
- * $(CLASSICSESSION_GETURI_RETURNS)
+ * $(CLASSICSESSION_GETURI)
  */
 #if DOXYGEN_JS
 String ClassicSession::getUri() {}
@@ -382,19 +293,11 @@ std::string ClassicSession::get_current_schema() {
   std::string name;
 
   if (_session && _session->is_open()) {
-    shcore::Argument_list query;
-    query.push_back(Value("select schema()"));
+    auto result = execute_sql("select schema()");
+    auto next_row = result->fetch_one();
 
-    Value res = run_sql(query);
-
-    std::shared_ptr<ClassicResult> rset = res.as_object<ClassicResult>();
-    Value next_row = rset->fetch_one(shcore::Argument_list());
-
-    if (next_row) {
-      std::shared_ptr<mysqlsh::Row> row = next_row.as_object<mysqlsh::Row>();
-      shcore::Value schema = row->get_member(0);
-
-      if (schema) name = schema.get_string();
+    if (next_row && !next_row->is_null(0)) {
+      name = next_row->get_string(0);
     }
   }
 
@@ -402,30 +305,26 @@ std::string ClassicSession::get_current_schema() {
 }
 
 void ClassicSession::set_current_schema(const std::string &name) {
-  execute_sql(sqlstring("use !", 0) << name);
+  get_core_session()->execute(sqlstring("use !", 0) << name);
 }
 
 std::shared_ptr<shcore::Object_bridge> ClassicSession::create(
-    const shcore::Argument_list &args) {
-  std::shared_ptr<ClassicSession> session(new ClassicSession());
-
-  auto connection_options =
-      mysqlsh::get_connection_options(args, mysqlsh::PasswordFormat::STRING);
-
+    const mysqlshdk::db::Connection_options &co_) {
+  auto co = co_;
   // DevAPI getClassicSession uses ssl-mode = REQUIRED by default if no
   // ssl-ca or ssl-capath are specified
-  if (!connection_options.get_ssl_options().has_mode() &&
-      !connection_options.has_value(mysqlshdk::db::kSslCa) &&
-      !connection_options.has_value(mysqlshdk::db::kSslCaPath)) {
-    connection_options.get_ssl_options().set_mode(
-        mysqlshdk::db::Ssl_mode::Required);
+  if (!co.get_ssl_options().has_mode() &&
+      !co.has_value(mysqlshdk::db::kSslCa) &&
+      !co.has_value(mysqlshdk::db::kSslCaPath)) {
+    co.get_ssl_options().set_mode(mysqlshdk::db::Ssl_mode::Required);
   }
 
-  session->connect(connection_options);
+  const auto session = std::make_shared<ClassicSession>();
+  session->connect(co);
 
   shcore::ShellNotifications::get()->notify("SN_SESSION_CONNECTED", session);
 
-  return std::dynamic_pointer_cast<shcore::Object_bridge>(session);
+  return session;
 }
 
 void ClassicSession::drop_schema(const std::string &name) {
@@ -448,39 +347,31 @@ std::string ClassicSession::db_object_exists(std::string &type,
   // their arguments it's enough to just escape the wildcards
   std::string escaped_name = escape_wildcards(name);
 
+  auto session = get_core_session();
   if (type == "Schema") {
     statement = sqlstring("show databases like ?", 0) << escaped_name;
-    auto val_result = execute_sql(statement, shcore::Array_t());
-    auto result = val_result.as_object<ClassicResult>();
-    auto val_row = result->fetch_one(shcore::Argument_list());
+    auto result = session->query(statement);
+    auto row = result->fetch_one();
 
-    if (val_row) {
-      auto row = val_row.as_object<mysqlsh::Row>();
-      if (row) ret_val = row->get_member(0).get_string();
-    }
+    if (row) ret_val = row->get_string(0);
   } else {
     statement = sqlstring("show full tables from ! like ?", 0)
                 << owner << escaped_name;
-    auto val_result = execute_sql(statement, shcore::Array_t());
-    auto result = val_result.as_object<ClassicResult>();
-    auto val_row = result->fetch_one(shcore::Argument_list());
+    auto result = session->query(statement);
+    auto row = result->fetch_one();
 
-    if (val_row) {
-      auto row = val_row.as_object<mysqlsh::Row>();
+    if (row) {
+      std::string db_type = row->get_string(1);
 
-      if (row) {
-        std::string db_type = row->get_member(1).get_string();
-
-        if (type == "Table" &&
-            (db_type == "BASE TABLE" || db_type == "LOCAL TEMPORARY"))
-          ret_val = row->get_member(0).get_string();
-        else if (type == "View" &&
-                 (db_type == "VIEW" || db_type == "SYSTEM VIEW"))
-          ret_val = row->get_member(0).get_string();
-        else if (type.empty()) {
-          ret_val = row->get_member(0).get_string();
-          type = db_type;
-        }
+      if (type == "Table" &&
+          (db_type == "BASE TABLE" || db_type == "LOCAL TEMPORARY"))
+        ret_val = row->get_string(0);
+      else if (type == "View" &&
+               (db_type == "VIEW" || db_type == "SYSTEM VIEW"))
+        ret_val = row->get_string(0);
+      else if (type.empty()) {
+        ret_val = row->get_string(0);
+        type = db_type;
       }
     }
   }
@@ -488,119 +379,6 @@ std::string ClassicSession::db_object_exists(std::string &type,
   return ret_val;
 }
 
-// Documentation of startTransaction function
-REGISTER_HELP_FUNCTION(startTransaction, ClassicSession);
-REGISTER_HELP(CLASSICSESSION_STARTTRANSACTION_BRIEF,
-              "Starts a transaction context on the server.");
-REGISTER_HELP(CLASSICSESSION_STARTTRANSACTION_RETURNS,
-              "@returns A ClassicResult object.");
-REGISTER_HELP(
-    CLASSICSESSION_STARTTRANSACTION_DETAIL,
-    "Calling this function will turn off the autocommit mode on the server.");
-REGISTER_HELP(CLASSICSESSION_STARTTRANSACTION_DETAIL1,
-              "All the operations executed after calling this function will "
-              "take place only when commit() is called.");
-REGISTER_HELP(CLASSICSESSION_STARTTRANSACTION_DETAIL2,
-              "All the operations executed after calling this function, will "
-              "be discarded is rollback() is called.");
-REGISTER_HELP(CLASSICSESSION_STARTTRANSACTION_DETAIL3,
-              "When commit() or rollback() are called, the server autocommit "
-              "mode will return back to it's state before calling "
-              "startTransaction().");
-
-/**
- * $(CLASSICSESSION_STARTTRANSACTION_BRIEF)
- *
- * $(CLASSICSESSION_STARTTRANSACTION_RETURNS)
- *
- * $(CLASSICSESSION_STARTTRANSACTION_DETAIL)
- *
- * $(CLASSICSESSION_STARTTRANSACTION_DETAIL1)
- *
- * $(CLASSICSESSION_STARTTRANSACTION_DETAIL2)
- *
- * $(CLASSICSESSION_STARTTRANSACTION_DETAIL3)
- */
-#if DOXYGEN_JS
-ClassicResult ClassicSession::startTransaction() {}
-#elif DOXYGEN_PY
-ClassicResult ClassicSession::start_transaction() {}
-#endif
-shcore::Value ClassicSession::_start_transaction(
-    const shcore::Argument_list &args) {
-  args.ensure_count(0, get_function_name("startTransaction").c_str());
-
-  return execute_sql("start transaction", shcore::Array_t());
-}
-
-// Documentation of commit function
-REGISTER_HELP_FUNCTION(commit, ClassicSession);
-REGISTER_HELP(
-    CLASSICSESSION_COMMIT_BRIEF,
-    "Commits all the operations executed after a call to startTransaction().");
-REGISTER_HELP(CLASSICSESSION_COMMIT_RETURNS,
-              "@returns A ClassicResult object.");
-REGISTER_HELP(CLASSICSESSION_COMMIT_DETAIL,
-              "All the operations executed after calling startTransaction() "
-              "will take place when this function is called.");
-REGISTER_HELP(CLASSICSESSION_COMMIT_DETAIL1,
-              "The server autocommit mode will return back to it's state "
-              "before calling startTransaction().");
-
-/**
- * $(CLASSICSESSION_COMMIT_BRIEF)
- *
- * $(CLASSICSESSION_COMMIT_RETURNS)
- *
- * $(CLASSICSESSION_COMMIT_DETAIL)
- *
- * $(CLASSICSESSION_COMMIT_DETAIL1)
- */
-#if DOXYGEN_JS
-ClassicResult ClassicSession::commit() {}
-#elif DOXYGEN_PY
-ClassicResult ClassicSession::commit() {}
-#endif
-shcore::Value ClassicSession::_commit(const shcore::Argument_list &args) {
-  args.ensure_count(0, get_function_name("commit").c_str());
-
-  return execute_sql("commit", shcore::Array_t());
-}
-
-// Documentation of rollback function
-REGISTER_HELP_FUNCTION(rollback, ClassicSession);
-REGISTER_HELP(
-    CLASSICSESSION_ROLLBACK_BRIEF,
-    "Discards all the operations executed after a call to startTransaction().");
-REGISTER_HELP(CLASSICSESSION_ROLLBACK_RETURNS,
-              "@returns A ClassicResult object.");
-REGISTER_HELP(CLASSICSESSION_ROLLBACK_DETAIL,
-              "All the operations executed after calling startTransaction() "
-              "will be discarded when this function is called.");
-REGISTER_HELP(CLASSICSESSION_ROLLBACK_DETAIL1,
-              "The server autocommit mode will return back to it's state "
-              "before calling startTransaction().");
-
-/**
- * $(CLASSICSESSION_ROLLBACK_BRIEF)
- *
- * $(CLASSICSESSION_ROLLBACK_RETURNS)
- *
- * $(CLASSICSESSION_ROLLBACK_DETAIL)
- *
- * $(CLASSICSESSION_ROLLBACK_DETAIL1)
- */
-#if DOXYGEN_JS
-ClassicResult ClassicSession::rollback() {}
-#elif DOXYGEN_PY
-ClassicResult ClassicSession::rollback() {}
-#endif
-shcore::Value ClassicSession::_rollback(const shcore::Argument_list &args) {
-  args.ensure_count(0, get_function_name("rollback").c_str());
-
-  return execute_sql("rollback", shcore::Array_t());
-}
-
 shcore::Value::Map_type_ref ClassicSession::get_status() {
   shcore::Value::Map_type_ref status(new shcore::Value::Map_type);
 
@@ -626,14 +404,21 @@ shcore::Value::Map_type_ref ClassicSession::get_status() {
 
       (*status)["SERVER_INFO"] = shcore::Value(_session->get_server_info());
 
-      (*status)["PROTOCOL_VERSION"] =
-          shcore::Value(std::string("classic ") +
-                        std::to_string(_session->get_protocol_info()));
+      (*status)["PROTOCOL_VERSION"] = shcore::Value(
+          (_session->is_compression_enabled() ? "Compressed " : "Classic ") +
+          std::to_string(_session->get_protocol_info()));
 
-      if (_session->is_compression_enabled())
-        (*status)["COMPRESSION"] = shcore::Value(std::string("Enabled"));
-      else
+      if (_session->is_compression_enabled()) {
+        std::string compr("Enabled");
+        auto res = _session->query("show status like 'Compression_algorithm';");
+        auto status_row = res->fetch_one();
+        if (status_row && !status_row->is_null(1))
+          compr += " (" + status_row->get_as_string(1) + ")";
+
+        (*status)["COMPRESSION"] = shcore::Value(compr);
+      } else {
         (*status)["COMPRESSION"] = shcore::Value(std::string("Disabled"));
+      }
 
       (*status)["CONNECTION"] = shcore::Value(_session->get_connection_info());
       //(*status)["INSERT_ID"] = shcore::Value(???);
@@ -654,7 +439,7 @@ shcore::Value::Map_type_ref ClassicSession::get_status() {
         "select @@character_set_client, @@character_set_connection, "
         "@@character_set_server, @@character_set_database, "
         "concat(@@version, \" \", @@version_comment) as version, "
-        "@@socket, @@port, @@datadir "
+        "@@socket, @@port, @@datadir, @@character_set_results "
         "limit 1");
 
     row = result->fetch_one();
@@ -665,6 +450,7 @@ shcore::Value::Map_type_ref ClassicSession::get_status() {
       (*status)["SERVER_CHARSET"] = shcore::Value(row->get_string(2));
       (*status)["SCHEMA_CHARSET"] = shcore::Value(row->get_string(3));
       (*status)["SERVER_VERSION"] = shcore::Value(row->get_string(4));
+      (*status)["RESULTS_CHARSET"] = shcore::Value(row->get_string(8));
 
       if (!_connection_options.has_transport_type() ||
           _connection_options.get_transport_type() ==
@@ -705,13 +491,69 @@ shcore::Value::Map_type_ref ClassicSession::get_status() {
 
   return status;
 }
+REGISTER_HELP_FUNCTION(startTransaction, ClassicSession);
+REGISTER_HELP_FUNCTION_TEXT(CLASSICSESSION_STARTTRANSACTION, R"*(
+Starts a transaction context on the server.
 
+@returns A ClassicResult object.
+
+Calling this function will turn off the autocommit mode on the server.
+
+All the operations executed after calling this function will take place only
+when commit() is called.
+
+All the operations executed after calling this function, will be discarded if
+rollback() is called.
+
+When commit() or rollback() are called, the server autocommit mode will return
+back to it's state before calling <<<startTransaction>>>().
+)*");
+/**
+ * $(CLASSICSESSION_STARTTRANSACTION_BRIEF)
+ *
+ * $(CLASSICSESSION_STARTTRANSACTION)
+ */
+#if DOXYGEN_JS
+ClassicResult ClassicSession::startTransaction() {}
+#elif DOXYGEN_PY
+ClassicResult ClassicSession::start_transaction() {}
+#endif
 void ClassicSession::start_transaction() {
   if (_tx_deep == 0) execute_sql("start transaction", shcore::Array_t());
 
   _tx_deep++;
 }
 
+std::shared_ptr<ClassicResult> ClassicSession::_start_transaction() {
+  auto result = std::dynamic_pointer_cast<mysqlshdk::db::mysql::Result>(
+      execute_sql("start transaction"));
+
+  return std::make_shared<ClassicResult>(result);
+}
+
+REGISTER_HELP_FUNCTION(commit, ClassicSession);
+REGISTER_HELP_FUNCTION_TEXT(CLASSICSESSION_COMMIT, R"*(
+Commits all the operations executed after a call to <<<startTransaction>>>().
+
+@returns A ClassicResult object.
+
+All the operations executed after calling <<<startTransaction>>>() will take place
+when this function is called.
+
+The server autocommit mode will return back to it's state before calling
+<<<startTransaction>>>().
+)*");
+/**
+ * $(CLASSICSESSION_COMMIT_BRIEF)
+ *
+ * $(CLASSICSESSION_COMMIT)
+ */
+#if DOXYGEN_JS
+ClassicResult ClassicSession::commit() {}
+#elif DOXYGEN_PY
+ClassicResult ClassicSession::commit() {}
+#endif
+
 void ClassicSession::commit() {
   _tx_deep--;
 
@@ -720,6 +562,36 @@ void ClassicSession::commit() {
   if (_tx_deep == 0) execute_sql("commit", shcore::Array_t());
 }
 
+std::shared_ptr<ClassicResult> ClassicSession::_commit() {
+  auto result = std::dynamic_pointer_cast<mysqlshdk::db::mysql::Result>(
+      execute_sql("commit"));
+
+  return std::make_shared<ClassicResult>(result);
+}
+
+// Documentation of rollback function
+REGISTER_HELP_FUNCTION(rollback, ClassicSession);
+REGISTER_HELP_FUNCTION_TEXT(CLASSICSESSION_ROLLBACK, R"*(
+Discards all the operations executed after a call to <<<startTransaction>>>().
+
+@returns A ClassicResult object.
+
+All the operations executed after calling <<<startTransaction>>>() will be discarded
+when this function is called.
+
+The server autocommit mode will return back to it's state before calling
+<<<startTransaction>>>().
+)*");
+/**
+ * $(CLASSICSESSION_ROLLBACK_BRIEF)
+ *
+ * $(CLASSICSESSION_ROLLBACK)
+ */
+#if DOXYGEN_JS
+ClassicResult ClassicSession::rollback() {}
+#elif DOXYGEN_PY
+ClassicResult ClassicSession::rollback() {}
+#endif
 void ClassicSession::rollback() {
   _tx_deep--;
 
@@ -732,18 +604,41 @@ uint64_t ClassicSession::get_connection_id() const {
   return _session->get_connection_id();
 }
 
+std::shared_ptr<ClassicResult> ClassicSession::_rollback() {
+  auto result = std::dynamic_pointer_cast<mysqlshdk::db::mysql::Result>(
+      execute_sql("rollback"));
+
+  return std::make_shared<ClassicResult>(result);
+}
+
+REGISTER_HELP_FUNCTION(isOpen, ClassicSession);
+REGISTER_HELP_FUNCTION_TEXT(CLASSICSESSION_ISOPEN, R"*(
+Returns true if session is known to be open.
+
+@returns A boolean value indicating if the session is still open.
+
+Returns true if the session is still open and false otherwise.
+
+@note This function may return true if connection is lost.
+)*");
+/**
+ * $(CLASSICSESSION_ISOPEN_BRIEF)
+ *
+ * $(CLASSICSESSION_ISOPEN)
+ */
+#if DOXYGEN_JS
+Bool ClassicSession::isOpen() {}
+#elif DOXYGEN_PY
+bool ClassicSession::is_open() {}
+#endif
 bool ClassicSession::is_open() const { return _session && _session->is_open(); }
 
 std::string ClassicSession::query_one_string(const std::string &query,
                                              int field) {
-  shcore::Value val_result = execute_sql(query, shcore::Array_t());
-  auto result = val_result.as_object<mysql::ClassicResult>();
-  shcore::Value val_row = result->fetch_one(shcore::Argument_list());
-  if (val_row) {
-    auto row = val_row.as_object<mysqlsh::Row>();
-    if (row) {
-      return row->get_member(field).get_string();
-    }
+  auto result = execute_sql(query);
+  auto row = result->fetch_one();
+  if (row) {
+    return row->get_string(field);
   }
   return "";
 }
@@ -762,3 +657,9 @@ void ClassicSession::kill_query() {
     log_warning("Error cancelling SQL query: %s", e.what());
   }
 }
+
+socket_t ClassicSession::_get_socket_fd() const {
+  if (!_session || !_session->is_open())
+    throw std::invalid_argument("Session is not open");
+  return _session->get_socket_fd();
+}
