@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -26,16 +26,22 @@
 
 #include "types_common.h"
 
+#include <algorithm>
+#include <cassert>
+#include <iterator>
+#include <list>
 #include <map>
 #include <memory>
 #include <set>
 #include <stdexcept>
 #include <string>
+#include <unordered_set>
+#include <utility>
 #include <vector>
 
+#include "mysqlshdk/include/mysqlshdk_export.h"
+#include "mysqlshdk/libs/utils/error.h"
 #include "mysqlshdk/libs/utils/nullable.h"
-#include "mysqlshdk/libs/utils/utils_json.h"
-#include "mysqlshdk_export.h"
 
 // For error codes used by the shell
 #define SHERR_FIRST 50000
@@ -63,15 +69,19 @@ enum Value_type {
   Function  //! A function reference, not serializable.
 };
 
-// kTypeConvertible[from_type][to_type] = is_convertible
-// from_type = row, to_type = column
-extern const bool kTypeConvertible[12][12];
+bool is_compatible_type(Value_type source_type, Value_type target_type);
 
 std::string type_description(Value_type type);
+std::string SHCORE_PUBLIC type_name(Value_type type);
 
 class Object_bridge;
 typedef std::shared_ptr<Object_bridge> Object_bridge_ref;
 
+/** Pointer to a function that may be implemented in any language.
+ */
+class Function_base;
+using Function_base_ref = std::shared_ptr<Function_base>;
+
 /** A generic value that can be used from any language we support.
 
  Anything that can be represented using this can be passed as a parameter to
@@ -178,14 +188,16 @@ struct SHCORE_PUBLIC Value {
       return _map[k];
     }
     bool operator==(const Map_type &other) const { return _map == other._map; }
+    bool operator<(const Map_type &other) const { return _map < other._map; }
+    bool operator<=(const Map_type &other) const { return _map <= other._map; }
 
     bool empty() const { return _map.empty(); }
     size_t size() const { return _map.size(); }
     size_t count(const std::string &k) const { return _map.count(k); }
 
     template <class T>
-    std::pair<iterator, bool> emplace(const std::string &key, const T &value) {
-      return _map.emplace(key, Value(value));
+    std::pair<iterator, bool> emplace(const std::string &key, const T &v) {
+      return _map.emplace(key, Value(v));
     }
 
    private:
@@ -209,8 +221,10 @@ struct SHCORE_PUBLIC Value {
 
   Value() : type(Undefined) {}
   Value(const Value &copy);
+  Value(Value &&other);
 
   explicit Value(const std::string &s);
+  explicit Value(std::string &&s);
   explicit Value(const char *);
   explicit Value(const char *, size_t n);
   explicit Value(int i);
@@ -220,11 +234,16 @@ struct SHCORE_PUBLIC Value {
   explicit Value(float f);
   explicit Value(double d);
   explicit Value(bool b);
-  explicit Value(std::shared_ptr<Function_base> f);
-  explicit Value(std::shared_ptr<Object_bridge> o);
-  explicit Value(std::weak_ptr<Map_type> n);
-  explicit Value(Map_type_ref n);
-  explicit Value(Array_type_ref n);
+  explicit Value(const std::shared_ptr<Function_base> &f);
+  explicit Value(std::shared_ptr<Function_base> &&f);
+  explicit Value(const std::shared_ptr<Object_bridge> &o);
+  explicit Value(std::shared_ptr<Object_bridge> &&o);
+  explicit Value(const std::weak_ptr<Map_type> &n);
+  explicit Value(std::weak_ptr<Map_type> &&n);
+  explicit Value(const Map_type_ref &n);
+  explicit Value(Map_type_ref &&n);
+  explicit Value(const Array_type_ref &n);
+  explicit Value(Array_type_ref &&n);
 
   static Value wrap(Object_bridge *o) {
     return Value(std::shared_ptr<Object_bridge>(o));
@@ -263,16 +282,31 @@ struct SHCORE_PUBLIC Value {
 
   //! parse a string returned by repr() back into a Value
   static Value parse(const std::string &s);
+  static Value parse(const char *s, size_t length);
 
   ~Value();
 
   Value &operator=(const Value &other);
+  Value &operator=(Value &&other);
 
   bool operator==(const Value &other) const;
 
   bool operator!=(const Value &other) const { return !(*this == other); }
 
-  operator bool() const { return type != Undefined && type != shcore::Null; }
+  bool operator<(const Value &other) const;
+
+  bool operator<=(const Value &other) const;
+
+  bool operator>(const Value &other) const { return !(*this <= other); }
+
+  bool operator>=(const Value &other) const { return !(*this < other); }
+
+  explicit operator bool() const {
+    return type != Undefined && type != shcore::Null;
+  }
+
+  // helper used by gtest
+  friend std::ostream &operator<<(std::ostream &os, const Value &v);
 
   //! returns a human-readable description text for the value.
   // if pprint is true, it will try to pretty-print it (like adding newlines)
@@ -293,6 +327,7 @@ struct SHCORE_PUBLIC Value {
   std::string &append_repr(std::string &s_out) const;
 
   void check_type(Value_type t) const;
+  Value_type get_type() const { return type; }
 
   bool as_bool() const;
   int64_t as_int() const;
@@ -326,15 +361,27 @@ struct SHCORE_PUBLIC Value {
     return type == shcore::Null ? nullptr : *value.array;
   }
 
-  std::vector<std::string> to_string_vector() const {
-    std::vector<std::string> vec;
+  template <class C>
+  C to_string_container() const {
+    C vec;
     check_type(Array);
-    for (const Value &v : *as_array()) {
-      vec.push_back(v.get_string());
-    }
+    auto arr = as_array();
+
+    std::transform(arr->begin(), arr->end(), std::inserter<C>(vec, vec.end()),
+                   [](const shcore::Value &v) { return v.get_string(); });
+
     return vec;
   }
 
+  std::map<std::string, std::string> to_string_map() const {
+    std::map<std::string, std::string> map;
+    check_type(Map);
+    for (const auto &v : *as_map()) {
+      map.emplace(v.first, v.second.get_string());
+    }
+    return map;
+  }
+
   std::shared_ptr<Function_base> as_function() const {
     check_type(Function);
     return std::dynamic_pointer_cast<Function_base>(
@@ -355,28 +402,33 @@ struct SHCORE_PUBLIC Value {
 typedef Value::Map_type_ref Dictionary_t;
 typedef Value::Array_type_ref Array_t;
 
-inline Dictionary_t make_dict() {
-  return Value::Map_type_ref(new Value::Map_type());
-}
+inline Dictionary_t make_dict() { return std::make_shared<Value::Map_type>(); }
 
-template <typename... Arg>
-inline Dictionary_t make_dict(const std::string &key,
-                              const shcore::Value &value, Arg... args) {
-  Dictionary_t dict = make_dict(args...);
-  dict->set(key, value);
+template <typename K, typename V, typename... Arg>
+inline Dictionary_t make_dict(K &&key, V &&value, Arg &&... args) {
+  Dictionary_t dict = make_dict(std::forward<Arg>(args)...);
+  dict->emplace(std::forward<K>(key), std::forward<V>(value));
   return dict;
 }
 
-inline Array_t make_array() {
-  return Value::Array_type_ref(new Value::Array_type());
+inline Array_t make_array() { return std::make_shared<Value::Array_type>(); }
+
+template <typename... Arg>
+inline Array_t make_array(Arg &&... args) {
+  auto array = make_array();
+  (void)std::initializer_list<int>{
+      (array->emplace_back(std::forward<Arg>(args)), 0)...};
+  return array;
 }
 
-class SHCORE_PUBLIC Exception : public std::exception {
+class SHCORE_PUBLIC Exception : public shcore::Error {
   std::shared_ptr<Value::Map_type> _error;
 
  public:
-  explicit Exception(const std::shared_ptr<Value::Map_type> e);
+  Exception(const std::string &message, int code,
+            const std::shared_ptr<Value::Map_type> &e);
   Exception(const std::string &message, int code);
+  Exception(const std::string &type, const std::string &message, int code);
 
   virtual ~Exception() noexcept {}
 
@@ -416,16 +468,14 @@ class SHCORE_PUBLIC Exception : public std::exception {
   bool is_mysql() const;
   bool is_mysqlsh() const;
   bool is_parser() const;
-
-  virtual const char *what() const noexcept;
+  bool is_runtime() const;
+  bool is_metadata() const;
 
   const char *type() const noexcept;
 
-  int code() const noexcept;
-
   std::shared_ptr<Value::Map_type> error() const { return _error; }
 
-  std::string format() const;
+  std::string format() const override;
 };
 
 class SHCORE_PUBLIC Argument_list {
@@ -465,6 +515,7 @@ class SHCORE_PUBLIC Argument_list {
   Value &operator[](size_t i) { return _args[i]; }
   const Value &operator[](size_t i) const { return _args[i]; }
   void clear() { _args.clear(); }
+  bool empty() const { return _args.empty(); }
 
   std::vector<Value>::const_iterator begin() const { return _args.begin(); }
   std::vector<Value>::const_iterator end() const { return _args.end(); }
@@ -473,7 +524,7 @@ class SHCORE_PUBLIC Argument_list {
 
   template <class T>
   void emplace_back(const T &value) {
-    return _args.emplace_back(Value(value));
+    _args.emplace_back(Value(value));
   }
 
  private:
@@ -539,18 +590,6 @@ struct value_type_for_native<bool> {
   static bool extract(const Value &value) { return value.as_bool(); }
 };
 template <>
-struct value_type_for_native<int64_t> {
-  static const Value_type type = Integer;
-
-  static int64_t extract(const Value &value) { return value.as_int(); }
-};
-template <>
-struct value_type_for_native<uint64_t> {
-  static const Value_type type = UInteger;
-
-  static uint64_t extract(const Value &value) { return value.as_uint(); }
-};
-template <>
 struct value_type_for_native<int> {
   static const Value_type type = Integer;
 
@@ -567,6 +606,38 @@ struct value_type_for_native<unsigned int> {
   }
 };
 template <>
+struct value_type_for_native<long int> {
+  static const Value_type type = Integer;
+
+  static long int extract(const Value &value) {
+    return static_cast<long int>(value.as_int());
+  }
+};
+template <>
+struct value_type_for_native<unsigned long int> {
+  static const Value_type type = UInteger;
+
+  static unsigned long int extract(const Value &value) {
+    return static_cast<unsigned long int>(value.as_uint());
+  }
+};
+template <>
+struct value_type_for_native<long long int> {
+  static const Value_type type = Integer;
+
+  static long long int extract(const Value &value) {
+    return static_cast<long long int>(value.as_int());
+  }
+};
+template <>
+struct value_type_for_native<unsigned long long int> {
+  static const Value_type type = UInteger;
+
+  static unsigned long long int extract(const Value &value) {
+    return static_cast<unsigned long long int>(value.as_uint());
+  }
+};
+template <>
 struct value_type_for_native<double> {
   static const Value_type type = Float;
 
@@ -590,7 +661,43 @@ struct value_type_for_native<std::vector<std::string>> {
   static const Value_type type = Array;
 
   static std::vector<std::string> extract(const Value &value) {
-    return value.to_string_vector();
+    return value.to_string_container<std::vector<std::string>>();
+  }
+};
+
+template <>
+struct value_type_for_native<std::list<std::string>> {
+  static const Value_type type = Array;
+
+  static std::list<std::string> extract(const Value &value) {
+    return value.to_string_container<std::list<std::string>>();
+  }
+};
+
+template <>
+struct value_type_for_native<std::set<std::string>> {
+  static const Value_type type = Array;
+
+  static std::set<std::string> extract(const Value &value) {
+    return value.to_string_container<std::set<std::string>>();
+  }
+};
+
+template <>
+struct value_type_for_native<std::unordered_set<std::string>> {
+  static const Value_type type = Array;
+
+  static std::unordered_set<std::string> extract(const Value &value) {
+    return value.to_string_container<std::unordered_set<std::string>>();
+  }
+};
+
+template <>
+struct value_type_for_native<std::map<std::string, std::string>> {
+  static const Value_type type = Map;
+
+  static std::map<std::string, std::string> extract(const Value &value) {
+    return value.to_string_map();
   }
 };
 
@@ -634,12 +741,28 @@ struct value_type_for_native<Value> {
   static Value extract(const Value &value) { return value; }
 };
 
-std::string SHCORE_PUBLIC type_name(Value_type type);
+template <typename T>
+struct value_type_for_native<mysqlshdk::utils::nullable<T>> {
+  static const Value_type type = value_type_for_native<T>::type;
+
+  static T extract(const Value &value) {
+    return value_type_for_native<T>::extract(value);
+  }
+};
+
+template <>
+struct value_type_for_native<Function_base_ref> {
+  static const Value_type type = Function;
+  static shcore::Function_base_ref extract(const Value &value) {
+    return value.as_function();
+  }
+};
 
 // Extract option values from an options dictionary, with validations
 // Replaces Argument_map
 class Option_unpacker {
  public:
+  Option_unpacker() = default;
   explicit Option_unpacker(const Dictionary_t &options);
 
   virtual ~Option_unpacker() {}
@@ -667,14 +790,6 @@ class Option_unpacker {
     return *this;
   }
 
-  template <typename T>
-  Option_unpacker &optional(const char *name,
-                            mysqlshdk::utils::nullable<T> *out_value) {
-    extract_value<T>(name, out_value,
-                     get_optional(name, value_type_for_native<T>::type));
-    return *this;
-  }
-
   // Extract optional option with exact type (no conversions)
   template <typename T>
   Option_unpacker &optional_exact(const char *name, T *out_value) {
@@ -683,15 +798,8 @@ class Option_unpacker {
     return *this;
   }
 
-  template <typename T>
-  Option_unpacker &optional_exact(const char *name,
-                                  mysqlshdk::utils::nullable<T> *out_value) {
-    extract_value<T>(name, out_value,
-                     get_optional_exact(name, value_type_for_native<T>::type));
-    return *this;
-  }
-
   // Case insensitive
+
   template <typename T>
   Option_unpacker &optional_ci(const char *name, T *out_value) {
     extract_value<T>(name, out_value,
@@ -701,6 +809,10 @@ class Option_unpacker {
 
   void end(const std::string &context = "");
 
+  void set_options(const shcore::Dictionary_t &options);
+
+  const shcore::Dictionary_t &options() const { return m_options; }
+
  protected:
   Dictionary_t m_options;
   std::set<std::string> m_unknown;
@@ -767,6 +879,12 @@ class SHCORE_PUBLIC Object_bridge {
     return !(*this == other);
   }
 
+  virtual bool operator<(const Object_bridge &) const { return false; }
+
+  virtual bool operator<=(const Object_bridge &other) const {
+    return (*this < other) || (*this == other);
+  }
+
   //! Returns the value of a member
   virtual Value get_member(const std::string &prop) const = 0;
 
@@ -819,12 +937,13 @@ class SHCORE_PUBLIC Function_base {
   // arglist must match the signature
   virtual Value invoke(const Argument_list &args) = 0;
 
-  virtual bool has_var_args() = 0;
-};
+  virtual std::string &append_descr(std::string *s_out, int indent = -1,
+                                    int quote_strings = 0) const;
 
-/** Pointer to a function that may be implemented in any language.
- */
-typedef std::shared_ptr<Function_base> Function_base_ref;
+  virtual std::string &append_repr(std::string *s_out) const;
+
+  virtual void append_json(JSON_dumper *dumper) const;
+};
 
 bool my_strnicmp(const char *c1, const char *c2, size_t n);
 }  // namespace shcore
