@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -27,8 +27,8 @@
 #include <fstream>
 #include <sstream>
 
-#include "my_dbug.h"
-
+#include "my_config.h"
+#include "mysqlshdk/libs/utils/debug.h"
 #include "mysqlshdk/libs/utils/utils_file.h"
 #include "mysqlshdk/libs/utils/utils_path.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
@@ -59,9 +59,13 @@ Config_file::Option_key Config_file::convert_option_to_key(
   std::replace(opt_key.key.begin(), opt_key.key.end(), '-', '_');
 
   // Ignore loose_ prefix
-  if (shcore::str_ibeginswith(opt_key.key, "loose_"))
+  if (shcore::str_ibeginswith(opt_key.key, "loose_")) {
     opt_key.key = opt_key.key.substr(6);
-
+  } else if (shcore::str_ibeginswith(opt_key.key, "group_replication_")) {
+    // if it doesn't start with loose_ and is a group_replication option,
+    // prepend the original_option (the one written to the file) with loose_.
+    opt_key.original_option = "loose_" + opt_key.original_option;
+  }
   return opt_key;
 }
 
@@ -334,7 +338,7 @@ void Config_file::read(const std::string &cnf_path,
                        const std::string &group_prefix) {
   container configmap(m_group_case == Case::SENSITIVE ? comp : icomp);
   read_recursive_aux(cnf_path, 0, &configmap, group_prefix);
-  // if no exception happned during the read of the files, then replace the
+  // if no exception happened during the read of the files, then replace the
   // current configuration map with the new one.
   m_configmap = std::move(configmap);
 }
@@ -657,7 +661,12 @@ void Config_file::read_recursive_aux(const std::string &cnf_path,
         cnf_path.c_str());
     return;  // exit function without doing anything.
   }
+#ifdef _WIN32
+  const auto wide_cnf_path = shcore::utf8_to_wide(cnf_path);
+  std::ifstream input_file(wide_cnf_path);
+#else
   std::ifstream input_file(cnf_path);
+#endif
   if (!input_file.good()) {
     throw std::runtime_error("Cannot open file: " + cnf_path + ".");
   }
@@ -780,13 +789,13 @@ std::string Config_file::parse_include_path(
 
     // Get absolute path for included file
     std::string base_dir = shcore::path::dirname(base_path);
-    res = shcore::get_absolute_path(base_dir, include_file_path);
+    res = shcore::get_absolute_path(include_file_path, base_dir);
   } else if (shcore::str_beginswith(line, "!includedir ")) {
     std::string include_dir_path = shcore::str_strip(line.substr(12));
 
     // Get absolute path for included dir
     std::string base_dir = shcore::path::dirname(base_path);
-    res = shcore::get_absolute_path(base_dir, include_dir_path);
+    res = shcore::get_absolute_path(include_dir_path, base_dir);
   } else {
     throw std::runtime_error(
         "No !include or !includedir directive found on "
@@ -798,18 +807,29 @@ std::string Config_file::parse_include_path(
 
 std::vector<std::string> get_default_config_paths(shcore::OperatingSystem os) {
   std::vector<std::string> default_paths;
+  // NOTE: According to the docs MySQL programs read startup options files
+  // according to a predefined location and order. Those my.cnf files for
+  // global options should be included in the default locations.
+  // See: https://dev.mysql.com/doc/en/option-files.html
   log_info("Getting default config paths for OS %s", to_string(os).c_str());
+  std::string sysconfdir;
+#ifdef DEFAULT_SYSCONFDIR
+  sysconfdir = std::string(DEFAULT_SYSCONFDIR);
+#endif
   switch (os) {
     case shcore::OperatingSystem::DEBIAN:
+      default_paths.push_back("/etc/my.cnf");
+      default_paths.push_back("/etc/mysql/my.cnf");
+      if (!sysconfdir.empty()) default_paths.push_back(sysconfdir + "/my.cnf");
       default_paths.push_back("/etc/mysql/mysql.conf.d/mysqld.cnf");
       break;
     case shcore::OperatingSystem::REDHAT:
     case shcore::OperatingSystem::SOLARIS:
-      default_paths.push_back("/etc/my.cnf");
-      break;
     case shcore::OperatingSystem::LINUX:
+    case shcore::OperatingSystem::MACOS:
       default_paths.push_back("/etc/my.cnf");
       default_paths.push_back("/etc/mysql/my.cnf");
+      if (!sysconfdir.empty()) default_paths.push_back(sysconfdir + "/my.cnf");
       break;
     case shcore::OperatingSystem::WINDOWS: {
       char *program_data_ptr = getenv("PROGRAMDATA");
@@ -820,11 +840,6 @@ std::vector<std::string> get_default_config_paths(shcore::OperatingSystem os) {
                                 R"(\MySQL\MySQL Server 8.0\my.ini)");
       }
     } break;
-    case shcore::OperatingSystem::MACOS:
-      default_paths.push_back("/etc/my.cnf");
-      default_paths.push_back("/etc/mysql/my.cnf");
-      default_paths.push_back("/usr/local/mysql/etc/my.cnf");
-      break;
     default:
       // The non-handled OS case will keep default_paths and cnfPath empty
       break;
