@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
 #include "mysqlshdk/libs/mysql/instance.h"
 #include "mysqlshdk/libs/utils/logger.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
+#include "mysqlshdk/libs/utils/utils_net.h"
 #include "mysqlshdk/libs/utils/utils_sqlstring.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
@@ -54,17 +55,19 @@ void Auth_options::set(mysqlshdk::db::Connection_options *copts) const {
   copts->set_ssl_options(ssl_options);
 }
 
-Instance::Instance(std::shared_ptr<db::ISession> session) : _session(session) {}
+Instance::Instance(const std::shared_ptr<db::ISession> &session)
+    : _session(session) {}
+
+void Instance::register_warnings_callback(const Warnings_callback &callback) {
+  m_warnings_callback = callback;
+}
 
 void Instance::refresh() {
   m_uuid.clear();
   m_group_name.clear();
 }
 
-std::string Instance::descr() const {
-  return _session->get_connection_options().as_uri(
-      db::uri::formats::only_transport());
-}
+std::string Instance::descr() const { return get_canonical_address(); }
 
 std::string Instance::get_canonical_hostname() const {
   if (m_hostname.empty()) {
@@ -97,7 +100,15 @@ std::string Instance::get_canonical_address() const {
     m_hostname = row->get_string(0, "");
     m_port = row->get_int(1);
   }
-  return m_hostname + ":" + std::to_string(m_port);
+  return mysqlshdk::utils::make_host_and_port(m_hostname, m_port);
+}
+
+uint32_t Instance::get_server_id() const {
+  if (m_server_id == 0) {
+    m_server_id = queryf_one_int(0, 0, "SELECT @@server_id");
+  }
+
+  return m_server_id;
 }
 
 const std::string &Instance::get_uuid() const {
@@ -107,6 +118,13 @@ const std::string &Instance::get_uuid() const {
   return m_uuid;
 }
 
+uint32_t Instance::get_id() const {
+  if (m_id.is_null()) {
+    m_id = queryf_one_int(0, 0, "SELECT @@server_id");
+  }
+  return *m_id;
+}
+
 const std::string &Instance::get_group_name() const {
   if (m_group_name.empty()) {
     m_group_name = get_sysvar_string("group_replication_group_name").get_safe();
@@ -114,6 +132,24 @@ const std::string &Instance::get_group_name() const {
   return m_group_name;
 }
 
+const std::string &Instance::get_version_compile_os() const {
+  if (m_version_compile_os.empty()) {
+    m_version_compile_os =
+        get_sysvar_string("version_compile_os", Var_qualifier::GLOBAL)
+            .get_safe();
+  }
+  return m_version_compile_os;
+}
+
+const std::string &Instance::get_version_compile_machine() const {
+  if (m_version_compile_machine.empty()) {
+    m_version_compile_machine =
+        get_sysvar_string("version_compile_machine", Var_qualifier::GLOBAL)
+            .get_safe();
+  }
+  return m_version_compile_machine;
+}
+
 namespace {
 bool sysvar_to_bool(const std::string &name, const std::string &str_value) {
   const char *value = str_value.c_str();
@@ -195,7 +231,8 @@ void Instance::set_sysvar(const std::string &name, const std::string &value,
   set_stmt << name;
   set_stmt << value;
   set_stmt.done();
-  execute(set_stmt);
+
+  query(set_stmt);
 }
 
 /**
@@ -219,7 +256,8 @@ void Instance::set_sysvar_default(const std::string &name,
   shcore::sqlstring set_stmt = shcore::sqlstring(set_stmt_fmt.c_str(), 0);
   set_stmt << name;
   set_stmt.done();
-  execute(set_stmt);
+
+  query(set_stmt);
 }
 
 /**
@@ -245,7 +283,8 @@ void Instance::set_sysvar(const std::string &name, const int64_t value,
   set_stmt << name;
   set_stmt << value;
   set_stmt.done();
-  execute(set_stmt);
+
+  query(set_stmt);
 }
 
 /**
@@ -272,7 +311,8 @@ void Instance::set_sysvar(const std::string &name, const bool value,
   set_stmt << name;
   set_stmt << str_value;
   set_stmt.done();
-  execute(set_stmt);
+
+  query(set_stmt);
 }
 
 std::map<std::string, utils::nullable<std::string>>
@@ -441,14 +481,6 @@ utils::nullable<std::string> Instance::get_plugin_status(
     return utils::nullable<std::string>();
 }
 
-const std::string &Instance::get_version_compile_os() const {
-  if (m_version_compile_os.empty()) {
-    m_version_compile_os =
-        *get_sysvar_string("version_compile_os", Var_qualifier::GLOBAL);
-  }
-  return m_version_compile_os;
-}
-
 std::string Instance::get_plugin_library_extension() const {
   std::string instance_os = shcore::str_upper(get_version_compile_os());
 
@@ -592,7 +624,7 @@ void Instance::drop_user(const std::string &user, const std::string &host,
 std::unique_ptr<User_privileges> Instance::get_user_privileges(
     const std::string &user, const std::string &host) const {
   return std::unique_ptr<User_privileges>(
-      new User_privileges(_session, user, host));
+      new User_privileges(*this, user, host));
 }
 
 bool Instance::is_read_only(bool super) const {
@@ -676,6 +708,22 @@ bool Instance::user_exists(const std::string &username,
   return true;
 }
 
+/**
+ * Set the password for a given mysql user account
+ *
+ * @param username account username
+ * @param hostname account hostname
+ * @param password password we want to set for the account
+ */
+void Instance::set_user_password(const std::string &username,
+                                 const std::string &hostname,
+                                 const std::string &password) const {
+  log_debug("Changing password for user %s@%s", username.c_str(),
+            hostname.c_str());
+  executef("ALTER USER /*(*/ ?@? /*)*/ IDENTIFIED BY /*((*/ ? /*))*/", username,
+           hostname, password);
+}
+
 utils::nullable<bool> Instance::is_set_persist_supported() const {
   // Check if the instance version is >= 8.0.11 to support the SET PERSIST.
   if (get_version() >= mysqlshdk::utils::Version(8, 0, 11)) {
@@ -746,7 +794,33 @@ void Instance::suppress_binary_log(bool flag) {
 
 std::shared_ptr<mysqlshdk::db::IResult> Instance::query(const std::string &sql,
                                                         bool buffered) const {
-  return get_session()->query(sql, buffered);
+  auto res = get_session()->query(sql, buffered);
+
+  // Call the Warnings_callback if registered
+  if (m_warnings_callback) {
+    // Get all the Warnings
+    while (auto warning = res->fetch_one_warning()) {
+      std::string warning_level;
+
+      switch (warning->level) {
+        case db::Warning::Level::Note:
+          warning_level = "NOTE";
+          break;
+        case db::Warning::Level::Warn:
+          warning_level = "WARNING";
+          break;
+        case db::Warning::Level::Error:
+          warning_level = "ERROR";
+          break;
+      }
+
+      m_warnings_callback(sql, warning->code, warning_level, warning->msg);
+
+      warning = res->fetch_one_warning();
+    }
+  }
+
+  return res;
 }
 
 void Instance::execute(const std::string &sql) const {
