@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -26,7 +26,9 @@
 #include <random>
 #include <string>
 #include <vector>
+#include "modules/adminapi/common/common.h"
 #include "mysqlshdk/libs/mysql/instance.h"
+#include "mysqlshdk/libs/utils/structured_text.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
@@ -56,7 +58,7 @@ std::string to_string(Replication_channel::Status status) {
 std::string to_string(Replication_channel::Receiver::State state) {
   switch (state) {
     case Replication_channel::Receiver::ON:
-      return "OK";
+      return "ON";
     case Replication_channel::Receiver::OFF:
       return "OFF";
     case Replication_channel::Receiver::CONNECTING:
@@ -65,10 +67,24 @@ std::string to_string(Replication_channel::Receiver::State state) {
   throw std::logic_error("internal error");
 }
 
+std::string to_string(Replication_channel::Receiver::Status status) {
+  switch (status) {
+    case Replication_channel::Receiver::Status::ON:
+      return "ON";
+    case Replication_channel::Receiver::Status::OFF:
+      return "OFF";
+    case Replication_channel::Receiver::Status::CONNECTING:
+      return "CONNECTING";
+    case Replication_channel::Receiver::Status::ERROR:
+      return "ERROR";
+  }
+  throw std::logic_error("internal error");
+}
+
 std::string to_string(Replication_channel::Coordinator::State state) {
   switch (state) {
     case Replication_channel::Coordinator::ON:
-      return "OK";
+      return "ON";
     case Replication_channel::Coordinator::OFF:
       return "OFF";
     case Replication_channel::Coordinator::NONE:
@@ -80,13 +96,29 @@ std::string to_string(Replication_channel::Coordinator::State state) {
 std::string to_string(Replication_channel::Applier::State state) {
   switch (state) {
     case Replication_channel::Applier::ON:
-      return "OK";
+      return "ON";
     case Replication_channel::Applier::OFF:
       return "OFF";
   }
   throw std::logic_error("internal error");
 }
 
+std::string to_string(Replication_channel::Applier::Status status) {
+  switch (status) {
+    case Replication_channel::Applier::Status::APPLYING:
+      return "APPLYING";
+    case Replication_channel::Applier::Status::APPLIED_ALL:
+      return "APPLIED_ALL";
+    case Replication_channel::Applier::Status::ON:
+      return "ON";
+    case Replication_channel::Applier::Status::ERROR:
+      return "ERROR";
+    case Replication_channel::Applier::Status::OFF:
+      return "OFF";
+  }
+  throw std::logic_error("internal error");
+}
+
 std::string to_string(const Replication_channel::Error &error) {
   if (error.code != 0) {
     return shcore::str_format("%s (%i) at %s", error.message.c_str(),
@@ -96,36 +128,45 @@ std::string to_string(const Replication_channel::Error &error) {
 }
 
 std::string format_status(const Replication_channel &channel, bool verbose) {
+  // This follows the KVP format
+
   std::string msg;
-  if (!verbose && channel.status() == Replication_channel::ON) {
-    msg = "Channel '" + channel.channel_name + "' is running.";
-  } else if (!verbose && channel.status() != Replication_channel::OFF) {
-    msg = "Channel '" + channel.channel_name + "' is stopped.";
-  } else {
-    msg = "Channel: " + channel.channel_name;
-    if (verbose) {
-      msg += "\nUser: " + channel.user;
-    }
-    msg += "\nReceiver state: " + to_string(channel.receiver.state);
+
+  msg += shcore::make_kvp(
+      "source", channel.host + ":" + std::to_string(channel.port), '"');
+  msg += " " + shcore::make_kvp("channel", channel.channel_name);
+  msg += " " + shcore::make_kvp("status", to_string(channel.status()));
+
+  if (verbose || channel.status() != Replication_channel::OFF) {
+    msg +=
+        " " + shcore::make_kvp("receiver", to_string(channel.receiver.state));
     if (channel.receiver.last_error.code != 0)
-      msg += "\n\tLast error: " + to_string(channel.receiver.last_error);
+      msg +=
+          " " + shcore::make_kvp("last_error",
+                                 to_string(channel.receiver.last_error), '"');
+
     if (channel.coordinator.state != Replication_channel::Coordinator::NONE) {
-      msg += "\nCoordinator state: " + to_string(channel.coordinator.state);
+      msg += " " + shcore::make_kvp("coordinator",
+                                    to_string(channel.coordinator.state));
       if (channel.coordinator.last_error.code != 0)
-        msg += "\n\tLast error: " + to_string(channel.coordinator.last_error);
+        msg += " " + shcore::make_kvp("last_error",
+                                      to_string(channel.coordinator.last_error),
+                                      '"');
     }
     if (channel.appliers.size() == 1) {
-      msg += "\nApplier state: " + to_string(channel.appliers.front().state);
+      msg += " " + shcore::make_kvp("applier",
+                                    to_string(channel.appliers.front().state));
       if (channel.appliers.front().last_error.code != 0)
-        msg +=
-            "\n\tLast error: " + to_string(channel.appliers.front().last_error);
+        msg += " " + shcore::make_kvp(
+                         "last_error",
+                         to_string(channel.appliers.front().last_error), '"');
     } else {
       int i = 0;
       for (const auto &a : channel.appliers) {
-        msg +=
-            "\nApplier " + std::to_string(i) + " state: " + to_string(a.state);
+        msg += " " + shcore::make_kvp(("applier" + std::to_string(i)).c_str(),
+                                      to_string(a.state));
         if (a.last_error.code != 0)
-          msg += "\n\tLast error: " + to_string(a.last_error);
+          msg += " " + shcore::make_kvp("last_error", to_string(a.last_error));
         ++i;
       }
     }
@@ -154,23 +195,44 @@ void unserialize_channel_info(const mysqlshdk::db::Row_ref_by_name &row,
   channel->source_uuid = row.get_string("source_uuid");
   channel->group_name =
       row.is_null("group_name") ? "" : row.get_string("group_name");
-  channel->last_heartbeat_timestamp =
-      row.get_string("last_heartbeat_timestamp");
+
+  channel->time_since_last_message =
+      row.has_field("time_since_last_message")
+          ? row.get_string("time_since_last_message", "")
+          : "";
+  channel->repl_lag_from_original =
+      row.has_field("lag_from_original")
+          ? row.get_string("lag_from_original", "")
+          : "";
+  channel->repl_lag_from_immediate =
+      row.has_field("lag_from_immediate")
+          ? row.get_string("lag_from_immediate", "")
+          : "";
+  channel->queued_gtid_set_to_apply =
+      row.get_string("queued_gtid_set_to_apply", "");
 
   if (!row.is_null("io_state")) {
     std::string state = row.get_string("io_state");
-    if (state == "ON")
+    if (state == "ON") {
       channel->receiver.state = Replication_channel::Receiver::ON;
-    else if (state == "OFF")
+      channel->receiver.status = Replication_channel::Receiver::Status::ON;
+    } else if (state == "OFF") {
       channel->receiver.state = Replication_channel::Receiver::OFF;
-    else if (state == "CONNECTING")
+      channel->receiver.status = Replication_channel::Receiver::Status::OFF;
+    } else if (state == "CONNECTING") {
       channel->receiver.state = Replication_channel::Receiver::CONNECTING;
-    else
+      channel->receiver.status =
+          Replication_channel::Receiver::Status::CONNECTING;
+    } else {
       assert(0);
+    }
 
     channel->receiver.thread_state = row.get_string("io_thread_state", "");
 
     extract_error(&channel->receiver.last_error, row, "io_");
+    if (channel->receiver.last_error.code != 0) {
+      channel->receiver.status = Replication_channel::Receiver::Status::ERROR;
+    }
   }
 
   if (!row.is_null("co_state")) {
@@ -200,18 +262,30 @@ void unserialize_channel_applier_info(const mysqlshdk::db::Row_ref_by_name &row,
     else
       assert(0);
 
+    if (applier.state == Replication_channel::Applier::ON) {
+      if (row.has_field("applier_busy_state")) {  // 8.0 only
+        if (row.get_string("applier_busy_state") == "APPLYING")
+          applier.status = Replication_channel::Applier::Status::APPLYING;
+        else
+          applier.status = Replication_channel::Applier::Status::APPLIED_ALL;
+      } else {
+        applier.status = Replication_channel::Applier::Status::ON;
+      }
+    } else {
+      if (applier.last_error.code != 0)
+        applier.status = Replication_channel::Applier::Status::ERROR;
+      else
+        applier.status = Replication_channel::Applier::Status::OFF;
+    }
+
     applier.thread_state = row.get_string("w_thread_state", "");
-    applier.last_applied_trx_delay = row.has_field("w_last_trx_delay")
-                                         ? row.get_uint("w_last_trx_delay")
-                                         : 0;
 
     extract_error(&applier.last_error, row, "w_");
     channel->appliers.push_back(applier);
   }
 }
 
-static const char *k_base_channel_query = R"*(
-  SELECT
+static const char *k_base_channel_query = R"*(SELECT
     c.channel_name, c.host, c.port, c.user,
     s.source_uuid, s.group_name, s.last_heartbeat_timestamp,
     s.service_state io_state, st.processlist_state io_thread_state,
@@ -222,10 +296,29 @@ static const char *k_base_channel_query = R"*(
     co.last_error_timestamp co_errtime,
     w.service_state w_state, wt.processlist_state w_thread_state,
     w.last_error_number w_errno, w.last_error_message w_errmsg,
-    w.last_error_timestamp w_errtime /*!80011 ,
-    CAST(w.last_applied_transaction_end_apply_timestamp
-      - w.last_applied_transaction_original_commit_timestamp as UNSIGNED)
-      as w_last_trx_delay */
+    w.last_error_timestamp w_errtime,
+    /*!80011 TIMEDIFF(NOW(6),
+      IF(TIMEDIFF(s.LAST_QUEUED_TRANSACTION_START_QUEUE_TIMESTAMP,
+          s.LAST_HEARTBEAT_TIMESTAMP) >= 0,
+        s.LAST_QUEUED_TRANSACTION_START_QUEUE_TIMESTAMP,
+        s.LAST_HEARTBEAT_TIMESTAMP
+      )) as time_since_last_message,
+    IF(s.LAST_QUEUED_TRANSACTION='' OR s.LAST_QUEUED_TRANSACTION=latest_w.LAST_APPLIED_TRANSACTION,
+      'IDLE',
+      'APPLYING') as applier_busy_state,
+    IF(s.LAST_QUEUED_TRANSACTION='' OR s.LAST_QUEUED_TRANSACTION=latest_w.LAST_APPLIED_TRANSACTION,
+      NULL,
+      TIMEDIFF(latest_w.LAST_APPLIED_TRANSACTION_END_APPLY_TIMESTAMP,
+        latest_w.LAST_APPLIED_TRANSACTION_ORIGINAL_COMMIT_TIMESTAMP)
+      ) as lag_from_original,
+    IF(s.LAST_QUEUED_TRANSACTION='' OR s.LAST_QUEUED_TRANSACTION=latest_w.LAST_APPLIED_TRANSACTION,
+      NULL,
+      TIMEDIFF(latest_w.LAST_APPLIED_TRANSACTION_END_APPLY_TIMESTAMP,
+        latest_w.LAST_APPLIED_TRANSACTION_IMMEDIATE_COMMIT_TIMESTAMP)
+      ) as lag_from_immediate,
+    */
+    GTID_SUBTRACT(s.RECEIVED_TRANSACTION_SET, @@global.gtid_executed)
+      as queued_gtid_set_to_apply
   FROM performance_schema.replication_connection_configuration c
   JOIN performance_schema.replication_connection_status s
     ON c.channel_name = s.channel_name
@@ -235,6 +328,13 @@ static const char *k_base_channel_query = R"*(
     ON c.channel_name = a.channel_name
   JOIN performance_schema.replication_applier_status_by_worker w
     ON c.channel_name = w.channel_name
+  LEFT JOIN
+  /* if parallel replication, fetch owner of most recently applied tx */
+    (SELECT *
+      FROM performance_schema.replication_applier_status_by_worker
+      /*!80011 ORDER BY LAST_APPLIED_TRANSACTION_END_APPLY_TIMESTAMP DESC */
+      LIMIT 1) latest_w
+    ON c.channel_name = latest_w.channel_name
   LEFT JOIN performance_schema.threads st
     ON s.thread_id = st.thread_id
   LEFT JOIN performance_schema.threads cot
@@ -270,6 +370,26 @@ std::vector<Replication_channel> get_incoming_channels(
   return channels;
 }
 
+std::vector<std::string> get_incoming_channel_names(
+    const mysqlshdk::mysql::IInstance &instance, bool include_stopped) {
+  const char *q_all =
+      "SELECT channel_name"
+      " FROM performance_schema.replication_connection_status";
+  const char *q =
+      "SELECT channel_name"
+      " FROM performance_schema.replication_connection_status"
+      " WHERE service_state <> 'OFF'";
+
+  std::vector<std::string> names;
+
+  auto res = instance.query(include_stopped ? q_all : q);
+  while (auto row = res->fetch_one()) {
+    names.emplace_back(row->get_string(0));
+  }
+
+  return names;
+}
+
 bool get_channel_status(const mysqlshdk::mysql::IInstance &instance,
                         const std::string &channel_name,
                         Replication_channel *out_channel) {
@@ -278,17 +398,158 @@ bool get_channel_status(const mysqlshdk::mysql::IInstance &instance,
       channel_name);
 
   if (auto row = result->fetch_one_named()) {
-    Replication_channel channel;
-    unserialize_channel_info(row, out_channel);
-    unserialize_channel_applier_info(row, out_channel);
-    while ((row = result->fetch_one_named())) {
-      if (out_channel) unserialize_channel_applier_info(row, out_channel);
+    if (out_channel) {
+      unserialize_channel_info(row, out_channel);
+      unserialize_channel_applier_info(row, out_channel);
+
+      while ((row = result->fetch_one_named())) {
+        unserialize_channel_applier_info(row, out_channel);
+      }
     }
     return true;
   }
   return false;
 }
 
+bool get_channel_state(const mysqlshdk::mysql::IInstance &instance,
+                       const std::string &channel_name,
+                       Replication_channel::Receiver::State *out_io_state,
+                       Replication_channel::Applier::State *out_sql_state) {
+  auto result =
+      instance.queryf("SHOW " + get_replica_keyword(instance.get_version()) +
+                          " STATUS FOR CHANNEL ?",
+                      channel_name);
+  if (auto row = result->fetch_one()) {
+    auto io_state = row->get_string(10);
+    auto sql_state = row->get_string(11);
+
+    if (io_state == "Yes") {
+      *out_io_state = Replication_channel::Receiver::ON;
+    } else if (io_state == "No") {
+      *out_io_state = Replication_channel::Receiver::OFF;
+    } else if (io_state == "Connecting") {
+      *out_io_state = Replication_channel::Receiver::ON;
+    } else {
+      throw std::logic_error("Unexpected value for Replica_IO_Running: " +
+                             io_state);
+    }
+
+    if (sql_state == "Yes") {
+      *out_sql_state = Replication_channel::Applier::ON;
+    } else if (sql_state == "No") {
+      *out_sql_state = Replication_channel::Applier::OFF;
+    } else {
+      throw std::logic_error("Unexpected value for Replica_SQL_Running: " +
+                             sql_state);
+    }
+    return true;
+  }
+  return false;
+}
+
+namespace {
+void unserialize_channel_master_info(
+    const mysqlshdk::db::Row_ref_by_name &row,
+    Replication_channel_master_info *out_master_info) {
+  out_master_info->master_log_name = row.get_string("Master_log_name");
+  out_master_info->master_log_pos = row.get_uint("Master_log_pos");
+  out_master_info->host = row.get_string("Host");
+  out_master_info->user_name = row.get_string("User_name");
+  out_master_info->port = row.get_int("Port");
+  out_master_info->connect_retry = row.get_uint("Connect_retry");
+  out_master_info->enabled_ssl = row.get_int("Enabled_ssl");
+  out_master_info->ssl_ca = row.get_string("Ssl_ca");
+  out_master_info->ssl_capath = row.get_string("Ssl_capath");
+  out_master_info->ssl_cert = row.get_string("Ssl_cert");
+  out_master_info->ssl_cipher = row.get_string("Ssl_cipher");
+  out_master_info->ssl_key = row.get_string("Ssl_key");
+  out_master_info->ssl_verify_server_cert =
+      row.get_int("Ssl_verify_server_cert");
+  out_master_info->heartbeat = row.get_double("Heartbeat");
+  out_master_info->bind = row.get_string("Bind");
+  out_master_info->ignored_server_ids = row.get_string("Ignored_server_ids");
+  out_master_info->uuid = row.get_string("Uuid");
+  out_master_info->retry_count = row.get_uint("Retry_count");
+  out_master_info->ssl_crl = row.get_string("Ssl_crl");
+  out_master_info->ssl_crlpath = row.get_string("Ssl_crlpath");
+  out_master_info->enabled_auto_position = row.get_int("Enabled_auto_position");
+  out_master_info->channel_name = row.get_string("Channel_name");
+  out_master_info->tls_version = row.get_string("Tls_version");
+
+  if (row.has_field("Public_key_path") && !row.is_null("Public_key_path"))
+    out_master_info->public_key_path = row.get_string("Public_key_path");
+  if (row.has_field("Get_public_key") && !row.is_null("Get_public_key"))
+    out_master_info->get_public_key = row.get_int("Get_public_key");
+  if (row.has_field("Network_namespace") && !row.is_null("Network_namespace"))
+    out_master_info->network_namespace = row.get_string("Network_namespace");
+  if (row.has_field("Master_compression_algorithm") &&
+      !row.is_null("Master_compression_algorithm"))
+    out_master_info->master_compression_algorithm =
+        row.get_string("Master_compression_algorithm");
+  if (row.has_field("Master_zstd_compression_level") &&
+      !row.is_null("Master_zstd_compression_level"))
+    out_master_info->master_zstd_compression_level =
+        row.get_uint("Master_zstd_compression_level");
+  if (row.has_field("Tls_ciphersuites") && !row.is_null("Tls_ciphersuites"))
+    out_master_info->tls_ciphersuites = row.get_string("Tls_ciphersuites");
+}
+
+void unserialize_channel_relay_log_info(
+    const mysqlshdk::db::Row_ref_by_name &row,
+    Replication_channel_relay_log_info *out_relay_log_info) {
+  out_relay_log_info->relay_log_name = row.get_string("Relay_log_name", "");
+  out_relay_log_info->relay_log_pos = row.get_uint("Relay_log_pos", 0);
+  out_relay_log_info->master_log_name = row.get_string("Master_log_name");
+  out_relay_log_info->master_log_pos = row.get_uint("Master_log_pos");
+  out_relay_log_info->sql_delay = row.get_uint("Sql_delay");
+  out_relay_log_info->number_of_workers = row.get_uint("Number_of_workers");
+  out_relay_log_info->id = row.get_uint("Id");
+  out_relay_log_info->channel_name = row.get_string("Channel_name");
+  if (row.has_field("Privilege_checks_username") &&
+      !row.is_null("Privilege_checks_username"))
+    out_relay_log_info->privilege_checks_username =
+        row.get_string("Privilege_checks_username");
+  if (row.has_field("Privilege_checks_hostname") &&
+      !row.is_null("Privilege_checks_hostname"))
+    out_relay_log_info->privilege_checks_hostname =
+        row.get_string("Privilege_checks_hostname");
+  if (row.has_field("Require_row_format") && !row.is_null("Require_row_format"))
+    out_relay_log_info->require_row_format = row.get_int("Require_row_format");
+}
+
+}  // namespace
+
+bool get_channel_info(const mysqlshdk::mysql::IInstance &instance,
+                      const std::string &channel_name,
+                      Replication_channel_master_info *out_master_info,
+                      Replication_channel_relay_log_info *out_relay_log_info) {
+  assert(out_master_info && out_relay_log_info);
+
+  {
+    auto result = instance.queryf(
+        "SELECT * FROM mysql.slave_master_info"
+        " WHERE channel_name = ?",
+        channel_name);
+    if (auto row = result->fetch_one_named()) {
+      unserialize_channel_master_info(row, out_master_info);
+    } else {
+      return false;
+    }
+  }
+
+  {
+    auto result = instance.queryf(
+        "SELECT * FROM mysql.slave_relay_log_info"
+        " WHERE channel_name = ?",
+        channel_name);
+    if (auto row = result->fetch_one_named()) {
+      unserialize_channel_relay_log_info(row, out_relay_log_info);
+    }
+  }
+
+  return true;
+}
+
 std::vector<Slave_host> get_slaves(
     const mysqlshdk::mysql::IInstance &instance) {
   std::vector<Slave_host> slaves;
@@ -302,13 +563,19 @@ std::vector<Slave_host> get_slaves(
     slaves.push_back(host);
   }
 
+  // Sort by host/name to ensure deterministic output
+  std::sort(slaves.begin(), slaves.end(),
+            [](const Slave_host &a, const Slave_host &b) {
+              return a.host < b.host || (a.host == b.host && a.port < b.port);
+            });
+
   return slaves;
 }
 
 Replication_channel::Status Replication_channel::status() const {
   bool applier_off = false;
 
-  int num_applier_off = 0;
+  size_t num_applier_off = 0;
   for (const auto &applier : appliers) {
     if (applier.state == Applier::OFF) {
       if (applier.last_error.code != 0) {
@@ -319,7 +586,7 @@ Replication_channel::Status Replication_channel::status() const {
       }
     }
   }
-  if (num_applier_off == static_cast<int>(appliers.size()))
+  if (num_applier_off == appliers.size())
     applier_off = true;
   else if (num_applier_off > 0)
     return APPLIER_ERROR;  // some ON and some OFF is treated as ERROR
@@ -335,10 +602,10 @@ Replication_channel::Status Replication_channel::status() const {
     }
   }
 
+  if (receiver.last_error.code != 0) {
+    return CONNECTION_ERROR;
+  }
   if (receiver.state == Receiver::OFF) {
-    if (receiver.last_error.code != 0) {
-      return CONNECTION_ERROR;
-    }
     if (applier_off) return OFF;
     return RECEIVER_OFF;
   } else if (receiver.state == Receiver::CONNECTING) {
@@ -353,13 +620,48 @@ Replication_channel::Status Replication_channel::status() const {
   return ON;
 }
 
-Replication_channel wait_replication_done_connecting(
-    const mysqlshdk::mysql::IInstance &slave, const std::string &channel_name,
-    int timeout) {
-  time_t start_time = time(nullptr);
-  // check that the connection thread for the channel switches to ON
-  // and if not, throw an exception
+Replication_channel::Status Replication_channel::applier_status() const {
+  bool applier_off = false;
 
+  size_t num_applier_off = 0;
+  for (const auto &applier : appliers) {
+    if (applier.state == Applier::OFF) {
+      if (applier.last_error.code != 0) {
+        // error is the strongest status
+        return APPLIER_ERROR;
+      } else {
+        num_applier_off++;
+      }
+    }
+  }
+  if (num_applier_off == appliers.size())
+    applier_off = true;
+  else if (num_applier_off > 0)
+    return APPLIER_ERROR;  // some ON and some OFF is treated as ERROR
+
+  if (coordinator.state == Coordinator::OFF) {
+    if (coordinator.last_error.code != 0) {
+      return APPLIER_ERROR;
+    }
+    // If we have a coordinator and it's OFF but the appliers are not, then
+    // it's an error
+    if (!applier_off) {
+      return APPLIER_ERROR;
+    }
+  }
+
+  if (applier_off) {
+    return APPLIER_OFF;
+  }
+
+  return ON;
+}
+
+Replication_channel wait_replication_done_connecting(
+    const mysqlshdk::mysql::IInstance &slave, const std::string &channel_name) {
+  // Check that the IO thread for the channel switches to ON/OFF.
+  // NOTE: It is assumed that the IO thread will not stay forever with the
+  //       state CONNECTING and will eventually switch to ON or OFF (stopped).
   Replication_channel channel;
 
   do {
@@ -375,9 +677,7 @@ Replication_channel wait_replication_done_connecting(
         return channel;
       }
     }
-  } while (time(nullptr) - start_time < timeout);
-
-  return channel;
+  } while (true);
 }
 
 bool wait_for_gtid_set(const mysqlshdk::mysql::IInstance &instance,
@@ -412,7 +712,7 @@ size_t estimate_gtid_set_size(const std::string &gtid_set) {
         size_t p = s.find(':');
         if (p != std::string::npos) {
           size_t begin, end;
-          switch (sscanf(&s[p + 1], "%zd-%zd", &begin, &end)) {
+          switch (sscanf(&s[p + 1], "%zu-%zu", &begin, &end)) {
             case 2:
               count += end - begin + 1;
               break;
@@ -427,19 +727,9 @@ size_t estimate_gtid_set_size(const std::string &gtid_set) {
   return count;
 }
 
-std::string get_executed_gtid_set(const mysqlshdk::mysql::IInstance &server,
-                                  bool include_received) {
+std::string get_executed_gtid_set(const mysqlshdk::mysql::IInstance &server) {
   // session GTID_EXECUTED is different from the global one in 5.7
-  if (include_received) {
-    return server.queryf_one_string(
-        0, "",
-        "SELECT CONCAT(@@GLOBAL.GTID_EXECUTED, "
-        "   (SELECT "
-        "     COALESCE(CONCAT(',', group_concat(received_transaction_set)), '')"
-        "    FROM performance_schema.replication_connection_status))");
-  } else {
-    return server.queryf_one_string(0, "", "SELECT @@GLOBAL.GTID_EXECUTED");
-  }
+  return server.queryf_one_string(0, "", "SELECT @@GLOBAL.GTID_EXECUTED");
 }
 
 std::string get_purged_gtid_set(const mysqlshdk::mysql::IInstance &server) {
@@ -450,12 +740,25 @@ std::string get_received_gtid_set(const mysqlshdk::mysql::IInstance &server,
                                   const std::string &channel_name) {
   return server.queryf_one_string(
       0, "",
-      "SELECT GROUP_CONCAT(received_transaction_set)"
+      "SELECT GTID_SUBTRACT(GROUP_CONCAT(received_transaction_set), '')"
       "   FROM performance_schema.replication_connection_status"
       "   WHERE channel_name = ?",
       channel_name);
 }
 
+std::string get_total_gtid_set(
+    const mysqlshdk::mysql::IInstance &server,
+    const std::vector<std::string> &known_channel_names) {
+  return server.queryf_one_string(
+      0, "",
+      "SELECT GTID_SUBTRACT(CONCAT(@@GLOBAL.GTID_EXECUTED, ',', ("
+      "   SELECT GROUP_CONCAT(received_transaction_set)"
+      "       FROM performance_schema.replication_connection_status"
+      "       WHERE channel_name IN (" +
+          shcore::str_join(known_channel_names, ",", shcore::quote_sql_string) +
+          "))), '')");
+}
+
 Gtid_set_relation compare_gtid_sets(const mysqlshdk::mysql::IInstance &server,
                                     const std::string &gtidset_a,
                                     const std::string &gtidset_b,
@@ -503,13 +806,30 @@ Gtid_set_relation compare_gtid_sets(const mysqlshdk::mysql::IInstance &server,
   }
 }
 
+std::string to_string(Replica_gtid_state state) {
+  switch (state) {
+    case Replica_gtid_state::NEW:
+      return "NEW";
+    case Replica_gtid_state::IDENTICAL:
+      return "IDENTICAL";
+    case Replica_gtid_state::RECOVERABLE:
+      return "RECOVERABLE";
+    case Replica_gtid_state::IRRECOVERABLE:
+      return "IRRECOVERABLE";
+    case Replica_gtid_state::DIVERGED:
+      return "DIVERGED";
+  }
+  throw std::logic_error("internal error");
+}
+
 Replica_gtid_state check_replica_gtid_state(
     const mysqlshdk::mysql::IInstance &master,
-    const mysqlshdk::mysql::IInstance &slave, bool include_received,
-    std::string *out_missing_gtids, std::string *out_errant_gtids) {
-  auto master_gtid = get_executed_gtid_set(master, include_received);
+    const mysqlshdk::mysql::IInstance &slave, std::string *out_missing_gtids,
+    std::string *out_errant_gtids) {
+  auto slave_gtid = get_executed_gtid_set(slave);
+  auto master_gtid = get_executed_gtid_set(master);
   auto master_purged_gtid = get_purged_gtid_set(master);
-  auto slave_gtid = get_executed_gtid_set(slave, include_received);
+
   return check_replica_gtid_state(master, master_gtid, master_purged_gtid,
                                   slave_gtid, out_missing_gtids,
                                   out_errant_gtids);
@@ -520,20 +840,28 @@ Replica_gtid_state check_replica_gtid_state(
     const std::string &master_gtidset, const std::string &master_purged_gtidset,
     const std::string &slave_gtidset, std::string *out_missing_gtids,
     std::string *out_errant_gtids) {
+  if (slave_gtidset.empty() && master_purged_gtidset.empty() &&
+      !master_gtidset.empty()) {
+    if (out_missing_gtids) {
+      *out_missing_gtids = master_gtidset;
+    }
+    if (out_errant_gtids) {
+      *out_errant_gtids = "";
+    }
+    return Replica_gtid_state::NEW;
+  }
+
   Gtid_set_relation rel =
       compare_gtid_sets(instance, master_gtidset, slave_gtidset,
                         out_errant_gtids, out_missing_gtids);
 
   switch (rel) {
-    case Gtid_set_relation::DISJOINT:
     case Gtid_set_relation::INTERSECTS:
+    case Gtid_set_relation::DISJOINT:
     case Gtid_set_relation::CONTAINED:
       return Replica_gtid_state::DIVERGED;
 
     case Gtid_set_relation::EQUAL:
-      if (slave_gtidset.empty()) {
-        return Replica_gtid_state::NEW;
-      }
       return Replica_gtid_state::IDENTICAL;
 
     case Gtid_set_relation::CONTAINS:
@@ -545,9 +873,6 @@ Replica_gtid_state check_replica_gtid_state(
                       slave_gtidset)
               ->fetch_one_or_throw()
               ->get_int(0)) {
-        if (slave_gtidset.empty()) {
-          return Replica_gtid_state::NEW;
-        }
         return Replica_gtid_state::RECOVERABLE;
       } else {
         return Replica_gtid_state::IRRECOVERABLE;
@@ -566,54 +891,37 @@ int64_t generate_server_id() {
   return distribution(rnd_gen);
 }
 
-bool is_async_replication_running(const mysqlshdk::mysql::IInstance &instance) {
-  auto session = instance.get_session();
-  std::string receiver_channel_state, applier_channel_state;
-
-  assert(session);
-
-  try {
-    // Get the state of the receiver and applier channels
-    // NOTE:
-    // - Values of receiver channel can be: ON, OFF, or CONNECTING
-    // - Values of applier channel can be: ON, OFF
-    std::string query(
-        "SELECT a.SERVICE_STATE AS RECEIVER, b.SERVICE_STATE "
-        "AS APPLIER FROM "
-        "performance_schema.replication_connection_status a, "
-        "performance_schema.replication_applier_status b WHERE "
-        "a.CHANNEL_NAME != 'group_replication_applier' AND "
-        "a.CHANNEL_NAME != 'group_replication_recovery' AND "
-        "b.CHANNEL_NAME != 'group_replication_applier' AND "
-        "b.CHANNEL_NAME != 'group_replication_recovery'");
-
-    log_debug("Executing query '%s'.", query.c_str());
-
-    auto resultset = session->query(query);
-    auto row = resultset->fetch_one();
-
-    // If the query returned no values it means async replication channels are
-    // not set
-    if (!row) {
-      log_debug("Query returned no results.");
-      return false;
-    }
-
-    receiver_channel_state = row->get_string(0);
-    applier_channel_state = row->get_string(1);
-
-    // If any of the channels is running, we can consider async replication
-    // is running
-    if ((receiver_channel_state != "OFF") || (applier_channel_state != "OFF")) {
-      return true;
-    } else {
-      return false;
-    }
-  } catch (const std::exception &e) {
-    log_error("Error checking asynchronous replication status: %s", e.what());
-    throw;
+std::string get_replica_keyword(const mysqlshdk::utils::Version &version) {
+  if (version < mysqlshdk::utils::Version(8, 0, 22)) {
+    return "SLAVE";
+  } else {
+    return "REPLICA";
   }
 }
 
+std::string get_replication_source_keyword(
+    const mysqlshdk::utils::Version &version, bool command) {
+  if (version < mysqlshdk::utils::Version(8, 0, 23)) {
+    return "MASTER";
+  } else {
+    return (command == true ? "REPLICATION SOURCE" : "SOURCE");
+  }
+}
+
+std::string get_replication_option_keyword(
+    const mysqlshdk::utils::Version &version, const std::string &option) {
+  std::string ret = option;
+
+  if (version >= mysqlshdk::utils::Version(8, 0, 26)) {
+    ret = shcore::str_replace(ret, "slave", "replica");
+    ret = shcore::str_replace(ret, "master", "source");
+  } else {
+    ret = shcore::str_replace(ret, "replica", "slave");
+    ret = shcore::str_replace(ret, "source", "master");
+  }
+
+  return ret;
+}
+
 }  // namespace mysql
 }  // namespace mysqlshdk
