@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -35,26 +35,114 @@
 #endif
 
 #include <deque>
+#include <mutex>
 #include <string>
+
+#include "mysqlshdk/include/scripting/type_info/custom.h"
+#include "mysqlshdk/include/scripting/type_info/generic.h"
+#include "mysqlshdk/include/scripting/types.h"
 #include "mysqlshdk/libs/utils/strformat.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 #include "mysqlshdk/shellcore/shell_console.h"
 
 namespace shcore {
+namespace {
+std::string hexify(const std::string &data) {
+  if (data.size() == 0) {
+    return std::string{};
+  }
+  std::string s(3 * data.size(), 'x');
+
+  std::string::iterator k = s.begin();
+
+  for (const unsigned char i : data) {
+    *k++ = "0123456789abcdef"[i >> 4];
+    *k++ = "0123456789abcdef"[i & 0x0F];
+    *k++ = ' ';
+  }
+  s.resize(3 * data.size() - 1);
+  return s;
+}
+}  // namespace
+
+const shcore::Option_pack_def<Document_reader_options>
+    &Document_reader_options::options() {
+  static const auto opts =
+      shcore::Option_pack_def<Document_reader_options>()
+          .optional("convertBsonTypes",
+                    &Document_reader_options::convert_bson_types)
+          .optional("convertBsonOid", &Document_reader_options::convert_bson_id)
+          .optional("extractOidTime",
+                    &Document_reader_options::extract_oid_time)
+          .optional("ignoreDate", &Document_reader_options::ignore_date)
+          .optional("ignoreTimestamp",
+                    &Document_reader_options::ignore_timestamp)
+          .optional("ignoreBinary", &Document_reader_options::ignore_binary)
+          .optional("ignoreRegex", &Document_reader_options::ignore_regexp)
+          .optional("ignoreRegexOptions",
+                    &Document_reader_options::ignore_regexp_options)
+          .optional("decimalAsDouble",
+                    &Document_reader_options::decimal_as_double)
+          .on_done(&Document_reader_options::on_unpacked_options);
+
+  return opts;
+}
+
+void Document_reader_options::on_unpacked_options() {
+  // The default value for convert_bson_id is the value of convert_bson_types
+  if (convert_bson_id.is_null()) convert_bson_id = convert_bson_types;
+
+  if (!extract_oid_time.is_null() && !convert_bson_id.get_safe(false)) {
+    throw shcore::Exception::argument_error(
+        "The 'extractOidTime' option can not be used if 'convertBsonOid' is "
+        "disabled.");
+  }
+
+  std::vector<std::string> used_options;
+
+  if (!ignore_date.is_null()) used_options.push_back("ignoreDate");
+  if (!ignore_timestamp.is_null()) used_options.push_back("ignoreTimestamp");
+  if (!ignore_binary.is_null()) used_options.push_back("ignoreBinary");
+  if (!ignore_regexp.is_null()) used_options.push_back("ignoreRegex");
+  if (!ignore_regexp_options.is_null())
+    used_options.push_back("ignoreRegexOptions");
+  if (!decimal_as_double.is_null()) used_options.push_back("decimalAsDouble");
+
+  if (!used_options.empty() && !convert_bson_types.get_safe(false)) {
+    throw shcore::Exception::argument_error(shcore::str_format(
+        "The following option%s can not be used if 'convertBsonTypes' is "
+        "disabled: %s",
+        used_options.size() > 1 ? "s" : "",
+        shcore::str_join(used_options, ", ", [](const std::string &data) {
+          return "'" + data + "'";
+        }).c_str()));
+  }
+
+  if (ignore_regexp.get_safe(false) && ignore_regexp_options.get_safe(false)) {
+    throw shcore::Exception::argument_error(
+        "The 'ignoreRegex' and 'ignoreRegexOptions' options can't both be "
+        "enabled");
+  }
+
+  if (!extract_oid_time.is_null() && (*extract_oid_time).empty()) {
+    throw shcore::Exception::runtime_error(
+        "Option 'extractOidTime' can not be empty.");
+  }
+}
 
 bool Document_reader_options::ignore_type(Bson_type type) const {
-  if (convert_bson_types) {
+  if (convert_bson_types.get_safe(false)) {
     switch (type) {
       case Bson_type::OBJECT_ID:
-        return !convert_bson_id;
+        return !convert_bson_id.get_safe(false);
       case Bson_type::DATE:
-        return ignore_date;
+        return ignore_date.get_safe(false);
       case Bson_type::TIMESTAMP:
-        return ignore_timestamp;
+        return ignore_timestamp.get_safe(false);
       case Bson_type::REGEX:
-        return ignore_regexp;
+        return ignore_regexp.get_safe(false);
       case Bson_type::BINARY:
-        return ignore_binary;
+        return ignore_binary.get_safe(false);
       case Bson_type::LONG:
       case Bson_type::INTEGER:
       case Bson_type::DECIMAL:
@@ -63,7 +151,8 @@ bool Document_reader_options::ignore_type(Bson_type type) const {
         break;
     }
   } else {
-    return type == Bson_type::OBJECT_ID ? !convert_bson_id : true;
+    return type == Bson_type::OBJECT_ID ? !convert_bson_id.get_safe(false)
+                                        : true;
   }
   return true;
 }
@@ -76,6 +165,52 @@ std::string Json_reader::next() {
   return parser.parse();
 }
 
+void Json_reader::parse_bom() {
+  std::string header;
+  header.reserve(4);
+  for (int i = 0; i < 4; i++) {
+    const auto c = m_source->peek();
+    if (c == '{' || ::isspace(c) || m_source->eof()) {
+      break;
+    } else {
+      header += m_source->get();
+    }
+  }
+
+  if (header.size() == 0) {
+    return;
+  }
+
+  if (std::string{'\xef', '\xbb', '\xbf'}.compare(header) == 0) {
+    // nop
+  } else if (std::string{'\x00', '\x00', '\xfe', '\xff'}.compare(header) == 0) {
+    // utf-32 be
+    throw std::runtime_error("UTF-32BE encoded document is not supported.");
+  } else if (std::string{'\xff', '\xfe', '\x00', '\x00'}.compare(header) == 0) {
+    // utf-32 le
+    throw std::runtime_error("UTF-32LE encoded document is not supported.");
+  } else if (std::string{'\xfe', '\xff'}.compare(header) == 0) {
+    // utf-16 be
+    throw std::runtime_error("UTF-16BE encoded document is not supported.");
+  } else if (std::string{'\xff', '\xfe'}.compare(header) == 0) {
+    // utf-16 le
+    throw std::runtime_error("UTF-16LE encoded document is not supported.");
+  } else {
+    throw std::runtime_error("JSON document contains invalid bytes (" +
+                             hexify(header) + ") at the begining of the file.");
+  }
+}
+
+Document_parser::Document_parser(Buffered_input *input,
+                                 const Document_reader_options &options,
+                                 size_t depth, bool as_array,
+                                 const std::string context)
+    : m_source(input),
+      m_options(options),
+      m_depth(depth),
+      m_as_array(as_array),
+      m_context(context) {}
+
 void Json_document_parser::throw_premature_end() {
   throw invalid_json("Premature end of input stream", m_source->offset());
 }
@@ -161,7 +296,8 @@ void Json_document_parser::parse(std::string *document) {
       get_string(m_document);
       m_last_attribute_end = m_document->size() - 1;
 
-      if (m_options.convert_bson_types || m_options.convert_bson_id) {
+      if (m_options.convert_bson_types.get_safe(false) ||
+          m_options.convert_bson_id.get_safe(false)) {
         auto type = get_bson_type();
         // If the first field is a mongo special field
         // The original document is translated based on
@@ -506,14 +642,14 @@ void Json_document_parser::parse_bson_decimal() {
 
   std::string decimal_str;
   std::string *custom_target =
-      m_options.decimal_as_double ? &decimal_str : m_document;
+      m_options.decimal_as_double.get_safe(false) ? &decimal_str : m_document;
   double number;
 
   // Gets the associated value
   get_bson_data({{':'}, {'N', "", custom_target, &number}, {'}'}},
                 "processing extended JSON for $numberDecimal");
 
-  if (m_options.decimal_as_double) {
+  if (m_options.decimal_as_double.get_safe(false)) {
     m_document->append(std::to_string(number));
   }
 }
@@ -617,7 +753,7 @@ void Json_document_parser::parse_bson_regex() {
   // Unquotes the options
   options = options.substr(1, options.size() - 2);
 
-  if (m_options.ignore_regexp_options) {
+  if (m_options.ignore_regexp_options.get_safe(true)) {
     m_document->append(regex);
     if (!options.empty()) {
       std::string warning = "The regular expression for " + m_context +
