@@ -26,6 +26,7 @@
 
 #include <algorithm>
 #include <cassert>
+#include <cstring>
 #include <functional>
 #include <map>
 #include <memory>
@@ -62,7 +63,7 @@ class Generic_option {
  public:
   Generic_option(const std::string &name, const char *environment_variable,
                  std::vector<std::string> &&command_line_names,
-                 const char *help);
+                 const std::string &help);
 
   virtual ~Generic_option() {}
 
@@ -96,7 +97,7 @@ class Generic_option {
 
   Source get_source() const { return source; }
 
-  const char *get_help() const { return help; }
+  const std::string &get_help() const { return help; }
 
   virtual void set(const std::string &new_value, Source source) = 0;
 
@@ -107,7 +108,7 @@ class Generic_option {
   std::vector<std::string> command_line_names;
   bool value_optional = true;
   bool no_cmdline_value = true;
-  const char *help;
+  const std::string help;
 };
 
 template <class T>
@@ -118,26 +119,33 @@ class Concrete_option : public Generic_option {
 
   using Serializer = typename std::function<std::string(const T &)>;
 
-  Concrete_option(T *landing_spot, T default_value, const std::string &name,
-                  const char *environment_variable,
-                  std::vector<std::string> &&command_line_names,
-                  const char *help, Validator validator,
-                  Serializer serializer = nullptr)
+  Concrete_option(T *landing_spot_, T default_value_, const std::string &name_,
+                  const char *environment_variable_,
+                  std::vector<std::string> &&command_line_names_,
+                  const std::string &help_, Validator validator_,
+                  Serializer serializer_ = nullptr)
       : Generic_option(
-            name, environment_variable,
-            std::forward<std::vector<std::string> &&>(command_line_names),
-            help),
-        landing_spot(landing_spot),
-        default_value(default_value),
-        validator(validator),
-        serializer(serializer) {
+            name_, environment_variable_,
+            std::forward<std::vector<std::string> &&>(command_line_names_),
+            help_),
+        landing_spot(landing_spot_),
+        default_value(default_value_),
+        validator(validator_),
+        serializer(serializer_) {
     assert(validator != nullptr);
     *landing_spot = default_value;
   }
 
-  void set(const std::string &new_value, Source source) override {
-    *landing_spot = validator(new_value, source);
-    this->source = source;
+  void set(const std::string &new_value, Source source_) override {
+    try {
+      *landing_spot = validator(new_value, source_);
+      this->source = source_;
+    } catch (const std::exception &e) {
+      if (name.empty() || source == Source::Command_line ||
+          strstr(e.what(), name.c_str()))
+        throw std::invalid_argument(e.what());
+      throw std::invalid_argument(name + ": " + e.what());
+    }
   }
 
   std::string get_value_as_string() const override {
@@ -161,13 +169,19 @@ class Concrete_option : public Generic_option {
         value = "";
       else
         throw std::invalid_argument("Option " + option + " needs value");
+    } else if (strcmp(value, "") == 0 && !std::is_same<T, std::string>::value) {
+      throw std::invalid_argument("Option " + option +
+                                  " does not accept empty string as a value");
     }
     try {
       set(value, Source::Command_line);
     } catch (const std::invalid_argument &e) {
       std::string err_msg(e.what());
-      auto it = err_msg.find(name);
-      if (it != std::string::npos) err_msg.replace(it, name.length(), option);
+      std::string::size_type it = std::string::npos;
+      if (!name.empty() && (it = err_msg.find(name)) != std::string::npos)
+        err_msg.replace(it, name.length(), option);
+      else
+        err_msg = option + ": " + err_msg;
       throw std::invalid_argument(err_msg);
     }
   }
@@ -195,8 +209,9 @@ class Proxy_option : public Generic_option {
       std::function<void(const std::string &optname, const char *new_value)>;
 
   Proxy_option(const char *environment_variable,
-               std::vector<std::string> &&command_line_names, const char *help,
-               Handler handler = nullptr, const std::string &name = "");
+               std::vector<std::string> &&command_line_names,
+               const std::string &help, Handler handler = nullptr,
+               const std::string &name = "");
 
   void set(const std::string &new_value, Source source) override;
 
@@ -222,9 +237,9 @@ Proxy_option::Handler assign_value(T *landing_spot, S value) {
 }
 
 template <class T>
-T convert(const std::string &data) {
+T convert(const std::string &data, Source s) {
   // assuming that option specification turns it on
-  if (data.empty()) return static_cast<T>(1);
+  if (data.empty() && s == Source::Command_line) return static_cast<T>(1);
   T t;
   std::istringstream iss(data);
   iss >> t;
@@ -241,7 +256,7 @@ T convert(const std::string &data) {
 }
 
 template <>
-std::string convert(const std::string &data);
+std::string convert(const std::string &data, Source);
 
 /// Standard serialization mechanism for options
 template <class T>
@@ -258,7 +273,9 @@ std::string serialize(const std::string &val);
 /// Validator that does some standard type conversion
 template <class T>
 struct Basic_type {
-  T operator()(const std::string &data, Source) { return convert<T>(data); }
+  T operator()(const std::string &data, Source s) {
+    return convert<T>(data, s);
+  }
 };
 
 /// Wrapper for validator that prohibits user from setting option value
@@ -274,7 +291,7 @@ struct Read_only : public Basic_type<T> {
 /// Extension of Basic_type validator to enable range checks
 template <class T>
 struct Range : public Basic_type<T> {
-  Range(T min, T max) : min(min), max(max) { assert(max >= min); }
+  Range(T min_, T max_) : min(min_), max(max_) { assert(max >= min); }
 
   T operator()(const std::string &data, Source source) {
     T t = Basic_type<T>::operator()(data, source);
@@ -296,7 +313,7 @@ class Options {
     template <class T, class S>
     Add_named_option_helper &operator()(
         T *landing_spot, S &&default_value, const std::string &optname,
-        const char *help,
+        const std::string &help,
         typename opts::Concrete_option<T>::Validator validator =
             opts::Basic_type<T>(),
         typename opts::Concrete_option<T>::Serializer serializer =
@@ -308,7 +325,7 @@ class Options {
     template <class T, class S>
     Add_named_option_helper &operator()(
         T *landing_spot, S &&default_value, const std::string &optname,
-        std::vector<std::string> &&command_line_names, const char *help,
+        std::vector<std::string> &&command_line_names, const std::string &help,
         typename opts::Concrete_option<T>::Validator validator =
             opts::Basic_type<T>(),
         typename opts::Concrete_option<T>::Serializer serializer =
@@ -322,7 +339,7 @@ class Options {
     Add_named_option_helper &operator()(
         T *landing_spot, S &&default_value, const std::string &optname,
         const char *envname, std::vector<std::string> &&command_line_names,
-        const char *help,
+        const std::string &help,
         typename opts::Concrete_option<T>::Validator validator =
             opts::Basic_type<T>(),
         typename opts::Concrete_option<T>::Serializer serializer =
@@ -342,7 +359,7 @@ class Options {
     template <class T, class S>
     Add_startup_option_helper &operator()(
         T *landing_spot, S &&default_value,
-        std::vector<std::string> &&command_line_names, const char *help,
+        std::vector<std::string> &&command_line_names, const std::string &help,
         typename opts::Concrete_option<T>::Validator validator =
             opts::Basic_type<T>()) {
       return (*this)(landing_spot, std::forward<S>(default_value), nullptr,
@@ -352,7 +369,7 @@ class Options {
     template <class T, class S>
     Add_startup_option_helper &operator()(
         T *landing_spot, S &&default_value, const char *envname,
-        std::vector<std::string> &&command_line_names, const char *help,
+        std::vector<std::string> &&command_line_names, const std::string &help,
         typename opts::Concrete_option<T>::Validator validator =
             opts::Basic_type<T>()) {
       parent.add_option(new opts::Concrete_option<T>(
@@ -364,7 +381,7 @@ class Options {
     // Proxy options - custom cmdline handle/just help definition.
     Add_startup_option_helper &operator()(
         std::vector<std::string> &&command_line_names,
-        const char *help,  // full help text
+        const std::string &help,  // full help text
         opts::Proxy_option::Handler handler = nullptr,
         const std::string &name = "") {
       parent.add_option(new opts::Proxy_option(
@@ -376,26 +393,17 @@ class Options {
     Add_startup_option_helper &operator()(
         std::vector<std::string> &&command_line_names,
         opts::Proxy_option::Handler handler = nullptr) {
-      return (*this)(std::move(command_line_names), nullptr, handler);
+      return (*this)(std::move(command_line_names), "", handler);
     }
 
     Options &parent;
   };
 
  public:
-  enum class Format {
-    INVALID,        /*!< Given argument does not match the expected one(s) */
-    MISSING_VALUE,  /*!< Given argument matches the expected one(s) but the
-                       required value is missing */
-    SEPARATE_VALUE, /*!< --option <value> or -o <value> */
-    SHORT,          /*!< -o<value> */
-    LONG            /*!< --option=<value> */
-  };
-
   class Cmdline_iterator {
    public:
-    Cmdline_iterator(int argc, char const *const *argv, int start)
-        : argv(argv), argc(argc), current(start) {}
+    Cmdline_iterator(int argc_, char const *const *argv_, int start)
+        : argv(argv_), argc(argc_), current(start) {}
 
     /// Return current cmdline argument.
     const char *peek() const { return argv[current]; }
@@ -417,12 +425,60 @@ class Options {
     int current;
   };
 
-  using Custom_cmdline_handler = std::function<bool(Cmdline_iterator *)>;
+  class Iterator {
+   public:
+    enum class Type {
+      VALUE,          /*!< value which does not begin with '-' */
+      NO_VALUE,       /*!< --option or -o but not followed with value */
+      SEPARATE_VALUE, /*!< --option <value> or -o <value> */
+      SHORT,          /*!< -o<value> */
+      LONG            /*!< --option=<value> */
+    };
 
-  static Format cmdline_arg_with_value(Cmdline_iterator *iterator,
-                                       const char *arg, const char *larg,
-                                       const char **value,
-                                       bool accept_null = false) noexcept;
+    explicit Iterator(const Options::Cmdline_iterator &iterator);
+
+    bool valid() const { return m_iterator.valid(); }
+
+    Type type() const { return m_type; }
+
+    std::string option() const { return m_option; }
+
+    const char *value() const { return m_value; }
+
+    bool has_non_empty_value() const {
+      return nullptr != value() && '\0' != value()[0];
+    }
+
+    Options::Cmdline_iterator *iterator() { return &m_iterator; }
+
+    /**
+     * Moves iterator to the next option without consuming the value.
+     *
+     * @throws std::logic_error if iterator is not valid
+     * @throws std::logic_error if type() is Type::LONG
+     */
+    void next_no_value();
+
+    /**
+     * Moves iterator to the next option.
+     *
+     * @throws std::logic_error if iterator is not valid
+     */
+    void next();
+
+   private:
+    void get_data();
+
+    void get_separate_value();
+
+    Options::Cmdline_iterator m_iterator;
+    std::string m_option;
+    const char *m_value;
+    Type m_type;
+    size_t m_short_value_pos = 0;
+  };
+
+  using Custom_cmdline_handler = std::function<bool(Iterator *)>;
 
   explicit Options(const std::string &config_file = "");
   virtual ~Options() {}
