@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -22,6 +22,9 @@
  */
 
 #include "mysqlshdk/libs/utils/utils_general.h"
+
+#include <utility>
+
 #include "mysqlshdk/libs/textui/textui.h"
 
 #include "include/mysh_config.h"
@@ -54,6 +57,7 @@ errno_t memset_s(void *__s, rsize_t __smax, int __c, rsize_t __n);
 #include <ctime>
 #include <locale>
 
+#include "mysqlshdk/include/shellcore/interrupt_handler.h"
 #include "mysqlshdk/libs/db/connection_options.h"
 #include "mysqlshdk/libs/db/uri_parser.h"
 #include "mysqlshdk/libs/utils/utils_file.h"
@@ -64,6 +68,16 @@ errno_t memset_s(void *__s, rsize_t __smax, int __c, rsize_t __n);
 #include <mysql_version.h>
 
 namespace shcore {
+
+Scoped_callback::~Scoped_callback() {
+  try {
+    call();
+  } catch (const std::exception &e) {
+    exception_ptr = std::current_exception();
+    log_error("Unexpected exception: %s", e.what());
+  }
+}
+
 bool is_valid_identifier(const std::string &name) {
   bool ret_val = false;
 
@@ -102,10 +116,7 @@ std::string strip_password(const std::string &connstring) {
   p = s.rfind('@');
   std::string user_part;
 
-  if (p == std::string::npos) {
-    // by default, connect using the current OS username
-    user_part = get_system_user();
-  } else {
+  if (p != std::string::npos) {
     user_part = s.substr(0, p);
   }
 
@@ -184,7 +195,9 @@ void update_connection_data(
     const std::string &database, const mysqlshdk::db::Ssl_options &ssl_options,
     const std::string &auth_method, bool get_server_public_key,
     const std::string &server_public_key_path,
-    const std::string &connect_timeout, bool compression) {
+    const std::string &connect_timeout, const std::string &compression,
+    const std::string &compress_algorithm,
+    mysqlshdk::utils::nullable<int64_t> compress_level) {
   if (!user.empty()) {
     connection_options->clear_user();
     connection_options->set_user(user);
@@ -261,6 +274,12 @@ void update_connection_data(
         ssl_options.get_tls_version());
   }
 
+  if (ssl_options.has_tls_ciphersuites()) {
+    connection_options->get_ssl_options().clear_tls_ciphersuites();
+    connection_options->get_ssl_options().set_tls_ciphersuites(
+        ssl_options.get_tls_ciphersuites());
+  }
+
   if (ssl_options.has_mode()) {
     connection_options->get_ssl_options().clear_mode();
     connection_options->get_ssl_options().set_mode(ssl_options.get_mode());
@@ -295,17 +314,23 @@ void update_connection_data(
     connection_options->set(mysqlshdk::db::kConnectTimeout, connect_timeout);
   }
 
-  if (compression) connection_options->set_compression(true);
+  if (!compression.empty()) connection_options->set_compression(compression);
+
+  if (!compress_algorithm.empty())
+    connection_options->set_compression_algorithms(compress_algorithm);
+
+  if (!compress_level.is_null())
+    connection_options->set_compression_level(*compress_level);
 }
 
 std::string get_system_user() {
-  std::string ret_val;
+  std::string ret_val{"UNKNOWN_USER"};
 
-#ifdef WIN32
-  char username[UNLEN + 1];
+#ifdef _WIN32
+  wchar_t username[UNLEN + 1] = {};
   DWORD username_len = UNLEN + 1;
-  if (GetUserName(username, &username_len)) {
-    ret_val.assign(username);
+  if (GetUserNameW(username, &username_len) != 0) {
+    ret_val = shcore::wide_to_utf8(username, username_len - 1);
   }
 #else
   if (geteuid() == 0) {
@@ -340,8 +365,6 @@ std::string get_system_user() {
 #elif HAVE_CUSERID
     char username[L_cuserid];
     if (cuserid(username)) ret_val.assign(username);
-#else
-    ret_val = "UNKNOWN_USER";
 #endif
   }
 #endif
@@ -518,7 +541,7 @@ std::string from_camel_case(const std::string &name) {
   // Uppercase letters will be converted to underscore+lowercase letter
   // except in two situations:
   // - When it is the first letter
-  // - When an underscore is already before the uppercase letter
+  // - When an underscore or a dash is already before the uppercase letter
   bool skip_underscore = true;
   for (auto character : name) {
     if (isupper(character)) {
@@ -530,7 +553,7 @@ std::string from_camel_case(const std::string &name) {
       new_name.append(1, tolower(character));
     } else {
       // if character is '_'
-      skip_underscore = character == '_';
+      skip_underscore = character == '_' || character == '-';
       if (skip_underscore) upper_count++;
       new_name.append(1, character);
     }
@@ -539,6 +562,10 @@ std::string from_camel_case(const std::string &name) {
   return new_name;
 }
 
+std::string from_camel_case_to_dashes(const std::string &name) {
+  return str_replace(from_camel_case(name), "_", "-");
+}
+
 static std::size_t span_quotable_identifier(const std::string &s, std::size_t p,
                                             std::string *out_string) {
   bool seen_not_a_digit = false;
@@ -793,7 +820,7 @@ static std::size_t span_account_hostname_relaxed(const std::string &s,
 
         try_quoting = !quoted;
       }
-    } catch (const std::runtime_error &e) {
+    } catch (const std::runtime_error &) {
       // In case of error parsing, tries quoting
       try_quoting = auto_quote_hosts;
     }
@@ -831,10 +858,18 @@ void split_account(const std::string &account, std::string *out_user,
       pos = span_quotable_string_literal(account, 0, out_user);
     } else {
       pos = account.rfind('@');
-      if (pos == 0) throw std::runtime_error("Empty user name: " + account);
+      if (pos == 0) throw std::runtime_error("User name must not be empty.");
+      // dont allow @ on the username unless it is quoted
+      if (account.rfind('@', pos - 1) != std::string::npos) {
+        throw std::runtime_error("Invalid user name: " +
+                                 account.substr(0, pos));
+      }
       if (out_user != nullptr) out_user->assign(account, 0, pos);
     }
+  } else {
+    throw std::runtime_error("User name must not be empty.");
   }
+
   if (std::string::npos != pos && account[pos] == '@' &&
       ++pos < account.length()) {
     if (account.compare(pos, std::string::npos, "skip-grants host") == 0) {
@@ -850,6 +885,12 @@ void split_account(const std::string &account, std::string *out_user,
                              "'");
 }
 
+Account split_account(const std::string &account, bool auto_quote_hosts) {
+  Account result;
+  split_account(account, &result.user, &result.host, auto_quote_hosts);
+  return result;
+}
+
 /** Join MySQL account components into a string suitable for use with GRANT
  *  and similar
  */
@@ -857,14 +898,126 @@ std::string make_account(const std::string &user, const std::string &host) {
   return shcore::sqlstring("?@?", 0) << user << host;
 }
 
-void sleep_ms(uint32_t ms) {
-#ifdef _WIN32
-  Sigint_event::get().wait(ms);
-#else
-  usleep(ms * 1000);
-#endif
+std::string make_account(const Account &account) {
+  return make_account(account.user, account.host);
 }
 
+void split_schema_and_table(const std::string &str, std::string *out_schema,
+                            std::string *out_table) {
+  std::string schema;
+  std::string table;
+
+  auto pos = span_quotable_identifier(str, 0, &schema);
+
+  if (pos < str.length()) {
+    if (str[pos] != '.') {
+      throw std::runtime_error(
+          std::string("Invalid object name, expected '.', but got: '") +
+          str[pos] + "'.");
+    }
+
+    pos = span_quotable_identifier(str, ++pos, &table);
+
+    if (pos < str.length()) {
+      throw std::runtime_error(
+          std::string("Invalid object name, expected end of name, but got: '") +
+          str[pos] + "'.");
+    }
+  } else {
+    // there's only table name
+    std::swap(schema, table);
+  }
+
+  if (table.empty()) {
+    throw std::runtime_error(
+        "Invalid object name, table name cannot be empty.");
+  }
+
+  if (out_schema) {
+    *out_schema = std::move(schema);
+  }
+
+  if (out_table) {
+    *out_table = std::move(table);
+  }
+}
+
+void split_priv_level(const std::string &str, std::string *out_schema,
+                      std::string *out_object, size_t *out_leftover) {
+  assert(out_schema && out_object);
+  // *
+  // *.*
+  // schema.*
+  // *.object
+  // schema.object
+  std::string schema;
+  std::string object;
+
+  *out_schema = "";
+  *out_object = "";
+
+  if (str.empty()) return;
+
+  std::string::size_type pos;
+
+  if (str.front() == '*') {
+    *out_schema = "*";
+    pos = 1;
+  } else {
+    pos = span_quotable_identifier(str, 0, out_schema);
+  }
+
+  if (pos < str.length()) {
+    if (str[pos] != '.') {
+      throw std::runtime_error(std::string("Invalid object name '" + str +
+                                           "', expected '.', but got: '") +
+                               str[pos] + "'.");
+    }
+    ++pos;
+
+    if (pos < str.length()) {
+      if (str[pos] == '*') {
+        *out_object = "*";
+        pos++;
+      } else {
+        pos = span_quotable_identifier(str, pos, out_object);
+      }
+    } else {
+      throw std::runtime_error("Invalid object name '" + str +
+                               "', expected object name after '.'");
+    }
+    if (pos < str.length()) {
+      if (out_leftover) {
+        *out_leftover = pos;
+      } else {
+        throw std::runtime_error(
+            std::string("Invalid object name '" + str +
+                        "', expected end of name, but got: '") +
+            str[pos] + "'.");
+      }
+    }
+  }
+}
+
+std::string SHCORE_PUBLIC unquote_identifier(const std::string &str) {
+  std::string object;
+  const auto pos = span_quotable_identifier(str, 0, &object);
+
+  if (pos < str.length()) {
+    throw std::runtime_error(
+        std::string("Invalid object name, expected end of name, but got: '") +
+        str[pos] + "'.");
+  }
+
+  if (object.empty()) {
+    throw std::runtime_error("Object name cannot be empty.");
+  }
+
+  return object;
+}
+
+void sleep_ms(uint32_t ms) { shcore::current_interrupt()->wait(ms); }
+
 /*
  * Determines the current Operating System
  *
@@ -928,6 +1081,8 @@ OperatingSystem get_os_type() {
   return os;
 }
 
+std::string get_machine_type() { return MACHINE_TYPE; }
+
 std::string to_string(OperatingSystem os_type) {
   switch (os_type) {
     case OperatingSystem::UNKNOWN:
@@ -1026,9 +1181,12 @@ bool _match_glob(const std::string &pat, const std::string &str) {
  */
 bool match_glob(const std::string &pattern, const std::string &s,
                 bool case_sensitive) {
-  std::string str = case_sensitive ? s : str_lower(s);
-  std::string pat = case_sensitive ? pattern : str_lower(pattern);
-  return _match_glob(pat, str);
+  if (!case_sensitive) {
+    const std::string &str = str_lower(s);
+    const std::string &pat = str_lower(pattern);
+    return _match_glob(pat, str);
+  }
+  return _match_glob(pattern, s);
 }
 
 const char *get_long_version() {
@@ -1053,6 +1211,12 @@ void clear_buffer(char *buffer, size_t size) {
 #endif
 }
 
+void clear_buffer(std::string *buffer) {
+  assert(buffer);
+  clear_buffer(&(*buffer)[0], buffer->capacity());
+  buffer->clear();
+}
+
 #ifdef _WIN32
 
 std::string SHCORE_PUBLIC last_error_to_string(DWORD code) {
@@ -1073,24 +1237,99 @@ std::string SHCORE_PUBLIC last_error_to_string(DWORD code) {
   return ret;
 }
 
-Sigint_event::Sigint_event() {
-  m_event = CreateEvent(nullptr, TRUE, FALSE, nullptr);
-}
-
-Sigint_event::~Sigint_event() { CloseHandle(m_event); }
-
-Sigint_event &Sigint_event::get() {
-  static Sigint_event instance;
-  return instance;
-}
-
-void Sigint_event::notify() { SetEvent(m_event); }
-
-void Sigint_event::wait(uint32_t ms) {
-  ResetEvent(m_event);
-  WaitForSingleObjectEx(m_event, ms, FALSE);
-}
-
 #endif  // _WIN32
 
+std::istream &getline(std::istream &in, std::string &out) {
+  if (std::getline(in, out)) {
+    if (!out.empty() && out.back() == '\r') {
+      out.pop_back();
+    }
+  }
+
+  return in;
+}
+
+bool verify_status_code(int status, std::string *error) {
+  assert(error);
+
+  if (status < 0) {
+    const auto code = errno;
+    *error = "failed to execute: " + errno_to_string(code);
+  } else {
+#ifdef _WIN32
+    const bool exited = true;
+    const auto exit_code = status;
+#else   // !_WIN32
+    const bool exited = WIFEXITED(status);
+    const auto exit_code = WEXITSTATUS(status);
+#endif  // !_WIN32
+
+    if (exited) {
+      if (0 != exit_code) {
+        *error = "returned exit code: " + std::to_string(exit_code);
+      } else {
+        return true;
+      }
+    }
+#ifndef _WIN32
+    else if (WIFSIGNALED(status)) {
+      *error = "was terminated by signal: " + std::to_string(WTERMSIG(status));
+    } else if (WIFSTOPPED(status)) {
+      *error = "was stopped by signal: " + std::to_string(WSTOPSIG(status));
+    } else {
+      *error = "failed with unknown code: " + std::to_string(status);
+    }
+#endif  // !_WIN32
+  }
+
+  return false;
+}
+
+bool setenv(const char *name, const char *value) {
+  if (nullptr == value || '\0' == *value) {
+    return shcore::unsetenv(name);
+  }
+
+  const auto ret =
+#ifdef _WIN32
+      _putenv_s(name, value);
+#else   // !_WIN32
+      ::setenv(name, value, 1 /* overwrite */);
+#endif  // !_WIN32
+  return ret == 0;
+}
+
+bool setenv(const char *name, const std::string &value) {
+  return shcore::setenv(name, value.c_str());
+}
+
+bool setenv(const std::string &name, const std::string &value) {
+  return shcore::setenv(name.c_str(), value.c_str());
+}
+
+bool setenv(const std::string &name_value) {
+  const auto pos = name_value.find('=');
+
+  if (std::string::npos != pos) {
+    return shcore::setenv(name_value.substr(0, pos),
+                          name_value.substr(pos + 1));
+  }
+
+  return false;
+}
+
+bool unsetenv(const char *name) {
+  const auto ret =
+#ifdef _WIN32
+      _putenv_s(name, "");
+#else   // !_WIN32
+      ::unsetenv(name);
+#endif  // !_WIN32
+  return ret == 0;
+}
+
+bool unsetenv(const std::string &name) {
+  return shcore::unsetenv(name.c_str());
+}
+
 }  // namespace shcore
