@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -21,9 +21,10 @@
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "utils/utils_sqlstring.h"
-#include "utils/utils_lexing.h"
-#include "utils/utils_string.h"
+#include "mysqlshdk/libs/utils/utils_sqlstring.h"
+#include "mysqlshdk/libs/utils/dtoa.h"
+#include "mysqlshdk/libs/utils/utils_lexing.h"
+#include "mysqlshdk/libs/utils/utils_string.h"
 
 // updated as of 5.7
 static const char *reserved_keywords[] = {"ACCESSIBLE",
@@ -387,7 +388,7 @@ std::string quote_sql_string(const std::string &s) {
 //--------------------------------------------------------------------------------------------------
 
 std::string quote_identifier(const std::string &identifier) {
-  return "`" + escape_backticks(identifier) + "`";
+  return "`" + str_replace(identifier, "`", "``") + "`";
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -444,7 +445,7 @@ sqlstring::sqlstring(const sqlstring &copy)
 sqlstring::sqlstring() : _format(0) {}
 
 std::string sqlstring::consume_until_next_escape() {
-  mysqlshdk::utils::SQL_string_iterator it(_format_string_left);
+  mysqlshdk::utils::SQL_iterator it(_format_string_left);
   for (; it.valid(); ++it)
     if (*it == '?' || *it == '!') break;
 
@@ -482,6 +483,10 @@ std::string sqlstring::str() const {
   return _formatted + _format_string_left;
 }
 
+std::size_t sqlstring::size() const {
+  return _formatted.size() + _format_string_left.size();
+}
+
 void sqlstring::done() const {
   if (!_format_string_left.empty() ||
       !(_format_string_left[0] != '!' && _format_string_left[0] != '?')) {
@@ -495,7 +500,7 @@ sqlstring &sqlstring::operator<<(const double v) {
     throw std::invalid_argument(
         "Error formatting SQL query: invalid escape for numeric argument");
 
-  append(std::to_string(v));
+  append(shcore::dtoa(v));
   append(consume_until_next_escape());
 
   return *this;
@@ -509,11 +514,10 @@ sqlstring &sqlstring::operator<<(const sqlstringformat format) {
 sqlstring &sqlstring::operator<<(const std::string &v) {
   int esc = next_escape();
   if (esc == '!') {
-    std::string escaped = escape_backticks(v);
     if ((_format._flags & QuoteOnlyIfNeeded) != 0)
-      append(quote_identifier_if_needed(escaped));
+      append(quote_identifier_if_needed(v));
     else
-      append(quote_identifier(escaped));
+      append(quote_identifier(v));
   } else if (esc == '?') {
     append("'").append(escape_sql_string(v)).append("'");
   } else {  // shouldn't happen
@@ -542,11 +546,10 @@ sqlstring &sqlstring::operator<<(const char *v) {
     if (!v)
       throw std::invalid_argument(
           "Error formatting SQL query: NULL value found for identifier");
-    std::string quoted = escape_backticks(v);
-    if (quoted == v && (_format._flags & QuoteOnlyIfNeeded))
-      append(quoted);
+    if (_format._flags & QuoteOnlyIfNeeded)
+      append(quote_identifier_if_needed(v));
     else
-      append("`").append(quoted).append("`");
+      append(quote_identifier(v));
   } else if (esc == '?') {
     if (v) {
       append(quote_sql_string(v));
