@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -24,8 +24,9 @@
 
 #include <tuple>
 
-#include "modules/devapi/base_resultset.h"
+#include "mysqlshdk/libs/utils/fault_injection.h"
 #include "mysqlshdk/libs/utils/logger.h"
+#include "mysqlshdk/libs/utils/threads.h"
 #include "mysqlshdk/shellcore/shell_console.h"
 #include "shellcore/base_session.h"
 #include "shellcore/interrupt_handler.h"
@@ -48,37 +49,54 @@
 
 namespace mysqlsh {
 
+namespace {
+const std::map<std::string, std::string> k_shell_hooks = {
+    {"ClassicResult", "dump"},       {"Result", "dump"},
+    {"DocResult", "dump"},           {"RowResult", "dump"},
+    {"SqlResult", "dump"},           {"CollectionAdd", "execute"},
+    {"CollectionFind", "execute"},   {"CollectionModify", "execute"},
+    {"CollectionRemove", "execute"}, {"TableDelete", "execute"},
+    {"TableInsert", "execute"},      {"TableSelect", "execute"},
+    {"TableUpdate", "execute"},      {"SqlExecute", "execute"},
+};
+}
+
 namespace {
 void print_diag(const std::string &s) { current_console()->print_diag(s); }
 
 void println(const std::string &s) { current_console()->println(s); }
+
+void print(const std::string &s) { current_console()->print(s); }
+
+void print_value(const shcore::Value &value, const std::string &s) {
+  current_console()->print_value(value, s);
+}
+
 }  // namespace
 
-Base_shell::Base_shell(std::shared_ptr<Shell_options> cmdline_options,
-                       shcore::Interpreter_delegate *custom_delegate)
+Base_shell::Base_shell(const std::shared_ptr<Shell_options> &cmdline_options)
     : m_shell_options{cmdline_options},
-      _deferred_output(new std::string()),
-      m_console_handler{
-          std::make_shared<mysqlsh::Shell_console>(custom_delegate)} {
-  m_console_handler.get()->set_verbose(options().verbose_level);
-
-  shcore::Interrupts::setup();
-
-  std::string log_path =
-      shcore::path::join_path(shcore::get_user_config_path(), "mysqlsh.log");
-
-  shcore::Logger::setup_instance(log_path.c_str(), options().log_to_stderr,
-                                 options().log_level);
+      _deferred_output(std::make_unique<std::string>()) {
+  shcore::current_interrupt()->setup();
 
   _input_mode = shcore::Input_state::Ok;
 
-  _shell.reset(new shcore::Shell_core(m_console_handler.get().get()));
-  _completer_object_registry.reset(new shcore::completer::Object_registry());
+  _shell = std::make_shared<shcore::Shell_core>();
+  _completer_object_registry =
+      std::make_shared<shcore::completer::Object_registry>();
 }
 
 Base_shell::~Base_shell() {}
 
+std::string Base_shell::get_shell_hook(const std::string &class_name) {
+  auto item = k_shell_hooks.find(class_name);
+
+  return item == k_shell_hooks.end() ? "" : item->second;
+}
+
 void Base_shell::finish_init() {
+  current_console()->set_verbose(options().verbose_level);
+
   shcore::IShell_core::Mode initial_mode = options().initial_mode;
   if (initial_mode == shcore::IShell_core::Mode::None) {
 #ifdef HAVE_V8
@@ -91,6 +109,15 @@ void Base_shell::finish_init() {
 #endif
 #endif
   }
+
+  // If we're not in the main thread, it means the shell object is created
+  // again, because of this we don't need need any mode initialized by default
+  // as it's very unlikely that we will be parsing any input directly. If
+  // however this will be needed, switch_shell_mode can be used to fix this.
+  if (!mysqlshdk::utils::in_main_thread()) {
+    initial_mode = shcore::IShell_core::Mode::None;
+  }
+
   // Final initialization that must happen outside the constructor
   switch_shell_mode(initial_mode, {}, true);
 
@@ -106,47 +133,60 @@ void Base_shell::finish_init() {
 void Base_shell::init_scripts(shcore::Shell_core::Mode mode) {
   std::string extension;
 
-  if (mode == shcore::Shell_core::Mode::JavaScript)
-    extension.append(".js");
-  else if (mode == shcore::Shell_core::Mode::Python)
-    extension.append(".py");
-  else
-    return;
+  switch (mode) {
+    case shcore::Shell_core::Mode::JavaScript:
+      extension.append(".js");
+      break;
 
-  std::vector<std::string> scripts_paths;
+    case shcore::Shell_core::Mode::Python:
+      extension.append(".py");
+      break;
 
-  std::string user_file = "";
+    case shcore::Shell_core::Mode::None:
+    case shcore::Shell_core::Mode::SQL:
+      return;
+  }
 
   try {
-    // Checks existence of global startup script
-    std::string path = shcore::get_global_config_path();
-    path.append("shellrc");
-    path.append(extension);
-    if (shcore::is_file(path)) scripts_paths.push_back(path);
+    std::vector<std::string> script_paths;
+    const auto startup_script = "mysqlshrc" + extension;
 
-    // Checks existence of startup script at MYSQLSH_HOME
-    // Or the binary location if not a standard installation
-    path = shcore::get_mysqlx_home_path();
-    if (!path.empty()) {
-      path.append("/share/mysqlsh/mysqlshrc");
-    } else {
-      path = shcore::get_binary_folder();
-      path.append("/mysqlshrc");
+    {
+      // Checks existence of global startup script
+      auto path = shcore::path::join_path(shcore::get_global_config_path(),
+                                          startup_script);
+      if (shcore::is_file(path)) script_paths.emplace_back(std::move(path));
     }
-    path.append(extension);
-    if (shcore::is_file(path)) scripts_paths.push_back(path);
 
-    // Checks existence of user startup script
-    path = shcore::get_user_config_path();
-    path.append("mysqlshrc");
-    path.append(extension);
-    if (shcore::is_file(path)) scripts_paths.push_back(path);
+    {
+      // Checks existence of startup script at MYSQLSH_HOME
+      // Or the binary location if not a standard installation
+      const auto home = shcore::get_mysqlx_home_path();
+      std::string path;
 
-    for (std::vector<std::string>::iterator i = scripts_paths.begin();
-         i != scripts_paths.end(); ++i) {
-      std::ifstream stream(*i);
-      if (stream && stream.peek() != std::ifstream::traits_type::eof())
-        process_file(*i, {*i});
+      if (!home.empty()) {
+        path =
+            shcore::path::join_path(home, "share", "mysqlsh", startup_script);
+      } else {
+        path = shcore::path::join_path(shcore::get_binary_folder(),
+                                       startup_script);
+      }
+
+      if (shcore::is_file(path)) script_paths.emplace_back(std::move(path));
+    }
+
+    {
+      // Checks existence of user startup script
+      auto path = shcore::path::join_path(shcore::get_user_config_path(),
+                                          startup_script);
+      if (shcore::is_file(path)) script_paths.emplace_back(std::move(path));
+    }
+
+    for (const auto &script : script_paths) {
+      std::ifstream stream(script);
+      if (stream && stream.peek() != std::ifstream::traits_type::eof()) {
+        process_file(script, {script});
+      }
     }
   } catch (const std::exception &e) {
     std::string error(e.what());
@@ -213,17 +253,26 @@ std::map<std::string, std::string> *Base_shell::prompt_variables() {
 }
 
 void Base_shell::request_prompt_variables_update(bool clear_cache) {
-  m_pending_update = clear_cache ? Prompt_variables_update_type::CLEAR_CACHE
-                                 : Prompt_variables_update_type::UPDATE;
+  const auto type = clear_cache ? Prompt_variables_update_type::CLEAR_CACHE
+                                : Prompt_variables_update_type::UPDATE;
+
+  if (type > m_pending_update) {
+    m_pending_update = type;
+  }
 }
 
 void Base_shell::update_prompt_variables() {
+  FI_SUPPRESS(mysql);
+  FI_SUPPRESS(mysqlx);
+
   if (Prompt_variables_update_type::CLEAR_CACHE == m_pending_update) {
     _prompt_variables.clear();
   }
 
   const auto session = _shell->get_dev_session();
   if (_prompt_variables.empty()) {
+    _prompt_variables["system_user"] = shcore::get_system_user();
+
     if (session && session->is_open()) {
       mysqlshdk::db::Connection_options options(session->uri());
       std::string socket;
@@ -255,6 +304,8 @@ void Base_shell::update_prompt_variables() {
       _prompt_variables["port"] = port;
       _prompt_variables["socket"] = socket;
       _prompt_variables["node_type"] = session->get_node_type();
+      _prompt_variables["connection_id"] =
+          std::to_string(session->get_connection_id());
     } else {
       _prompt_variables["ssl"] = "";
       _prompt_variables["uri"] = "";
@@ -264,6 +315,7 @@ void Base_shell::update_prompt_variables() {
       _prompt_variables["socket"] = "";
       _prompt_variables["session"] = "";
       _prompt_variables["node_type"] = "";
+      _prompt_variables["connection_id"] = "";
     }
   }
   if (session && session->is_open()) {
@@ -387,7 +439,7 @@ bool Base_shell::switch_shell_mode(shcore::Shell_core::Mode mode,
  * @param str text to be printed.
  */
 void Base_shell::println_deferred(const std::string &str) {
-  // This can't be called once the deferred output is flusehd
+  // This can't be called once the deferred output is flushed
   assert(_deferred_output != nullptr);
   _deferred_output->append(str + "\n");
 }
@@ -401,14 +453,17 @@ void Base_shell::clear_input() {
 void Base_shell::process_line(const std::string &line) {
   std::string to_history;
 
-  if (_input_mode == shcore::Input_state::ContinuedBlock && line.empty())
+  if (_input_mode == shcore::Input_state::ContinuedBlock && line.empty()) {
     _input_mode = shcore::Input_state::Ok;
+  }
 
   // Appends the line, no matter if it is an empty line
   _input_buffer.append(_shell->preprocess_input_line(line));
 
   // Appends the new line if anything has been added to the buffer
-  if (!_input_buffer.empty()) _input_buffer.append("\n");
+  if (!_input_buffer.empty()) {
+    _input_buffer.append("\n");
+  }
 
   if (_input_mode != shcore::Input_state::ContinuedBlock &&
       !_input_buffer.empty()) {
@@ -420,7 +475,7 @@ void Base_shell::process_line(const std::string &line) {
         to_history = _shell->get_handled_input();
       }
     } catch (shcore::Exception &exc) {
-      m_console_handler.get()->print_value(shcore::Value(exc.error()), "error");
+      print_value(shcore::Value(exc.error()), "error");
       to_history = _input_buffer;
     } catch (const std::exception &exc) {
       std::string error(exc.what());
@@ -432,10 +487,14 @@ void Base_shell::process_line(const std::string &line) {
     // TODO: Do we need this cleanup? i.e. in case of exceptions above??
     // Clears the buffer if OK, if continued, buffer will contain
     // the non executed code
-    if (_input_mode == shcore::Input_state::Ok) _input_buffer.clear();
+    if (_input_mode == shcore::Input_state::Ok) {
+      _input_buffer.clear();
+    }
   }
 
-  if (!to_history.empty()) notify_executed_statement(to_history);
+  if (!to_history.empty()) {
+    notify_executed_statement(to_history);
+  }
 }
 
 void Base_shell::notify_executed_statement(const std::string &line) {
@@ -446,8 +505,8 @@ void Base_shell::notify_executed_statement(const std::string &line) {
 }
 
 void Base_shell::process_sql_result(
-    std::shared_ptr<mysqlshdk::db::IResult> result,
-    const shcore::Sql_result_info &info) {
+    const std::shared_ptr<mysqlshdk::db::IResult> &result,
+    const shcore::Sql_result_info & /*info*/) {
   if (!result) {
     // Return value of undefined implies an error processing
     // TODO(alfredo) - signaling of errors should be moved down
@@ -455,62 +514,35 @@ void Base_shell::process_sql_result(
   }
 }
 
-void Base_shell::print_result(shcore::Value result) {
+void Base_shell::print_result(const shcore::Value &result) {
   if (result) {
-    shcore::Value shell_hook;
-    std::shared_ptr<shcore::Object_bridge> object;
     if (result.type == shcore::Object) {
-      object = result.as_object();
-      if (object && object->has_member("__shell_hook__"))
-        shell_hook = object->get_member("__shell_hook__");
+      auto object = result.as_object();
+      auto shell_hook = get_shell_hook(object->class_name());
+      if (!shell_hook.empty()) {
+        if (object->has_member(shell_hook)) {
+          shcore::Value hook_result = object->call(shell_hook, {});
 
-      if (shell_hook) {
-        shcore::Argument_list args;
-        shcore::Value hook_result = object->call("__shell_hook__", args);
+          // Recursive call to continue processing shell hooks if any
+          process_result(hook_result, false);
 
-        // Recursive call to continue processing shell hooks if any
-        process_result(hook_result, false);
+          return;
+        }
       }
     }
 
-    // If the function is not found the values still needs to be printed
-    if (!shell_hook) {
-      // Resultset objects get printed
-      if (object && object->class_name().find("Result") != std::string::npos) {
-        std::shared_ptr<mysqlsh::ShellBaseResult> resultset =
-            std::static_pointer_cast<mysqlsh::ShellBaseResult>(object);
+    // In JSON mode: the json representation is used for Object, Array and
+    // Map For anything else a map is printed with the "value" key
+    std::string tag;
+    if (result.type != shcore::Object && result.type != shcore::Array &&
+        result.type != shcore::Map)
+      tag = "value";
 
-        // TODO(alfredo) - dumping of SqlResults should be redirected to
-        // process_sql_result()
-
-        // Result buffering will be done ONLY if on any of the scripting
-        // interfaces
-        Resultset_dumper dumper(
-            resultset->get_result(),
-            _shell->interactive_mode() != shcore::IShell_core::Mode::SQL);
-
-        bool is_doc_result = resultset->is_doc_result();
-        bool is_query = is_doc_result || resultset->is_row_result();
-        std::string item_label = is_doc_result
-                                     ? "document"
-                                     : resultset->is_result() ? "item" : "row";
-
-        dumper.dump(item_label, is_query, is_doc_result);
-      } else {
-        // In JSON mode: the json representation is used for Object, Array and
-        // Map For anything else a map is printed with the "value" key
-        std::string tag;
-        if (result.type != shcore::Object && result.type != shcore::Array &&
-            result.type != shcore::Map)
-          tag = "value";
-
-        m_console_handler.get()->print_value(result, tag);
-      }
-    }
+    print_value(result, tag);
   }
 }
 
-void Base_shell::process_result(shcore::Value result, bool got_error) {
+void Base_shell::process_result(const shcore::Value &result, bool got_error) {
   assert(_shell->interactive_mode() != shcore::Shell_core::Mode::SQL);
 
   if (options().interactive) print_result(result);
@@ -525,8 +557,6 @@ int Base_shell::process_file(const std::string &path,
 
   if (path.empty()) {
     print_diag("Invalid filename");
-  } else if (_shell->is_module(path)) {
-    _shell->execute_module(path, argv);
   } else {
     std::string file = shcore::path::expand_user(path);
     if (shcore::is_folder(file)) {
@@ -537,7 +567,12 @@ int Base_shell::process_file(const std::string &path,
       return ret_val;
     }
 
+#ifdef _WIN32
+    std::ifstream s(shcore::utf8_to_wide(file));
+#else
     std::ifstream s(file.c_str());
+#endif
+
     if (!s.fail()) {
       // The return value now depends on the stream processing
       ret_val = process_stream(s, file, argv);
@@ -557,20 +592,36 @@ int Base_shell::process_file(const std::string &path,
   return ret_val;
 }
 
+int Base_shell::run_module(const std::string &module,
+                           const std::vector<std::string> &argv) {
+  if (module.empty()) {
+    print_diag("Invalid module name");
+    return 1;
+  }
+
+  _shell->execute_module(module, argv);
+
+  return 0;
+}
+
 int Base_shell::process_stream(std::istream &stream, const std::string &source,
                                const std::vector<std::string> &argv,
                                bool force_batch) {
+  _shell->set_argv(argv);
+
   // If interactive is set, it means that the shell was started with the option
   // to Emulate interactive mode while processing the stream
   if (!force_batch && options().interactive) {
-    if (options().full_interactive) m_console_handler.get()->print(prompt());
+    if (options().full_interactive) print(prompt());
 
     bool comment_first_js_line =
         _shell->interactive_mode() == shcore::IShell_core::Mode::JavaScript;
-    while (!stream.eof()) {
-      std::string line;
 
-      std::getline(stream, line);
+    std::string line;
+    while (!stream.eof()) {
+      line.clear();
+
+      shcore::getline(stream, line);
 
       // When processing JavaScript files, validates the very first line to
       // start with #! If that's the case, it is replaced by a comment indicator
@@ -585,24 +636,29 @@ int Base_shell::process_stream(std::istream &stream, const std::string &source,
 
       process_line(line);
 
-      if (options().full_interactive) m_console_handler.get()->print(prompt());
+      if (options().full_interactive) print(prompt());
+    }
+
+    // If stream ended, but doesn't have newline at the end, then last statement
+    // won't be executed.  We force process empty line to be able to escape from
+    // ContinuedBlock state and execute last statement.
+    if (_input_mode == shcore::Input_state::ContinuedBlock) {
+      process_line(std::string());
     }
 
     // Being interactive, we do not care about the return value
     return 0;
   } else {
-    return _shell->process_stream(stream, source, argv);
+    return _shell->process_stream(stream, source, false);
   }
 }
 
 void Base_shell::set_global_object(
-    const std::string &name, std::shared_ptr<shcore::Cpp_object_bridge> object,
+    const std::string &name,
+    const std::shared_ptr<shcore::Cpp_object_bridge> &object,
     shcore::IShell_core::Mode_mask modes) {
   if (object) {
-    _shell->set_global(
-        name,
-        shcore::Value(std::dynamic_pointer_cast<shcore::Object_bridge>(object)),
-        modes);
+    _shell->set_global(name, shcore::Value(object), modes);
   } else {
     _shell->set_global(name, shcore::Value(), modes);
   }
