@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -23,6 +23,7 @@
 
 #include "mysqlshdk/shellcore/shell_console.h"
 
+#include <cstdio>
 #include <memory>
 #include <string>
 #include <utility>
@@ -34,6 +35,7 @@
 #include "mysqlshdk/libs/textui/term_vt100.h"
 #include "mysqlshdk/libs/textui/textui.h"
 #include "mysqlshdk/libs/utils/logger.h"
+#include "mysqlshdk/libs/utils/strformat.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_json.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
@@ -105,17 +107,12 @@ class Shell_pager : public IPager {
       current_console()->remove_print_handler(&m_handler);
 
       // inform of any errors
-#ifdef _WIN32
-      const auto exit_code = status;
-      const bool error_occurred = 0 != exit_code;
-#else   // !_WIN32
-      const auto exit_code = WEXITSTATUS(status);
-      const bool error_occurred = WIFEXITED(status) && 0 != exit_code;
-#endif  // !_WIN32
-      if (error_occurred) {
-        current_console()->print_error(
-            "Pager \"" + current_shell_options()->get().pager +
-            "\" returned exit code: " + std::to_string(exit_code) + ".");
+      std::string error;
+
+      if (!shcore::verify_status_code(status, &error)) {
+        current_console()->print_error("Pager \"" +
+                                       current_shell_options()->get().pager +
+                                       "\" " + error + ".");
       }
     }
   }
@@ -134,7 +131,6 @@ class Shell_pager : public IPager {
 };
 
 namespace {
-int g_dont_log = 0;
 
 void log_hook(const shcore::Logger::Log_entry &log, void *data) {
   Shell_console *self = reinterpret_cast<Shell_console *>(data);
@@ -148,22 +144,25 @@ void log_hook(const shcore::Logger::Log_entry &log, void *data) {
     // verbose=4, DEBUG3
     switch (self->get_verbose()) {
       case 4:
-        if (log.level == shcore::Logger::LOG_DEBUG3) show_prefix = "verbose";
+        if (log.level == shcore::Logger::LOG_DEBUG3) show_prefix = "";
+        // fallthrough
       case 3:
-        if (log.level == shcore::Logger::LOG_DEBUG2) show_prefix = "verbose";
+        if (log.level == shcore::Logger::LOG_DEBUG2) show_prefix = "";
+        // fallthrough
       case 2:
-        if (log.level == shcore::Logger::LOG_DEBUG) show_prefix = "verbose";
+        if (log.level == shcore::Logger::LOG_DEBUG) show_prefix = "";
+        // fallthrough
       case 1:
         switch (log.level) {
           case shcore::Logger::LOG_INFO:
-            show_prefix = "verbose";
+            show_prefix = "";
             break;
           case shcore::Logger::LOG_WARNING:
-            show_prefix = "verbose: warning";
+            show_prefix = "warning: ";
             break;
           case shcore::Logger::LOG_ERROR:
           case shcore::Logger::LOG_INTERNAL_ERROR:
-            show_prefix = "verbose: error";
+            show_prefix = "error: ";
             break;
           default:
             break;
@@ -171,14 +170,32 @@ void log_hook(const shcore::Logger::Log_entry &log, void *data) {
         break;
     }
 
-    if (show_prefix && g_dont_log == 0) {
+    if (show_prefix && shcore::current_logger()->log_allowed()) {
+      static char color_on[16] = {0};
+      static char color_off[16] = {0};
+      std::string ts = mysqlshdk::utils::fmttime(
+          "%Y-%m-%dT%H:%M:%SZ", mysqlshdk::utils::Time_type::LOCAL,
+          &log.timestamp);
+
+      if (self->use_colors() && color_on[0] == '\0') {
+        snprintf(color_on, sizeof(color_on), "%s",
+                 mysqlshdk::vt100::attr(-1, -1, mysqlshdk::vt100::Dim).c_str());
+        snprintf(color_off, sizeof(color_off), "%s",
+                 mysqlshdk::vt100::attr().c_str());
+      } else if (!self->use_colors() && color_on[0] != '\0') {
+        memset(color_on, 0, sizeof(color_on));
+        memset(color_off, 0, sizeof(color_off));
+      }
+
       if (log.domain && *log.domain)
-        self->raw_print(shcore::str_format("%s: %s: %s\n", show_prefix,
-                                           log.domain, log.message),
+        self->raw_print(shcore::str_format("%sverbose: %s: %s%s: %s%s\n",
+                                           color_on, ts.c_str(), show_prefix,
+                                           log.domain, log.message, color_off),
                         Output_stream::STDERR);
       else
         self->raw_print(
-            shcore::str_format("%s: %s\n", show_prefix, log.message),
+            shcore::str_format("%sverbose: %s: %s%s%s\n", color_on, ts.c_str(),
+                               show_prefix, log.message, color_off),
             Output_stream::STDERR);
     }
   }
@@ -188,13 +205,14 @@ void log_hook(const shcore::Logger::Log_entry &log, void *data) {
 Shell_console::Shell_console(shcore::Interpreter_delegate *deleg)
     : m_ideleg(deleg) {
   m_print_handlers.emplace_back(deleg);
-  // Capture logging output and if verbose is enabled, show them in the console
-  shcore::Logger::singleton()->attach_log_hook(log_hook, this, true);
+#ifndef _WIN32
+  if (isatty(2)) {
+    m_use_colors = true;
+  }
+#endif
 }
 
-Shell_console::~Shell_console() {
-  shcore::Logger::singleton()->detach_log_hook(log_hook);
-}
+Shell_console::~Shell_console() { detach_log_hook(); }
 
 void Shell_console::dump_json(const char *tag, const std::string &s) const {
   delegate_print(json_obj(tag, s).c_str());
@@ -203,16 +221,16 @@ void Shell_console::dump_json(const char *tag, const std::string &s) const {
 void Shell_console::raw_print(const std::string &text, Output_stream stream,
                               bool format_json) const {
   using Print_func = void (Shell_console::*)(const char *) const;
-  Print_func print = stream == Output_stream::STDOUT
-                         ? &Shell_console::delegate_print
-                         : &Shell_console::delegate_print_diag;
+  Print_func func = stream == Output_stream::STDOUT
+                        ? &Shell_console::delegate_print
+                        : &Shell_console::delegate_print_diag;
 
   // format_json=false means bypass JSON wrapping
   if (use_json() && format_json) {
     const char *tag = stream == Output_stream::STDOUT ? "info" : "error";
     dump_json(tag, text);
   } else {
-    (this->*print)(text.c_str());
+    (this->*func)(text.c_str());
   }
 }
 
@@ -233,115 +251,161 @@ void Shell_console::println(const std::string &text) const {
 }
 
 namespace {
-std::string format(const std::string &text) {
-  return shcore::str_subvars(text,
-                             [](const std::string &var) {
-                               return shcore::get_member_name(
-                                   var, shcore::current_naming_style());
-                             },
-                             "<<<", ">>>");
+std::string format_vars(const std::string &text) {
+  return shcore::str_subvars(
+      text,
+      [](const std::string &var) {
+        return shcore::get_member_name(var, shcore::current_naming_style());
+      },
+      "<<<", ">>>");
 }
+
+std::string fit_screen(const std::string &text) {
+  // int r, c;
+  // if (!mysqlshdk::vt100::get_screen_size(&r, &c)) c = 80;
+
+  // if (c >= 120)
+  //   c = 120;
+  // else if (c >= 80)
+  //   c = 80;
+  // else
+  //   return text;
+  constexpr const int c = 80;
+
+  return mysqlshdk::textui::format_markup_text(shcore::str_split(text, "\n"), c,
+                                               0, false);
+}
+
 }  // namespace
 
 void Shell_console::print_diag(const std::string &text) const {
-  std::string ftext = format(text);
+  std::string ftext = format_vars(text);
   if (use_json()) {
     dump_json("error", ftext);
   } else {
     delegate_print_diag(ftext.c_str());
   }
-  if (g_dont_log == 0) {
-    g_dont_log++;
+  if (shcore::current_logger()->log_allowed()) {
+    shcore::Log_reentrant_protector lock;
     log_debug("%s", ftext.c_str());
-    g_dont_log--;
   }
 }
 
 void Shell_console::print_error(const std::string &text) const {
-  std::string ftext = format(text);
+  std::string ftext = format_vars(text);
   if (use_json()) {
     dump_json("error", ftext);
   } else {
     delegate_print_error(
         (mysqlshdk::textui::error("ERROR: ") + ftext + "\n").c_str());
   }
-  if (g_dont_log == 0) {
-    g_dont_log++;
+  if (shcore::current_logger()->log_allowed()) {
+    shcore::Log_reentrant_protector lock;
     log_error("%s", ftext.c_str());
-    g_dont_log--;
   }
 }
 
 void Shell_console::print_warning(const std::string &text) const {
-  std::string ftext = format(text);
+  std::string ftext = format_vars(text);
   if (use_json()) {
     dump_json("warning", ftext);
   } else {
     delegate_print_error(
         (mysqlshdk::textui::warning("WARNING: ") + ftext + "\n").c_str());
   }
-  if (g_dont_log == 0) {
-    g_dont_log++;
+  if (shcore::current_logger()->log_allowed()) {
+    shcore::Log_reentrant_protector lock;
     log_warning("%s", ftext.c_str());
-    g_dont_log--;
   }
 }
 
 void Shell_console::print_note(const std::string &text) const {
-  std::string ftext = format(text);
+  std::string ftext = format_vars(text);
   if (use_json()) {
     dump_json("note", ftext);
   } else {
     delegate_print_error(
         (mysqlshdk::textui::notice("NOTE: ") + ftext + "\n").c_str());
   }
-  if (g_dont_log == 0) {
-    g_dont_log++;
-    log_debug("%s", text.c_str());
-    g_dont_log--;
+  if (shcore::current_logger()->log_allowed()) {
+    shcore::Log_reentrant_protector lock;
+    log_info("%s", text.c_str());
   }
 }
 
 void Shell_console::print_info(const std::string &text) const {
-  std::string ftext = format(text);
+  std::string ftext = format_vars(text);
   if (use_json()) {
     dump_json("info", ftext);
   } else {
     delegate_print_error((ftext + "\n").c_str());
   }
-  if (g_dont_log == 0 && !text.empty()) {
-    g_dont_log++;
+  if (shcore::current_logger()->log_allowed() && !text.empty()) {
+    shcore::Log_reentrant_protector lock;
     log_debug("%s", ftext.c_str());
-    g_dont_log--;
   }
 }
 
-bool Shell_console::prompt(const std::string &prompt, std::string *ret_val,
-                           Validator validator) const {
+void Shell_console::print_status(const std::string &text) const {
+  std::string ftext = format_vars(text);
+  if (use_json()) {
+    dump_json("status", ftext);
+  } else {
+    delegate_print_error((ftext + "\n").c_str());
+  }
+  if (shcore::current_logger()->log_allowed() && !text.empty()) {
+    shcore::Log_reentrant_protector lock;
+    log_debug("%s", ftext.c_str());
+  }
+}
+
+void Shell_console::print_para(const std::string &text) const {
+  std::string ftext = format_vars(text);
+  if (use_json()) {
+    dump_json("info", ftext);
+  } else {
+    delegate_print_error((fit_screen(ftext) + "\n\n").c_str());
+  }
+  if (shcore::current_logger()->log_allowed() && !text.empty()) {
+    shcore::Log_reentrant_protector lock;
+    log_debug2("%s", ftext.c_str());
+  }
+}
+
+shcore::Prompt_result Shell_console::call_prompt(
+    const std::string &prompt, std::string *out_val, Validator validator,
+    const char *tag, shcore::Interpreter_delegate::Prompt func) const {
   std::string text;
   if (use_json()) {
-    text = json_obj("prompt", prompt);
+    text = json_obj(tag, prompt);
   } else {
     text = mysqlshdk::textui::bold(prompt);
   }
 
-  while (1) {
-    shcore::Prompt_result result = m_ideleg->prompt(text.c_str(), ret_val);
-    if (result == shcore::Prompt_result::Cancel)
-      throw shcore::cancelled("Cancelled");
+  shcore::Prompt_result result;
+  bool valid = true;
+  do {
+    result = (m_ideleg->*func)(text.c_str(), out_val);
+
     if (result == shcore::Prompt_result::Ok) {
       if (validator) {
-        std::string msg = validator(*ret_val);
+        std::string msg = validator(*out_val);
 
-        if (msg.empty()) return true;
+        valid = msg.empty();
 
-        print_warning(msg);
-      } else {
-        return true;
+        if (!valid) print_warning(msg);
       }
     }
-  }
-  return false;
+  } while (result == shcore::Prompt_result::Ok && !valid);
+
+  return result;
+}
+
+shcore::Prompt_result Shell_console::prompt(const std::string &prompt,
+                                            std::string *ret_val,
+                                            Validator validator) const {
+  return call_prompt(prompt, ret_val, validator, "prompt",
+                     &shcore::Interpreter_delegate::prompt);
 }
 
 static char process_label(const std::string &s, std::string *out_display,
@@ -425,7 +489,13 @@ Prompt_answer Shell_console::confirm(const std::string &prompt,
   }
 
   while (final_ans == Prompt_answer::NONE) {
-    if (this->prompt(prompt + " " + def_str, &ans)) {
+    auto res = this->prompt(prompt + " " + def_str, &ans);
+    if (res == shcore::Prompt_result::Cancel)
+      throw shcore::cancelled("Cancelled");
+    // Since fixing the prompt we need to preserve original behavior of the
+    // functionality, which was repeating prompt when user did press ctrl+d.
+    if (res == shcore::Prompt_result::CTRL_D) continue;
+    if (res == shcore::Prompt_result::Ok) {
       if (ans.empty()) {
         final_ans = def;
       } else {
@@ -474,7 +544,13 @@ bool Shell_console::select(const std::string &prompt_text, std::string *result,
   mysqlshdk::utils::nullable<std::string> good_answer;
 
   while (!valid && good_answer.is_null()) {
-    if (prompt(text, &answer)) {
+    auto res = prompt(text, &answer);
+    if (res == shcore::Prompt_result::Cancel)
+      throw shcore::cancelled("Cancelled");
+    // Since fixing the prompt we need to preserve original behavior of the
+    // functionality, which was repeating prompt when user did press ctrl+d.
+    if (res == shcore::Prompt_result::CTRL_D) continue;
+    if (res == shcore::Prompt_result::Ok) {
       int option = static_cast<int>(default_option);
 
       try {
@@ -519,30 +595,8 @@ bool Shell_console::select(const std::string &prompt_text, std::string *result,
 shcore::Prompt_result Shell_console::prompt_password(
     const std::string &prompt, std::string *out_val,
     Validator validator) const {
-  std::string text;
-  if (use_json()) {
-    text = json_obj("password", prompt);
-  } else {
-    text = mysqlshdk::textui::bold(prompt);
-  }
-
-  shcore::Prompt_result result;
-  bool valid = true;
-  do {
-    result = m_ideleg->password(text.c_str(), out_val);
-
-    if (result == shcore::Prompt_result::Ok) {
-      if (validator) {
-        std::string msg = validator(*out_val);
-
-        valid = msg.empty();
-
-        if (!valid) print_warning(msg);
-      }
-    }
-  } while (result == shcore::Prompt_result::Ok && !valid);
-
-  return result;
+  return call_prompt(prompt, out_val, validator, "password",
+                     &shcore::Interpreter_delegate::password);
 }
 
 void Shell_console::print_value(const shcore::Value &value,
@@ -639,29 +693,33 @@ void Shell_console::delegate_print_diag(const char *msg) const {
 }
 
 void Shell_console::delegate(
-    const char *msg, shcore::Interpreter_print_handler::Print print) const {
+    const char *msg, shcore::Interpreter_print_handler::Print func) const {
   for (auto it = m_print_handlers.crbegin(); it != m_print_handlers.crend();
        ++it) {
-    if ((*it->*print)(msg)) {
+    if ((*it->*func)(msg)) {
       return;
     }
   }
 }
 
-std::string fit_screen(const std::string &text) {
-  // int r, c;
-  // if (!mysqlshdk::vt100::get_screen_size(&r, &c)) c = 80;
+void Shell_console::on_set_verbose() {
+  if (get_verbose() > 0) {
+    if (!m_hook_registered) {
+      // if verbose is enabled, capture logging output and show them in the
+      // console
+      shcore::current_logger()->attach_log_hook(log_hook, this, true);
+      m_hook_registered = true;
+    }
+  } else {
+    detach_log_hook();
+  }
+}
 
-  // if (c >= 120)
-  //   c = 120;
-  // else if (c >= 80)
-  //   c = 80;
-  // else
-  //   return text;
-  constexpr const int c = 80;
-
-  return mysqlshdk::textui::format_markup_text(shcore::str_split(text, "\n"), c,
-                                               0, false);
+void Shell_console::detach_log_hook() {
+  if (m_hook_registered) {
+    shcore::current_logger()->detach_log_hook(log_hook);
+    m_hook_registered = false;
+  }
 }
 
 }  // namespace mysqlsh
