@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -32,6 +32,7 @@
 #include "mysqlshdk/include/shellcore/base_shell.h"
 #include "mysqlshdk/include/shellcore/console.h"
 #include "mysqlshdk/libs/db/column.h"
+#include "mysqlshdk/libs/db/row_copy.h"
 #include "mysqlshdk/libs/utils/dtoa.h"
 #include "mysqlshdk/libs/utils/strformat.h"
 #include "mysqlshdk/libs/utils/utils_json.h"
@@ -41,6 +42,10 @@
 
 #define MAX_DISPLAY_LENGTH 1024
 
+// max # of rows to pre-fetch when dumping resultsets with table formatting,
+// in order to calculate column widths
+static constexpr const int k_pre_fetch_result_rows = 1000;
+
 namespace mysqlsh {
 
 /* Calculates the required buffer size and display size considering:
@@ -221,7 +226,7 @@ enum class ResultFormat { VERTICAL, TABBED, TABLE };
 class Field_formatter {
  public:
   Field_formatter(ResultFormat format, const mysqlshdk::db::Column &column)
-      : m_buffer(nullptr),
+      : m_allocated(0),
         m_max_display_length(0),
         m_max_buffer_length(0),
         m_max_mb_holes(0),
@@ -257,34 +262,6 @@ class Field_formatter {
     }
   }
 
-  Field_formatter(Field_formatter &&other) {
-    this->operator=(std::move(other));
-  }
-
-  void operator=(Field_formatter &&other) {
-    m_allocated = other.m_allocated;
-    m_zerofill = other.m_zerofill;
-    m_align_right = other.m_align_right;
-    m_buffer = std::move(other.m_buffer);
-
-    other.m_buffer = nullptr;
-    other.m_allocated = 0;
-
-    m_max_display_length = other.m_max_display_length;
-    m_max_buffer_length = other.m_max_buffer_length;
-    m_max_mb_holes = other.m_max_mb_holes;
-
-    // Length cache for each data to be printed with this formatter
-    m_display_lengths = std::move(other.m_display_lengths);
-    m_buffer_lengths = std::move(other.m_buffer_lengths);
-
-    m_format = other.m_format;
-    m_flags = other.m_flags;
-    m_type = other.m_type;
-
-    m_is_numeric = other.m_is_numeric;
-  }
-
   void process(const mysqlshdk::db::IRow *row, size_t index) {
     // This function is meant to be called only for tables
     assert(m_format == ResultFormat::TABLE);
@@ -296,6 +273,10 @@ class Field_formatter {
     } else if (m_is_numeric) {
       auto str = get_number_string(row, index);
       dlength = blength = str.length();
+    } else if (m_type == mysqlshdk::db::Type::Bit) {
+      auto value = row->get_bit(index);
+      auto str = shcore::lexical_cast<std::string>(value);
+      dlength = blength = str.length();
     } else if (m_type == mysqlshdk::db::Type::Bytes) {
       auto data = row->get_string_data(index);
       auto fsizes = get_utf8_sizes(data.first, data.second, m_flags);
@@ -309,83 +290,84 @@ class Field_formatter {
     }
 
     m_max_mb_holes = std::max<size_t>(m_max_mb_holes, blength - dlength);
-
     m_max_display_length = std::max<size_t>(m_max_display_length, dlength);
-
     m_max_buffer_length = std::max<size_t>(m_max_buffer_length, blength);
-
-    m_display_lengths.push_back(dlength);
-    m_buffer_lengths.push_back(blength);
-
-    m_max_mb_holes = std::max<size_t>(m_max_mb_holes, blength - dlength);
   }
 
   ~Field_formatter() {}
 
   std::string get_number_string(const mysqlshdk::db::IRow *row, size_t index) {
-    std::string tmp;
     if (m_type == mysqlshdk::db::Type::Float) {
-      auto value = row->get_float(index);
-      char buffer[32];
-      size_t len;
-      len = my_gcvt(value, MY_GCVT_ARG_FLOAT, sizeof(buffer) - 1, buffer, NULL);
-      tmp.assign(buffer, len);
+      return shcore::ftoa(row->get_float(index));
     } else if (m_type == mysqlshdk::db::Type::Integer) {
       return std::to_string(row->get_int(index));
     } else if (m_type == mysqlshdk::db::Type::UInteger) {
       return std::to_string(row->get_uint(index));
     } else if (m_type == mysqlshdk::db::Type::Double) {
-      auto value = row->get_double(index);
-      char buffer[32];
-      size_t len;
-      len =
-          my_gcvt(value, MY_GCVT_ARG_DOUBLE, sizeof(buffer) - 1, buffer, NULL);
-      tmp.assign(buffer, len);
+      return shcore::dtoa(row->get_double(index));
     } else {
       return row->get_as_string(index);
     }
-
-    return tmp;
   }
 
   bool put(const mysqlshdk::db::IRow *row, size_t index) {
     reset();
 
+    std::string tmp;
+    const char *data;
+    size_t length;
+
+    size_t display_size;
+    size_t buffer_size;
+
     if (row->is_null(index)) {
-      append("NULL", 4);
+      data = "NULL";
+      display_size = buffer_size = length = 4;
     } else if (m_is_numeric) {
-      std::string tmp = get_number_string(row, index);
+      tmp = get_number_string(row, index);
 
       if (m_zerofill > tmp.length()) {
         tmp = std::string(m_zerofill - tmp.length(), '0').append(tmp);
-
-        // Updates the display length with the new size
-        if (m_format == ResultFormat::TABLE) {
-          m_display_lengths[0] = tmp.length();
-        }
       }
-      append(tmp.data(), tmp.length());
-    } else if (m_type == mysqlshdk::db::Type::Bytes) {
-      auto data = row->get_string_data(index);
-      return append(data.first, data.second);
+      data = tmp.data();
+      display_size = buffer_size = length = tmp.length();
     } else if (m_type == mysqlshdk::db::Type::Bit) {
       auto value = row->get_bit(index);
-      auto tmp = shcore::lexical_cast<std::string>(value);
-      return append(tmp.data(), tmp.length());
+      tmp = shcore::lexical_cast<std::string>(value);
+      data = tmp.data();
+      display_size = buffer_size = length = tmp.length();
+    } else if (m_type == mysqlshdk::db::Type::Bytes) {
+      std::tie(data, length) = row->get_string_data(index);
+
+      std::tie(display_size, buffer_size) =
+          get_utf8_sizes(data, length, m_flags);
     } else {
-      auto data = row->get_as_string(index);
-      return append(data.c_str(), data.size());
+      tmp = row->get_as_string(index);
+      data = tmp.data();
+      length = tmp.length();
+
+      std::tie(display_size, buffer_size) =
+          get_utf8_sizes(data, length, m_flags);
     }
 
+    if (!append(data, length, display_size, buffer_size)) {
+      if (m_is_numeric || m_type == mysqlshdk::db::Type::Bit) {
+        // if a number is larger than expected (e.g. floating pt with lots of
+        // decimals)
+        m_buffer.assign(data, length);
+      } else {
+        return false;
+      }
+    }
     return true;
   }
 
-  const char *c_str() const { return m_buffer.get(); }
+  const std::string &str() const { return m_buffer; }
   size_t get_max_display_length() const { return m_max_display_length; }
   size_t get_max_buffer_length() const { return m_max_buffer_length; }
 
  private:
-  std::unique_ptr<char[]> m_buffer;
+  std::string m_buffer;
   size_t m_allocated;
   size_t m_zerofill;
   bool m_align_right;
@@ -394,10 +376,6 @@ class Field_formatter {
   size_t m_max_buffer_length;
   size_t m_max_mb_holes;
 
-  // Length cache for each data to be printed with this formatter
-  std::deque<size_t> m_display_lengths;
-  std::deque<size_t> m_buffer_lengths;
-
   ResultFormat m_format;
   Print_flags m_flags;
   mysqlshdk::db::Type m_type;
@@ -405,37 +383,23 @@ class Field_formatter {
 
   void reset() {
     // sets the buffer only once
-    if (!m_buffer) {
+    if (m_buffer.empty()) {
       if (m_format == ResultFormat::TABLE) {
         m_allocated = std::max<size_t>(m_max_display_length,
-                                       m_max_buffer_length + m_max_mb_holes) +
-                      1;
+                                       m_max_buffer_length + m_max_mb_holes);
 
         if (m_allocated > MAX_DISPLAY_LENGTH) m_allocated = MAX_DISPLAY_LENGTH;
       } else {
         m_allocated = MAX_DISPLAY_LENGTH;
       }
-
-      m_buffer.reset(new char[m_allocated]);
     }
+    m_buffer.resize(m_allocated);
 
-    memset(m_buffer.get(), ' ', m_allocated);
+    memset(&m_buffer[0], ' ', m_buffer.size());
   }
 
-  bool append(const char *text, size_t length) {
-    size_t display_size;
-    size_t buffer_size;
-    if (m_format == ResultFormat::TABLE) {
-      display_size = m_display_lengths.front();
-      buffer_size = m_buffer_lengths.front();
-      m_display_lengths.pop_front();
-      m_buffer_lengths.pop_front();
-    } else {
-      auto fsizes = get_utf8_sizes(text, length, m_flags);
-      display_size = std::get<0>(fsizes);
-      buffer_size = std::get<1>(fsizes);
-    }
-
+  bool append(const char *text, size_t length, size_t display_size,
+              size_t buffer_size) {
     if (buffer_size > m_allocated) return false;
 
     size_t next_index = 0;
@@ -445,7 +409,7 @@ class Field_formatter {
       }
     }
 
-    auto buffer = m_buffer.get();
+    auto buffer = &m_buffer[0];
     for (size_t index = 0; index < length; index++) {
       if (m_flags.is_set(Print_flag::PRINT_0_AS_ESC) && text[index] == '\0') {
         buffer[next_index++] = '\\';
@@ -471,16 +435,12 @@ class Field_formatter {
     }
 
     if (m_format == ResultFormat::TABLE) {
-      if (buffer_size > display_size) {
-        // It means some multibyte characters were found, and so we need to
-        // truncate the buffer adding the 'lost' characters
-        buffer[m_max_display_length + (buffer_size - display_size)] = 0;
-      } else {
-        // Otherwise, we truncate at _column_width
-        buffer[m_max_display_length] = 0;
-      }
+      // If some multibyte characters were found, we need to truncate the buffer
+      // adding the 'lost' characters
+      buffer[std::min(m_allocated,
+                      m_max_display_length + (buffer_size - display_size))] = 0;
     } else {
-      buffer[next_index] = 0;
+      m_buffer.resize(next_index);
     }
 
     return true;
@@ -536,23 +496,21 @@ Resultset_dumper_base::Resultset_dumper_base(
   if (m_format == "ndjson") m_format = "json/raw";
 }
 
-Resultset_dumper::Resultset_dumper(mysqlshdk::db::IResult *target,
-                                   bool buffer_data)
-    : Resultset_dumper(
-          target, mysqlsh::current_shell_options()->get().wrap_json,
-          mysqlsh::current_shell_options()->get().result_format, buffer_data,
-          mysqlsh::current_shell_options()->get().show_warnings,
-          mysqlsh::current_shell_options()->get().interactive) {}
+Resultset_dumper::Resultset_dumper(mysqlshdk::db::IResult *target)
+    : Resultset_dumper(target,
+                       mysqlsh::current_shell_options()->get().wrap_json,
+                       mysqlsh::current_shell_options()->get().result_format,
+                       mysqlsh::current_shell_options()->get().show_warnings,
+                       mysqlsh::current_shell_options()->get().interactive) {}
 
 Resultset_dumper::Resultset_dumper(mysqlshdk::db::IResult *target,
                                    const std::string &wrap_json,
-                                   const std::string &format, bool buffer_data,
+                                   const std::string &format,
                                    bool show_warnings, bool show_stats)
-    : Resultset_dumper_base(target, shcore::make_unique<Console_printer>(),
+    : Resultset_dumper_base(target, std::make_unique<Console_printer>(),
                             wrap_json, format),
       m_show_warnings(show_warnings),
-      m_show_stats(show_stats),
-      m_buffer_data(buffer_data) {}
+      m_show_stats(show_stats) {}
 
 size_t Resultset_dumper::dump(const std::string &item_label, bool is_query,
                               bool is_doc_result) {
@@ -577,10 +535,6 @@ size_t Resultset_dumper::dump(const std::string &item_label, bool is_query,
       }
 
       if (m_result->has_resultset()) {
-        // Data requires to be buffered on table format because it will be
-        // traversed once for proper formatting and once for printing it
-        if (m_buffer_data || m_format == "table") m_result->buffer();
-
         if (is_doc_result || m_format.find("json") != std::string::npos)
           count = dump_documents(is_doc_result);
         else if (m_format == "vertical")
@@ -629,9 +583,6 @@ size_t Resultset_dumper::dump(const std::string &item_label, bool is_query,
     m_printer->println(
         "Result printing interrupted, rows may be missing from the output.");
 
-  // Restores data
-  if (m_buffer_data) m_result->rewind();
-
   return total_count;
 }
 
@@ -666,11 +617,11 @@ void dump_json_row(shcore::JSON_dumper *dumper,
     } else if (type == mysqlshdk::db::Type::UInteger) {
       dumper->append_uint64(row->get_uint(col_index));
     } else if (type == mysqlshdk::db::Type::Float) {
-      dumper->append_float(row->get_float(col_index));
+      dumper->append_float(static_cast<double>(row->get_float(col_index)));
     } else if (type == mysqlshdk::db::Type::Double) {
       dumper->append_float(row->get_double(col_index));
     } else if (type == mysqlshdk::db::Type::Decimal) {
-      dumper->append_float(row->get_float(col_index));
+      dumper->append_float(static_cast<double>(row->get_float(col_index)));
     } else if (type == mysqlshdk::db::Type::Bit) {
       dumper->append_int64(row->get_bit(col_index));
     }
@@ -683,11 +634,11 @@ void dump_json_row(shcore::JSON_dumper *dumper,
  * being processed.
  */
 size_t Resultset_dumper_base::dump_documents(bool is_doc_result) {
-  auto metadata = m_result->get_metadata();
+  const auto &metadata = m_result->get_metadata();
   auto row = m_result->fetch_one();
   size_t row_count = 0;
-  bool as_array = m_format == "json/array";
-  bool pretty = m_format != "json/raw" && m_format != "json/array";
+  const bool as_array = m_format == "json/array";
+  const bool pretty = m_format != "json/raw" && m_format != "json/array";
 
   if (!row) return row_count;
 
@@ -719,7 +670,7 @@ size_t Resultset_dumper_base::dump_documents(bool is_doc_result) {
 }
 
 size_t Resultset_dumper_base::dump_tabbed() {
-  auto metadata = m_result->get_metadata();
+  const auto &metadata = m_result->get_metadata();
   auto row = m_result->fetch_one();
   size_t row_index = 0;
 
@@ -744,7 +695,7 @@ size_t Resultset_dumper_base::dump_tabbed() {
     for (size_t field_index = 0; field_index < field_count; field_index++) {
       auto column = metadata[field_index];
       if (fmt[field_index].put(row, field_index)) {
-        m_printer->print(fmt[field_index].c_str());
+        m_printer->print(fmt[field_index].str());
       } else {
         mysqlshdk::db::is_string_type(column.get_type());
         m_printer->print(row->get_string(field_index));
@@ -760,15 +711,20 @@ size_t Resultset_dumper_base::dump_tabbed() {
 }
 
 size_t Resultset_dumper_base::dump_vertical() {
-  auto metadata = m_result->get_metadata();
+  return format_vertical(true, true, 0);
+}
 
-  std::string star_separator(27, '*');
+size_t Resultset_dumper_base::format_vertical(bool has_header, bool align_right,
+                                              size_t min_label_width) {
+  const auto &metadata = m_result->get_metadata();
+
+  const std::string star_separator(27, '*');
 
   std::vector<Field_formatter> fmt;
 
   // Calculate length of a longest column description, used to right align
   // column descriptions
-  std::size_t max_col_len = 0;
+  std::size_t max_col_len = min_label_width;
   for (const auto &column : metadata) {
     max_col_len = std::max(max_col_len, column.get_column_label().length());
     fmt.emplace_back(ResultFormat::VERTICAL, column);
@@ -776,22 +732,30 @@ size_t Resultset_dumper_base::dump_vertical() {
 
   auto row = m_result->fetch_one();
   size_t row_index = 0;
-  while (row) {
-    std::string row_header = star_separator + " " +
-                             std::to_string(row_index + 1) + ". row " +
-                             star_separator + "\n";
+  while (row && !m_cancelled) {
+    if (has_header) {
+      std::string row_header = star_separator + " " +
+                               std::to_string(row_index + 1) + ". row " +
+                               star_separator + "\n";
 
-    m_printer->print(row_header);
+      m_printer->print(row_header);
+    }
 
     for (size_t col_index = 0; col_index < metadata.size(); col_index++) {
       auto column = metadata[col_index];
 
       std::string padding(max_col_len - column.get_column_label().size(), ' ');
-      std::string label = padding + column.get_column_label() + ": ";
+      std::string label = column.get_column_label() + ": ";
+
+      if (align_right) {
+        label = padding + label;
+      } else {
+        label += padding;
+      }
 
       m_printer->print(label);
       if (fmt[col_index].put(row, col_index)) {
-        m_printer->print(fmt[col_index].c_str());
+        m_printer->print(fmt[col_index].str());
       } else {
         assert(mysqlshdk::db::is_string_type(column.get_type()));
         m_printer->print(row->get_string(col_index));
@@ -801,8 +765,6 @@ size_t Resultset_dumper_base::dump_vertical() {
 
     row = m_result->fetch_one();
     row_index++;
-
-    if (m_cancelled) return row_index;
   }
   return row_index;
 }
@@ -831,7 +793,7 @@ size_t Resultset_dumper_base::dump_json(const std::string &item_label,
 
   size_t row_count = 0;
   if (m_result->has_resultset()) {
-    auto metadata = m_result->get_metadata();
+    const auto &metadata = m_result->get_metadata();
     auto row = m_result->fetch_one();
     while (row) {
       if (is_doc_result) {
@@ -900,36 +862,36 @@ size_t Resultset_dumper_base::dump_json(const std::string &item_label,
 }
 
 size_t Resultset_dumper_base::dump_table() {
-  auto metadata = m_result->get_metadata();
-
-  std::vector<std::string> columns;
+  const auto &metadata = m_result->get_metadata();
   std::vector<Field_formatter> fmt;
-
-  size_t field_count = metadata.size();
+  std::vector<mysqlshdk::db::Row_copy> pre_fetched_rows;
+  size_t num_records = 0;
+  const size_t field_count = metadata.size();
   if (field_count == 0) return 0;
 
   // Updates the max_length array with the maximum length between column name,
   // min column length and column max length
   for (size_t field_index = 0; field_index < field_count; field_index++) {
     auto column = metadata[field_index];
-
-    columns.push_back(column.get_column_label());
     fmt.emplace_back(ResultFormat::TABLE, column);
   }
 
-  std::vector<const mysqlshdk::db::IRow *> records;
-  auto row = m_result->fetch_one();
-  while (row && !m_cancelled) {
-    records.push_back(row);
-    for (size_t field_index = 0; field_index < field_count; field_index++) {
-      fmt[field_index].process(row, field_index);
+  pre_fetched_rows.reserve(k_pre_fetch_result_rows);
+  {
+    auto row = m_result->fetch_one();
+    while (row && !m_cancelled) {
+      pre_fetched_rows.emplace_back(*row);
+
+      for (size_t field_index = 0; field_index < field_count; field_index++) {
+        fmt[field_index].process(row, field_index);
+      }
+
+      if (pre_fetched_rows.size() >= k_pre_fetch_result_rows) break;
+
+      row = m_result->fetch_one();
     }
-    row = m_result->fetch_one();
   }
-
-  m_result->rewind();
-
-  if (m_cancelled || records.empty()) return 0;
+  if (m_cancelled || pre_fetched_rows.empty()) return 0;
 
   //-----------
 
@@ -956,29 +918,48 @@ size_t Resultset_dumper_base::dump_table() {
   }
   m_printer->print(separator);
 
-  // Now prints the records
-  row = m_result->fetch_one();
-  while (row && !m_cancelled) {
+  // Print pre-fetched records
+  for (const auto &row : pre_fetched_rows) {
+    ++num_records;
     m_printer->print("| ");
 
     for (size_t field_index = 0; field_index < field_count; field_index++) {
-      if (fmt[field_index].put(row, field_index)) {
-        m_printer->print(fmt[field_index].c_str());
+      if (fmt[field_index].put(&row, field_index)) {
+        m_printer->print(fmt[field_index].str());
       } else {
         assert(mysqlshdk::db::is_string_type(metadata[field_index].get_type()));
-        m_printer->print(row->get_as_string(field_index));
+        m_printer->print(row.get_as_string(field_index));
       }
       if (field_index < field_count - 1) m_printer->print(" | ");
     }
     m_printer->print(" |\n");
-    row = m_result->fetch_one();
+
+    if (m_cancelled) break;
   }
+  // Now prints the remaining records
+  if (!m_cancelled) {
+    auto row = m_result->fetch_one();
+    while (row && !m_cancelled) {
+      ++num_records;
+      m_printer->print("| ");
 
-  m_result->rewind();
+      for (size_t field_index = 0; field_index < field_count; field_index++) {
+        if (fmt[field_index].put(row, field_index)) {
+          m_printer->print(fmt[field_index].str());
+        } else {
+          assert(
+              mysqlshdk::db::is_string_type(metadata[field_index].get_type()));
+          m_printer->print(row->get_as_string(field_index));
+        }
+        if (field_index < field_count - 1) m_printer->print(" | ");
+      }
+      m_printer->print(" |\n");
+      row = m_result->fetch_one();
+    }
+  }
+  m_printer->print(separator);
 
-  m_printer->print(separator.c_str());
-
-  return records.size();
+  return num_records;
 }
 
 std::string Resultset_dumper::get_affected_stats(
@@ -1021,49 +1002,46 @@ int Resultset_dumper::get_warning_and_execution_time_stats(
 }
 
 void Resultset_dumper_base::dump_warnings() {
-  if (m_result->get_warning_count()) {
-    auto warning = m_result->fetch_one_warning();
-    while (warning && !m_cancelled) {
-      std::string type;
-      switch (warning->level) {
-        case mysqlshdk::db::Warning::Level::Note:
-          type = "Note";
-          break;
-        case mysqlshdk::db::Warning::Level::Warn:
-          type = "Warning";
-          break;
-        case mysqlshdk::db::Warning::Level::Error:
-          type = "Error";
-          break;
-      }
-
-      m_printer->print(
-          (shcore::str_format("%s (code %d): %s\n", type.c_str(), warning->code,
-                              warning->msg.c_str())));
-
-      warning = m_result->fetch_one_warning();
+  auto warning = m_result->fetch_one_warning();
+  while (warning && !m_cancelled) {
+    std::string type;
+    switch (warning->level) {
+      case mysqlshdk::db::Warning::Level::Note:
+        type = "Note";
+        break;
+      case mysqlshdk::db::Warning::Level::Warn:
+        type = "Warning";
+        break;
+      case mysqlshdk::db::Warning::Level::Error:
+        type = "Error";
+        break;
     }
+
+    m_printer->print((shcore::str_format("%s (code %d): %s\n", type.c_str(),
+                                         warning->code, warning->msg.c_str())));
+
+    warning = m_result->fetch_one_warning();
   }
 }
 
 Resultset_writer::Resultset_writer(mysqlshdk::db::IResult *target)
     : Resultset_dumper_base(
-          target, shcore::make_unique<String_printer>(),
+          target, std::make_unique<String_printer>(),
           mysqlsh::current_shell_options()->get().wrap_json,
           mysqlsh::current_shell_options()->get().result_format) {}
 
 std::string Resultset_writer::write_table() {
-  return write([this]() {
-    // dump_table() iterates over rows twice, data needs to be buffered
-    m_result->buffer();
-    dump_table();
-  });
+  return write([this]() { dump_table(); });
 }
 
 std::string Resultset_writer::write_vertical() {
   return write([this]() { dump_vertical(); });
 }
 
+std::string Resultset_writer::write_status() {
+  return write([this]() { format_vertical(false, false, 24); });
+}
+
 std::string Resultset_writer::write(const std::function<void()> &dump) {
   const auto printer = dynamic_cast<String_printer *>(m_printer.get());
   printer->reset();
