@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -34,6 +34,8 @@
 #include <io.h>
 #define isatty _isatty
 #else
+#include <fcntl.h>
+#include <sys/ioctl.h>
 #include <unistd.h>
 #endif
 
@@ -41,13 +43,18 @@
 #include "modules/devapi/base_resultset.h"
 #include "modules/mod_shell_options.h"  // <---
 #include "mysqlshdk/libs/utils/logger.h"
+#include "mysqlshdk/libs/utils/structured_text.h"
 #include "mysqlshdk/libs/utils/utils_file.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
+#include "mysqlshdk/libs/utils/utils_path.h"
 #include "mysqlshdk/libs/utils/utils_sqlstring.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
+#include "mysqlshdk/shellcore/shell_console.h"
 #include "shellcore/base_session.h"
 #include "shellcore/interrupt_handler.h"
 #include "shellcore/utils_help.h"
+#include "src/mysqlsh/commands/command_edit.h"
+#include "src/mysqlsh/commands/command_system.h"
 
 #ifdef WIN32
 #undef max
@@ -190,6 +197,113 @@ void print_diag(const std::string &s) { current_console()->print_diag(s); }
 
 void println(const std::string &s = "") { current_console()->println(s); }
 
+template <typename T>
+void display_info(const std::shared_ptr<T> &object, const std::string &variable,
+                  const std::string &context, bool print_name = true) {
+  static constexpr auto var_begin = "${";
+  static constexpr auto var_end = "}";
+  const auto replace = [&object, &variable, &context](const std::string &var) {
+    if (var == "name") {
+      return object->impl()->get_name();
+    } else if (var == "class") {
+      return object->class_name();
+    } else if (var == "var") {
+      return variable;
+    } else if (var == "context") {
+      return context;
+    }
+
+    return std::string{"ERROR"};
+  };
+
+  if (object) {
+    const auto console = current_console();
+
+    if (print_name) {
+      console->print_info(shcore::str_subvars(
+          "You are connected to a member of ${context} '${name}'.", replace,
+          var_begin, var_end));
+    }
+
+    console->print_info(shcore::str_subvars(
+        "Variable '${var}' is set.\nUse ${var}.status() in scripting mode to "
+        "get status of this ${context} or \\? ${class} for more commands.",
+        replace, var_begin, var_end));
+  }
+}
+
+/**
+ * Checks if standard input is a TTY.
+ *
+ * @return true if standard input is a TTY
+ */
+bool is_stdin_a_tty() {
+  const auto fd =
+#ifdef _WIN32
+      _fileno(stdin)
+#else   // !_WIN32
+      STDIN_FILENO
+#endif  // !_WIN32
+      ;
+  return isatty(fd);
+}
+
+/**
+ * Sends CTRL-C sequence to the controlling terminal/console.
+ *
+ * @return true if sequence was sent
+ */
+bool send_ctrl_c_to_terminal() {
+  bool success = false;
+#ifdef _WIN32
+  const auto handle = CreateFile("CONIN$",         // lpFileName
+                                 GENERIC_WRITE,    // dwDesiredAccess
+                                 FILE_SHARE_READ,  // dwShareMode
+                                 nullptr,          // lpSecurityAttributes
+                                 OPEN_EXISTING,    // dwCreationDisposition
+                                 0,                // dwFlagsAndAttributes
+                                 nullptr           // hTemplateFile
+  );
+
+  if (INVALID_HANDLE_VALUE != handle) {
+    constexpr DWORD count = 1;
+    INPUT_RECORD ir[count];
+
+    ir[0].EventType = KEY_EVENT;
+    ir[0].Event.KeyEvent.bKeyDown = TRUE;
+    ir[0].Event.KeyEvent.dwControlKeyState = 0;
+    ir[0].Event.KeyEvent.uChar.UnicodeChar =
+        MapVirtualKey(VK_CANCEL, MAPVK_VK_TO_CHAR);
+    ir[0].Event.KeyEvent.wRepeatCount = 1;
+    ir[0].Event.KeyEvent.wVirtualKeyCode = VK_CANCEL;
+    ir[0].Event.KeyEvent.wVirtualScanCode =
+        MapVirtualKey(VK_CANCEL, MAPVK_VK_TO_VSC);
+
+    DWORD written = 0;
+    success = WriteConsoleInput(handle, ir, count, &written);
+
+    CloseHandle(handle);
+  }
+#else   // !_WIN32
+  char controlling_terminal[L_ctermid];
+  int fd = -1;
+
+  // get controlling terminal name, open it only for writing, do not set that
+  // terminal as a controlling if process does not have one
+  if (ctermid(controlling_terminal) &&
+      (fd = ::open(controlling_terminal, O_WRONLY | O_NOCTTY)) >= 0) {
+    // simulate user input
+    if (-1 != ioctl(fd, TIOCSTI, CTRL_C_STR)) {
+      success = true;
+    }
+
+    ::close(fd);
+  }
+#endif  // !_WIN32
+
+  return success;
+}
+
 }  // namespace
 
 REGISTER_HELP(CMD_HISTORY_BRIEF, "View and edit command line history.");
@@ -267,6 +381,46 @@ REGISTER_HELP(CMD_PAGER_EXAMPLE4_DESC, "Sets pager to \"more -10\".");
 REGISTER_HELP(CMD_NOPAGER_BRIEF, "Disables the current pager.");
 REGISTER_HELP(CMD_NOPAGER_SYNTAX, "<b>\\nopager</b>");
 
+REGISTER_HELP(CMD_SYSTEM_BRIEF, "Execute a system shell command.");
+REGISTER_HELP(CMD_SYSTEM_SYNTAX, "<b>\\system</b> [command [arguments...]]");
+REGISTER_HELP(CMD_SYSTEM_SYNTAX1, "<b>\\!</b> [command [arguments...]]");
+REGISTER_HELP(CMD_SYSTEM_EXAMPLE, "<b>\\system</b>");
+REGISTER_HELP(CMD_SYSTEM_EXAMPLE_DESC,
+              "With no arguments, this command displays this help.");
+REGISTER_HELP(CMD_SYSTEM_EXAMPLE1, "<b>\\system</b> ls");
+REGISTER_HELP(CMD_SYSTEM_EXAMPLE1_DESC,
+              "Executes 'ls' in the current working directory and displays the "
+              "result.");
+REGISTER_HELP(CMD_SYSTEM_EXAMPLE2, "<b>\\!</b> ls > list.txt");
+REGISTER_HELP(CMD_SYSTEM_EXAMPLE2_DESC,
+              "Executes 'ls' in the current working directory, storing the "
+              "result in the 'list.txt' file.");
+
+REGISTER_HELP(CMD_EDIT_BRIEF,
+              "Launch a system editor to edit a command to be executed.");
+REGISTER_HELP(CMD_EDIT_DETAIL,
+              "The system editor is selected using the <b>EDITOR</b> and "
+              "<b>VISUAL</b> environment variables.");
+#ifdef _WIN32
+REGISTER_HELP(CMD_EDIT_DETAIL1,
+              "If these are not set, falls back to notepad.exe.");
+#else   // !_WIN32
+REGISTER_HELP(CMD_EDIT_DETAIL1, "If these are not set, falls back to vi.");
+#endif  // !_WIN32
+REGISTER_HELP(
+    CMD_EDIT_DETAIL2,
+    "It is also possible to invoke this command by pressing CTRL-X CTRL-E.");
+REGISTER_HELP(CMD_EDIT_SYNTAX, "<b>\\edit</b> [arguments...]");
+REGISTER_HELP(CMD_EDIT_SYNTAX1, "<b>\\e</b> [arguments...]");
+REGISTER_HELP(CMD_EDIT_EXAMPLE, "<b>\\edit</b>");
+REGISTER_HELP(CMD_EDIT_EXAMPLE_DESC,
+              "Allows to edit the last command in history.");
+REGISTER_HELP(CMD_EDIT_EXAMPLE_DESC1,
+              "If there are no commands in history, editor will be blank.");
+REGISTER_HELP(CMD_EDIT_EXAMPLE1, "<b>\\e</b> print('hello world!')");
+REGISTER_HELP(CMD_EDIT_EXAMPLE1_DESC,
+              "Allows to edit the commands given as arguments.");
+
 Command_line_shell::Command_line_shell(
     std::shared_ptr<Shell_options> cmdline_options,
     std::unique_ptr<shcore::Interpreter_delegate> delegate,
@@ -293,6 +447,8 @@ Command_line_shell::Command_line_shell(
   _history.set_limit(std::min<int64_t>(options().history_max_size,
                                        std::numeric_limits<int>::max()));
 
+  m_syslog.enable(options().history_sql_syslog);
+
   observe_notification(SN_SHELL_OPTION_CHANGED);
 
   linenoiseSetCompletionCallback(auto_complete);
@@ -302,6 +458,32 @@ Command_line_shell::Command_line_shell(
   SET_SHELL_COMMAND("\\pager|\\P", "CMD_PAGER", Command_line_shell::cmd_pager);
   SET_SHELL_COMMAND("\\nopager", "CMD_NOPAGER",
                     Command_line_shell::cmd_nopager);
+  SET_CUSTOM_SHELL_COMMAND("\\system|\\!", "CMD_SYSTEM",
+                           [this](const std::vector<std::string> &args) {
+                             return Command_system(_shell)(args);
+                           });
+  SET_CUSTOM_SHELL_COMMAND("\\edit|\\e", "CMD_EDIT",
+                           [this](const std::vector<std::string> &args) {
+                             return Command_edit(_shell, this, &_history)(args);
+                           });
+
+  linenoiseRegisterCustomCommand(
+      "\x18\x05",  // CTRL-X, CTRL-E
+      [](const char *, void *data, char **) {
+        const auto that = static_cast<Command_line_shell *>(data);
+
+        if (that->set_pending_command([that](const std::string &line) {
+              Command_edit(that->_shell, that, &that->_history)
+                  .execute(that->_input_buffer + line);
+            })) {
+          // command was registered, simulate user pressing enter
+          return 0x0A;
+        } else {
+          // it was not possible to register the edit command, continue
+          return -1;
+        }
+      },
+      this);
 
   if (!m_default_pager.empty()) {
     log_info("%s", get_pager_message(m_default_pager).c_str());
@@ -377,18 +559,30 @@ Command_line_shell::~Command_line_shell() {
   current_console()->disable_global_pager();
   // disable verbose output, since it also uses the delegate
   current_console()->set_verbose(0);
+
+  linenoiseRemoveCustomCommand("\x18\x05");
 }
 
 void Command_line_shell::load_prompt_theme(const std::string &path) {
   if (!path.empty()) {
-    std::ifstream f;
-    f.open(path);
+#ifdef _WIN32
+    std::ifstream f(shcore::utf8_to_wide(path));
+#else
+    std::ifstream f(path);
+#endif
+
     if (f.good()) {
       std::stringstream buffer;
       buffer << f.rdbuf();
+
       try {
-        shcore::Value theme(shcore::Value::parse(buffer.str().c_str()));
-        _prompt.set_theme(theme);
+        const auto data = buffer.str();
+
+        if (!shcore::is_valid_utf8(data)) {
+          throw std::invalid_argument("File contains invalid UTF-8 sequence.");
+        }
+
+        _prompt.set_theme(shcore::Value::parse(data));
       } catch (const std::exception &e) {
         log_warning("Error loading prompt theme '%s': %s", path.c_str(),
                     e.what());
@@ -441,7 +635,7 @@ void Command_line_shell::save_state(shcore::Shell_core::Mode mode) {
 bool Command_line_shell::switch_shell_mode(shcore::Shell_core::Mode mode,
                                            const std::vector<std::string> &args,
                                            bool initializing,
-                                           bool prompt_variables_update) {
+                                           bool /* prompt_variables_update */) {
   shcore::Shell_core::Mode old_mode = _shell->interactive_mode();
   bool ret = Mysql_shell::switch_shell_mode(mode, args, initializing);
 
@@ -718,66 +912,128 @@ void Command_line_shell::handle_interrupt() { _interrupted = true; }
 shcore::Prompt_result Command_line_shell::deleg_prompt(void *cdata,
                                                        const char *prompt,
                                                        std::string *ret) {
-  Command_line_shell *self = reinterpret_cast<Command_line_shell *>(cdata);
-  self->_interrupted = false;
-  char *tmp = Command_line_shell::readline(prompt);
-  if (tmp && strcmp(tmp, CTRL_C_STR) == 0) self->_interrupted = true;
-  if (!tmp || self->_interrupted) {
-    if (tmp) free(tmp);
-    *ret = "";
-    if (self->_interrupted)
-      return shcore::Prompt_result::Cancel;
-    else
-      return shcore::Prompt_result::CTRL_D;
-  }
-  *ret = tmp;
-  free(tmp);
-  return shcore::Prompt_result::Ok;
+  const auto self = reinterpret_cast<Command_line_shell *>(cdata);
+
+  return self->do_prompt(Command_line_shell::readline, false, prompt, ret);
 }
 
 shcore::Prompt_result Command_line_shell::deleg_password(void *cdata,
                                                          const char *prompt,
                                                          std::string *ret) {
-  Command_line_shell *self = reinterpret_cast<Command_line_shell *>(cdata);
-  self->_interrupted = false;
-  shcore::Interrupt_handler inth([self]() {
-    self->handle_interrupt();
+  const auto self = reinterpret_cast<Command_line_shell *>(cdata);
+  const auto passwords_from_stdin = self->options().passwords_from_stdin;
+
+  return self->do_prompt(passwords_from_stdin ? shcore::mysh_get_stdin_password
+                                              : mysh_get_tty_password,
+                         true, prompt, ret);
+}
+
+shcore::Prompt_result Command_line_shell::do_prompt(
+    char *(*get_response)(const char *), bool is_password, const char *text,
+    std::string *ret) {
+  char *tmp = nullptr;
+
+  shcore::on_leave_scope cleanup([tmp, is_password]() {
+    if (tmp) {
+      if (is_password) {
+        // BUG#28915716: Cleans up the memory containing the password
+        shcore::clear_buffer(tmp, ::strlen(tmp));
+      }
+
+      free(tmp);
+    }
+  });
+
+  shcore::Interrupt_handler second_handler([get_response]() {
+#ifdef _WIN32
+    // this handler is not called if shell is running interactively
+    // this handler is called when shell is running interactively with
+    // --passwords-from-stdin option and a password prompt is canceled
+    // this handler is called when shell is running in non-interactive mode
+    // (i.e. launched by another process)
+
+    // cancel any pending I/O, if signal was sent using
+    // GenerateConsoleCtrlEvent() these are not going to be interrupted on its
+    // own
+    CancelIoEx(GetStdHandle(STD_INPUT_HANDLE), nullptr);
+
+    // mysh_get_tty_password() reads directly from the console, CancelIoEx()
+    // will not interrupt this, need to send CTRL-C
+    if (mysh_get_tty_password == get_response) {
+#else   // !_WIN32
+    // if signal is sent to us from another process and we're reading from
+    // terminal, we need to generate CTRL-C sequence to interrupt read loops
+    if (mysh_get_tty_password == get_response || is_stdin_a_tty()) {
+#endif  // !_WIN32
+      send_ctrl_c_to_terminal();
+    }
+
+    // always suppress further processing of the interrupt handlers, if a call
+    // to prompt*() throws in response to Prompt_result::Cancel, we want to make
+    // sure that this exception is properly propagated
+    return false;
+  });
+
+  shcore::Interrupt_handler first_handler([this]() {
+    // we need another Interrupt_handler, which is going to be executed first
+    // and returns true, to make sure that any interrupts generated by the
+    // handler registered first (and executed second) are suppressed
+    handle_interrupt();
     return true;
   });
-  char *tmp = self->options().passwords_from_stdin
-                  ? shcore::mysh_get_stdin_password(prompt)
-                  : mysh_get_tty_password(prompt);
-  if (tmp && strcmp(tmp, CTRL_C_STR) == 0) self->_interrupted = true;
-  if (!tmp || self->_interrupted) {
-    if (tmp) free(tmp);
+
+  _interrupted = false;
+
+  tmp = get_response(text);
+
+#ifdef _WIN32
+  if (!tmp && !_interrupted) {
+    // If prompt returned no result but it was not interrupted, it means that
+    // either user pressed CTRL-D/CTRL-Z or there was a signal which
+    // interrupted read(), but was not yet delivered (signals are asynchronous
+    // on Windows) - this may happen i.e. if shell was started with
+    // --passwords-from-stdin and password prompt is interrupted with CTRL-C.
+    // Wait a bit here, if signal arrives, sleep is going to be interrupted.
+    // Worst case scenario: CTRL-D/CTRL-Z was pressed or signal arrived after
+    // flag was checked but before we went to sleep.
+    shcore::sleep_ms(100);
+  }
+#endif  // _WIN32
+
+  if (tmp && strcmp(tmp, CTRL_C_STR) == 0) _interrupted = true;
+
+  if (!tmp || _interrupted) {
     *ret = "";
-    if (self->_interrupted)
+
+    if (_interrupted)
       return shcore::Prompt_result::Cancel;
     else
       return shcore::Prompt_result::CTRL_D;
   }
+
   *ret = tmp;
 
-  // BUG#28915716: Cleans up the memory containing the password
-  shcore::clear_buffer(tmp, ret->size());
-
-  free(tmp);
   return shcore::Prompt_result::Ok;
 }
 
 std::string Command_line_shell::history_file(shcore::Shell_core::Mode mode) {
-  return shcore::get_user_config_path() + "/history." +
-         to_string(mode == shcore::Shell_core::Mode::None ? interactive_mode()
-                                                          : mode);
+  return shcore::path::join_path(
+      shcore::get_user_config_path(),
+      "history." + to_string(mode == shcore::Shell_core::Mode::None
+                                 ? interactive_mode()
+                                 : mode));
+}
+
+void Command_line_shell::pre_command_loop() {
+  display_info(m_default_cluster, "cluster", "cluster");
+  display_info(m_default_clusterset, "clusterset", "clusterset");
+
+  // information about replicaset name is displayed by the dba
+  display_info(m_default_replicaset, "rs", "replicaset", false);
 }
 
 void Command_line_shell::command_loop() {
-  bool using_tty = false;
-#if defined(WIN32)
-  using_tty = isatty(_fileno(stdin));
-#else
-  using_tty = isatty(STDIN_FILENO);
-#endif
+  bool using_tty = is_stdin_a_tty();
 
   if (_deferred_output && !_deferred_output->empty()) {
     println(*_deferred_output);
@@ -830,7 +1086,13 @@ void Command_line_shell::command_loop() {
           mysqlsh::current_console()->raw_print(
               "\n", mysqlsh::Output_stream::STDOUT, false);
         }
+
+        m_set_pending_command = true;
+
         char *tmp = Command_line_shell::readline(prompt().c_str());
+
+        m_set_pending_command = false;
+
         if (tmp && strcmp(tmp, CTRL_C_STR) != 0) {
           cmd = tmp;
           free(tmp);
@@ -856,7 +1118,7 @@ void Command_line_shell::command_loop() {
           mysqlsh::current_console()->raw_print(
               prompt(), mysqlsh::Output_stream::STDOUT, false);
         }
-        if (!std::getline(std::cin, cmd)) {
+        if (!shcore::getline(std::cin, cmd)) {
           if (_interrupted || !std::cin.eof()) {
             _interrupted = false;
             continue;
@@ -880,13 +1142,11 @@ void Command_line_shell::print_banner() {
   std::string welcome_msg("MySQL Shell ");
   welcome_msg += MYSH_FULL_VERSION;
   welcome_msg += "\n\n";
-  welcome_msg +=
-      "Copyright (c) 2016, " PACKAGE_YEAR
-      ", Oracle and/or its affiliates. All rights "
-      "reserved.\n"
-      "Oracle is a registered trademark of Oracle Corporation and/or its "
-      "affiliates.\nOther names may be trademarks of their respective "
-      "owners.";
+  welcome_msg += "Copyright (c) 2016, " PACKAGE_YEAR
+                 ", Oracle and/or its affiliates.\n"
+                 "Oracle is a registered trademark of Oracle Corporation "
+                 "and/or its affiliates.\n"
+                 "Other names may be trademarks of their respective owners.";
   println(welcome_msg);
   println();
   println("Type '\\help' or '\\?' for help; '\\quit' to exit.");
@@ -900,18 +1160,16 @@ void Command_line_shell::print_cmd_line_helper() {
   println("");
   // Splitting line in two so the git hook does not complain
   println("Copyright (c) 2016, " PACKAGE_YEAR ", "
-          "Oracle and/or its affiliates. All rights reserved.");
-  println("");
-  println("Oracle is a registered trademark of Oracle Corporation and/or its");
-  println("affiliates. Other names may be trademarks of their respective");
-  println("owners.");
+          "Oracle and/or its affiliates.");
+  println("Oracle is a registered trademark of Oracle Corporation and/or its affiliates.");
+  println("Other names may be trademarks of their respective owners.");
   println("");
   println("Usage: mysqlsh [OPTIONS] [URI]");
-  println("       mysqlsh [OPTIONS] [URI] -f <path> [script args...]");
-  println("       mysqlsh [OPTIONS] [URI] --dba [command]");
+  println("       mysqlsh [OPTIONS] [URI] -f <path> [<script-args>...]");
+  println("       mysqlsh [OPTIONS] [URI] --dba enableXProtocol");
   println("       mysqlsh [OPTIONS] [URI] --cluster");
-  println("       mysqlsh [OPTIONS] [URI] -- <object> <method> [method args...]");
-  println("       mysqlsh [OPTIONS] [URI] --import file|- [collection] | [table [, column]");
+  println("       mysqlsh [OPTIONS] [URI] -- <object> <method> [<method-args>...]");
+  println("       mysqlsh [OPTIONS] [URI] --import {<file>|-} [<collection>|<table> <column>]");
   println("");
   // clang-format on
   std::vector<std::string> details = Shell_options(0, nullptr).get_details();
@@ -933,23 +1191,16 @@ void Command_line_shell::print_cmd_line_helper() {
 bool Command_line_shell::cmd_process_file(
     const std::vector<std::string> &params) {
   pause_history(true);
-  try {
-    bool r = Mysql_shell::cmd_process_file(params);
-    pause_history(false);
-    return r;
-  } catch (...) {
-    pause_history(false);
-    throw;
-  }
+  shcore::on_leave_scope resume_history{[this]() { pause_history(false); }};
+
+  return Mysql_shell::cmd_process_file(params);
 }
 
 void Command_line_shell::handle_notification(
-    const std::string &name, const shcore::Object_bridge_ref &sender,
+    const std::string &name, const shcore::Object_bridge_ref & /* sender */,
     shcore::Value::Map_type_ref data) {
   if (name == "SN_STATEMENT_EXECUTED") {
-    std::string executed = data->get_string("statement");
-    while (executed.back() == '\n' || executed.back() == '\r')
-      executed.pop_back();
+    const auto executed = shcore::str_strip(data->get_string("statement"));
     auto mode = interactive_mode();
     auto sql = executed;
     if (shcore::str_beginswith(executed, "\\sql ") && executed.length() > 5) {
@@ -958,6 +1209,10 @@ void Command_line_shell::handle_notification(
     }
     if (mode != shcore::Shell_core::Mode::SQL || sql_safe_for_logging(sql)) {
       _history.add(executed);
+
+      if (shcore::Shell_core::Mode::SQL == mode) {
+        syslog(sql);
+      }
     } else {
       // add but delete after the next command and
       // don't let it get saved to disk either
@@ -989,6 +1244,8 @@ void Command_line_shell::handle_notification(
       const auto console = current_console();
 
       console->set_verbose(data->get_int("value"));
+    } else if (SHCORE_HISTORY_SQL_SYSLOG == option) {
+      m_syslog.enable(data->get_bool("value"));
     }
   }
 }
@@ -1017,4 +1274,107 @@ void Command_line_shell::detect_session_change() {
   }
 }
 
+void Command_line_shell::set_next_input(const std::string &input) {
+  if (!input.empty()) {
+    auto lines = shcore::str_split(input, "\n");
+
+    // trim whitespace from the right side of the lines
+    std::transform(
+        lines.begin(), lines.end(), lines.begin(),
+        [](const std::string &line) { return shcore::str_rstrip(line); });
+
+    // remove consecutive empty lines at the end
+    while (lines.size() > 1 && lines.back().empty() &&
+           std::prev(lines.end(), 2)->empty()) {
+      lines.pop_back();
+    }
+
+    // if there are only two lines, and the last one is empty, remove it as well
+    if (lines.size() == 2 && lines.back().empty()) {
+      lines.pop_back();
+    }
+
+    // the last line goes to the linenoise and can be edited
+    const auto last_line = lines.back();
+    lines.pop_back();
+
+    const auto console = current_console();
+
+    // emulate user writing the input line by line
+    _input_buffer = "";
+    _input_mode = shcore::Input_state::Ok;
+
+    for (const auto &line : lines) {
+      console->raw_print(prompt() + line + "\n", mysqlsh::Output_stream::STDOUT,
+                         false);
+
+      _input_buffer += line + "\n";
+      _input_mode = shcore::Input_state::ContinuedBlock;
+    }
+
+    // feed the last line to the linenoise
+    if (!last_line.empty()) {
+      linenoisePreloadBuffer(last_line.c_str());
+    }
+  }
+}
+
+bool Command_line_shell::set_pending_command(const Pending_command &command) {
+  if (m_set_pending_command) {
+    m_pending_command = command;
+  }
+
+  return m_set_pending_command;
+}
+
+void Command_line_shell::process_line(const std::string &line) {
+  if (m_pending_command) {
+    m_pending_command(line);
+    m_pending_command = nullptr;
+  } else {
+    Mysql_shell::process_line(line);
+  }
+}
+
+void Command_line_shell::syslog(const std::string &statement) {
+  // log SQL statements and \source commands
+  if (m_syslog.active() &&
+      ('\\' != statement[0] || shcore::str_beginswith(statement, "\\source ") ||
+       shcore::str_beginswith(statement, "\\. "))) {
+    m_syslog.log(shcore::syslog::Level::INFO, syslog_format(statement));
+  }
+}
+
+std::string Command_line_shell::syslog_format(const std::string &statement) {
+  const std::size_t max_log_length = 1024;
+  const auto variables = prompt_variables();
+
+  std::vector<std::string> data;
+
+  const auto add_data = [&data](const char *key, const std::string &value) {
+    data.emplace_back(shcore::make_kvp(key, value.empty() ? "--" : value));
+  };
+
+  const auto add_data_from_prompt =
+      [&variables, &add_data](const char *key, const std::string &var) {
+        const auto value = variables->find(var);
+
+        add_data(key, variables->end() != value ? value->second : "");
+      };
+
+  add_data_from_prompt("SYSTEM_USER", "system_user");
+  add_data_from_prompt("MYSQL_USER", "user");
+  add_data_from_prompt("CONNECTION_ID", "connection_id");
+  add_data_from_prompt("DB_SERVER", "host");
+  add_data_from_prompt("DB", "schema");
+  add_data("QUERY", statement);
+
+  return shcore::truncate(shcore::str_join(data, " "), max_log_length);
+}
+
+void Command_line_shell::pause_history(bool flag) {
+  _history.pause(flag);
+  m_syslog.pause(flag);
+}
+
 }  // namespace mysqlsh
