@@ -1,6 +1,5 @@
-
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -207,10 +206,9 @@ class Completer_frontend : public Shell_core_test_wrapper {
     execute("del globals()['db2']");
     wipe_all();
     execute("sys.version.split(' ')[0]");
-    if (output_handler.std_out.find("2.6") != std::string::npos)
-      _python26 = true;
+    m_python_version = output_handler.std_out;
   }
-  bool _python26 = false;
+  std::string m_python_version;
 
  public:
   std::pair<std::string, std::vector<std::string>> complete(
@@ -276,18 +274,30 @@ class Completer_frontend : public Shell_core_test_wrapper {
     if (_target_server_version < mysqlshdk::utils::Version("8.0")) {
       method_args.erase(
           remove_if(method_args.begin(), method_args.end(),
-                    [](const decltype(method_args)::value_type &other) {
+                    [](const auto &other) {
                       return other.first.compare("replaceOne") == 0 ||
                              other.first.compare("createIndex") == 0;
                     }),
           method_args.end());
-      completions.erase(
-          remove_if(completions.begin(), completions.end(),
-                    [](const decltype(completions)::value_type &other) {
-                      return other.compare("replaceOne()") == 0 ||
-                             other.compare("createIndex()") == 0;
-                    }),
-          completions.end());
+      completions.erase(remove_if(completions.begin(), completions.end(),
+                                  [](const auto &other) {
+                                    return other.compare("replaceOne()") == 0 ||
+                                           other.compare("createIndex()") == 0;
+                                  }),
+                        completions.end());
+    }
+
+    if (_target_server_version < mysqlshdk::utils::Version("8.0.19")) {
+      method_args.erase(remove_if(method_args.begin(), method_args.end(),
+                                  [](const auto &other) {
+                                    return other.first == "modifyCollection";
+                                  }),
+                        method_args.end());
+      completions.erase(remove_if(completions.begin(), completions.end(),
+                                  [](const auto &other) {
+                                    return other == "modifyCollection()";
+                                  }),
+                        completions.end());
     }
 
     // first execute the methods with special handling given by the test
@@ -388,8 +398,8 @@ class Completer_frontend : public Shell_core_test_wrapper {
 
     std::vector<std::string> expected;
     for (const shcore::Value &value : *result.as_array()) {
-      if (value.get_string()[0] == '_')  // skip __methods from python
-        continue;
+      // skip __methods from python
+      if (shcore::str_beginswith(value.get_string(), "__")) continue;
       expected.push_back(value.get_string());
     }
     std::sort(expected.begin(), expected.end());
@@ -499,12 +509,12 @@ TEST_F(Completer_frontend, builtin_others) {
   EXPECT_AFTER_TAB("\\hi", "\\history");
   EXPECT_AFTER_TAB("\\he", "\\help");
 
-  auto expect =
-      strv({"\\",       "\\connect", "\\exit",       "\\help",   "\\history",
-            "\\js",     "\\nopager", "\\nowarnings", "\\option", "\\pager",
-            "\\py",     "\\quit",    "\\reconnect",  "\\rehash", "\\show",
-            "\\source", "\\sql",     "\\status",     "\\use",    "\\warnings",
-            "\\watch"});
+  auto expect = strv({"\\",        "\\connect",    "\\disconnect", "\\edit",
+                      "\\exit",    "\\help",       "\\history",    "\\js",
+                      "\\nopager", "\\nowarnings", "\\option",     "\\pager",
+                      "\\py",      "\\quit",       "\\reconnect",  "\\rehash",
+                      "\\show",    "\\source",     "\\sql",        "\\status",
+                      "\\system",  "\\use",        "\\warnings",   "\\watch"});
 
 #ifndef HAVE_V8
   expect.erase(std::find(expect.begin(), expect.end(), "\\js"));
@@ -655,13 +665,16 @@ TEST_F(Completer_frontend, js_shell) {
   CHECK_OBJECT_COMPLETIONS("shell");
 
   EXPECT_AFTER_TAB("sh", "shell");
-  EXPECT_AFTER_TAB("shell.con", "shell.connect()");
+  EXPECT_AFTER_TAB("shell.rec", "shell.reconnect()");
   EXPECT_AFTER_TAB_TAB("shell.", strv({"addExtensionObjectMember()",
                                        "connect()",
+                                       "connectToPrimary()",
+                                       "createContext()",
                                        "createExtensionObject()",
                                        "deleteAllCredentials()",
                                        "deleteCredential()",
                                        "disablePager()",
+                                       "disconnect()",
                                        "dumpRows()",
                                        "enablePager()",
                                        "getSession()",
@@ -669,6 +682,7 @@ TEST_F(Completer_frontend, js_shell) {
                                        "listCredentialHelpers()",
                                        "listCredentials()",
                                        "log()",
+                                       "openSession()",
                                        "options",
                                        "parseUri()",
                                        "prompt()",
@@ -680,7 +694,8 @@ TEST_F(Completer_frontend, js_shell) {
                                        "setSession()",
                                        "status()",
                                        "storeCredential()",
-                                       "unparseUri()"}));
+                                       "unparseUri()",
+                                       "version"}));
 
   EXPECT_TAB_DOES_NOTHING("shell.conect()");
 
@@ -705,14 +720,26 @@ TEST_F(Completer_frontend, js_adminapi) {
   CHECK_OBJECT_COMPLETIONS("dba");
 
   // TS_FR5.2_C02, TS_FR5.2_X02
-  EXPECT_AFTER_TAB_TAB(
-      "dba.",
-      strv({"checkInstanceConfiguration()", "configureInstance()",
-            "configureLocalInstance()", "createCluster()",
-            "deleteSandboxInstance()", "deploySandboxInstance()",
-            "dropMetadataSchema()", "getCluster()", "help()",
-            "killSandboxInstance()", "rebootClusterFromCompleteOutage()",
-            "startSandboxInstance()", "stopSandboxInstance()", "verbose"}));
+  EXPECT_AFTER_TAB_TAB("dba.", strv({"checkInstanceConfiguration()",
+                                     "configureInstance()",
+                                     "configureLocalInstance()",
+                                     "configureReplicaSetInstance()",
+                                     "createCluster()",
+                                     "createReplicaSet()",
+                                     "deleteSandboxInstance()",
+                                     "deploySandboxInstance()",
+                                     "dropMetadataSchema()",
+                                     "getCluster()",
+                                     "getClusterSet()",
+                                     "getReplicaSet()",
+                                     "help()",
+                                     "killSandboxInstance()",
+                                     "rebootClusterFromCompleteOutage()",
+                                     "session",
+                                     "startSandboxInstance()",
+                                     "stopSandboxInstance()",
+                                     "upgradeMetadata()",
+                                     "verbose"}));
   EXPECT_AFTER_TAB("dba.depl", "dba.deploySandboxInstance()");
 }
 
@@ -1044,11 +1071,15 @@ TEST_F(Completer_frontend, js_devapi_members_x) {
       {"rollbackTo", "('name')"},
       {"releaseSavepoint", "('name')"},
       {"rollback", "()"},
-      {"runSql", "('select 1')"}};
+      {"runSql", "('select 1')"},
+      {"_enableNotices", ""},
+      {"_fetchNotice", ""},
+      {"_getSocketFd", ""}};
   CHECK_OBJECT_MEMBER_COMPLETIONS("session", session_calls);
 
   std::vector<std::pair<std::string, std::string>> db_calls{
       {"createCollection", "('dropme')"},
+      {"modifyCollection", "('dropme', {'validation': {'level':'off'}})"},
       {"dropCollection", "('dropme')"},
       {"getCollection", "('testcol')"},
       {"getTable", "('testtab')"},
@@ -1096,7 +1127,8 @@ TEST_F(Completer_frontend, js_devapi_members_classic) {
       {"help", ""},
       {"close", ""},
       {"runSql", "('select 1')"},
-      {"query", "('select 1')"}};
+      {"query", "('select 1')"},
+      {"_getSocketFd", ""}};
   CHECK_OBJECT_MEMBER_COMPLETIONS("session", session_calls);
 }
 #endif
@@ -1150,13 +1182,16 @@ TEST_F(Completer_frontend, py_shell) {
   CHECK_OBJECT_COMPLETIONS("shell");
 
   EXPECT_AFTER_TAB("sh", "shell");
-  EXPECT_AFTER_TAB("shell.con", "shell.connect()");
+  EXPECT_AFTER_TAB("shell.rec", "shell.reconnect()");
   EXPECT_AFTER_TAB_TAB("shell.", strv({"add_extension_object_member()",
                                        "connect()",
+                                       "connect_to_primary()",
+                                       "create_context()",
                                        "create_extension_object()",
                                        "delete_all_credentials()",
                                        "delete_credential()",
                                        "disable_pager()",
+                                       "disconnect()",
                                        "dump_rows()",
                                        "enable_pager()",
                                        "get_session()",
@@ -1164,6 +1199,7 @@ TEST_F(Completer_frontend, py_shell) {
                                        "list_credential_helpers()",
                                        "list_credentials()",
                                        "log()",
+                                       "open_session()",
                                        "options",
                                        "parse_uri()",
                                        "prompt()",
@@ -1175,7 +1211,8 @@ TEST_F(Completer_frontend, py_shell) {
                                        "set_session()",
                                        "status()",
                                        "store_credential()",
-                                       "unparse_uri()"}));
+                                       "unparse_uri()",
+                                       "version"}));
 
   EXPECT_TAB_DOES_NOTHING("shell.conect()");
 
@@ -1186,19 +1223,21 @@ TEST_F(Completer_frontend, py_shell) {
   // a dynamically created global var
   execute("testobj = {'key_one':1, 'another_key': 2}");
   EXPECT_AFTER_TAB("testo", "testobj");
-  strv dict_options;
-  if (_python26)
-    dict_options = {"clear()",      "copy()",   "fromkeys()",  "get()",
-                    "has_key()",    "items()",  "iteritems()", "iterkeys()",
-                    "itervalues()", "keys()",   "pop()",       "popitem()",
-                    "setdefault()", "update()", "values()"};
-  else
-    dict_options = {"clear()",      "copy()",      "fromkeys()",  "get()",
-                    "has_key()",    "items()",     "iteritems()", "iterkeys()",
-                    "itervalues()", "keys()",      "pop()",       "popitem()",
-                    "setdefault()", "update()",    "values()",    "viewitems()",
-                    "viewkeys()",   "viewvalues()"};
-  EXPECT_AFTER_TAB_TAB("testobj.", strv(dict_options));
+  std::set<std::string> dict_options = {
+      "clear()", "copy()",    "fromkeys()",   "get()",    "items()", "keys()",
+      "pop()",   "popitem()", "setdefault()", "update()", "values()"};
+  if (shcore::str_beginswith(m_python_version, "2.")) {
+    dict_options.emplace("has_key()");
+    dict_options.emplace("iteritems()");
+    dict_options.emplace("iterkeys()");
+    dict_options.emplace("itervalues()");
+  }
+  if (shcore::str_beginswith(m_python_version, "2.7")) {
+    dict_options.emplace("viewitems()");
+    dict_options.emplace("viewkeys()");
+    dict_options.emplace("viewvalues()");
+  }
+  EXPECT_AFTER_TAB_TAB("testobj.", dict_options);
   EXPECT_AFTER_TAB("testobj.k", "testobj.keys()");
   EXPECT_TAB_DOES_NOTHING("testobj.key_one");
 
@@ -1212,14 +1251,26 @@ TEST_F(Completer_frontend, py_adminapi) {
   CHECK_OBJECT_COMPLETIONS("dba");
 
   // TS_FR5.2_C02, TS_FR5.2_X02
-  EXPECT_AFTER_TAB_TAB(
-      "dba.",
-      strv({"check_instance_configuration()", "configure_instance()",
-            "configure_local_instance()", "create_cluster()",
-            "delete_sandbox_instance()", "deploy_sandbox_instance()",
-            "drop_metadata_schema()", "get_cluster()", "help()",
-            "kill_sandbox_instance()", "reboot_cluster_from_complete_outage()",
-            "start_sandbox_instance()", "stop_sandbox_instance()", "verbose"}));
+  EXPECT_AFTER_TAB_TAB("dba.", strv({"check_instance_configuration()",
+                                     "configure_instance()",
+                                     "configure_local_instance()",
+                                     "configure_replica_set_instance()",
+                                     "create_cluster()",
+                                     "create_replica_set()",
+                                     "delete_sandbox_instance()",
+                                     "deploy_sandbox_instance()",
+                                     "drop_metadata_schema()",
+                                     "get_cluster()",
+                                     "get_cluster_set()",
+                                     "get_replica_set()",
+                                     "help()",
+                                     "kill_sandbox_instance()",
+                                     "reboot_cluster_from_complete_outage()",
+                                     "session",
+                                     "start_sandbox_instance()",
+                                     "stop_sandbox_instance()",
+                                     "upgrade_metadata()",
+                                     "verbose"}));
   EXPECT_AFTER_TAB("dba.depl", "dba.deploy_sandbox_instance()");
 }
 
@@ -1447,7 +1498,7 @@ TEST_F(Completer_frontend, py_devapi_table) {
                    "productTable.select([]).execute()");
   EXPECT_AFTER_TAB("f = productTable.select([]).b",
                    "f = productTable.select([]).bind()");
-  execute("findOp = db.productTable.select();");
+  execute("findOp = " DB_PRODUCTTABLE ".select();");
 
   EXPECT_TAB_DOES_NOTHING("findOp.sel");
   EXPECT_AFTER_TAB("findOp.exe", "findOp.execute()");
