@@ -1,23 +1,25 @@
-/* Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
-
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License, version 2.0,
- as published by the Free Software Foundation.
-
- This program is also distributed with certain software (including
- but not limited to OpenSSL) that is licensed under separate terms, as
- designated in a particular file or component or in included license
- documentation.  The authors of MySQL hereby grant you an additional
- permission to link the program and your derivative works with the
- separately licensed software that they have included with MySQL.
- This program is distributed in the hope that it will be useful,  but
- WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
- the GNU General Public License, version 2.0, for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software Foundation, Inc.,
- 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
+/*
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
+ *
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have included with MySQL.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <iostream>
 #include <queue>
@@ -35,6 +37,11 @@ namespace import_table {
 
 constexpr const int kBufferSize = BUFFER_SIZE;
 
+struct Range {
+  size_t begin{0};
+  size_t end{0};
+};
+
 TEST(import_table, double_buffer_iteration) {
   const std::string pattern{"abcdefghijklmnopqrstuvwxyz01234567890"};
   std::string test_string;
@@ -47,7 +54,8 @@ TEST(import_table, double_buffer_iteration) {
 
   for (int reserved : {0, 1, 2, 3, 10, 500, 512, 1024, kBufferSize / 4,
                        kBufferSize / 2, (kBufferSize / 2 + 1)}) {
-    File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     auto first = fh.begin(reserved);
     auto last = fh.end(reserved);
 
@@ -149,10 +157,11 @@ TEST(import_table, chunks) {
   shcore::create_file(path, test_string, true);
   const size_t needle_size = line_terminator.size();
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, 1, &r);
+                       line_terminator, 1, &r, File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -160,14 +169,15 @@ TEST(import_table, chunks) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -211,10 +221,11 @@ TEST(import_table, chunks_2) {
   shcore::create_file(path, test_string, true);
   const size_t needle_size = line_terminator.size();
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, 1, &r);
+                       line_terminator, 1, &r, File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -222,14 +233,15 @@ TEST(import_table, chunks_2) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -269,10 +281,11 @@ TEST(import_table, false_line_termination) {
   const size_t needle_size = line_terminator.size();
 
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, escape, 1, &r);
+                       line_terminator, escape, 1, &r, File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -280,14 +293,15 @@ TEST(import_table, false_line_termination) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -329,10 +343,11 @@ TEST(import_table, escape_char_in_the_middle_if_line_terminator) {
   shcore::create_file(path, test_string, true);
   const size_t needle_size = line_terminator.size();
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, escape, 1, &r);
+                       line_terminator, escape, 1, &r, File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -340,14 +355,15 @@ TEST(import_table, escape_char_in_the_middle_if_line_terminator) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -396,10 +412,11 @@ TEST(import_table, escape_at_the_end_of_a_buffer) {
   shcore::create_file(path, test_string, true);
   const size_t needle_size = line_terminator.size();
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, escape, 1, &r);
+                       line_terminator, escape, 1, &r, File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -407,14 +424,15 @@ TEST(import_table, escape_at_the_end_of_a_buffer) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -502,11 +520,11 @@ TEST(import_table, multichar_line_terminator_with_buffer_invalidation) {
   const size_t needle_size = line_terminator.size();
 
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
-    const size_t needle_size = line_terminator.size();
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, kSkipBytes, &r);
+                       line_terminator, kSkipBytes, &r, File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -514,14 +532,15 @@ TEST(import_table, multichar_line_terminator_with_buffer_invalidation) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -570,10 +589,11 @@ TEST(import_table, false_line_terminator_after_eof) {
   const size_t needle_size = line_terminator.size();
 
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, kSkipBytes, &r);
+                       line_terminator, kSkipBytes, &r, File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -581,15 +601,16 @@ TEST(import_table, false_line_terminator_after_eof) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
 
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -645,10 +666,12 @@ TEST(import_table, last_character_same_as_terminator) {
   const size_t needle_size = line_terminator.size();
 
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, escape, kSkipBytes, &r);
+                       line_terminator, escape, kSkipBytes, &r,
+                       File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -656,15 +679,16 @@ TEST(import_table, last_character_same_as_terminator) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
 
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -736,10 +760,12 @@ TEST(import_table, onechar_a_line_terminator_same_as_row_data) {
   const size_t needle_size = line_terminator.size();
 
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, escape, kSkipBytes, &r);
+                       line_terminator, escape, kSkipBytes, &r,
+                       File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -747,15 +773,16 @@ TEST(import_table, onechar_a_line_terminator_same_as_row_data) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
 
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -829,10 +856,12 @@ TEST(import_table, twochar_ab_line_terminator_same_as_row_data) {
   const size_t needle_size = line_terminator.size();
 
   {
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, escape, kSkipBytes, &r);
+                       line_terminator, escape, kSkipBytes, &r,
+                       File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -840,15 +869,16 @@ TEST(import_table, twochar_ab_line_terminator_same_as_row_data) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
   }
 
   {
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
@@ -924,10 +954,12 @@ TEST(import_table, twochar_aa_line_terminator_same_as_row_data) {
     expected_ranges.push_back(Range{566, 640});
     expected_ranges.push_back(Range{640, 717});
 
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, escape, kSkipBytes, &r);
+                       line_terminator, escape, kSkipBytes, &r,
+                       File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -935,8 +967,8 @@ TEST(import_table, twochar_aa_line_terminator_same_as_row_data) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
@@ -950,10 +982,11 @@ TEST(import_table, twochar_aa_line_terminator_same_as_row_data) {
     expected_ranges.push_back(Range{368, 640});
     expected_ranges.push_back(Range{640, 717});
 
-    mysqlsh::import_table::File_handler fh{path};
-    std::queue<Range> r;
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
+    std::queue<File_import_info> r;
     chunk_by_max_bytes(fh.begin(needle_size), fh.end(needle_size),
-                       line_terminator, escape, 254, &r);
+                       line_terminator, escape, 254, &r, File_import_info());
 
     EXPECT_EQ(expected_ranges.size(), r.size());
 
@@ -961,8 +994,8 @@ TEST(import_table, twochar_aa_line_terminator_same_as_row_data) {
       auto range = r.front();
       r.pop();
 
-      EXPECT_EQ(expected_ranges[i].begin, range.begin);
-      EXPECT_EQ(expected_ranges[i].end, range.end);
+      EXPECT_EQ(expected_ranges[i].begin, range.range.first);
+      EXPECT_EQ(expected_ranges[i].end, range.range.second);
     }
 
     EXPECT_TRUE(r.empty());
@@ -982,7 +1015,8 @@ TEST(import_table, twochar_aa_line_terminator_same_as_row_data) {
     expected_ranges.push_back(Range{566, 640});
     expected_ranges.push_back(Range{640, 717});
 
-    mysqlsh::import_table::File_handler fh{path};
+    auto fh_ptr = mysqlshdk::storage::make_file(path);
+    File_handler fh{fh_ptr.get()};
     std::queue<Range> r;
     auto row = fh.begin(needle_size);
     auto last = fh.end(needle_size);
