@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -36,21 +36,26 @@
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_path.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
+#include "mysqlshdk/libs/utils/version.h"
 #include "unittest/gtest_clean.h"
 #include "unittest/test_utils/mocks/gmock_clean.h"
 #include "unittest/test_utils/shell_test_wrapper.h"
 
+using mysql::secret_store::api::get_available_helpers;
+using mysql::secret_store::api::get_helper;
 using mysql::secret_store::api::Helper_interface;
 using mysql::secret_store::api::Helper_name;
 using mysql::secret_store::api::Secret_spec;
 using mysql::secret_store::api::Secret_type;
-using mysql::secret_store::api::get_available_helpers;
-using mysql::secret_store::api::get_helper;
 
 namespace tests {
 
 namespace {
 
+const auto g_format_parameter = [](const auto &info) {
+  return shcore::str_replace(info.param, "-", "_");
+};
+
 template <typename T>
 ::testing::AssertionResult is_empty(const T &container) {
   if (container.empty()) {
@@ -424,7 +429,7 @@ class Helper_executable_tester : public Helper_tester {
         paths.begin(), paths.end(),
         [&name](const std::string &p) { return get_helper_name(p) == name; });
     ASSERT_NE(paths.end(), helper);
-    m_invoker = std::unique_ptr<Helper_invoker>{new Helper_invoker{*helper}};
+    m_invoker = std::make_unique<Helper_invoker>(*helper);
     ASSERT_NE(nullptr, m_invoker);
   }
 
@@ -820,6 +825,11 @@ class Config_editor_invoker {
 
   void clear() { invoke({"reset"}); }
 
+  mysqlshdk::utils::Version version() const {
+    return mysqlshdk::utils::Version{
+        shcore::str_split(invoke({"--version"}), " ", -1, true)[2]};
+  }
+
   std::string invoke(const std::vector<std::string> &args,
                      bool uses_terminal = false,
                      const std::string &password = "") const {
@@ -1193,29 +1203,35 @@ void test_available_helpers() {
     }                                                                         \
     Config_editor_invoker invoker;                                            \
     invoker.clear();                                                          \
+    const auto quoted =                                                       \
+        invoker.version() >= mysqlshdk::utils::Version(8, 0, 24);             \
+    const auto host = [&quoted](const std::string &h) {                       \
+      return "host = " + (quoted ? shcore::quote_string(h, '"') : h);         \
+    };                                                                        \
     STORE_AND_CHECK("user@[::1]", "one");                                     \
     auto output = invoker.invoke({"print", "--all"});                         \
-    EXPECT_THAT(output, ::testing::HasSubstr("host = ::1"));                  \
+    EXPECT_THAT(output, ::testing::HasSubstr(host("::1")));                   \
     invoker.clear();                                                          \
     STORE_AND_CHECK("user@[fe80::850a:5a7c:6ab7:aec4]:4321", "two");          \
     output = invoker.invoke({"print", "--all"});                              \
     EXPECT_THAT(output,                                                       \
-                ::testing::HasSubstr("host = fe80::850a:5a7c:6ab7:aec4"));    \
+                ::testing::HasSubstr(host("fe80::850a:5a7c:6ab7:aec4")));     \
     EXPECT_THAT(output, ::testing::HasSubstr("port = 4321"));                 \
     invoker.clear();                                                          \
     STORE_AND_CHECK("user@[fe80::850a:5a7c:6ab7:aec4%25enp0s3]", "three");    \
     output = invoker.invoke({"print", "--all"});                              \
     EXPECT_THAT(output, ::testing::HasSubstr(                                 \
-                            "host = fe80::850a:5a7c:6ab7:aec4%enp0s3"));      \
+                            host("fe80::850a:5a7c:6ab7:aec4%enp0s3")));       \
     invoker.clear();                                                          \
   }
 
-#define REGISTER_TESTS(test_case_name)                               \
-  namespace {                                                        \
-  const auto test_case_name##_list = test_case_name::list_helpers(); \
-  }                                                                  \
-  INSTANTIATE_TEST_CASE_P(Helpers, test_case_name,                   \
-                          ::testing::ValuesIn(test_case_name##_list))
+#define REGISTER_TESTS(test_case_name)                                 \
+  namespace {                                                          \
+  const auto test_case_name##_list = test_case_name::list_helpers();   \
+  }                                                                    \
+  INSTANTIATE_TEST_SUITE_P(Helpers, test_case_name,                    \
+                           ::testing::ValuesIn(test_case_name##_list), \
+                           g_format_parameter)
 
 }  // namespace
 
@@ -1225,7 +1241,7 @@ void test_available_helpers() {
 class Mysql_secret_store_api_test
     : public Parametrized_helper_test<Mysql_secret_store_api_tester> {};
 
-ADD_TESTS(Mysql_secret_store_api_test);
+ADD_TESTS(Mysql_secret_store_api_test)
 
 #undef VALIDATION_TEST
 #undef NORMALIZATION_TEST
@@ -1281,7 +1297,7 @@ REGISTER_TESTS(Mysql_secret_store_api_test);
 
 class Shell_api_test : public Parametrized_helper_test<Shell_api_tester> {};
 
-ADD_TESTS(Shell_api_test);
+ADD_TESTS(Shell_api_test)
 
 #undef VALIDATION_TEST
 #undef NORMALIZATION_TEST
@@ -1304,7 +1320,7 @@ REGISTER_TESTS(Shell_api_test);
 class Helper_executable_test
     : public Parametrized_helper_test<Helper_executable_tester> {};
 
-ADD_TESTS(Helper_executable_test);
+ADD_TESTS(Helper_executable_test)
 
 #undef VALIDATION_TEST
 #undef NORMALIZATION_TEST
