@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -43,6 +43,11 @@ class Config_server_handler_test : public tests::Shell_base_test {
     // Create instance and Open the session for the tests.
     m_connection_options = shcore::get_connection_options(_mysql_uri);
     m_session->connect(m_connection_options);
+
+    m_report_host =
+        m_session->query("SELECT coalesce(@@report_host, @@hostname)")
+            ->fetch_one()
+            ->get_string(0);
   }
 
   void TearDown() {
@@ -55,6 +60,8 @@ class Config_server_handler_test : public tests::Shell_base_test {
   std::shared_ptr<mysqlshdk::db::ISession> m_session =
       mysqlshdk::db::mysql::Session::create();
   mysqlshdk::db::Connection_options m_connection_options;
+
+  std::string m_report_host;
 };
 
 TEST_F(Config_server_handler_test, config_interface_global_vars) {
@@ -558,10 +565,9 @@ TEST_F(Config_server_handler_test, apply_errors) {
     SCOPED_TRACE("Error applying bool value setting (with context).");
     Config_server_handler cfg_h(&instance, Var_qualifier::GLOBAL);
     cfg_h.set("not_exit_bool", nullable<bool>(true), "notExistBool");
-    EXPECT_THROW_LIKE(
-        cfg_h.apply(), std::runtime_error,
-        "Unable to set value 'true' for 'notExistBool': localhost:" +
-            _mysql_port + ": Unknown system variable 'not_exit_bool'");
+    EXPECT_THROW_LIKE(cfg_h.apply(), std::runtime_error,
+                      "Unable to set value 'true' for 'notExistBool': Unknown "
+                      "system variable 'not_exit_bool'");
   }
 
   {
@@ -576,10 +582,9 @@ TEST_F(Config_server_handler_test, apply_errors) {
     SCOPED_TRACE("Error applying int value setting (with context).");
     Config_server_handler cfg_h(&instance, Var_qualifier::GLOBAL);
     cfg_h.set("not_exit_int", nullable<int64_t>(1234), "notExistInt");
-    EXPECT_THROW_LIKE(
-        cfg_h.apply(), std::runtime_error,
-        "Unable to set value '1234' for 'notExistInt': localhost:" +
-            _mysql_port + ": Unknown system variable 'not_exit_int'");
+    EXPECT_THROW_LIKE(cfg_h.apply(), std::runtime_error,
+                      "Unable to set value '1234' for 'notExistInt': Unknown "
+                      "system variable 'not_exit_int'");
   }
 
   {
@@ -595,10 +600,9 @@ TEST_F(Config_server_handler_test, apply_errors) {
     Config_server_handler cfg_h(&instance, Var_qualifier::GLOBAL);
     cfg_h.set("not_exit_string", nullable<std::string>("mystr"),
               "notExistString");
-    EXPECT_THROW_LIKE(
-        cfg_h.apply(), std::runtime_error,
-        "Unable to set value 'mystr' for 'notExistString': localhost:" +
-            _mysql_port + ": Unknown system variable 'not_exit_string'");
+    EXPECT_THROW_LIKE(cfg_h.apply(), std::runtime_error,
+                      "Unable to set value 'mystr' for 'notExistString': "
+                      "Unknown system variable 'not_exit_string'");
   }
 }
 
