@@ -1,23 +1,25 @@
-/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
-
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License, version 2.0,
- as published by the Free Software Foundation.
-
- This program is also distributed with certain software (including
- but not limited to OpenSSL) that is licensed under separate terms, as
- designated in a particular file or component or in included license
- documentation.  The authors of MySQL hereby grant you an additional
- permission to link the program and your derivative works with the
- separately licensed software that they have included with MySQL.
- This program is distributed in the hope that it will be useful,  but
- WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
- the GNU General Public License, version 2.0, for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software Foundation, Inc.,
- 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
+/*
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
+ *
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have included with MySQL.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include "mysqlshdk/libs/db/uri_parser.h"
 #include "mysqlshdk/libs/utils/nullable.h"
@@ -77,8 +79,8 @@ void validate_uri(
     const mysqlshdk::utils::nullable<const char *> &host,
     const mysqlshdk::utils::nullable<int> &port,
     const mysqlshdk::utils::nullable<const char *> &sock,
-    const mysqlshdk::utils::nullable<const char *> &db, bool has_password,
-    bool has_port, Transport_type target_type,
+    const mysqlshdk::utils::nullable<const char *> &db, bool /* has_password */,
+    bool /* has_port */, Transport_type target_type,
     const std::map<std::string, std::vector<std::string>> *attributes = 0) {
   SCOPED_TRACE(connstring);
 
@@ -358,6 +360,9 @@ TEST(Uri_parser, parse_host_ipv4) {
   validate_uri("mysqlx://user@10.150.123.45:2845", "mysqlx", "user",
                NO_PASSWORD, "10.150.123.45", 2845, NO_SOCK, NO_DB,
                HAS_NO_PASSWORD, HAS_PORT, Transport_type::Tcp);
+  validate_uri("mysqlx://user@10.150.123.45:0", "mysqlx", "user", NO_PASSWORD,
+               "10.150.123.45", 0, NO_SOCK, NO_DB, HAS_NO_PASSWORD, HAS_PORT,
+               Transport_type::Tcp);
 
   //                0    0    1    1    2    2    3    3    4    4    5    5
   //                0    5    0    5    0    5    0    5    0    5    0    5
@@ -378,6 +383,22 @@ TEST(Uri_parser, parse_host_ipv4) {
   validate_bad_uri("mysqlx://user@10.150.123.45:", "Missing port number");
   validate_bad_uri("mysqlx://user@10.150.123.45:2845f",
                    "Illegal character [f] found at position 32");
+  validate_bad_uri("mysqlx://user@10.150.123.45:2147483647",
+                   "Port is out of the valid range: 0 - 65535");
+  validate_bad_uri("mysqlx://user@10.150.123.45:2147483648",
+                   "Port is out of the valid range: 0 - 65535");
+  validate_bad_uri("mysqlx://user@10.150.123.45:4294967295",
+                   "Port is out of the valid range: 0 - 65535");
+  validate_bad_uri("mysqlx://user@10.150.123.45:4294967296",
+                   "Port is out of the valid range: 0 - 65535");
+  validate_bad_uri("mysqlx://user@10.150.123.45:928482174821947214627846278",
+                   "Port is out of the valid range: 0 - 65535");
+  validate_bad_uri("mysqlx://user@10.150.123.45:-4294967296",
+                   "Illegal character [-] found at position 28");
+  validate_bad_uri("mysqlx://user@10.150.123.45:-1",
+                   "Illegal character [-] found at position 28");
+  validate_bad_uri("mysqlx://user@10.150.123.45:-0",
+                   "Illegal character [-] found at position 28");
   validate_bad_uri("mysqlx://user@10.150.123.45:invalid",
                    "Illegal character [i] found at position 28");
 }
@@ -496,6 +517,26 @@ TEST(Uri_parser, parse_host_ipv6) {
   validate_bad_uri("mysqlx://user:sample@[1:2:3:4:5:6:7]:8]:2540/testing",
                    "Invalid IPv6: the number of segments does not match the "
                    "specification");
+  validate_bad_uri("mysqlx://user:sample@[[1:2:3:4:5:6:7:8]]:2540/testing",
+                   "Unexpected data [[] found at position 22");
+  validate_bad_uri("mysqlx://user:sample@[.1:2:3:4:5:6:7:8]:2540/testing",
+                   "Unexpected data [.] found at position 22");
+  validate_bad_uri("mysqlx://user:sample@[1[:2:3:4:5:6:7:8]:2540/testing",
+                   "Expected token type : at position 23 but found type [ ([)");
+  validate_bad_uri("mysqlx://user:sample@[1.:2:3:4:5:6:7:8]:2540/testing",
+                   "Expected token type : at position 23 but found type . (.)");
+  validate_bad_uri("mysqlx://user:sample@[1:[2:3:4:5:6:7:8]:2540/testing",
+                   "Unexpected data [[] found at position 24");
+  validate_bad_uri("mysqlx://user:sample@[1:.2:3:4:5:6:7:8]:2540/testing",
+                   "Unexpected data [.] found at position 24");
+  validate_bad_uri("mysqlx://user:sample@[1:2[2:3:4:5:6:7:8]:2540/testing",
+                   "Expected token type : at position 25 but found type [ ([)");
+  validate_bad_uri("mysqlx://user:sample@[1:2.2:3:4:5:6:7:8]:2540/testing",
+                   "Expected token type : at position 25 but found type . (.)");
+  validate_bad_uri("mysqlx://user:sample@[1:2[:3:4:5:6:7:8]:2540/testing",
+                   "Expected token type : at position 25 but found type [ ([)");
+  validate_bad_uri("mysqlx://user:sample@[1:2.:3:4:5:6:7:8]:2540/testing",
+                   "Expected token type : at position 25 but found type . (.)");
 
   // zone ID
   validate_ipv6("[fe80::850a:5a7c:6ab7:aec4%25enp0s3]",
@@ -628,16 +669,6 @@ TEST(Uri_parser, parse_host_ipv6) {
 }
 
 TEST(Uri_parser, parse_socket) {
-#ifndef _WIN32
-  validate_uri("mysqlx://user:password@/path%2Fto%2Fsocket/schema", "mysqlx",
-               "user", "password", NO_HOST, NO_PORT, "/path/to/socket",
-               "schema", HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
-  validate_uri("mysqlx://user:password@/path%2Fto%2Fsocket/", "mysqlx", "user",
-               "password", NO_HOST, NO_PORT, "/path/to/socket", NO_SCHEMA,
-               HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
-  validate_uri("mysqlx://user:password@/path%2Fto%2Fsocket", "mysqlx", "user",
-               "password", NO_HOST, NO_PORT, "/path/to/socket", NO_SCHEMA,
-               HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
   validate_uri("mysqlx://user:password@/socket/schema", "mysqlx", "user",
                "password", NO_HOST, NO_PORT, "/socket", "schema", HAS_PASSWORD,
                HAS_NO_PORT, Transport_type::Socket);
@@ -653,6 +684,17 @@ TEST(Uri_parser, parse_socket) {
   validate_uri("mysqlx://user:password@a/", "mysqlx", "user", "password", "a",
                NO_PORT, NO_SOCK, NO_SCHEMA, HAS_PASSWORD, HAS_NO_PORT,
                Transport_type::Tcp);
+
+  validate_uri("mysqlx://user:password@/path%2Fto%2Fsocket/schema", "mysqlx",
+               "user", "password", NO_HOST, NO_PORT, "/path/to/socket",
+               "schema", HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
+  validate_uri("mysqlx://user:password@/path%2Fto%2Fsocket/", "mysqlx", "user",
+               "password", NO_HOST, NO_PORT, "/path/to/socket", NO_SCHEMA,
+               HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
+  validate_uri("mysqlx://user:password@/path%2Fto%2Fsocket", "mysqlx", "user",
+               "password", NO_HOST, NO_PORT, "/path/to/socket", NO_SCHEMA,
+               HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
+
   validate_uri("mysqlx://user:password@(/path/to/mysql.sock)/schema", "mysqlx",
                "user", "password", NO_HOST, NO_PORT, "/path/to/mysql.sock",
                "schema", HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
@@ -663,6 +705,17 @@ TEST(Uri_parser, parse_socket) {
                "password", NO_HOST, NO_PORT, "/path/to/mysql.sock", NO_SCHEMA,
                HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
 
+  validate_uri("mysqlx://user:password@(/path\\to\\mysql.sock)/schema",
+               "mysqlx", "user", "password", NO_HOST, NO_PORT,
+               "/path\\to\\mysql.sock", "schema", HAS_PASSWORD, HAS_NO_PORT,
+               Transport_type::Socket);
+  validate_uri("mysqlx://user:password@(/path\\to\\mysql.sock)/", "mysqlx",
+               "user", "password", NO_HOST, NO_PORT, "/path\\to\\mysql.sock",
+               NO_SCHEMA, HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
+  validate_uri("mysqlx://user:password@(/path\\to\\mysql.sock)", "mysqlx",
+               "user", "password", NO_HOST, NO_PORT, "/path\\to\\mysql.sock",
+               NO_SCHEMA, HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
+
   validate_uri("mysqlx://user:password@.%2Fpath%2Fto%2Fsocket/schema", "mysqlx",
                "user", "password", NO_HOST, NO_PORT, "./path/to/socket",
                "schema", HAS_PASSWORD, HAS_NO_PORT, Transport_type::Socket);
@@ -710,71 +763,9 @@ TEST(Uri_parser, parse_socket) {
                    "Illegal character [/] found at position 24");
   validate_bad_uri("mysqlx://user:password@../path/to/socket/schema",
                    "Illegal character [/] found at position 25");
-#else   // _WIN32
-  validate_bad_uri("mysqlx://user:password@/path%2Fto%2Fsocket/schema",
-                   "Illegal character [/] found at position 23");
-  validate_bad_uri("mysqlx://user:password@/path%2Fto%2Fsocket/",
-                   "Illegal character [/] found at position 23");
-  validate_bad_uri("mysqlx://user:password@/path%2Fto%2Fsocket",
-                   "Illegal character [/] found at position 23");
-  validate_bad_uri("mysqlx://user:password@/socket/schema",
-                   "Illegal character [/] found at position 23");
-  validate_bad_uri("mysqlx://user:password@/socket/",
-                   "Illegal character [/] found at position 23");
-  validate_bad_uri("mysqlx://user:password@/socket",
-                   "Illegal character [/] found at position 23");
-  validate_uri("mysqlx://user:password@a/schema", "mysqlx", "user", "password",
-               "a", NO_PORT, NO_SOCK, "schema", HAS_PASSWORD, HAS_NO_PORT,
-               Transport_type::Tcp);
-  validate_uri("mysqlx://user:password@a/", "mysqlx", "user", "password", "a",
-               NO_PORT, NO_SOCK, NO_SCHEMA, HAS_PASSWORD, HAS_NO_PORT,
-               Transport_type::Tcp);
-  validate_bad_uri("mysqlx://user:password@(/path/to/mysql.sock)/schema",
-                   "Illegal character [/] found at position 24");
-  validate_bad_uri("mysqlx://user:password@(/path/to/mysql.sock)/",
-                   "Illegal character [/] found at position 24");
-  validate_bad_uri("mysqlx://user:password@(/path/to/mysql.sock)",
-                   "Illegal character [/] found at position 24");
-
-  validate_bad_uri("mysqlx://user:password@.%2Fpath%2Fto%2Fsocket/schema",
-                   "Unexpected character [.] found at position 23");
-  validate_bad_uri("mysqlx://user:password@.%2Fpath%2Fto%2Fsocket/",
-                   "Unexpected character [.] found at position 23");
-  validate_bad_uri("mysqlx://user:password@.%2Fpath%2Fto%2Fsocket",
-                   "Unexpected character [.] found at position 23");
-
-  validate_bad_uri("mysqlx://user:password@(./path/to/socket)/schema",
-                   "Illegal character [/] found at position 25");
-  validate_bad_uri("mysqlx://user:password@(./path/to/socket)/",
-                   "Illegal character [/] found at position 25");
-  validate_bad_uri("mysqlx://user:password@(./path/to/socket)",
-                   "Illegal character [/] found at position 25");
-
-  validate_bad_uri("mysqlx://user:password@..%2Fpath%2Fto%2Fsocket/schema",
-                   "Unexpected character [.] found at position 23");
-  validate_bad_uri("mysqlx://user:password@..%2Fpath%2Fto%2Fsocket/",
-                   "Unexpected character [.] found at position 23");
-  validate_bad_uri("mysqlx://user:password@..%2Fpath%2Fto%2Fsocket",
-                   "Unexpected character [.] found at position 23");
-
-  validate_bad_uri("mysqlx://user:password@(../path/to/socket)/schema",
-                   "Illegal character [/] found at position 26");
-  validate_bad_uri("mysqlx://user:password@(../path/to/socket)/",
-                   "Illegal character [/] found at position 26");
-  validate_bad_uri("mysqlx://user:password@(../path/to/socket)",
-                   "Illegal character [/] found at position 26");
-
-  validate_bad_uri("mysqlx://user:password@/path/to/socket/schema",
-                   "Illegal character [/] found at position 23");
-  validate_bad_uri("mysqlx://user:password@./path/to/socket/schema",
-                   "Unexpected character [.] found at position 23");
-  validate_bad_uri("mysqlx://user:password@../path/to/socket/schema",
-                   "Unexpected character [.] found at position 23");
-#endif  // _WIN32
 }
 
 TEST(Uri_parser, parse_pipe) {
-#ifdef _WIN32
   validate_uri("mysql://user:password@\\\\.\\d%3A%5Cpath%5Cto%5Csocket/schema",
                "mysql", "user", "password", NO_HOST, NO_PORT,
                "d:\\path\\to\\socket", "schema", HAS_PASSWORD, HAS_NO_PORT,
@@ -801,6 +792,19 @@ TEST(Uri_parser, parse_pipe) {
                "d:\\path\\to\\socket", NO_SCHEMA, HAS_PASSWORD, HAS_NO_PORT,
                Transport_type::Pipe);
 
+  validate_uri("mysql://user:password@\\\\.\\d%3A%2Fpath%2Fto%2Fsocket/schema",
+               "mysql", "user", "password", NO_HOST, NO_PORT,
+               "d:/path/to/socket", "schema", HAS_PASSWORD, HAS_NO_PORT,
+               Transport_type::Pipe);
+  validate_uri("mysql://user:password@\\\\.\\d%3A%2Fpath%2Fto%2Fsocket/",
+               "mysql", "user", "password", NO_HOST, NO_PORT,
+               "d:/path/to/socket", NO_SCHEMA, HAS_PASSWORD, HAS_NO_PORT,
+               Transport_type::Pipe);
+  validate_uri("mysql://user:password@\\\\.\\d%3A%2Fpath%2Fto%2Fsocket",
+               "mysql", "user", "password", NO_HOST, NO_PORT,
+               "d:/path/to/socket", NO_SCHEMA, HAS_PASSWORD, HAS_NO_PORT,
+               Transport_type::Pipe);
+
   validate_uri("mysql://user:password@(\\\\.\\d:/path/to/socket)/schema",
                "mysql", "user", "password", NO_HOST, NO_PORT,
                "d:/path/to/socket", "schema", HAS_PASSWORD, HAS_NO_PORT,
@@ -832,57 +836,13 @@ TEST(Uri_parser, parse_pipe) {
 
   validate_bad_uri("mysqlx://user:password@(\\\\.\\d:/path/to/socket)",
                    "Pipe can only be used with Classic session");
+  validate_bad_uri("mysql://user:password@\\(\\\\.\\d:\\path\\to\\socket)",
+                   "Illegal character [\\] found at position 22");
 
   validate_bad_uri("mysql://user:password@\\\\.\\",
                    "Named pipe cannot be empty.");
   validate_bad_uri("mysql://user:password@(\\\\.\\)",
                    "Named pipe cannot be empty.");
-#else   // !_WIN32
-  validate_bad_uri(
-      "mysql://user:password@\\\\.\\d%3A%5Cpath%5Cto%5Csocket/schema",
-      "Illegal character [\\] found at position 22");
-  validate_bad_uri("mysql://user:password@\\\\.\\d%3A%5Cpath%5Cto%5Csocket/",
-                   "Illegal character [\\] found at position 22");
-  validate_bad_uri("mysql://user:password@\\\\.\\d%3A%5Cpath%5Cto%5Csocket",
-                   "Illegal character [\\] found at position 22");
-
-  validate_bad_uri(
-      "mysql://user:password@(\\\\.\\d:%5Cpath%5Cto%5Csocket)/schema",
-      "Illegal character [\\] found at position 23");
-  validate_bad_uri("mysql://user:password@(\\\\.\\d:%5Cpath%5Cto%5Csocket)/",
-                   "Illegal character [\\] found at position 23");
-  validate_bad_uri("mysql://user:password@(\\\\.\\d:%5Cpath%5Cto%5Csocket)",
-                   "Illegal character [\\] found at position 23");
-
-  validate_bad_uri("mysql://user:password@(\\\\.\\d:/path/to/socket)/schema",
-                   "Illegal character [\\] found at position 23");
-  validate_bad_uri("mysql://user:password@(\\\\.\\d:/path/to/socket)/",
-                   "Illegal character [\\] found at position 23");
-  validate_bad_uri("mysql://user:password@(\\\\.\\d:/path/to/socket)",
-                   "Illegal character [\\] found at position 23");
-
-  validate_bad_uri("mysql://user:password@(\\\\.\\d:\\path\\to\\socket)/schema",
-                   "Illegal character [\\] found at position 23");
-  validate_bad_uri("mysql://user:password@(\\\\.\\d:\\path\\to\\socket)/",
-                   "Illegal character [\\] found at position 23");
-  validate_bad_uri("mysql://user:password@\(\\\\.\\d:\\path\\to\\socket)",
-                   "Illegal character [\\] found at position 23");
-
-  validate_bad_uri(
-      "mysql://user:password@\\\\.\\d:%5Cpath%5Cto%5Csocket/schema",
-      "Illegal character [\\] found at position 22");
-  validate_bad_uri(
-      "mysql://user:password@\\\\.\\d%3A\\path%5Cto%5Csocket/schema",
-      "Illegal character [\\] found at position 22");
-
-  validate_bad_uri("mysqlx://user:password@(\\\\.\\d:/path/to/socket)",
-                   "Illegal character [\\] found at position 24");
-
-  validate_bad_uri("mysql://user:password@\\\\.\\",
-                   "Illegal character [\\] found at position 22");
-  validate_bad_uri("mysql://user:password@(\\\\.\\)",
-                   "Illegal character [\\] found at position 23");
-#endif  // !_WIN32
 }
 
 TEST(Uri_parser, parse_path) {
@@ -936,5 +896,6 @@ TEST(Uri_parser, parse_query) {
       "mysqlx", "user", NO_PASSWORD, "10.150.123.45", 2845, NO_SOCK, "world",
       HAS_NO_PASSWORD, HAS_PORT, Transport_type::Tcp, &atts);
 }
-};  // namespace proj_parser_tests
-};  // namespace testing
+
+}  // namespace proj_parser_tests
+}  // namespace testing
