@@ -1,9 +1,9 @@
 // Assumptions: smart deployment functions available
 
 function print_metadata_instance_addresses(session) {
-    var res = session.runSql("select * from mysql_innodb_cluster_metadata.instances").fetchAll();
+    var res = session.runSql("select instance_name, addresses from mysql_innodb_cluster_metadata.instances").fetchAll();
     for (var i = 0; i < res.length; i++) {
-        print(res[i][4] + " = " + res[i][7] + "\n");
+        print(res[i][0] + " = " + res[i][1] + "\n");
     }
     print("\n");
 }
@@ -25,190 +25,8 @@ function number_of_non_expiring_pwd_accounts(session, user) {
     return row[0];
 }
 
-// WL#12049 AdminAPI: option to shutdown server when dropping out of the
-// cluster
-//
-// In 8.0.12 and 5.7.24, Group Replication introduced an option to allow
-// defining the behaviour of a cluster member whenever it drops the cluster
-// unintentionally, i.e. not manually removed by the user (DBA). The
-// behaviour defined can be either automatically shutting itself down or
-// switching itself to super-read-only (current behaviour). The option is:
-// group_replication_exit_state_action.
-//
-// In order to support defining such option, the AdminAPI was extended by
-// introducing a new optional parameter, named 'exitStateAction', in the
-// following functions:
-//
-// - dba.createCluster()
-// - Cluster.addInstance()
-//
 
-//@ WL#12049: Initialization
-testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
-testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
 
-shell.connect(__sandbox_uri1);
-
-// Test the option on the addInstance() command
-//@ WL#12049: Create cluster 1 {VER(>=5.7.24)}
-var c = dba.createCluster('test', {clearReadOnly: true, gtidSetIsComplete: true})
-
-//@ WL#12049: addInstance() errors using exitStateAction option {VER(>=5.7.24)}
-// F1.2 - The exitStateAction option shall be a string value.
-// NOTE: GR validates the value, which is an Enumerator, and accepts the values
-// `ABORT_SERVER` or `READ_ONLY`, or 1 or 0.
-c.addInstance(__sandbox_uri2, {exitStateAction: ""});
-
-c.addInstance(__sandbox_uri2, {exitStateAction: " "});
-
-c.addInstance(__sandbox_uri2, {exitStateAction: ":"});
-
-c.addInstance(__sandbox_uri2, {exitStateAction: "AB"});
-
-c.addInstance(__sandbox_uri2, {exitStateAction: "10"});
-
-//@ WL#12049: Add instance using a valid exitStateAction 1 {VER(>=5.7.24)}
-c.addInstance(__sandbox_uri2, {exitStateAction: "ABORT_SERVER"});
-
-//@ WL#12049: Dissolve cluster 1 {VER(>=5.7.24)}
-c.dissolve({force: true});
-
-// Verify if exitStateAction is persisted on >= 8.0.12
-
-// F2 - On a successful [dba.]createCluster() or [Cluster.]addInstance() call
-// using the option exitStateAction, the GR sysvar
-// group_replication_exit_state_action must be persisted using SET PERSIST at
-// the target instance, if the MySQL Server instance version is >= 8.0.12.
-
-//@ WL#12049: Create cluster 2 {VER(>=8.0.12)}
-var c = dba.createCluster('test', {clearReadOnly: true, groupName: "ca94447b-e6fc-11e7-b69d-4485005154dc", exitStateAction: "READ_ONLY", gtidSetIsComplete: true});
-
-//@ WL#12049: Add instance using a valid exitStateAction 2 {VER(>=8.0.12)}
-c.addInstance(__sandbox_uri2, {exitStateAction: "READ_ONLY"})
-
-session.close()
-shell.connect(__sandbox_uri2);
-
-//@<> WL#12049: exitStateAction must be persisted on mysql >= 8.0.12 {VER(>=8.0.12)}
-EXPECT_EQ("READ_ONLY", get_sysvar(session,  "group_replication_exit_state_action", "PERSISTED"));
-
-//@ WL#12049: Dissolve cluster 2 {VER(>=8.0.12)}
-c.dissolve({force: true});
-
-// Verify that group_replication_exit_state_action is not persisted when not used
-// We need a clean instance for that because dissolve does not unset the previously set variables
-
-//@ WL#12049: Initialize new instance
-session.close();
-testutil.destroySandbox(__mysql_sandbox_port1);
-testutil.destroySandbox(__mysql_sandbox_port2);
-testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
-testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
-
-shell.connect(__sandbox_uri1);
-
-//@ WL#12049: Create cluster 3 {VER(>=8.0.12)}
-var c = dba.createCluster('test', {groupName: "ca94447b-e6fc-11e7-b69d-4485005154dc", gtidSetIsComplete: true});
-
-//@ WL#12049: Add instance without using exitStateAction {VER(>=8.0.12)}
-c.addInstance(__sandbox_uri2)
-
-//@<> BUG#28701263: DEFAULT VALUE OF EXITSTATEACTION TOO DRASTIC {VER(>=8.0.12)}
-EXPECT_EQ("READ_ONLY", get_sysvar(session, "group_replication_exit_state_action"));
-
-//@ WL#12049: Finalization
-session.close();
-testutil.destroySandbox(__mysql_sandbox_port1);
-testutil.destroySandbox(__mysql_sandbox_port2);
-
-// WL#11032 AdminAPI: Configure member weight for automatic primary election on
-// failover
-//
-// In MySQL 8.0.2 and 5.7.20, Group Replication introduces an option to control
-// the outcome of the primary election algorithm in single-primary mode. With
-// this option, the user can influence the primary member election by providing
-// a member weight value for each member node. That weight value is used for
-// electing the primary member instead of the member uuid which is the default
-// method used for the election.
-//
-// In order to support defining such option, the AdminAPI was extended by
-// introducing a new optional parameter, named 'memberWeight', in the
-// following functions:
-//
-// - dba.createCluster()
-// - Cluster.addInstance()
-//
-
-//@ WL#11032: Initialization
-testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
-testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
-
-shell.connect(__sandbox_uri1);
-
-// Test the option on the addInstance() command
-//@ WL#11032: Create cluster 1 {VER(>=5.7.20)}
-var c = dba.createCluster('test', {clearReadOnly: true, gtidSetIsComplete: true})
-
-//@ WL#11032: addInstance() errors using memberWeight option {VER(>=5.7.20)}
-// F1.2 - The memberWeight option shall be an integer value.
-c.addInstance(__sandbox_uri2, {memberWeight: ""});
-
-c.addInstance(__sandbox_uri2, {memberWeight: true});
-
-c.addInstance(__sandbox_uri2, {memberWeight: "AB"});
-
-c.addInstance(__sandbox_uri2, {memberWeight: 10.5});
-
-//@ WL#11032: Add instance using a valid value for memberWeight (25) {VER(>=5.7.20)}
-c.addInstance(__sandbox_uri2, {memberWeight: 25});
-
-//@ WL#11032: Dissolve cluster 1 {VER(>=5.7.20)}
-c.dissolve({force: true});
-
-// Verify if memberWeight is persisted on >= 8.0.11
-
-// F2 - On a successful [dba.]createCluster() or [Cluster.]addInstance() call
-// using the option memberWeight, the GR sysvar
-// group_replication_member_weight must be persisted using SET PERSIST at
-// the target instance, if the MySQL Server instance version is >= 8.0.11.
-
-//@ WL#11032: Create cluster 2 {VER(>=8.0.11)}
-var c = dba.createCluster('test', {clearReadOnly: true, groupName: "ca94447b-e6fc-11e7-b69d-4485005154dc", memberWeight: 75, gtidSetIsComplete: true});
-
-//@ WL#11032: Add instance using a valid value for memberWeight (-50) {VER(>=8.0.11)}
-c.addInstance(__sandbox_uri2, {memberWeight: -50})
-
-//@<> WL#11032: memberWeight must be persisted on mysql >= 8.0.11 {VER(>=8.0.11)}
-EXPECT_EQ("0", get_sysvar(__mysql_sandbox_port2, "group_replication_member_weight", "PERSISTED"));
-
-//@ WL#11032: Dissolve cluster 2 {VER(>=8.0.11)}
-c.dissolve({force: true});
-
-// Verify that group_replication_member_weight is not persisted when not used
-// We need a clean instance for that because dissolve does not unset the previously set variables
-
-//@ WL#11032: Initialize new instance
-session.close();
-testutil.destroySandbox(__mysql_sandbox_port1);
-testutil.destroySandbox(__mysql_sandbox_port2);
-testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
-testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
-
-shell.connect(__sandbox_uri1);
-
-//@ WL#11032: Create cluster 3 {VER(>=8.0.11)}
-var c = dba.createCluster('test', {groupName: "ca94447b-e6fc-11e7-b69d-4485005154dc", gtidSetIsComplete: true});
-
-//@ WL#11032: Add instance without using memberWeight {VER(>=8.0.11)}
-c.addInstance(__sandbox_uri2)
-
-//@<> WL#11032: memberWeight must not be persisted on mysql >= 8.0.11 if not set {VER(>=8.0.11)}
-EXPECT_EQ("", get_sysvar(__mysql_sandbox_port2, "group_replication_member_weight", "PERSISTED"));
-
-//@ WL#11032: Finalization
-session.close();
-testutil.destroySandbox(__mysql_sandbox_port1);
-testutil.destroySandbox(__mysql_sandbox_port2);
 
 // BUG#27084767: CREATECLUSTER()/ ADDINSTANCE() DOES NOT CORRECTLY SET AUTO_INCREMENT_OFFSET
 //
@@ -400,8 +218,8 @@ c = dba.createCluster('noxplugin', {gtidSetIsComplete: true});
 c.addInstance(__sandbox_uri2);
 c.addInstance(__sandbox_uri3);
 
-var msg1 = "The X plugin is not enabled on instance '" + localhost + ":" + __mysql_sandbox_port1 + "'. No value will be assumed for the X protocol address.";
-var msg2 = "The X plugin is not enabled on instance '" + localhost + ":" + __mysql_sandbox_port2 + "'. No value will be assumed for the X protocol address.";
+var msg1 = "The X plugin is not enabled on instance '" + hostname + ":" + __mysql_sandbox_port1 + "'. No value will be assumed for the X protocol address.";
+var msg2 = "The X plugin is not enabled on instance '" + hostname + ":" + __mysql_sandbox_port2 + "'. No value will be assumed for the X protocol address.";
 EXPECT_SHELL_LOG_CONTAINS(msg1);
 EXPECT_SHELL_LOG_CONTAINS(msg2);
 
@@ -431,6 +249,7 @@ var wrong_pwd_uri = "root:wrongpawd@" + hostname + ":" + __mysql_sandbox_port2;
 c.removeInstance(wrong_pwd_uri, {force: true});
 
 //@<> BUG28056944 Error adding instance already in group but not in Metadata.
+// This should pass by just adding the instance to the metadata
 c.addInstance(__hostname_uri2);
 
 //@ BUG28056944 clean-up.
@@ -439,179 +258,17 @@ session.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
 testutil.destroySandbox(__mysql_sandbox_port2);
 
-// WL#12067 AdminAPI: Define failover consistency
-//
-// In 8.0.14, Group Replication introduces an option to specify the failover
-// guarantees (eventual or read_your_writes) when a primary failover happens in
-// single-primary mode). The new option defines the behavior of a new fencing
-// mechanism when a new primary is being promoted in a group. The fencing will
-// restrict connections from writing and reading from the new primary until it
-// has applied all the pending backlog of changes that came from the old
-// primary (read_your_writes). Applications will not see time going backward for
-// a short period of time (during the new primary promotion).
-
-// In order to support defining such option, the AdminAPI was extended by
-// introducing a new optional parameter, named 'consistency', in the
-// dba.createCluster function.
-//
-//@ WL#12067: Initialization {VER(>=8.0.14)}
-testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
-testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
-var s1 = mysql.getSession(__sandbox_uri1);
-var s2 = mysql.getSession(__sandbox_uri2);
-
-shell.connect(__sandbox_uri1);
-
-// FR2 The value for group_replication_consistency must be the same on all cluster members that support the option
-//@ WL#12067: TSF2_1 The value for group_replication_consistency must be the same on all cluster members (single-primary) {VER(>=8.0.14)}
-var c = dba.createCluster('test', {consistency: "1", gtidSetIsComplete: true});
-c.addInstance(__sandbox_uri2);
-session.close();
-
-//@<> WL#12067: TSF2_1 Confirm group_replication_consistency value is the same on all cluster members (single-primary) {VER(>=8.0.14)}
-EXPECT_EQ("BEFORE_ON_PRIMARY_FAILOVER", get_sysvar(__mysql_sandbox_port1, "group_replication_consistency"));
-EXPECT_EQ("BEFORE_ON_PRIMARY_FAILOVER", get_sysvar(__mysql_sandbox_port2, "group_replication_consistency"));
-
-//@<> WL#12067: TSF2_1 Confirm group_replication_consistency value was persisted (single-primary) {VER(>=8.0.14)}
-EXPECT_EQ("BEFORE_ON_PRIMARY_FAILOVER", get_sysvar(__mysql_sandbox_port1, "group_replication_consistency", "PERSISTED"));
-EXPECT_EQ("BEFORE_ON_PRIMARY_FAILOVER", get_sysvar(__mysql_sandbox_port2, "group_replication_consistency", "PERSISTED"));
-
-//@ WL#12067: Dissolve cluster 1{VER(>=8.0.14)}
-c.dissolve({force: true});
-
-// FR2 The value for group_replication_consistency must be the same on all cluster members that support the option
-//@ WL#12067: TSF2_2 The value for group_replication_consistency must be the same on all cluster members (multi-primary) {VER(>=8.0.14)}
-shell.connect(__sandbox_uri1);
-var c = dba.createCluster('test', { clearReadOnly:true, consistency: "1", multiPrimary:true, force: true, gtidSetIsComplete: true});
-c.addInstance(__sandbox_uri2);
-session.close();
-
-//@<> WL#12067: TSF2_2 Confirm group_replication_consistency value is the same on all cluster members (multi-primary) {VER(>=8.0.14)}
-EXPECT_EQ("BEFORE_ON_PRIMARY_FAILOVER", get_sysvar(__mysql_sandbox_port1, "group_replication_consistency"));
-EXPECT_EQ("BEFORE_ON_PRIMARY_FAILOVER", get_sysvar(__mysql_sandbox_port2, "group_replication_consistency"));
-
-//@<> WL#12067: TSF2_2 Confirm group_replication_consistency value was persisted (multi-primary) {VER(>=8.0.14)}
-EXPECT_EQ("BEFORE_ON_PRIMARY_FAILOVER", get_sysvar(__mysql_sandbox_port1, "group_replication_consistency", "PERSISTED"));
-EXPECT_EQ("BEFORE_ON_PRIMARY_FAILOVER", get_sysvar(__mysql_sandbox_port2, "group_replication_consistency", "PERSISTED"));
-
-//@ WL#12067: Finalization {VER(>=8.0.14)}
-testutil.destroySandbox(__mysql_sandbox_port1);
-testutil.destroySandbox(__mysql_sandbox_port2);
-
-// WL#12050 AdminAPI: Define the timeout for evicting previously active cluster members
-//
-// In 8.0.13, Group Replication introduced an option to allow defining
-// the timeout for evicting previously active nodes.  In order to support
-// defining such option, the AdminAPI was extended by introducing a new
-// optional parameter, named 'expelTimeout', in the dba.createCluster()
-// function.
-
-//@ WL#12050: Initialization {VER(>=8.0.13)}
-testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
-testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
-var s1 = mysql.getSession(__sandbox_uri1);
-var s2 = mysql.getSession(__sandbox_uri2);
-
-shell.connect(__sandbox_uri1);
-
-// FR2 The value for group_replication_member_expel_timeout must be the same on all cluster members that support the option
-//@ WL#12050: TSF2_1 The value for group_replication_member_expel_timeout must be the same on all cluster members (single-primary) {VER(>=8.0.13)}
-var c = dba.createCluster('test', {expelTimeout: 100, gtidSetIsComplete: true});
-c.addInstance(__sandbox_uri2);
-session.close();
-
-//@<> WL#12050: TSF2_1 Confirm group_replication_member_expel_timeout value is the same on all cluster members (single-primary) {VER(>=8.0.13)}
-EXPECT_EQ(100, get_sysvar(__mysql_sandbox_port1, "group_replication_member_expel_timeout"));
-EXPECT_EQ(100, get_sysvar(__mysql_sandbox_port2, "group_replication_member_expel_timeout"));
-
-//@<> WL#12050: TSF2_1 Confirm group_replication_member_expel_timeout value was persisted (single-primary) {VER(>=8.0.13)}
-EXPECT_EQ("100", get_sysvar(__mysql_sandbox_port1, "group_replication_member_expel_timeout", "PERSISTED"));
-EXPECT_EQ("100", get_sysvar(__mysql_sandbox_port2, "group_replication_member_expel_timeout", "PERSISTED"));
-
-//@ WL#12050: Dissolve cluster 1 {VER(>=8.0.13)}
-c.dissolve({force: true});
-
-// FR2 The value for group_replication_member_expel_timeout must be the same on all cluster members that support the option
-//@ WL#12050: TSF2_2 The value for group_replication_member_expel_timeout must be the same on all cluster members (multi-primary) {VER(>=8.0.13)}
-shell.connect(__sandbox_uri1);
-var c = dba.createCluster('test', { clearReadOnly:true, expelTimeout: 200, multiPrimary:true, force: true, gtidSetIsComplete: true});
-c.addInstance(__sandbox_uri2);
-session.close();
-
-//@<> WL#12050: TSF2_2 Confirm group_replication_member_expel_timeout value is the same on all cluster members (multi-primary) {VER(>=8.0.13)}
-EXPECT_EQ(200, get_sysvar(__mysql_sandbox_port1, "group_replication_member_expel_timeout"));
-EXPECT_EQ(200, get_sysvar(__mysql_sandbox_port2, "group_replication_member_expel_timeout"));
-
-//@<> WL#12050: TSF2_2 Confirm group_replication_member_expel_timeout value was persisted (multi-primary) {VER(>=8.0.13)}
-EXPECT_EQ("200", get_sysvar(__mysql_sandbox_port1, "group_replication_member_expel_timeout", "PERSISTED"));
-EXPECT_EQ("200", get_sysvar(__mysql_sandbox_port2, "group_replication_member_expel_timeout", "PERSISTED"));
-
-//@ WL#12050: Finalization {VER(>=8.0.13)}
-testutil.destroySandbox(__mysql_sandbox_port1);
-testutil.destroySandbox(__mysql_sandbox_port2);
-
-
-// WL#12066 AdminAPI: option to define the number of auto-rejoins
-//
-// In 8.0.16, Group Replication introduced an option to allow setting the
-// number of auto-rejoin attempts an instance will do after being
-// expelled from the group.  In order to support defining such option,
-// the AdminAPI was extended by introducing a new optional parameter,
-// named 'autoRejoinTries', in the dba.createCluster(),
-// cluster.addInstance, cluster.setOption and cluster.setInstanceOption
-// functions.
-
-//@ WL#12066: Initialization {VER(>=8.0.16)}
-testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
-testutil.snapshotSandboxConf(__mysql_sandbox_port1);
-testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
-testutil.snapshotSandboxConf(__mysql_sandbox_port2);
-testutil.deploySandbox(__mysql_sandbox_port3, "root", {report_host: hostname});
-var s1 = mysql.getSession(__sandbox_uri1);
-var s2 = mysql.getSession(__sandbox_uri2);
-var s3 = mysql.getSession(__sandbox_uri3);
-shell.connect(__sandbox_uri1);
-var c = dba.createCluster('test', {autoRejoinTries: 2, gtidSetIsComplete: true});
-
-// FR1 - A new option autoRejoinTries shall be added to the [dba.]createCluster() and [cluster.]addInstance() to allow users to set the Group Replication (GR) system variable group_replication_autorejoin_tries.
-//@ WL#12066: TSF1_4 Validate that an exception is thrown if the value specified is not an unsigned integer. {VER(>=8.0.16)}
-c.addInstance(__sandbox_uri2, {autoRejoinTries: -5});
-
-//@ WL#12066: TSF1_5 Validate that an exception is thrown if the value  is not in the range 0 to 2016. {VER(>=8.0.16)}
-c.addInstance(__sandbox_uri2, {autoRejoinTries: 2017});
-
-//@ WL#12066: TSF1_1, TSF6_1 Validate that the functions [dba.]createCluster() and [cluster.]addInstance() support a new option named autoRejoinTries. {VER(>=8.0.16)}
-c.addInstance(__sandbox_uri2, {autoRejoinTries: 2016});
-c.addInstance(__sandbox_uri3);
-session.close();
-
-//@<> WL#12066: TSF1_3, TSF1_6 Validate that when calling the functions [dba.]createCluster() and [cluster.]addInstance(), the GR variable group_replication_autorejoin_tries is persisted with the value given by the user on the target instance.{VER(>=8.0.16)}
-EXPECT_EQ(2, get_sysvar(__mysql_sandbox_port1, "group_replication_autorejoin_tries"));
-EXPECT_EQ(2016, get_sysvar(__mysql_sandbox_port2, "group_replication_autorejoin_tries"));
-EXPECT_EQ(0, get_sysvar(__mysql_sandbox_port3, "group_replication_autorejoin_tries"));
-
-//@<> WL#12066: TSF1_3, TSF1_6 Confirm group_replication_autorejoin_tries value was persisted {VER(>=8.0.16)}
-EXPECT_EQ("2", get_sysvar(__mysql_sandbox_port1, "group_replication_autorejoin_tries", "PERSISTED"));
-EXPECT_EQ("2016", get_sysvar(__mysql_sandbox_port2, "group_replication_autorejoin_tries", "PERSISTED"));
-EXPECT_EQ("", get_sysvar(__mysql_sandbox_port3, "group_replication_autorejoin_tries", "PERSISTED"));
-
-//@ WL#12066: Finalization {VER(>=8.0.16)}
-// NOTE: re-use the cluster for the next test. Remove only instances 2 and 3
-c.removeInstance(__sandbox_uri2);
-c.removeInstance(__sandbox_uri3);
-testutil.destroySandbox(__mysql_sandbox_port3);
-
 // BUG#29305551: ADMINAPI FAILS TO DETECT INSTANCE IS RUNNING ASYNCHRONOUS REPLICATION
 //
 // dba.checkInstance() reports that a target instance which is running the Slave
-// SQL and IO threads is valid for InnoDB cluster usage.
+// SQL and IO threads is valid to be used in an InnoDB cluster.
 //
 // As a consequence, the AdminAPI fails to detects that an instance has
 // asynchronous replication running and both addInstance() and rejoinInstance()
 // fail with useless/unfriendly errors on this scenario. There's not even
 // information on how to overcome the issue.
 
-//@<> BUG#29305551: Initialization {VER(<8.0)}
+//@<> BUG#29305551: Initialization
 testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
 testutil.snapshotSandboxConf(__mysql_sandbox_port1);
 testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
@@ -633,9 +290,20 @@ shell.connect(__sandbox_uri2);
 session.runSql("CHANGE MASTER TO MASTER_HOST='" + hostname + "', MASTER_PORT=" + __mysql_sandbox_port1 + ", MASTER_USER='repl', MASTER_PASSWORD='password', MASTER_AUTO_POSITION=1, MASTER_SSL=1");
 session.runSql("START SLAVE");
 
+testutil.waitMemberTransactions(__mysql_sandbox_port2, __mysql_sandbox_port1);
+
 //@ AddInstance async replication error
 c.addInstance(__sandbox_uri2);
 
+// BUG#32197197: ADMINAPI DOES NOT PROPERLY CHECK FOR PRECONFIGURED REPLICATION CHANNELS
+//
+// Even if replication is not running but configured, the warning/error has to
+// be provided as implemented in BUG#29305551
+session.runSql("STOP SLAVE");
+
+//@ AddInstance async replication error with channels stopped
+c.addInstance(__sandbox_uri2);
+
 //@ BUG#29305551: Finalization
 session.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
@@ -718,8 +386,8 @@ EXPECT_EQ(1, number);
 
 //@<> WL#12773: FR1.2 - The host-name shall be %
 result = session.runSql("SELECT Host FROM mysql.user WHERE User = 'mysql_innodb_cluster_22222'");
-var hostname = result.fetchOne()[0];
-EXPECT_EQ("%", hostname);
+var user_host = result.fetchOne()[0];
+EXPECT_EQ("%", user_host);
 
 //@ WL#12773: FR4 - The ipWhitelist shall not change the behavior defined by FR1
 result = session.runSql("SELECT COUNT(*) FROM mysql.user");
@@ -732,8 +400,185 @@ result = session.runSql("SELECT COUNT(*) FROM mysql.user");
 var new_account_number = result.fetchOne()[0];
 EXPECT_EQ(old_account_number + 1, new_account_number);
 
+// Tests for deprecation of ipWhitelist in favor of ipAllowlist
+
+//@<> Remove an instance to add it back using ipWhitelist
+c.removeInstance(__sandbox_uri2);
+
+//@<> Verify that ipWhitelist sets the right sysvars depending on the version
+var ip_white_list = "10.10.10.1/15,127.0.0.1," + hostname_ip;
+
+c.addInstance(__sandbox_uri2, {ipWhitelist: ip_white_list});
+shell.connect(__sandbox_uri2);
+
+//@<> Verify that ipWhitelist sets group_replication_ip_allowlist in 8.0.22 {VER(>=8.0.22)}
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_allowlist"));
+
+//@<> Verify that ipWhitelist sets group_replication_ip_whitelist in versions < 8.0.22 {VER(<8.0.22)}
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_whitelist"));
+
+//@<> Remove the instance to add it back using ipAllowlist
+c.removeInstance(__sandbox_uri2);
+
+//@<> Verify the new option ipAllowlist sets the right sysvars depending on the version
+ip_white_list = "10.1.1.0/15,127.0.0.1," + hostname_ip;
+
+c.addInstance(__sandbox_uri2, {ipAllowlist:ip_white_list});
+shell.connect(__sandbox_uri2);
+
+//@<> Verify the new option ipAllowlist sets group_replication_ip_allowlist in 8.0.22 {VER(>=8.0.22)}
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_allowlist"));
+
+//@<> Verify the new option ipAllowlist sets group_replication_ip_whitelist in versions < 8.0.22 {VER(<8.0.22)}
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_whitelist"));
+
 //@<> WL#12773: Cleanup
 session.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
 testutil.destroySandbox(__mysql_sandbox_port2);
 testutil.destroySandbox(__mysql_sandbox_port3);
+
+// BUG#25503159: addInstance() does not drop recovery user when it fails
+//@<> BUG#25503159: deploy sandboxes.
+testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
+testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
+
+//@<> BUG#25503159: create cluster.
+shell.connect(__hostname_uri1);
+var c = dba.createCluster('test', {gtidSetIsComplete: true});
+//var c = dba.createCluster('test');
+
+//@ BUG#25503159: number of recovery users before executing addInstance().
+var num_recovery_users = number_of_gr_recovery_accounts(session);
+print("Number of recovery user before addInstance(): " + num_recovery_users + "\n");
+
+//@ BUG#25503159: add instance fail (using an invalid localaddress).
+c.addInstance(__hostname_uri2, {localAddress: "invalid_host"});
+
+//@<> BUG#25503159: number of recovery users is the same.
+var num_recovery_users_after = number_of_gr_recovery_accounts(session);
+EXPECT_EQ(num_recovery_users, num_recovery_users_after);
+
+// BUG#30281908: MYSQL SHELL CRASHED WHILE ADDING A INSTANCE TO INNODB CLUSTER
+// This bug was caused by a segmentation fault in the post-clone restart handling
+// that did not cover the timeout waiting for the instance to start after clone restarts it.
+// To test it, we change the sandbox configuration file to introduce a bogus setting which will
+// cause the failure of the instance restart, simulating then a timeout.
+
+//@ BUG#30281908: add instance using clone and simulating a restart timeout {VER(>= 8.0.17)}
+testutil.changeSandboxConf(__mysql_sandbox_port2, "foo", "bar");
+// Also tests the restartWaitTimeout option
+shell.options["dba.restartWaitTimeout"] = 1;
+c.addInstance(__sandbox_uri2, {interactive:true, recoveryMethod:"clone"});
+shell.options["dba.restartWaitTimeout"] = 60;
+
+//@<> BUG#30281908: restart the instance manually {VER(>= 8.0.17)}
+testutil.removeFromSandboxConf(__mysql_sandbox_port2, "foo");
+testutil.startSandbox(__mysql_sandbox_port2);
+testutil.waitMemberState(__mysql_sandbox_port2, "ONLINE");
+
+//@ BUG#30281908: complete the process with .rescan() {VER(>= 8.0.17)}
+testutil.expectPrompt("Would you like to add it to the cluster metadata? [Y/n]:", "y");
+c.rescan({interactive:true});
+
+//@<> BUG#25503159: clean-up.
+c.disconnect();
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+testutil.destroySandbox(__mysql_sandbox_port2);
+
+//@<> Initialization canonical IPv6 addresses supported WL#12758 {VER(>= 8.0.14)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: "::1"});
+testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: "::1"});
+shell.connect(__sandbox_uri1);
+var cluster = dba.createCluster("cluster", {gtidSetIsComplete: true});
+
+//@<> canonical IPv6 addresses are supported on addInstance WL#12758 {VER(>= 8.0.14)}
+cluster.addInstance(__sandbox_uri2);
+testutil.waitMemberState(__mysql_sandbox_port2, "ONLINE");
+// Bug #30548843 Validate that IPv6 values stored on metadata for mysqlx are valid
+print_metadata_instance_addresses(session);
+
+//@<> Cleanup canonical IPv6 addresses supported WL#12758 {VER(>= 8.0.14)}
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+testutil.destroySandbox(__mysql_sandbox_port2);
+
+//@<> Initialization sandbox WL#12758 IPv6 addresses supported {VER(>= 8.0.14)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: "::1"});
+testutil.deploySandbox(__mysql_sandbox_port2, "root");
+shell.connect(__sandbox_uri1);
+var cluster = dba.createCluster("cluster", {gtidSetIsComplete: true});
+
+//@<> IPv6 addresses are supported on localAddress, groupSeeds and ipWhitelist WL#12758 {VER(>= 8.0.14)}
+var local_address = "[::1]:" + __mysql_sandbox_gr_port2;
+var ip_white_list = "::1, 127.0.0.1";
+var group_seeds = "[::1]:" + __mysql_sandbox_gr_port1; + ", [::1]:" + __mysql_sandbox_gr_port2;
+cluster.addInstance(__sandbox_uri2, {ipWhitelist:ip_white_list, groupSeeds:group_seeds, localAddress:local_address});
+testutil.waitMemberState(__mysql_sandbox_port2, "ONLINE");
+shell.connect(__sandbox_uri2);
+EXPECT_EQ(local_address, get_sysvar(session, "group_replication_local_address"));
+EXPECT_EQ(group_seeds, get_sysvar(session, "group_replication_group_seeds"));
+
+//@<> Confirm that ipWhitelist is set {VER(>= 8.0.14) && VER(<8.0.22)}
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_whitelist"));
+
+//@<> Confirm that ipWhitelist is set {VER(>= 8.0.22)}
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_allowlist"));
+
+//@<> Cleanup WL#12758 IPv6 addresses supported {VER(>= 8.0.14)}
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+testutil.destroySandbox(__mysql_sandbox_port2);
+
+//@<> Initialization canonical IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root");
+testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: "::1"});
+shell.connect(__sandbox_uri1);
+var cluster = dba.createCluster("cluster", {gtidSetIsComplete: true});
+
+//@ canonical IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+cluster.addInstance(__sandbox_uri2);
+
+//@<> Cleanup canonical IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+testutil.destroySandbox(__mysql_sandbox_port2);
+
+//@<> Initialization IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root");
+testutil.deploySandbox(__mysql_sandbox_port2, "root");
+shell.connect(__sandbox_uri1);
+var cluster = dba.createCluster("cluster", {gtidSetIsComplete: true});
+
+//@ IPv6 local_address is not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+var local_address = "[::1]:" + __mysql_sandbox_gr_port1;
+cluster.addInstance(__sandbox_uri2, {localAddress: local_address});
+
+//@ IPv6 on ipWhitelist is not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+cluster.addInstance(__sandbox_uri2, {ipWhitelist: "::1, 127.0.0.1"});
+
+//@ IPv6 on groupSeeds is not supported below 8.0.14 - 1 WL#12758 {VER(< 8.0.14)}
+var group_seeds = "127.0.0.1:" + __mysql_sandbox_gr_port1 + ", [::1]:" + __mysql_sandbox_gr_port2;
+cluster.addInstance(__sandbox_uri2, {groupSeeds: group_seeds});
+
+//@ IPv6 on groupSeeds is not supported below 8.0.14 - 2 WL#12758 {VER(< 8.0.14)}
+var group_seeds = "127.0.0.1:" + __mysql_sandbox_gr_port1 + ", [::1]:" + __mysql_sandbox_gr_port2 + " , [fe80::7e36:f49a:63c8:8ad6]:" + __mysql_sandbox_gr_port1;
+cluster.addInstance(__sandbox_uri2, {groupSeeds:group_seeds});
+
+//@<> Cleanup IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+testutil.destroySandbox(__mysql_sandbox_port2);
+
+//@<> cluster.addInstance does not error if using instance with binlog_checksum enabled BUG#31329024 {VER(>= 8.0.21)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname, binlog_checksum: "NONE"});
+testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname, binlog_checksum: "CRC32"});
+
+shell.connect(__sandbox_uri1);
+cluster = dba.createCluster("sample", {gtidSetIsComplete: true});
+cluster.addInstance(__sandbox_uri2);
+EXPECT_STDERR_EMPTY();
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+testutil.destroySandbox(__mysql_sandbox_port2);
