@@ -30,12 +30,16 @@ function mark_clone_disabled(flag) {
         session.runSql("UPDATE mysql_innodb_cluster_metadata.clusters SET attributes = JSON_REMOVE(attributes, '$.opt_disableClone')");
 }
 
-function install_clone() {
-    ensure_plugin_enabled("clone", session2, "mysql_clone");
+function install_clone(session) {
+    ensure_plugin_enabled("clone", session, "mysql_clone");
 }
 
-function clone_installed() {
-    var row = session2.runSql("select plugin_status from information_schema.plugins where plugin_name='clone'").fetchOne();
+function uninstall_clone(session) {
+    ensure_plugin_disabled("clone", session);
+}
+
+function clone_installed(session) {
+    var row = session.runSql("select plugin_status from information_schema.plugins where plugin_name='clone'").fetchOne();
     if (row)
         return row[0] == "ACTIVE";
     return false;
@@ -52,7 +56,8 @@ function clone_threshold() {
 testutil.deploySandbox(__mysql_sandbox_port1, "root");
 testutil.deploySandbox(__mysql_sandbox_port2, "root");
 
-session2= mysql.getSession(__sandbox_uri2);
+session1 = mysql.getSession(__sandbox_uri1);
+session2 = mysql.getSession(__sandbox_uri2);
 
 // Tests for recovery method selection logic
 //---------------------------------------------
@@ -63,7 +68,8 @@ session2= mysql.getSession(__sandbox_uri2);
 // | !auto     | -        | true             | true or gtidSetComplete=1  | Continue without clone (regular distributed recovery) |
 // | auto      | true     | true             | false                      | PROMPT: Clone, Recovery, Abort                        |
 // | auto      | false    | true             | false                      | PROMPT: Recovery, Abort                               |
-// | auto      | true     | false            | -                          | PROMPT: Clone, Abort                                  |
+// | auto      | true     | false            | -                          | GTID-SET DIVERGED: PROMPT: Clone, Abort               |
+// | auto      | true     | false            | -                          | GTID-SET COMPATIBLE: Continue with clone              |
 // | auto      | false    | false            | -                          | ERROR: Manual provisioning required                   |
 
 // | regular   | -        | true             | -                          | Continue without clone (regular distributed recovery) |
@@ -78,7 +84,8 @@ c = dba.createCluster("mycluster");
 
 var gtid_executed = session.runSql("SELECT @@global.gtid_executed").fetchOne()[0];
 
-testutil.dbugSet("+d,dba_abort_join_group");
+testutil.clearTraps();
+testutil.setTrap("mysql", ["sql regex start group_replication"], {msg: "debug"});
 
 //@ bogus recoveryMethod (should fail)
 c.addInstance(__sandbox_uri2, {interactive:false, recoveryMethod:"foobar"});
@@ -88,26 +95,51 @@ c.addInstance(__sandbox_uri2, {interactive:false, recoveryMethod:"clone"});
 
 //@ Ensure clone disabled on incremental recovery {VER(>=8.0.17)}
 session2.runSql("RESET MASTER");
-install_clone();
+install_clone(session2);
 
 c.addInstance(__sandbox_uri2, {interactive:false, recoveryMethod:"incremental"});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
-//@ Ensure clone enabled on clone recovery {VER(>=8.0.17)}
+// Regression test for BUG#29954085 and BUG#29960838:
+//
+// Regardless if:
+//  - The cluster was set up using Incremental recovery only, clone usage was
+//    not disabled.
+//  - The cluster was created on an instance supporting clone but using a Shell
+//    older than 8.0.16
+//
+// The clone plugin must be installed on all cluster members.
+// The same logic applies when disableClone was set: the plugin must be
+// uninstalled on all cluster members
+
+//@ Ensure clone enabled on all cluster members on clone recovery {VER(>=8.0.17)}
 c.addInstance(__sandbox_uri2, {interactive:false, recoveryMethod:"clone"});
 
-EXPECT_TRUE(clone_installed());
+EXPECT_TRUE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
 //EXPECT_EQ(1, clone_threshold()); this check doesn't work because addInstance will revert the threshold when it finishes up
 
+//@ Ensure clone enabled on all cluster members on clone recovery when cluster created with old shell {VER(>=8.0.17)}
+// NOTE: To simulate a cluster created with an old shell, on an instance
+// supporting clone, we simply ensure the plugin is uninstalled
+uninstall_clone(session1);
+EXPECT_FALSE(clone_installed(session1));
+c.addInstance(__sandbox_uri2, {interactive:false, recoveryMethod:"clone"});
+
+EXPECT_TRUE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
+
 //@ recoveryMethod:auto, interactive, empty GTID -> prompt c/i/a {VER(>=8.0.17)}
 testutil.expectPrompt("Please select a recovery method [C]lone/[I]ncremental recovery/[A]bort (default Clone): ", "i");
 mark_gtid_set_complete(false);
 
 c.addInstance(__sandbox_uri2, {interactive:true});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 //@ recoveryMethod:auto, interactive, cloneDisabled, empty GTID -> prompt i/a
@@ -117,7 +149,8 @@ mark_clone_disabled(true);
 
 c.addInstance(__sandbox_uri2, {interactive:true});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 mark_clone_disabled(false);
 
@@ -126,7 +159,8 @@ mark_gtid_set_complete(true);
 
 c.addInstance(__sandbox_uri2, {interactive:true});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 mark_gtid_set_complete(false);
@@ -137,7 +171,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed]);
 
 c.addInstance(__sandbox_uri2, {interactive:true});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 //@ recoveryMethod:auto, interactive, errant GTIDs -> prompt c/a {VER(>=8.0.17)}
@@ -147,7 +182,8 @@ testutil.expectPrompt("Please select a recovery method [C]lone/[A]bort (default
 
 c.addInstance(__sandbox_uri2, {interactive:true});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 //@ recoveryMethod:auto, interactive, errant GTIDs -> error 5.7 {VER(<8.0.17)}
@@ -156,7 +192,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed+",00025721-1111-1111-
 
 c.addInstance(__sandbox_uri2, {interactive:true});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 //@ recoveryMethod:auto, interactive, cloneDisabled, errant GTIDs -> error
@@ -167,19 +204,21 @@ mark_clone_disabled(true);
 
 c.addInstance(__sandbox_uri2, {interactive:true});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 mark_clone_disabled(false);
 
 // ====
 
-//@ recoveryMethod:auto, non-interactive, empty GTID -> error
+//@ recoveryMethod:auto, non-interactive, empty GTID -> error {VER(>=8.0.17)}
 session2.runSql("RESET MASTER");
 mark_gtid_set_complete(false);
 
 c.addInstance(__sandbox_uri2, {interactive:false});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 //@ recoveryMethod:auto, non-interactive, cloneDisabled, empty GTID -> error {VER(>=8.0.17)}
@@ -188,7 +227,8 @@ mark_clone_disabled(true);
 
 c.addInstance(__sandbox_uri2, {interactive:false});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 mark_clone_disabled(false);
 
@@ -197,7 +237,8 @@ mark_gtid_set_complete(true);
 
 c.addInstance(__sandbox_uri2, {interactive:false});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 mark_gtid_set_complete(false);
@@ -208,7 +249,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed]);
 
 c.addInstance(__sandbox_uri2, {interactive:false});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 //@ recoveryMethod:auto, non-interactive, errant GTIDs -> error
@@ -217,7 +259,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed+",00025721-1111-1111-
 
 c.addInstance(__sandbox_uri2, {interactive:false});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 
 //@ recoveryMethod:auto, non-interactive, cloneDisabled, errant GTIDs -> error
@@ -228,7 +271,8 @@ mark_clone_disabled(true);
 
 c.addInstance(__sandbox_uri2, {interactive:false});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 EXPECT_NE(1, clone_threshold());
 mark_clone_disabled(false);
 
@@ -247,7 +291,8 @@ mark_gtid_set_complete(false);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"incremental"});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 
 //@ recoveryMethod:incremental, cloneDisabled, empty GTID -> incr
 mark_gtid_set_complete(false);
@@ -255,7 +300,8 @@ mark_clone_disabled(true);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"incremental"});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 mark_clone_disabled(false);
 
 //@ recoveryMethod:incremental, empty GTIDs + gtidSetIsComplete -> incr
@@ -263,7 +309,8 @@ mark_gtid_set_complete(true);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"incremental"});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 mark_gtid_set_complete(false);
 
 //@ recoveryMethod:incremental, subset GTIDs -> incr
@@ -272,7 +319,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed]);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"incremental"});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 
 //@ recoveryMethod:incremental, errant GTIDs -> error
 session2.runSql("RESET MASTER");
@@ -280,7 +328,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed+",00025721-1111-1111-
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"incremental"});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 
 //@ recoveryMethod:incremental, cloneDisabled, errant GTIDs -> error
 session2.runSql("RESET MASTER");
@@ -290,8 +339,10 @@ mark_clone_disabled(true);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"incremental"});
 
-EXPECT_FALSE(clone_installed());
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_FALSE(clone_installed(session2));
 mark_clone_disabled(false);
+session2.runSql("RESET MASTER");
 
 
 // ====
@@ -307,7 +358,8 @@ mark_gtid_set_complete(false);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"clone"});
 
-EXPECT_TRUE(clone_installed());
+EXPECT_TRUE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
 
 //@ recoveryMethod:clone, cloneDisabled, empty GTID -> err {VER(>=8.0.17)}
 mark_gtid_set_complete(false);
@@ -322,7 +374,8 @@ mark_gtid_set_complete(true);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"clone"});
 
-EXPECT_TRUE(clone_installed());
+EXPECT_TRUE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
 mark_gtid_set_complete(false);
 
 //@ recoveryMethod:clone, subset GTIDs -> clone {VER(>=8.0.17)}
@@ -331,7 +384,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed]);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"clone"});
 
-EXPECT_TRUE(clone_installed());
+EXPECT_TRUE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
 
 //@ recoveryMethod:clone, errant GTIDs -> clone {VER(>=8.0.17)}
 session2.runSql("RESET MASTER");
@@ -339,7 +393,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed+",00025721-1111-1111-
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"clone"});
 
-EXPECT_TRUE(clone_installed());
+EXPECT_TRUE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
 
 //@ recoveryMethod:clone, cloneDisabled, errant GTIDs -> error {VER(>=8.0.17)}
 session2.runSql("RESET MASTER");
@@ -351,6 +406,61 @@ c.addInstance(__sandbox_uri2, {recoveryMethod:"clone"});
 
 mark_clone_disabled(false);
 
+//@<> Ensure clone is installed on all cluster members when failover happens {VER(>=8.0.17)}
+//
+// Regression test for BUG#29954085
+//
+// NOTE: When a cluster is created with a shell version < 8.0.17, the cluster
+// plugin won't be installed even if the target mysql instance supports it.
+// If an instance is added using incremental recovery and then the seed is
+// removed for some reason, whenever adding another new instance using
+// clone recovery the "new" seed must have the clone plugin installed.
+//
+// To simulate a cluster created with a shell version < 8.0.17 we simply uninstall
+// the clone plugin on the seed if installed.
+
+// Disable debug
+testutil.clearTraps();
+
+testutil.deploySandbox(__mysql_sandbox_port3, "root");
+session3 = mysql.getSession(__sandbox_uri3);
+
+ensure_plugin_disabled("clone", session1);
+EXPECT_FALSE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
+
+session2.runSql("RESET MASTER");
+mark_gtid_set_complete(true);
+
+// Add the second instance to the cluster using incremental recovery
+c.addInstance(__sandbox_uri2, {interactive:true, recoveryMethod:"incremental"});
+
+// Remove the seed
+c.removeInstance(__sandbox_uri1);
+
+session.close();
+c.disconnect();
+shell.connect(__sandbox_uri2);
+
+var c = dba.getCluster();
+
+// Add a third instance using clone recovery
+c.addInstance(__sandbox_uri3, {recoveryMethod:"clone"});
+
+c.removeInstance(__sandbox_uri2);
+
+// Re-enable debug
+testutil.setTrap("mysql", ["sql regex start group_replication"], {msg: "debug"});
+
+session.close();
+c.disconnect();
+shell.connect(__sandbox_uri3);
+var c = dba.getCluster();
+
+set_sysvar(session, "super_read_only", 0);
+
+session1 = mysql.getSession(__sandbox_uri3);
+session2 = mysql.getSession(__sandbox_uri2);
 
 // Tests with purged GTIDs
 // ----------------------------------------
@@ -358,12 +468,33 @@ mark_clone_disabled(false);
 //@ purge GTIDs from cluster
 session.runSql("FLUSH BINARY LOGS");
 session.runSql("PURGE BINARY LOGS BEFORE DATE_ADD(NOW(6), INTERVAL 1 DAY)");
+
+//@ recoveryMethod:auto, interactive, purged GTID, new -> prompt c/a {VER(>=8.0.17)}
+testutil.expectPrompt("Please select a recovery method [C]lone/[A]bort (default Clone): ", "a");
+mark_gtid_set_complete(false);
 session2.runSql("RESET MASTER");
 
+c.addInstance(__sandbox_uri2, {interactive:true});
 
-//@ recoveryMethod:auto, interactive, purged GTID -> prompt c/a {VER(>=8.0.17)}
-testutil.expectPrompt("Please select a recovery method [C]lone/[A]bort (default Abort): ", "a");
+//@ recoveryMethod:auto, no-interactive, purged GTID, new -> error {VER(>=8.0.17)}
 mark_gtid_set_complete(false);
+session2.runSql("RESET MASTER");
+
+c.addInstance(__sandbox_uri2, {interactive:false});
+
+// BUG#30884590: ADDING AN INSTANCE WITH COMPATIBLE GTID SET SHOULDN'T PROMPT FOR CLONE
+//@ recoveryMethod:auto, interactive, purged GTID, subset gtid -> clone, no prompt {VER(>=8.0.17)}
+mark_gtid_set_complete(false);
+session2.runSql("RESET MASTER");
+session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed]);
+
+c.addInstance(__sandbox_uri2, {interactive:true});
+
+// BUG#30884590: ADDING AN INSTANCE WITH COMPATIBLE GTID SET SHOULDN'T PROMPT FOR CLONE
+//@ recoveryMethod:auto, no-interactive, purged GTID, subset gtid -> clone, no prompt {VER(>=8.0.17)}
+mark_gtid_set_complete(false);
+session2.runSql("RESET MASTER");
+session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed]);
 
 c.addInstance(__sandbox_uri2, {interactive:true});
 
@@ -434,7 +565,8 @@ mark_gtid_set_complete(false);
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"clone"});
 
-EXPECT_TRUE(clone_installed());
+EXPECT_TRUE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
 
 //@ recoveryMethod:clone, cloneDisabled, purged GTID -> err {VER(>=8.0.17)}
 mark_gtid_set_complete(false);
@@ -450,7 +582,8 @@ session2.runSql("SET GLOBAL gtid_purged=?", [gtid_executed+",00025721-1111-1111-
 
 c.addInstance(__sandbox_uri2, {recoveryMethod:"clone"});
 
-EXPECT_TRUE(clone_installed());
+EXPECT_TRUE(clone_installed(session1));
+EXPECT_TRUE(clone_installed(session2));
 
 //@ recoveryMethod:clone, cloneDisabled, errant GTIDs + purged GTIDs -> error {VER(>=8.0.17)}
 session2.runSql("RESET MASTER");
@@ -464,5 +597,11 @@ mark_clone_disabled(false);
 
 
 //@ Cleanup
+testutil.clearTraps();
+
+session.close();
+session1.close();
+session2.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
-testutil.destroySandbox(__mysql_sandbox_port2);
\ No newline at end of file
+testutil.destroySandbox(__mysql_sandbox_port2);
+testutil.destroySandbox(__mysql_sandbox_port3);
