@@ -30,13 +30,26 @@ function gtid_contains_server_uuid(session) {
     return gtids.search(uuid) >= 0;
 }
 
-// WL#12011: AdminAPI: Refactor dba.createCluster()
-//@ WL#12011: Initialization
+function print_metadata_instance_addresses(session) {
+    var res = session.runSql("select * from mysql_innodb_cluster_metadata.instances");
+    var row = res.fetchOneObject()
+    while(row) {
+        print(row.instance_name + " = " + row.addresses + "\n");
+        row = res.fetchOneObject();
+    }
+    print("\n");
+}
+
+
+//@<> init
 testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
+testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
 testutil.snapshotSandboxConf(__mysql_sandbox_port1);
+testutil.snapshotSandboxConf(__mysql_sandbox_port2);
 
 shell.connect(__sandbox_uri1);
 
+// WL#12011: AdminAPI: Refactor dba.createCluster()
 //@<> WL#12011: FR2-04 - invalid value for interactive option.
 var c = dba.createCluster('test', {interactive: "not a valid type"});
 
@@ -47,10 +60,50 @@ var c = dba.createCluster('test', {interactive: true, multiPrimary: true});
 //@<> WL#12011: FR2-03 - no interactive option (default: non-interactive).
 var c = dba.createCluster('test', {multiPrimary: true, force: true});
 
+c.dissolve();
+
+//@<> Create cluster without group_replication_start_on_boot
+session.runSql("set global group_replication_start_on_boot=1");
+var start_on_boot = session.runSql("select @@group_replication_start_on_boot").fetchOne()[0];
+EXPECT_EQ(1, start_on_boot);
+
+// this also ensures the ensure_updates_everywhere_checks option
+// is updated correctly when we're switching from multi-primary to single-primary
+session.runSql("set global group_replication_single_primary_mode=0");
+session.runSql("set global group_replication_enforce_update_everywhere_checks=1");
+
+c = dba.createCluster("test", {manualStartOnBoot:true});
+start_on_boot = session.runSql("select @@group_replication_start_on_boot").fetchOne()[0];
+EXPECT_EQ(0, start_on_boot);
+
+c.addInstance(__sandbox_uri2, {recoveryMethod:"incremental"});
+session2 = mysql.getSession(__sandbox_uri2);
+start_on_boot = session2.runSql("select @@group_replication_start_on_boot").fetchOne()[0];
+EXPECT_EQ(0, start_on_boot);
+
+c.dissolve();
+
+//@<> check error log dumping on create if supported {VER(>=8.0.22)}
+
+// force GR start to fail by passing an invalid localAddress
+EXPECT_THROWS(function(){dba.createCluster("test", {localAddress:"0.0.0.0:1"})}, "Group Replication failed to start:");
+
+EXPECT_OUTPUT_CONTAINS("The MySQL error_log contains the following messages:");
+EXPECT_OUTPUT_CONTAINS("Plugin group_replication reported: '[GCS] There is no local IP address matching the one configured for the local node");
+
+//@<> check error log dumping on add if supported {VER(>=8.0.22)}
+var c = dba.createCluster("test");
+
+EXPECT_THROWS(function(){c.addInstance(__sandbox_uri2, {localAddress:"0.0.0.0:1", recoveryMethod:"incremental"})}, "Group Replication failed to start:");
+
+EXPECT_OUTPUT_CONTAINS("The MySQL error_log contains the following messages:");
+EXPECT_OUTPUT_CONTAINS("Plugin group_replication reported: '[GCS] There is no local IP address matching the one configured for the local node");
+
 //@ WL#12011: Finalization.
 c.disconnect();
 session.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
+testutil.destroySandbox(__mysql_sandbox_port2);
 
 // WL#12049 AdminAPI: option to shutdown server when dropping out of the
 // cluster
@@ -101,6 +154,10 @@ var c = dba.createCluster('test', {exitStateAction: "ABORT_SERVER"});
 // user.
 EXPECT_EQ("ABORT_SERVER", get_sysvar(session, "group_replication_exit_state_action"));
 
+//@<> group_replication_view_change_uuid must not be set for Clusters running versions < 8.0.27 {VER(<8.0.27)}
+var view_change_uuid_md = session.runSql("select (attributes->>'$.group_replication_view_change_uuid') from mysql_innodb_cluster_metadata.clusters where cluster_name='newcluster'").fetchOne();
+EXPECT_EQ(null, view_change_uuid_md);
+
 //@ WL#12049: Dissolve cluster 1 {VER(>=5.7.24)}
 c.dissolve({force: true});
 
@@ -175,9 +232,12 @@ shell.connect(__sandbox_uri1);
 //@ WL#12049: Create cluster 2 {VER(>=8.0.12)}
 var c = dba.createCluster('test', {groupName: "ca94447b-e6fc-11e7-b69d-4485005154dc"});
 
-//@<OUT> BUG#28701263: DEFAULT VALUE OF EXITSTATEACTION TOO DRASTIC {VER(>=8.0.12)}
+//@<OUT> BUG#28701263: DEFAULT VALUE OF EXITSTATEACTION TOO DRASTIC {VER(>=8.0.12) && VER(<8.0.16)}
 print(get_persisted_gr_sysvars(__mysql_sandbox_port1));
 
+//@<> BUG#29037274: ADMINAPI MUST NOT SET DEFAULT VALUE FOR G_R_EXIT_STATE_ACTION IN 8.0.16 {VER(>=8.0.16)}
+EXPECT_EQ(null, session.runSql("SELECT * from performance_schema.persisted_variables WHERE Variable_name like 'group_replication_exit_state_action'").fetchOne());
+
 //@ WL#12049: Finalization
 session.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
@@ -484,7 +544,7 @@ var c = dba.createCluster('test', {clearReadOnly:true});
 c.disconnect();
 
 //@<> WL#12050: TSF1_2 Confirm group_replication_member_expel_timeout is set correctly (0) {VER(>=8.0.13)}
-EXPECT_EQ(0, get_sysvar(session, "group_replication_member_expel_timeout"));
+EXPECT_EQ(__default_gr_expel_timeout, get_sysvar(session, "group_replication_member_expel_timeout"));
 
 //@<> WL#12050: TSF1_2 Confirm group_replication_member_expel_timeout was not persisted since no value was provided. {VER(>=8.0.13)}
 EXPECT_EQ("", get_sysvar(session, "group_replication_member_expel_timeout", "PERSISTED"));
@@ -576,7 +636,7 @@ session.close();
 
 //@<> WL#12066: TSF1_3, TSF1_6 Validate that when calling the functions [dba.]createCluster() and [cluster.]addInstance(), the GR variable group_replication_autorejoin_tries is persisted with the value given by the user on the target instance.{VER(>=8.0.16)}
 EXPECT_EQ(2016, get_sysvar(__mysql_sandbox_port1, "group_replication_autorejoin_tries"));
-EXPECT_EQ(0, get_sysvar(__mysql_sandbox_port2, "group_replication_autorejoin_tries"));
+EXPECT_EQ(__default_gr_auto_rejoin_tries, get_sysvar(__mysql_sandbox_port2, "group_replication_autorejoin_tries"));
 
 //@<> WL#12066: TSF1_3, TSF1_6 Confirm group_replication_autorejoin_tries value was persisted {VER(>=8.0.16)}
 EXPECT_EQ("2016", get_sysvar(__mysql_sandbox_port1, "group_replication_autorejoin_tries", "PERSISTED"));
@@ -608,17 +668,21 @@ print(get_query_single_result(session, number_of_rpl_users_query) + "\n");
 //NOTE: Do not destroy sandbox 2 to be used on the next tests
 session.close();
 
-// BUG#29305551: ADMINAPI FAILS TO DETECT INSTANCE IS RUNNING ASYNCHRONOUS REPLICATION
+// BUG#32197222: ADMINAPI CREATECLUSTER() SHOULD NOT ALLOW EXISTING ASYNC REPLICATION CHANNELS
 //
-// dba.checkInstance() reports that a target instance which is running the Slave
-// SQL and IO threads is valid for InnoDB cluster usage.
+// With BUG#29305551, dba.checkInstanceConfiguration() was extended to
+// include a check to verify if asynchronous replication is configured and
+// running on the target instance and print a warning if that's the case.
+// On top of that, the same check is used in <Cluster>.addInstance() and
+// <Cluster>.rejoinInstance() to terminate the commands with an error if
+// such scenario is verified.
+// The same check is also used in dba.rebootClusterFromCompleteOutage()
+// whenever there are instances to be rejoined to the cluster.
 //
-// As a consequence, the AdminAPI fails to detects that an instance has
-// asynchronous replication running and both addInstance() and rejoinInstance()
-// fail with useless/unfriendly errors on this scenario. There's not even
-// information on how to overcome the issue.
+// However, dba.createCluster() was skipping that check.
 //
-// dba.createCluster() shall not fail if asynchronous replication is running
+// dba.createCluster() must fail if asynchronous replication is running, unless
+// the 'force' option is
 
 //@ BUG#29305551: Initialization
 testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
@@ -637,9 +701,12 @@ shell.connect(__sandbox_uri2);
 session.runSql("CHANGE MASTER TO MASTER_HOST='" + hostname + "', MASTER_PORT=" + __mysql_sandbox_port1 + ", MASTER_USER='repl', MASTER_PASSWORD='password', MASTER_AUTO_POSITION=1, MASTER_SSL=1");
 session.runSql("START SLAVE");
 
-//@ Create cluster async replication OK
+//@ Create cluster async replication (should fail)
 dba.createCluster('testAsync', {clearReadOnly: true});
 
+//@<> Create cluster async replication using force option (should pass)
+EXPECT_NO_THROWS(function(){dba.createCluster('testAsync', {clearReadOnly: true, force: true});});
+
 //@ BUG#29305551: Finalization
 session.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
@@ -710,7 +777,7 @@ shell.connect(__sandbox_uri1);
 //@ WL#12773: FR4 - The ipWhitelist shall not change the behavior defined by FR1
 var result = session.runSql("SELECT COUNT(*) FROM mysql.user");
 var old_account_number = result.fetchOne()[0];
-dba.createCluster('test', {ipWhitelist:"192.168.2.1/15,127.0.0.1," + hostname_ip});
+testutil.callMysqlsh([__sandbox_uri1, "--", "dba", "create-cluster", "test", "--ip-whitelist=192.168.2.1/15,127.0.0.1," + hostname_ip])
 print(get_all_gr_recovery_accounts(session));
 
 result = session.runSql("SELECT COUNT(*) FROM mysql.user");
@@ -737,7 +804,7 @@ dba.createCluster('test', {disableClone: {}});
 var c = dba.createCluster('test');
 
 //@<> WL#13208: TS_FR1_3 verify disableClone is false.
-c.options();
+normalize_cluster_options(c.options());
 
 //@<> WL#13208: TS_FR1_3 dissolve cluster.
 c.dissolve();
@@ -746,7 +813,7 @@ c.dissolve();
 var c = dba.createCluster('test', {disableClone: true});
 
 //@ WL#13208: TS_FR1_1 verify disableClone match the value set (true). {VER(>=8.0.17)}
-c.options();
+normalize_cluster_options(c.options());
 
 //@<> WL#13208: TS_FR1_1 dissolve cluster. {VER(>=8.0.17)}
 c.dissolve();
@@ -758,3 +825,82 @@ dba.createCluster('test', {disableClone: true});
 session.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
 testutil.destroySandbox(__mysql_sandbox_port2);
+
+//@<> Initialization canonical IPv6 addresses are supported WL#12758 {VER(>= 8.0.14)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: "::1"});
+shell.connect(__sandbox_uri1);
+
+//@ canonical IPv6 addresses are supported WL#12758 {VER(>= 8.0.14)}
+c = dba.createCluster("cluster");
+// Bug #30548843 Validate that IPv6 value stored on metadata for mysqlx is valid
+print_metadata_instance_addresses(session);
+
+//@<> Cleanup canonical IPv6 addresses are supported WL#12758 {VER(>= 8.0.14)}
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+
+//@<> Initialization IPv6 addresses are supported on localAddress, groupSeeds and ipWhitelist WL#12758 {VER(>= 8.0.14)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root");
+shell.connect(__sandbox_uri1);
+
+//@ IPv6 addresses are supported on localAddress, groupSeeds and ipWhitelist WL#12758 {VER(>= 8.0.14)}
+var local_address = "[::1]:" + __mysql_sandbox_gr_port1;
+var ip_white_list = "::1, 127.0.0.1";
+var group_seeds = "[::1]:" + __mysql_sandbox_gr_port1; + ", [::1]:" + __mysql_sandbox_gr_port2;
+c = dba.createCluster("cluster", {ipWhitelist:ip_white_list, groupSeeds:group_seeds, localAddress:local_address});
+
+//@<> If the target instance is >= 8.0.22, ipWhitelist is automatically used to set ipAllowlist {VER(>=8.0.22)}
+shell.connect(__sandbox_uri1);
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_allowlist"));
+
+//@<> If the target instance is >= 8.0.22, ipWhitelist is automatically used to set ipAllowlist {VER(>=8.0.14) && VER(<8.0.22)}
+shell.connect(__sandbox_uri1);
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_whitelist"));
+
+EXPECT_EQ(local_address, get_sysvar(session, "group_replication_local_address"));
+EXPECT_EQ(group_seeds, get_sysvar(session, "group_replication_group_seeds"));
+
+//@<> Dissolve the cluster {VER(>=8.0)}
+c.dissolve({force: true});
+
+//@<> Verify the new option ipAllowlist that deprecates ipWhitelist sets the allowlist {VER(>=8.0.22)}
+c = dba.createCluster("cluster", {ipAllowlist: ip_white_list});
+EXPECT_EQ(ip_white_list, get_sysvar(session, "group_replication_ip_allowlist"));
+
+//@<> Cleanup IPv6 addresses are supported on localAddress, groupSeeds and ipWhitelist WL#12758 {VER(>= 8.0.14)}
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+
+//@<> Initialization canonical IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: "::1"});
+shell.connect(__sandbox_uri1);
+
+//@ canonical IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+c = dba.createCluster("cluster");
+
+//@<> Cleanup canonical IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
+
+//@<> Initialization IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+testutil.deploySandbox(__mysql_sandbox_port1, "root");
+shell.connect(__sandbox_uri1);
+
+//@ IPv6 local_address is not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+var local_address = "[::1]:" + __mysql_sandbox_gr_port1;
+var c = dba.createCluster("cluster", {localAddress:local_address});
+
+//@ IPv6 on ipWhitelist is not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+var c = dba.createCluster("cluster", {ipWhitelist:"::1, 127.0.0.1"});
+
+//@ IPv6 on groupSeeds is not supported below 8.0.14 - 1 WL#12758 {VER(< 8.0.14)}
+var group_seeds = "127.0.0.1:" + __mysql_sandbox_gr_port1 + ", [::1]:" + __mysql_sandbox_gr_port2;
+var c = dba.createCluster("cluster", {groupSeeds:group_seeds});
+
+//@ IPv6 on groupSeeds is not supported below 8.0.14 - 2 WL#12758 {VER(< 8.0.14)}
+var group_seeds = "127.0.0.1:" + __mysql_sandbox_gr_port1 + ", [::1]:" + __mysql_sandbox_gr_port2 + " , [fe80::7e36:f49a:63c8:8ad6]:" + __mysql_sandbox_gr_port1;
+var c = dba.createCluster("cluster", {groupSeeds:group_seeds});
+
+//@<> Cleanup IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+session.close();
+testutil.destroySandbox(__mysql_sandbox_port1);
