@@ -27,7 +27,7 @@ Replication was disabled but user data was left intact.
 ||
 
 //@<OUT> Dissolve stopped because instance is unreachable and user answer 'no' to continue.
-The cluster still has the following registered ReplicaSets:
+The cluster still has the following registered instances:
 {
     "clusterName": "c",
     "defaultReplicaSet": {
@@ -36,12 +36,12 @@ The cluster still has the following registered ReplicaSets:
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
@@ -52,7 +52,7 @@ The cluster still has the following registered ReplicaSets:
         "topologyMode": "Single-Primary"
     }
 }
-WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from their ReplicaSet and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
+WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from the cluster and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
 
 Are you sure you want to dissolve the cluster? [y/N]:
 ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' cannot be removed because it is on a '(MISSING)' state. Please bring the instance back ONLINE and try to dissolve the cluster again. If the instance is permanently not reachable, then you can choose to proceed with the operation and only remove the instance from the Cluster Metadata.
@@ -64,7 +64,7 @@ Do you want to continue anyway (only the instance metadata will be removed)? [y/
 Cluster.dissolve: The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' is '(MISSING)' (RuntimeError)
 
 //@<OUT> Dissolve stopped because one instance is unreachable (force: false).
-The cluster still has the following registered ReplicaSets:
+The cluster still has the following registered instances:
 {
     "clusterName": "c",
     "defaultReplicaSet": {
@@ -73,12 +73,12 @@ The cluster still has the following registered ReplicaSets:
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
@@ -89,7 +89,7 @@ The cluster still has the following registered ReplicaSets:
         "topologyMode": "Single-Primary"
     }
 }
-WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from their ReplicaSet and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
+WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from the cluster and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
 
 Are you sure you want to dissolve the cluster? [y/N]:
 ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' cannot be removed because it is on a '(MISSING)' state. Please bring the instance back ONLINE and try to dissolve the cluster again. If the instance is permanently not reachable, then please use <Cluster>.dissolve() with the force option set to true to proceed with the operation and only remove the instance from the Cluster Metadata.
@@ -99,7 +99,7 @@ ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' cannot be remov
 Cluster.dissolve: The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' is '(MISSING)' (RuntimeError)
 
 //@<OUT> Dissolve continues because instance is unreachable and user answer 'yes' to continue.
-The cluster still has the following registered ReplicaSets:
+The cluster still has the following registered instances:
 {
     "clusterName": "c",
     "defaultReplicaSet": {
@@ -108,12 +108,12 @@ The cluster still has the following registered ReplicaSets:
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
@@ -124,12 +124,15 @@ The cluster still has the following registered ReplicaSets:
         "topologyMode": "Single-Primary"
     }
 }
-WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from their ReplicaSet and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
+WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from the cluster and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
 
 Are you sure you want to dissolve the cluster? [y/N]:
 ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' cannot be removed because it is on a '(MISSING)' state. Please bring the instance back ONLINE and try to dissolve the cluster again. If the instance is permanently not reachable, then you can choose to proceed with the operation and only remove the instance from the Cluster Metadata.
 
 Do you want to continue anyway (only the instance metadata will be removed)? [y/N]:
+* Waiting for instance to synchronize with the primary...
+* Waiting for instance to synchronize with the primary...
+* Waiting for instance to synchronize with the primary...
 Instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is attempting to leave the cluster...
 <<<(__version_num<80011)?"WARNING: On instance '"+hostname+":"+__mysql_sandbox_port2+"' configuration cannot be persisted since MySQL version "+__version+" does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please set the 'group_replication_start_on_boot' variable to 'OFF' in the server configuration file, otherwise it might rejoin the cluster upon restart.\n":""\>>>
 Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' is attempting to leave the cluster...
@@ -161,14 +164,11 @@ WARNING: The cluster was successfully dissolved, but the following instance was
 //@ Connect to seed and get cluster
 ||
 
-//@ Change shell option dba.gtidWaitTimeout to 1 second (to issue error faster)
-||
-
 //@ Execute trx that will lead to error on instance2
 ||
 
 //@<OUT> Dissolve stopped because instance cannot catch up with cluster and user answer 'n' to continue.
-The cluster still has the following registered ReplicaSets:
+The cluster still has the following registered instances:
 {
     "clusterName": "c",
     "defaultReplicaSet": {
@@ -177,33 +177,43 @@ The cluster still has the following registered ReplicaSets:
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             }
         ],
         "topologyMode": "Single-Primary"
     }
 }
-WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from their ReplicaSet and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
+WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from the cluster and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
 
 Are you sure you want to dissolve the cluster? [y/N]:
+
+//@<OUT> Dissolve stopped because instance cannot catch up with cluster and user answer 'n' to continue. {VER(<8.0.23)}
+ERROR: Applier error in replication channel 'group_replication_applier': [[*]]
 ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was unable to catch up with cluster transactions. There might be too many transactions to apply or some replication error. In the former case, you can retry the operation (using a higher timeout value by setting the global shell option 'dba.gtidWaitTimeout'). In the later case, analyze and fix any replication error. You can also choose to skip this error using the 'force: true' option, but it might leave the instance in an inconsistent state and lead to errors if you want to reuse it.
 
 Do you want to continue anyway (only the instance metadata will be removed)? [y/N]:
 
+//@<OUT> Dissolve stopped because instance cannot catch up with cluster and user answer 'n' to continue. {VER(>=8.0.23)}
+ERROR: Coordinator error in replication channel 'group_replication_applier': [[*]]
+ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was unable to catch up with cluster transactions. There might be too many transactions to apply or some replication error. In the former case, you can retry the operation (using a higher timeout value by setting the global shell option 'dba.gtidWaitTimeout'). In the later case, analyze and fix any replication error. You can also choose to skip this error using the 'force: true' option, but it might leave the instance in an inconsistent state and lead to errors if you want to reuse it.
 
-//@<ERR> Dissolve stopped because instance cannot catch up with cluster and user answer 'n' to continue.
-Cluster.dissolve: Timeout reached waiting for cluster transactions to be applied on instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' (RuntimeError)
+Do you want to continue anyway (only the instance metadata will be removed)? [y/N]:
 
+//@<ERR> Dissolve stopped because instance cannot catch up with cluster and user answer 'n' to continue. {VER(<8.0.23)}
+Cluster.dissolve: <<<hostname>>>:<<<__mysql_sandbox_port2>>>: Error found in replication applier thread (MYSQLSH 51145)
+
+//@<ERR> Dissolve stopped because instance cannot catch up with cluster and user answer 'n' to continue. {VER(>=8.0.23)}
+Cluster.dissolve: <<<hostname>>>:<<<__mysql_sandbox_port2>>>: Error found in replication coordinator thread (MYSQLSH 51144)
 
 //@ Connect to instance on port2 to fix error and add new one
 ||
@@ -215,7 +225,7 @@ Cluster.dissolve: Timeout reached waiting for cluster transactions to be applied
 ||
 
 //@<OUT> Dissolve stopped because instance cannot catch up with cluster (force: false).
-The cluster still has the following registered ReplicaSets:
+The cluster still has the following registered instances:
 {
     "clusterName": "c",
     "defaultReplicaSet": {
@@ -224,30 +234,39 @@ The cluster still has the following registered ReplicaSets:
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             }
         ],
         "topologyMode": "Single-Primary"
     }
 }
-WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from their ReplicaSet and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
+WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from the cluster and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
 
 Are you sure you want to dissolve the cluster? [y/N]:
+
+//@<OUT> Dissolve stopped because instance cannot catch up with cluster (force: false). {VER(<8.0.23)}
+ERROR: Applier error in replication channel 'group_replication_applier': Error [[*]]
 ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was unable to catch up with cluster transactions. There might be too many transactions to apply or some replication error. In the former case, you can retry the operation (using a higher timeout value by setting the global shell option 'dba.gtidWaitTimeout'). In the later case, analyze and fix any replication error. You can also choose to skip this error using the 'force: true' option, but it might leave the instance in an inconsistent state and lead to errors if you want to reuse it.
 
+//@<OUT> Dissolve stopped because instance cannot catch up with cluster (force: false). {VER(>=8.0.23)}
+ERROR: Coordinator error in replication channel 'group_replication_applier': [[*]]
+ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was unable to catch up with cluster transactions. There might be too many transactions to apply or some replication error. In the former case, you can retry the operation (using a higher timeout value by setting the global shell option 'dba.gtidWaitTimeout'). In the later case, analyze and fix any replication error. You can also choose to skip this error using the 'force: true' option, but it might leave the instance in an inconsistent state and lead to errors if you want to reuse it.
 
-//@<ERR> Dissolve stopped because instance cannot catch up with cluster (force: false).
-Cluster.dissolve: Timeout reached waiting for cluster transactions to be applied on instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' (RuntimeError)
+//@<ERR> Dissolve stopped because instance cannot catch up with cluster (force: false). {VER(<8.0.23)}
+Cluster.dissolve: <<<hostname>>>:<<<__mysql_sandbox_port2>>>: Error found in replication applier thread (MYSQLSH 51145)
+
+//@<ERR> Dissolve stopped because instance cannot catch up with cluster (force: false). {VER(>=8.0.23)}
+Cluster.dissolve: <<<hostname>>>:<<<__mysql_sandbox_port2>>>: Error found in replication coordinator thread (MYSQLSH 51144)
 
 //@ Connect to instance on port2 to fix error and add new one, one last time
 ||
@@ -259,7 +278,7 @@ Cluster.dissolve: Timeout reached waiting for cluster transactions to be applied
 ||
 
 //@<OUT> Dissolve continues because instance cannot catch up with cluster and user answer 'y' to continue.
-The cluster still has the following registered ReplicaSets:
+The cluster still has the following registered instances:
 {
     "clusterName": "c",
     "defaultReplicaSet": {
@@ -268,32 +287,44 @@ The cluster still has the following registered ReplicaSets:
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
-                "role": "HA"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "role": "HA"
             }
         ],
         "topologyMode": "Single-Primary"
     }
 }
-WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from their ReplicaSet and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
+WARNING: You are about to dissolve the whole cluster and lose the high availability features provided by it. This operation cannot be reverted. All members will be removed from the cluster and replication will be stopped, internal recovery user accounts and the cluster metadata will be dropped. User data will be maintained intact in all instances.
 
 Are you sure you want to dissolve the cluster? [y/N]:
+
+//@<OUT> Dissolve continues because instance cannot catch up with cluster and user answer 'y' to continue. {VER(<8.0.23)}
+ERROR: Applier error in replication channel 'group_replication_applier': Error [[*]]
 ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was unable to catch up with cluster transactions. There might be too many transactions to apply or some replication error. In the former case, you can retry the operation (using a higher timeout value by setting the global shell option 'dba.gtidWaitTimeout'). In the later case, analyze and fix any replication error. You can also choose to skip this error using the 'force: true' option, but it might leave the instance in an inconsistent state and lead to errors if you want to reuse it.
 
 Do you want to continue anyway (only the instance metadata will be removed)? [y/N]:
+
+//@<OUT> Dissolve continues because instance cannot catch up with cluster and user answer 'y' to continue. {VER(>=8.0.23)}
+ERROR: Coordinator error in replication channel 'group_replication_applier': [[*]]
+ERROR: The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was unable to catch up with cluster transactions. There might be too many transactions to apply or some replication error. In the former case, you can retry the operation (using a higher timeout value by setting the global shell option 'dba.gtidWaitTimeout'). In the later case, analyze and fix any replication error. You can also choose to skip this error using the 'force: true' option, but it might leave the instance in an inconsistent state and lead to errors if you want to reuse it.
+
+Do you want to continue anyway (only the instance metadata will be removed)? [y/N]:
+
+//@<OUT> Dissolve continues because instance cannot catch up with cluster and user answer 'y' to continue.
 WARNING: An error occurred when trying to catch up with cluster transactions and instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' might have been left in an inconsistent state that will lead to errors if it is reused.
 
 Instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is attempting to leave the cluster...
 <<<(__version_num<80011)?"WARNING: On instance '"+hostname+":"+__mysql_sandbox_port2+"' configuration cannot be persisted since MySQL version "+__version+" does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please set the 'group_replication_start_on_boot' variable to 'OFF' in the server configuration file, otherwise it might rejoin the cluster upon restart.\n":""\>>>
+* Waiting for instance to synchronize with the primary...
 Instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' is attempting to leave the cluster...
 <<<(__version_num<80011)?"WARNING: On instance '"+hostname+":"+__mysql_sandbox_port3+"' configuration cannot be persisted since MySQL version "+__version+" does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please set the 'group_replication_start_on_boot' variable to 'OFF' in the server configuration file, otherwise it might rejoin the cluster upon restart.\n":""\>>>
 Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' is attempting to leave the cluster...
