@@ -16,17 +16,21 @@
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<(__version_num>=80011) ?  "\n                \"replicationLag\": [[*]],":"">>>
                 "role": "HA",
-                "status": "ONLINE"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<(__version_num>=80011) ?  "\n                \"replicationLag\": [[*]],":"">>>
                 "role": "HA",
-                "status": "ONLINE"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -46,17 +50,21 @@
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<(__version_num>=80011) ?  "\n                \"replicationLag\": [[*]],":"">>>
                 "role": "HA",
-                "status": "ONLINE"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<(__version_num>=80011) ?  "\n                \"replicationLag\": [[*]],":"">>>
                 "role": "HA",
-                "status": "ONLINE"<<<(__version_num>=80011)?",\n[[*]]\"version\": \"" + __version + "\"":"">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -65,7 +73,7 @@
 }
 
 //@# disconnect the cluster object
-||The cluster object is disconnected. Please use <Dba>.getCluster to obtain a fresh cluster handle.
+||The cluster object is disconnected. Please use dba.getCluster to obtain a fresh cluster handle.
 
 //@ SP - getCluster() on primary
 |TCP port:                     <<<__mysql_sandbox_port1>>>|
@@ -131,103 +139,103 @@
 
 //@ SP - Connect with --redirect-primary while connected to the primary
 |NOTE: --redirect-primary ignored because target is already a PRIMARY|
-|Session type:                 Classic|
+|Protocol version:             Classic 10|
 |TCP port:                     <<<__mysql_sandbox_port1>>>|
 
 //@ SP - Connect with --redirect-primary while connected to a secondary
-|Reconnecting to PRIMARY instance of the InnoDB cluster (mysql://<<<hostname>>>:<<<__mysql_sandbox_port1>>>)...|
-|Session type:                 Classic|
+|Reconnecting to the PRIMARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port1>>>)...|
+|Protocol version:             Classic 10|
 |TCP port:                     <<<__mysql_sandbox_port1>>>|
 
 //@ SP - Connect with --redirect-primary while connected to a non-cluster member (error)
 |While handling --redirect-primary:|
-|InnoDB cluster error 10001: Cluster metadata not found: Cluster metadata schema not found|
+|Metadata schema of an InnoDB cluster or ReplicaSet not found|
 
 //@ SP - Connect with --redirect-secondary while connected to the primary
-|Reconnecting to SECONDARY instance of the InnoDB cluster (mysql://<<<hostname>>>:<<<__mysql_sandbox_port2>>>)...|
-|Session type:                 Classic|
+|Reconnecting to the SECONDARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port2>>>)...|
+|Protocol version:             Classic 10|
 |TCP port:                     <<<__mysql_sandbox_port2>>>|
 
 //@ SP - Connect with --redirect-secondary while connected to a secondary
 |NOTE: --redirect-secondary ignored because target is already a SECONDARY|
-|Session type:                 Classic|
+|Protocol version:             Classic 10|
 |TCP port:                     <<<__mysql_sandbox_port2>>>|
 
 //@ SP - Connect with --redirect-secondary while connected to a non-cluster member (error)
 |While handling --redirect-secondary:|
-|InnoDB cluster error 10001: Cluster metadata not found: Cluster metadata schema not found|
+|Metadata schema of an InnoDB cluster or ReplicaSet not found|
 
 //@ SPX - Connect with no options and ensure it will connect to the specified member
-|Session type:                 X|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port1>>>0|
 
 //@ SPX - Connect with no options and ensure it will connect to the specified member 2
-|Session type:                 X|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port2>>>0|
 
 //@ SPX - Connect with --redirect-primary while connected to the primary
 |NOTE: --redirect-primary ignored because target is already a PRIMARY|
-|Session type:                 X|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port1>>>0|
 
 //@ SPX - Connect with --redirect-primary while connected to a secondary
-|Reconnecting to PRIMARY instance of the InnoDB cluster (mysqlx://<<<hostname>>>:<<<__mysql_sandbox_port1>>>0)...|
-|Session type:                 X|
+|Reconnecting to the PRIMARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port1>>>0)...|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port1>>>0|
 
 //@ SPX - Connect with --redirect-primary while connected to a non-cluster member (error)
 |While handling --redirect-primary:|
-|InnoDB cluster error 10001: Cluster metadata not found: Cluster metadata schema not found|
+|Metadata schema of an InnoDB cluster or ReplicaSet not found|
 
 //@ SPX - Connect with --redirect-secondary while connected to the primary
-|Reconnecting to SECONDARY instance of the InnoDB cluster (mysqlx://<<<hostname>>>:<<<__mysql_sandbox_port2>>>0)...|
-|Session type:                 X|
+|Reconnecting to the SECONDARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port2>>>0)...|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port2>>>0|
 
 //@ SPX - Connect with --redirect-secondary while connected to a secondary
 |NOTE: --redirect-secondary ignored because target is already a SECONDARY|
-|Session type:                 X|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port2>>>0|
 
 //@ SPX - Connect with --redirect-secondary while connected to a non-cluster member (error)
 |While handling --redirect-secondary:|
-|InnoDB cluster error 10001: Cluster metadata not found: Cluster metadata schema not found|
+|Metadata schema of an InnoDB cluster or ReplicaSet not found|
 
 //@ SPX implicit - Connect with no options and ensure it will connect to the specified member
-|Session type:                 X|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port1>>>0|
 
 //@ SPX implicit - Connect with no options and ensure it will connect to the specified member 2
-|Session type:                 X|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port2>>>0|
 
 //@ SPX implicit - Connect with --redirect-primary while connected to the primary
 |NOTE: --redirect-primary ignored because target is already a PRIMARY|
-|Session type:                 X|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port1>>>0|
 
 //@ SPX implicit - Connect with --redirect-primary while connected to a secondary
-|Reconnecting to PRIMARY instance of the InnoDB cluster (mysqlx://<<<hostname>>>:<<<__mysql_sandbox_port1>>>0)...|
-|Session type:                 X|
+|Reconnecting to the PRIMARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port1>>>0)...|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port1>>>0|
 
 //@ SPX implicit - Connect with --redirect-primary while connected to a non-cluster member (error)
 |While handling --redirect-primary:|
-|InnoDB cluster error 10001: Cluster metadata not found: Cluster metadata schema not found|
+|Metadata schema of an InnoDB cluster or ReplicaSet not found|
 
 //@ SPX implicit - Connect with --redirect-secondary while connected to the primary
-|Reconnecting to SECONDARY instance of the InnoDB cluster (mysqlx://<<<hostname>>>:<<<__mysql_sandbox_port2>>>0)...|
-|Session type:                 X|
+|Reconnecting to the SECONDARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port2>>>0)...|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port2>>>0|
 
 //@ SPX implicit - Connect with --redirect-secondary while connected to a secondary
 |NOTE: --redirect-secondary ignored because target is already a SECONDARY|
-|Session type:                 X|
+|Protocol version:             X protocol|
 |TCP port:                     <<<__mysql_sandbox_port2>>>0|
 
 //@ SPX implicit - Connect with --redirect-secondary while connected to a non-cluster member (error)
 |While handling --redirect-secondary:|
-|InnoDB cluster error 10001: Cluster metadata not found: Cluster metadata schema not found|
+|Metadata schema of an InnoDB cluster or ReplicaSet not found|
 
 //@ SP - Connect with --cluster 1
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
@@ -239,33 +247,72 @@
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SP - Connect with --cluster on a non-cluster member + cmd (error)
-|Option --cluster requires a session to a member of a InnoDB cluster.|
-|ERROR: RuntimeError: Dba.getCluster: This function is not available through a session to a standalone instance|
+|Option --cluster requires a session to a member of an InnoDB cluster.|
+|ERROR: MYSQLSH 51300: This function is not available through a session to a standalone instance|
 
 //@ SP - Connect with --cluster on a non-cluster member interactive (error)
-|Option --cluster requires a session to a member of a InnoDB cluster.|
-|ERROR: RuntimeError: Dba.getCluster: This function is not available through a session to a standalone instance|
+|Option --cluster requires a session to a member of an InnoDB cluster.|
+|ERROR: MYSQLSH 51300: This function is not available through a session to a standalone instance|
 
 //@ SP - Connect with --cluster on a non-cluster member (error)
-|Option --cluster requires a session to a member of a InnoDB cluster.|
-|ERROR: RuntimeError: Dba.getCluster: This function is not available through a session to a standalone instance|
+|Option --cluster requires a session to a member of an InnoDB cluster.|
+|ERROR: MYSQLSH 51300: This function is not available through a session to a standalone instance|
+
+//@ SP - Connect with --replicaset, expect error {VER(>8.0.0)}
+|Option --replicaset requires a session to a member of an InnoDB ReplicaSet.|
+|This function is not available through a session to an instance already in an InnoDB Cluster|
+
+//@ SP - Connect with --replicaset, expect error {VER(<8.0.0)}
+|Unsupported server version: This AdminAPI operation requires MySQL version 8.0 or newer, but target is |
 
 //@ SP - Connect with --cluster + --redirect-primary 1
 |NOTE: --redirect-primary ignored because target is already a PRIMARY|
 |    "groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SP - Connect with --cluster + --redirect-primary 2
-|Reconnecting to PRIMARY instance of the InnoDB cluster (mysql://<<<hostname>>>:<<<__mysql_sandbox_port1>>>)...|
+|Reconnecting to the PRIMARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port1>>>)...|
 |    "groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SP - Connect with --cluster + --redirect-secondary 1
-|Reconnecting to SECONDARY instance of the InnoDB cluster (mysql://<<<hostname>>>:<<<__mysql_sandbox_port2>>>)...|
+|Reconnecting to the SECONDARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port2>>>)...|
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SP - Connect with --cluster + --redirect-secondary 2
 |NOTE: --redirect-secondary ignored because target is already a SECONDARY|
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
+//@ SP - Connect with --replicaset + --redirect-primary 1, expect error {VER(>8.0.0)}
+|NOTE: --redirect-primary ignored because target is already a PRIMARY|
+|Option --replicaset requires a session to a member of an InnoDB ReplicaSet.|
+|This function is not available through a session to an instance already in an InnoDB Cluster|
+
+//@ SP - Connect with --replicaset + --redirect-primary 1, expect error {VER(<8.0.0)}
+|Unsupported server version: This AdminAPI operation requires MySQL version 8.0 or newer, but target is |
+
+//@ SP - Connect with --replicaset + --redirect-primary 2, expect error {VER(>8.0.0)}
+|Reconnecting to the PRIMARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port1>>>)...|
+|Option --replicaset requires a session to a member of an InnoDB ReplicaSet.|
+|This function is not available through a session to an instance already in an InnoDB Cluster|
+
+//@ SP - Connect with --replicaset + --redirect-primary 2, expect error {VER(<8.0.0)}
+|Unsupported server version: This AdminAPI operation requires MySQL version 8.0 or newer, but target is |
+
+//@ SP - Connect with --replicaset + --redirect-secondary 1, expect error {VER(>8.0.0)}
+|Reconnecting to the SECONDARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port2>>>)...|
+|Option --replicaset requires a session to a member of an InnoDB ReplicaSet.|
+|This function is not available through a session to an instance already in an InnoDB Cluster|
+
+//@ SP - Connect with --replicaset + --redirect-secondary 1, expect error {VER(<8.0.0)}
+|Unsupported server version: This AdminAPI operation requires MySQL version 8.0 or newer, but target is |
+
+//@ SP - Connect with --replicaset + --redirect-secondary 2, expect error {VER(>8.0.0)}
+|NOTE: --redirect-secondary ignored because target is already a SECONDARY|
+|Option --replicaset requires a session to a member of an InnoDB ReplicaSet.|
+|This function is not available through a session to an instance already in an InnoDB Cluster|
+
+//@ SP - Connect with --replicaset + --redirect-secondary 2, expect error {VER(<8.0.0)}
+|Unsupported server version: This AdminAPI operation requires MySQL version 8.0 or newer, but target is |
+
 //@ SPX - Connect with --cluster 1
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
@@ -276,25 +323,41 @@
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SPX - Connect with --cluster on a non-cluster member (error)
-|Option --cluster requires a session to a member of a InnoDB cluster.|
-|ERROR: RuntimeError: Dba.getCluster: This function is not available through a session to a standalone instance|
+|Option --cluster requires a session to a member of an InnoDB cluster.|
+|ERROR: MYSQLSH 51300: This function is not available through a session to a standalone instance|
 
 //@ SPX - Connect with --cluster + --redirect-primary 1
 |NOTE: --redirect-primary ignored because target is already a PRIMARY|
 |    "groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SPX - Connect with --cluster + --redirect-primary 2
-|Reconnecting to PRIMARY instance of the InnoDB cluster (mysqlx://<<<hostname>>>:<<<__mysql_sandbox_port1>>>0)...|
+|Reconnecting to the PRIMARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port1>>>0)...|
 |    "groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SPX - Connect with --cluster + --redirect-secondary 1
-|Reconnecting to SECONDARY instance of the InnoDB cluster (mysqlx://<<<hostname>>>:<<<__mysql_sandbox_port2>>>0)...|
+|Reconnecting to the SECONDARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port2>>>0)...|
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SPX - Connect with --cluster + --redirect-secondary 2
 |NOTE: --redirect-secondary ignored because target is already a SECONDARY|
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
+//@ SPX - Connect with --replicaset + --redirect-primary 2, expect error {VER(>8.0.0)}
+|Reconnecting to the PRIMARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_x_port1>>>)...|
+|Option --replicaset requires a session to a member of an InnoDB ReplicaSet.|
+|This function is not available through a session to an instance already in an InnoDB Cluster|
+
+//@ SPX - Connect with --replicaset + --redirect-primary 2, expect error {VER(<8.0.0)}
+|Unsupported server version: This AdminAPI operation requires MySQL version 8.0 or newer, but target is |
+
+//@ SPX - Connect with --replicaset + --redirect-secondary 1, expect error {VER(>8.0.0)}
+|Reconnecting to the SECONDARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_x_port2>>>)...|
+|Option --replicaset requires a session to a member of an InnoDB ReplicaSet.|
+|This function is not available through a session to an instance already in an InnoDB Cluster|
+
+//@ SPX - Connect with --replicaset + --redirect-secondary 1, expect error {VER(<8.0.0)}
+|Unsupported server version: This AdminAPI operation requires MySQL version 8.0 or newer, but target is |
+
 //@ SPX implicit - Connect with --cluster 1
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
@@ -305,19 +368,19 @@
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SPX implicit - Connect with --cluster on a non-cluster member (error)
-|Option --cluster requires a session to a member of a InnoDB cluster.|
-|ERROR: RuntimeError: Dba.getCluster: This function is not available through a session to a standalone instance|
+|Option --cluster requires a session to a member of an InnoDB cluster.|
+|ERROR: MYSQLSH 51300: This function is not available through a session to a standalone instance|
 
 //@ SPX implicit - Connect with --cluster + --redirect-primary 1
 |NOTE: --redirect-primary ignored because target is already a PRIMARY|
 |    "groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SPX implicit - Connect with --cluster + --redirect-primary 2
-|Reconnecting to PRIMARY instance of the InnoDB cluster (mysqlx://<<<hostname>>>:<<<__mysql_sandbox_port1>>>0)...|
+|Reconnecting to the PRIMARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port1>>>0)...|
 |    "groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SPX implicit - Connect with --cluster + --redirect-secondary 1
-|Reconnecting to SECONDARY instance of the InnoDB cluster (mysqlx://<<<hostname>>>:<<<__mysql_sandbox_port2>>>0)...|
+|Reconnecting to the SECONDARY instance of an InnoDB cluster (<<<hostname>>>:<<<__mysql_sandbox_port2>>>0)...|
 |"groupInformationSourceMember": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>"|
 
 //@ SPX implicit - Connect with --cluster + --redirect-secondary 2
