@@ -1,8 +1,6 @@
-//@ WL#12011: Initialization
-||
 
 //@<ERR> WL#12011: FR2-04 - invalid value for interactive option.
-Dba.createCluster: Option 'interactive' Bool expected, but value is String (TypeError)
+Dba.createCluster: Argument #2: Option 'interactive' Bool expected, but value is String (TypeError)
 
 //@<OUT> WL#12011: FR2-01 - interactive = true.
 A new InnoDB cluster will be created on instance 'localhost:<<<__mysql_sandbox_port1>>>'.
@@ -15,6 +13,7 @@ proceeding.
 
 I have read the MySQL InnoDB cluster manual and I understand the requirements
 and limitations of advanced Multi-Primary Mode.
+
 Confirm [y/N]:
 
 //@<ERR> WL#12011: FR2-01 - interactive = true.
@@ -23,15 +22,17 @@ Dba.createCluster: Cancelled (RuntimeError)
 //@<OUT> WL#12011: FR2-03 - no interactive option (default: non-interactive).
 A new InnoDB cluster will be created on instance 'localhost:<<<__mysql_sandbox_port1>>>'.
 
-Validating instance at localhost:<<<__mysql_sandbox_port1>>>...
+Validating instance configuration at localhost:<<<__mysql_sandbox_port1>>>...
 NOTE: Instance detected as a sandbox.
 Please note that sandbox instances are only suitable for deploying test clusters for use within the same host.
 
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port1>>>
 
 Instance configuration is suitable.
-<<<(__version_num<80011)?"WARNING: Instance '"+localhost+":"+__mysql_sandbox_port1+"' cannot persist Group Replication configuration since MySQL version "+__version+" does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the <Dba>.configureLocalInstance() command locally to persist the changes.\n":""\>>>
-Creating InnoDB cluster 'test' on 'localhost:<<<__mysql_sandbox_port1>>>'...
+NOTE: Group Replication will communicate with other members using '<<<hostname>>>:<<<__mysql_sandbox_gr_port1>>>'. Use the localAddress option to override.
+
+<<<(__version_num<80011)?"WARNING: Instance '"+hostname+":"+__mysql_sandbox_port1+"' cannot persist Group Replication configuration since MySQL version "+__version+" does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the dba.configureLocalInstance() command locally to persist the changes.\n":""\>>>
+Creating InnoDB cluster 'test' on '<<<hostname>>>:<<<__mysql_sandbox_port1>>>'...
 
 Adding Seed Instance...
 Cluster successfully created. Use Cluster.addInstance() to add MySQL instances.
@@ -50,9 +51,9 @@ one server failure.
 //@ WL#12049: Create cluster errors using exitStateAction option {VER(>=5.7.24)}
 ||Invalid value for exitStateAction, string value cannot be empty.
 ||Invalid value for exitStateAction, string value cannot be empty.
-||Unable to set value ':' for 'exitStateAction': localhost:<<<__mysql_sandbox_port1>>>: Variable 'group_replication_exit_state_action' can't be set to the value of ':'
-||Unable to set value 'AB' for 'exitStateAction': localhost:<<<__mysql_sandbox_port1>>>: Variable 'group_replication_exit_state_action' can't be set to the value of 'AB'
-||Unable to set value '10' for 'exitStateAction': localhost:<<<__mysql_sandbox_port1>>>: Variable 'group_replication_exit_state_action' can't be set to the value of '10'
+||Unable to set value ':' for 'exitStateAction': Variable 'group_replication_exit_state_action' can't be set to the value of ':'
+||Unable to set value 'AB' for 'exitStateAction': Variable 'group_replication_exit_state_action' can't be set to the value of 'AB'
+||Unable to set value '10' for 'exitStateAction': Variable 'group_replication_exit_state_action' can't be set to the value of '10'
 
 //@ WL#12049: Create cluster specifying a valid value for exitStateAction (ABORT_SERVER) {VER(>=5.7.24)}
 ||
@@ -83,8 +84,8 @@ one server failure.
 
 //@<OUT> WL#12049: exitStateAction must be persisted on mysql >= 8.0.12 {VER(>=8.0.12)}
 group_replication_bootstrap_group = OFF
+group_replication_enforce_update_everywhere_checks = OFF
 group_replication_exit_state_action = READ_ONLY
-group_replication_force_members =
 group_replication_group_name = ca94447b-e6fc-11e7-b69d-4485005154dc
 group_replication_group_seeds =
 group_replication_local_address = <<<hostname>>>:<<<__mysql_sandbox_gr_port1>>>
@@ -102,7 +103,7 @@ group_replication_start_on_boot = ON
 //@ WL#12049: Create cluster 2 {VER(>=8.0.12)}
 ||
 
-//@<OUT> BUG#28701263: DEFAULT VALUE OF EXITSTATEACTION TOO DRASTIC {VER(>=8.0.12)}
+//@<OUT> BUG#28701263: DEFAULT VALUE OF EXITSTATEACTION TOO DRASTIC
 group_replication_exit_state_action = READ_ONLY
 group_replication_group_name = ca94447b-e6fc-11e7-b69d-4485005154dc
 group_replication_local_address = <<<hostname>>>:<<<__mysql_sandbox_gr_port1>>>
@@ -150,8 +151,10 @@ group_replication_start_on_boot = ON
 
 //@<OUT> WL#11032: memberWeight must be persisted on mysql >= 8.0.11 {VER(>=8.0.12)}
 group_replication_bootstrap_group = OFF
+group_replication_enforce_update_everywhere_checks = OFF
+?{VER(<8.0.16)}
 group_replication_exit_state_action = READ_ONLY
-group_replication_force_members =
+?{}
 group_replication_group_name = ca94447b-e6fc-11e7-b69d-4485005154dc
 group_replication_group_seeds =
 group_replication_local_address = <<<hostname>>>:<<<__mysql_sandbox_gr_port1>>>
@@ -171,7 +174,9 @@ group_replication_start_on_boot = ON
 ||
 
 //@<OUT> WL#11032: memberWeight must not be persisted on mysql >= 8.0.11 if not set {VER(>=8.0.12)}
+?{VER(<8.0.16)}
 group_replication_exit_state_action = READ_ONLY
+?{}
 group_replication_group_name = ca94447b-e6fc-11e7-b69d-4485005154dc
 group_replication_local_address = <<<hostname>>>:<<<__mysql_sandbox_gr_port1>>>
 group_replication_recovery_use_ssl = ON
@@ -189,11 +194,11 @@ group_replication_start_on_boot = ON
 ||Option 'consistency' not supported on target server version: '<<<__version>>>'
 
 //@ WL#12067: Create cluster errors using consistency option {VER(>=8.0.14)}
-||Invalid value for consistency, string value cannot be empty.
-||Invalid value for consistency, string value cannot be empty.
-||Unable to set value ':' for 'consistency': localhost:<<<__mysql_sandbox_port1>>>: Variable 'group_replication_consistency' can't be set to the value of ':'
-||Unable to set value 'AB' for 'consistency': localhost:<<<__mysql_sandbox_port1>>>: Variable 'group_replication_consistency' can't be set to the value of 'AB'
-||Unable to set value '10' for 'consistency': localhost:<<<__mysql_sandbox_port1>>>: Variable 'group_replication_consistency' can't be set to the value of '10'
+||Invalid value for 'consistency', string value cannot be empty.
+||Invalid value for 'consistency', string value cannot be empty.
+||Unable to set value ':' for 'consistency': Variable 'group_replication_consistency' can't be set to the value of ':'
+||Unable to set value 'AB' for 'consistency': Variable 'group_replication_consistency' can't be set to the value of 'AB'
+||Unable to set value '10' for 'consistency': Variable 'group_replication_consistency' can't be set to the value of '10'
 ||Option 'consistency' is expected to be of type String, but is Integer (TypeError)
 ||Cannot use the failoverConsistency and consistency options simultaneously. The failoverConsistency option is deprecated, please use the consistency option instead. (ArgumentError)
 
@@ -283,7 +288,7 @@ group_replication_start_on_boot = ON
 ||
 
 //@ BUG#25867733: createCluster error with performance_schema=off
-|ERROR: Instance 'localhost:<<<__mysql_sandbox_port1>>>' has the performance_schema disabled (performance_schema=OFF). Instances must have the performance_schema enabled to for InnoDB Cluster usage.|performance_schema disabled on target instance. (RuntimeError)
+|ERROR: Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' has the performance_schema disabled (performance_schema=OFF). Instances must have the performance_schema enabled to for InnoDB Cluster usage.|performance_schema disabled on target instance. (RuntimeError)
 
 //@ BUG#25867733: createCluster no error with performance_schema=on
 ||
@@ -299,27 +304,27 @@ group_replication_start_on_boot = ON
 
 //@<OUT> BUG#29246110: check instance error with non supported host.
 This instance reports its own address as 127.0.1.1:<<<__mysql_sandbox_port1>>>
-ERROR: Cannot use host '127.0.1.1' for instance 'localhost:<<<__mysql_sandbox_port1>>>' because it resolves to an IP address (127.0.1.1) that does not match a real network interface, thus it is not supported by the Group Replication communication layer. Change your system settings and/or set the MySQL server 'report_host' variable to a hostname that resolves to a supported IP address.
+ERROR: Cannot use host '127.0.1.1' for instance '127.0.1.1:<<<__mysql_sandbox_port1>>>' because it resolves to an IP address (127.0.1.1) that does not match a real network interface, thus it is not supported. Change your system settings and/or set the MySQL server 'report_host' variable to a hostname that resolves to a supported IP address.
 
 //@<ERR> BUG#29246110: check instance error with non supported host.
-Dba.checkInstanceConfiguration: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which is not supported by Group Replication. (RuntimeError)
+Dba.checkInstanceConfiguration: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which is not supported. (RuntimeError)
 
 //@<OUT> BUG#29246110: createCluster error with non supported host.
 This instance reports its own address as 127.0.1.1:<<<__mysql_sandbox_port1>>>
-ERROR: Cannot use host '127.0.1.1' for instance 'localhost:<<<__mysql_sandbox_port1>>>' because it resolves to an IP address (127.0.1.1) that does not match a real network interface, thus it is not supported by the Group Replication communication layer. Change your system settings and/or set the MySQL server 'report_host' variable to a hostname that resolves to a supported IP address.
+ERROR: Cannot use host '127.0.1.1' for instance '127.0.1.1:<<<__mysql_sandbox_port1>>>' because it resolves to an IP address (127.0.1.1) that does not match a real network interface, thus it is not supported. Change your system settings and/or set the MySQL server 'report_host' variable to a hostname that resolves to a supported IP address.
 
 //@<ERR> BUG#29246110: createCluster error with non supported host.
-Dba.createCluster: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which is not supported by Group Replication. (RuntimeError)
+Dba.createCluster: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which is not supported. (RuntimeError)
 
 //@ BUG#29246110: create cluster succeed with supported host.
 ||
 
 //@<OUT> BUG#29246110: add instance error with non supported host.
 This instance reports its own address as 127.0.1.1:<<<__mysql_sandbox_port1>>>
-ERROR: Cannot use host '127.0.1.1' for instance 'localhost:<<<__mysql_sandbox_port1>>>' because it resolves to an IP address (127.0.1.1) that does not match a real network interface, thus it is not supported by the Group Replication communication layer. Change your system settings and/or set the MySQL server 'report_host' variable to a hostname that resolves to a supported IP address.
+ERROR: Cannot use host '127.0.1.1' for instance '127.0.1.1:<<<__mysql_sandbox_port1>>>' because it resolves to an IP address (127.0.1.1) that does not match a real network interface, thus it is not supported. Change your system settings and/or set the MySQL server 'report_host' variable to a hostname that resolves to a supported IP address.
 
 //@<ERR> BUG#29246110: add instance error with non supported host.
-Cluster.addInstance: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which is not supported by Group Replication. (RuntimeError)
+Cluster.addInstance: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which is not supported. (RuntimeError)
 
 //@ BUG#29246110: finalization
 ||
@@ -328,10 +333,10 @@ Cluster.addInstance: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which i
 ||
 
 //@ WL#12066: TSF1_4 Validate that an exception is thrown if the value specified is not an unsigned integer. {VER(>=8.0.16)}
-||Dba.createCluster: Unable to set value '-1' for 'autoRejoinTries': localhost:<<<__mysql_sandbox_port1>>>: Variable 'group_replication_autorejoin_tries' can't be set to the value of '-1' (RuntimeError)
+||Unable to set value '-1' for 'autoRejoinTries': Variable 'group_replication_autorejoin_tries' can't be set to the value of '-1' (RuntimeError)
 
 //@ WL#12066: TSF1_5 Validate that an exception is thrown if the value  is not in the range 0 to 2016. {VER(>=8.0.16)}
-||Dba.createCluster: Unable to set value '2017' for 'autoRejoinTries': localhost:<<<__mysql_sandbox_port1>>>: Variable 'group_replication_autorejoin_tries' can't be set to the value of '2017' (RuntimeError)
+||Unable to set value '2017' for 'autoRejoinTries': Variable 'group_replication_autorejoin_tries' can't be set to the value of '2017' (RuntimeError)
 
 //@ WL#12066: TSF1_1 Validate that the functions [dba.]createCluster() and [cluster.]addInstance() support a new option named autoRejoinTries. {VER(>=8.0.16)}
 |WARNING: The member will only proceed according to its exitStateAction if auto-rejoin fails (i.e. all retry attempts are exhausted).|
@@ -351,8 +356,11 @@ Cluster.addInstance: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which i
 //@ BUG#29305551: Initialization
 ||
 
-//@ Create cluster async replication OK
-||
+//@<OUT> Create cluster async replication (should fail)
+ERROR: Cannot create cluster on instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' because it has asynchronous (source-replica) replication channel(s) configured. MySQL InnoDB Cluster does not support manually configured channels as they are not managed using the AdminAPI (e.g. when PRIMARY moves to another member) which may cause cause replication to break or even create split-brain scenarios (data loss). Use the 'force' option to skip this validation on a temporary scenario (e.g. migrating from a replication topology to InnoDB Cluster).
+
+//@<ERR> Create cluster async replication (should fail)
+Dba.createCluster: The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' has asynchronous replication configured. (RuntimeError)
 
 //@ BUG#29305551: Finalization
 ||
@@ -364,15 +372,17 @@ Cluster.addInstance: Invalid host/IP '127.0.1.1' resolves to '127.0.1.1' which i
 //@<OUT> BUG#29361352: no warning or prompt for multi-primary (interactive: true, multiPrimary: false).
 A new InnoDB cluster will be created on instance 'localhost:<<<__mysql_sandbox_port1>>>'.
 
-Validating instance at localhost:<<<__mysql_sandbox_port1>>>...
+Validating instance configuration at localhost:<<<__mysql_sandbox_port1>>>...
 NOTE: Instance detected as a sandbox.
 Please note that sandbox instances are only suitable for deploying test clusters for use within the same host.
 
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port1>>>
 
 Instance configuration is suitable.
-<<<(__version_num<80011)?"WARNING: Instance '"+localhost+":"+__mysql_sandbox_port1+"' cannot persist Group Replication configuration since MySQL version "+__version+" does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the <Dba>.configureLocalInstance() command locally to persist the changes.\n":""\>>>
-Creating InnoDB cluster 'test' on 'localhost:<<<__mysql_sandbox_port1>>>'...
+NOTE: Group Replication will communicate with other members using '<<<hostname>>>:<<<__mysql_sandbox_gr_port1>>>'. Use the localAddress option to override.
+
+<<<(__version_num<80011)?"WARNING: Instance '"+hostname+":"+__mysql_sandbox_port1+"' cannot persist Group Replication configuration since MySQL version "+__version+" does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the dba.configureLocalInstance() command locally to persist the changes.\n":""\>>>
+Creating InnoDB cluster 'test' on '<<<hostname>>>:<<<__mysql_sandbox_port1>>>'...
 
 Adding Seed Instance...
 Cluster successfully created. Use Cluster.addInstance() to add MySQL instances.
@@ -398,10 +408,10 @@ one server failure.
 |mysql_innodb_cluster_11111, %|
 
 //@ WL#13208: TS_FR1_2 validate errors for disableClone (only boolean values).
-||Dba.createCluster: Option 'disableClone' Bool expected, but value is String (TypeError)
-||Dba.createCluster: Option 'disableClone' Bool expected, but value is String (TypeError)
-||Dba.createCluster: Option 'disableClone' is expected to be of type Bool, but is Array (TypeError)
-||Dba.createCluster: Option 'disableClone' is expected to be of type Bool, but is Map (TypeError)
+||Option 'disableClone' Bool expected, but value is String (TypeError)
+||Option 'disableClone' Bool expected, but value is String (TypeError)
+||Option 'disableClone' is expected to be of type Bool, but is Array (TypeError)
+||Option 'disableClone' is expected to be of type Bool, but is Map (TypeError)
 
 //@ WL#13208: TS_FR1_3 validate default for disableClone is false.
 ||
@@ -426,8 +436,19 @@ one server failure.
                 "value": <<<(__version_num>=80017)?"false":"true">>>
             }
         ],
+        "tags": {
+            ".global": [],
+            "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": []
+        },
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": [
+?{VER(>=8.0.16)}
+                {
+                    "option": "autoRejoinTries",
+                    "value": "<<<__default_gr_auto_rejoin_tries>>>",
+                    "variable": "group_replication_autorejoin_tries"
+                },
+?{}
                 {
                     "option": "consistency",
                     "value": "EVENTUAL",
@@ -440,7 +461,7 @@ one server failure.
                 },
                 {
                     "option": "expelTimeout",
-                    "value": "0",
+                    "value": "<<<__default_gr_expel_timeout>>>",
                     "variable": "group_replication_member_expel_timeout"
                 },
                 {
@@ -448,6 +469,16 @@ one server failure.
                     "value": "",
                     "variable": "group_replication_group_seeds"
                 },
+                {
+                    "option": "ipAllowlist",
+?{VER(>=8.0.22)}
+                    "value": "AUTOMATIC",
+?{}
+?{VER(<8.0.22)}
+                    "value": null,
+?{}
+                    "variable": "group_replication_ip_allowlist"
+                },
                 {
                     "option": "ipWhitelist",
                     "value": "AUTOMATIC",
@@ -462,6 +493,46 @@ one server failure.
                     "option": "memberWeight",
                     "value": "50",
                     "variable": "group_replication_member_weight"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "WRITESET",
+?{}
+?{VER(<8.0.23)}
+                    "value": "COMMIT_ORDER",
+?{}
+                    "variable": "binlog_transaction_dependency_tracking"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "LOGICAL_CLOCK",
+?{}
+?{VER(<8.0.23)}
+                    "value": "DATABASE",
+?{}
+                    "variable": "<<<__replica_keyword>>>_parallel_type"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "4",
+?{}
+?{VER(<8.0.23)}
+                    "value": "0",
+?{}
+                    "variable": "<<<__replica_keyword>>>_parallel_workers"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "ON",
+?{}
+?{VER(<8.0.23)}
+                    "value": "OFF",
+?{}
+                    "variable": "<<<__replica_keyword>>>_preserve_commit_order"
+                },
+                {
+                    "value": "XXHASH64",
+                    "variable": "transaction_write_set_extraction"
                 }
             ]
         }
@@ -488,8 +559,17 @@ one server failure.
                 "value": true
             }
         ],
+        "tags": {
+            ".global": [],
+            "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": []
+        },
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": [
+                {
+                    "option": "autoRejoinTries",
+                    "value": null,
+                    "variable": "group_replication_autorejoin_tries"
+                },
                 {
                     "option": "consistency",
                     "value": null,
@@ -510,6 +590,11 @@ one server failure.
                     "value": "",
                     "variable": "group_replication_group_seeds"
                 },
+                {
+                    "option": "ipAllowlist",
+                    "value": null,
+                    "variable": "group_replication_ip_allowlist"
+                },
                 {
                     "option": "ipWhitelist",
                     "value": "AUTOMATIC",
@@ -524,6 +609,46 @@ one server failure.
                     "option": "memberWeight",
                     "value": "50",
                     "variable": "group_replication_member_weight"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "WRITESET",
+?{}
+?{VER(<8.0.23)}
+                    "value": "COMMIT_ORDER",
+?{}
+                    "variable": "binlog_transaction_dependency_tracking"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "LOGICAL_CLOCK",
+?{}
+?{VER(<8.0.23)}
+                    "value": "DATABASE",
+?{}
+                    "variable": "<<<__replica_keyword>>>_parallel_type"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "4",
+?{}
+?{VER(<8.0.23)}
+                    "value": "0",
+?{}
+                    "variable": "<<<__replica_keyword>>>_parallel_workers"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "ON",
+?{}
+?{VER(<8.0.23)}
+                    "value": "OFF",
+?{}
+                    "variable": "<<<__replica_keyword>>>_preserve_commit_order"
+                },
+                {
+                    "value": "XXHASH64",
+                    "variable": "transaction_write_set_extraction"
                 }
             ]
         }
@@ -553,8 +678,17 @@ one server failure.
                 "value": true
             }
         ],
+        "tags": {
+            ".global": [],
+            "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": []
+        },
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": [
+                {
+                    "option": "autoRejoinTries",
+                    "value": "<<<__default_gr_auto_rejoin_tries>>>",
+                    "variable": "group_replication_autorejoin_tries"
+                },
                 {
                     "option": "consistency",
                     "value": "EVENTUAL",
@@ -567,7 +701,7 @@ one server failure.
                 },
                 {
                     "option": "expelTimeout",
-                    "value": "0",
+                    "value": "<<<__default_gr_expel_timeout>>>",
                     "variable": "group_replication_member_expel_timeout"
                 },
                 {
@@ -575,6 +709,16 @@ one server failure.
                     "value": "",
                     "variable": "group_replication_group_seeds"
                 },
+                {
+                    "option": "ipAllowlist",
+?{VER(>=8.0.22)}
+                    "value": "AUTOMATIC",
+?{}
+?{VER(<8.0.22)}
+                    "value": null,
+?{}
+                    "variable": "group_replication_ip_allowlist"
+                },
                 {
                     "option": "ipWhitelist",
                     "value": "AUTOMATIC",
@@ -589,6 +733,46 @@ one server failure.
                     "option": "memberWeight",
                     "value": "50",
                     "variable": "group_replication_member_weight"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "WRITESET",
+?{}
+?{VER(<8.0.23)}
+                    "value": "COMMIT_ORDER",
+?{}
+                    "variable": "binlog_transaction_dependency_tracking"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "LOGICAL_CLOCK",
+?{}
+?{VER(<8.0.23)}
+                    "value": "DATABASE",
+?{}
+                    "variable": "<<<__replica_keyword>>>_parallel_type"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "4",
+?{}
+?{VER(<8.0.23)}
+                    "value": "0",
+?{}
+                    "variable": "<<<__replica_keyword>>>_parallel_workers"
+                },
+                {
+?{VER(>=8.0.23)}
+                    "value": "ON",
+?{}
+?{VER(<8.0.23)}
+                    "value": "OFF",
+?{}
+                    "variable": "<<<__replica_keyword>>>_preserve_commit_order"
+                },
+                {
+                    "value": "XXHASH64",
+                    "variable": "transaction_write_set_extraction"
                 }
             ]
         }
@@ -596,4 +780,28 @@ one server failure.
 }
 
 //@ WL#13208: TS_FR1_1 disableClone option is not supported for server version < 8.0.17. {VER(<8.0.17)}
-||Dba.createCluster: Option 'disableClone' not supported on target server version: '<<<__version>>>' (RuntimeError)
+||Option 'disableClone' not supported on target server version: '<<<__version>>>' (RuntimeError)
+
+//@ canonical IPv6 addresses are supported WL#12758 {VER(>= 8.0.14)}
+|[::1]:<<<__mysql_sandbox_port1>>> = {"mysqlX": "[::1]:<<<__mysql_sandbox_x_port1>>>", "grLocal": "[::1]:<<<__mysql_sandbox_gr_port1>>>", "mysqlClassic": "[::1]:<<<__mysql_sandbox_port1>>>"}|
+
+//If the target instance is >= 8.0.22, when ipWhitelist is used a deprecation warning must be printed
+//@ IPv6 addresses are supported on localAddress, groupSeeds and ipWhitelist WL#12758 {VER(>=8.0.22)}
+|WARNING: The ipWhitelist option is deprecated in favor of ipAllowlist. ipAllowlist will be set instead.|
+
+//@ canonical IPv6 addresses are not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+|ERROR: Cannot use host '::1' for instance '[::1]:<<<__mysql_sandbox_port1>>>' because it is an IPv6 address which is only supported by Group Replication from MySQL version >= 8.0.14. Set the MySQL server 'report_host' variable to an IPv4 address or hostname that resolves an IPv4 address.|
+||Unsupported IP address '::1'. IPv6 is only supported by Group Replication on MySQL version >= 8.0.14. (RuntimeError)
+
+//@ IPv6 local_address is not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+|ERROR: Cannot create cluster on instance '127.0.0.1:<<<__mysql_sandbox_port1>>>': unsupported localAddress value.|
+||Cannot use value '[::1]:<<<__mysql_sandbox_gr_port1>>>' for option localAddress because it has an IPv6 address which is only supported by Group Replication from MySQL version >= 8.0.14 and the target instance version is <<<__version>>>. (ArgumentError)
+
+//@ IPv6 on ipWhitelist is not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
+||Invalid value for ipWhitelist '::1': IPv6 not supported (version >= 8.0.14 required for IPv6 support). (ArgumentError)
+
+//@ IPv6 on groupSeeds is not supported below 8.0.14 - 1 WL#12758 {VER(< 8.0.14)}
+||Instance does not support the following groupSeed value :'[::1]:<<<__mysql_sandbox_gr_port2>>>'. IPv6 addresses/hostnames are only supported by Group Replication from MySQL version >= 8.0.14 and the target instance version is <<<__version>>>. (ArgumentError)
+
+//@ IPv6 on groupSeeds is not supported below 8.0.14 - 2 WL#12758 {VER(< 8.0.14)}
+||Instance does not support the following groupSeed values :'[::1]:<<<__mysql_sandbox_gr_port2>>>, [fe80::7e36:f49a:63c8:8ad6]:<<<__mysql_sandbox_gr_port1>>>'. IPv6 addresses/hostnames are only supported by Group Replication from MySQL version >= 8.0.14 and the target instance version is <<<__version>>>. (ArgumentError)
