@@ -12,8 +12,8 @@ use the following wildcards:
 
 The following are the main help categories:
 
- - AdminAPI       Introduces to the dba global object and the InnoDB cluster
-                  administration API.
+ - AdminAPI       The AdminAPI is an API that enables configuring and managing
+                  InnoDB clusters and replicasets, among other things.
  - Shell Commands Provides details about the available built-in shell commands.
  - ShellAPI       Contains information about the shell and util global objects
                   as well as the mysql module that enables executing SQL on
@@ -44,6 +44,8 @@ The following shell commands are available:
  - \                   Start multi-line input when in SQL mode.
  - \connect    (\c)    Connects the shell to a MySQL server and assigns the
                        global session.
+ - \disconnect         Disconnects the global session.
+ - \edit       (\e)    Launch a system editor to edit a command to be executed.
  - \exit               Exits the MySQL Shell, same as \quit.
  - \help       (\?,\h) Prints help information about a specific topic.
  - \history            View and edit command line history.
@@ -62,6 +64,7 @@ The following shell commands are available:
  - \sql                Executes SQL statement or switches to SQL processing
                        mode when no statement is given.
  - \status     (\s)    Print information about the current global session.
+ - \system     (\!)    Execute a system shell command.
  - \use        (\u)    Sets the active schema.
  - \warnings   (\W)    Show warnings after every statement.
  - \watch              Executes the given report with provided options and
@@ -75,6 +78,8 @@ The following modules and objects are ready for use when the shell starts:
  - mysql    Support for connecting to MySQL servers using the classic MySQL
             protocol.
  - mysqlx   Used to work with X Protocol sessions using the MySQL X DevAPI.
+ - os       Gives access to functions which allow to interact with the
+            operating system.
  - shell    Gives access to general purpose functions and properties.
  - sys      Gives access to system specific parameters.
  - testutil
@@ -110,8 +115,8 @@ use the following wildcards:
 
 The following are the main help categories:
 
- - AdminAPI       Introduces to the dba global object and the InnoDB cluster
-                  administration API.
+ - AdminAPI       The AdminAPI is an API that enables configuring and managing
+                  InnoDB clusters and replicasets, among other things.
  - Shell Commands Provides details about the available built-in shell commands.
  - ShellAPI       Contains information about the shell and util global objects
                   as well as the mysql module that enables executing SQL on
@@ -157,6 +162,8 @@ The following shell commands are available:
  - \                   Start multi-line input when in SQL mode.
  - \connect    (\c)    Connects the shell to a MySQL server and assigns the
                        global session.
+ - \disconnect         Disconnects the global session.
+ - \edit       (\e)    Launch a system editor to edit a command to be executed.
  - \exit               Exits the MySQL Shell, same as \quit.
  - \G                  Send command to mysql server, display result vertically.
  - \g                  Send command to mysql server.
@@ -177,6 +184,7 @@ The following shell commands are available:
  - \sql                Executes SQL statement or switches to SQL processing
                        mode when no statement is given.
  - \status     (\s)    Print information about the current global session.
+ - \system     (\!)    Execute a system shell command.
  - \use        (\u)    Sets the active schema.
  - \warnings   (\W)    Show warnings after every statement.
  - \watch              Executes the given report with provided options and
@@ -210,26 +218,93 @@ Use \? \help for additional details.
 Switching to JavaScript mode...
 
 //@<OUT> Help on Admin API Category
-MySQL InnoDB cluster provides a complete high availability solution for MySQL.
-
-The AdminAPI is an interactive API that enables configuring and administering
-InnoDB clusters.
-
-Use the dba global object to:
-
-- Verify if a MySQL server is suitable for InnoDB cluster.
-- Configure a MySQL server to be used as an InnoDB cluster instance.
-- Create an InnoDB cluster.
-- Get a handle for performing operations on an InnoDB cluster.
-- Other InnoDB cluster maintenance tasks.
-
-In the AdminAPI, an InnoDB cluster is represented as an instance of the Cluster
-class.
+The AdminAPI can be used interactively from the MySQL Shell prompt and
+non-interactively from JavaScript and Python scripts and directly from the
+command line.
 
 For more information about the dba object use: \? dba
 
+In the AdminAPI, an InnoDB cluster is represented as an instance of the Cluster
+class, while replicasets are represented as an instance of the ReplicaSet
+class, and ClusterSets are represented as an instance of the ClusterSet class.
+
 For more information about the Cluster class use: \? Cluster
 
+For more information about the ClusterSet class use: \? ClusterSet
+
+For more information about the ReplicaSet class use: \? ReplicaSet
+
+Scripting
+
+Through the JavaScript and Python bindings of the MySQL Shell, the AdminAPI can
+be used from scripts, which can in turn be used interactively or
+non-interactively. To execute a script, use the -f command line option,
+followed by the script file path. Options that follow the path are passed
+directly to the script being executed, which can access them from sys.argv
+    mysqlsh root@localhost -f myscript.py arg1 arg2
+
+If the script finishes successfully, the Shell will exit with code 0, while
+uncaught exceptions/errors cause it to exist with a non-0 code.
+
+By default, the AdminAPI enables interactivity, which will cause functions to
+prompt for missing passwords, confirmations and bits of information that cannot
+be obtained automatically.
+
+Prompts can be completely disabled with the --no-wizard command line option or
+using the "interactive" boolean option available in some of functions. If
+interactivity is disabled and some information is missing (e.g. a password), an
+error will be raised instead of being prompted.
+
+Secure Password Handling
+
+Passwords can be safely stored locally, using the system's native secrets
+storage functionality (or login-paths in Linux). Whenever the Shell needs a
+password, it will first query for the password in the system, before prompting
+for it.
+
+Passwords can be stored during interactive use, by confirming in the Store
+Password prompt. They can also be stored programmatically, using the
+shell.storeCredential() function.
+
+You can also use environment variables to pass information to your scripts. In
+JavaScript, the os.getenv() function can be used to access them.
+
+Command Line Interface
+
+In addition to the scripting interface, the MySQL Shell supports generic
+command line integration, which allows calling AdminAPI functions directly from
+the system shell (e.g. bash). Examples:
+    $ mysqlsh -- dba configure-instance root@localhost
+
+    is equivalent to:
+
+    > dba.configureInstance("root@localhost")
+
+    $ mysqlsh root@localhost -- cluster status --extended
+
+    is equivalent to:
+
+    > dba.getCluster().status({extended:true})
+
+The mapping from AdminAPI function signatures works as follows:
+
+- The first argument after a -- can be a shell global object, such as dba. As a
+  special case, cluster and rs are also accepted.
+- The second argument is the name of the function of the object to be called.
+  The naming convention is automatically converted from camelCase/snake_case to
+  lower case separated by dashes.
+- The rest of the arguments are used in the same order as their JS/Python
+  counterparts. Instances can be given as URIs. Option dictionaries can be
+  passed as --options, where the option name is the same as in JS/Python.
+
+OBJECTS
+ - dba InnoDB cluster and replicaset management functions.
+
+CLASSES
+ - Cluster    Represents an InnoDB cluster.
+ - ClusterSet Represents an InnoDB ClusterSet.
+ - ReplicaSet Represents an InnoDB ReplicaSet.
+
 //@<OUT> Help on shell commands
 The shell commands allow executing specific operations including updating the
 shell configuration.
@@ -239,6 +314,8 @@ The following shell commands are available:
  - \                   Start multi-line input when in SQL mode.
  - \connect    (\c)    Connects the shell to a MySQL server and assigns the
                        global session.
+ - \disconnect         Disconnects the global session.
+ - \edit       (\e)    Launch a system editor to edit a command to be executed.
  - \exit               Exits the MySQL Shell, same as \quit.
  - \help       (\?,\h) Prints help information about a specific topic.
  - \history            View and edit command line history.
@@ -257,6 +334,7 @@ The following shell commands are available:
  - \sql                Executes SQL statement or switches to SQL processing
                        mode when no statement is given.
  - \status     (\s)    Print information about the current global session.
+ - \system     (\!)    Execute a system shell command.
  - \use        (\u)    Sets the active schema.
  - \warnings   (\W)    Show warnings after every statement.
  - \watch              Executes the given report with provided options and
@@ -273,6 +351,8 @@ Contains information about the shell and util global objects as well as the
 mysql module that enables executing SQL on MySQL Servers.
 
 OBJECTS
+ - os    Gives access to functions which allow to interact with the operating
+         system.
  - shell Gives access to general purpose functions and properties.
  - sys   Gives access to system specific parameters.
  - util  Global object that groups miscellaneous tools like upgrade checker and
@@ -282,6 +362,9 @@ FUNCTIONS
       dir(object)
             Returns a list of enumerable properties on the target object.
 
+      require(module_name_or_path)
+            Loads the specified JavaScript module.
+
 CLASSES
  - Column Represents the metadata for a column in a result.
  - Row    Represents the a Row in a Result.
@@ -301,7 +384,7 @@ WHERE
       object: The object whose properties will be listed.
 
 RETURNS
-       The list of enumerable properties on the object.
+      The list of enumerable properties on the object.
 
 DESCRIPTION
       Traverses the object retrieving its enumerable properties. The content of
@@ -313,6 +396,90 @@ DESCRIPTION
       Behavior of the function passing other types of objects is undefined and
       also unsupported.
 
+//@<OUT> WL13119-TSFR10_1: Validate that the help system contains an entry for the require() function.
+NAME
+      require - Loads the specified JavaScript module.
+
+SYNTAX
+      require(module_name_or_path)
+
+WHERE
+      module_name_or_path: The name or a path to the module to be loaded.
+
+RETURNS
+      The exported content of the loaded module.
+
+DESCRIPTION
+      The module_name_or_path parameter can be either a name of the built-in
+      module (i.e. mysql or mysqlx) or a path to the JavaScript module on the
+      local file system. The local module is searched for in the following
+      folders:
+
+      - if module_name_or_path begins with either './' or '../' characters, use
+        the folder which contains the JavaScipt file or module which is
+        currently being executed or the current working directory if there is
+        no such file or module (i.e. shell is running in interactive mode),
+      - folders listed in the sys.path variable.
+
+      The file containing the module to be loaded is located by iterating
+      through these folders, for each folder path:
+
+      - append module_name_or_path, if such file exists, use it,
+      - append module_name_or_path, append '.js' extension, if such file
+        exists, use it,
+      - append module_name_or_path, if such folder exists, append 'init.js'
+        file name, if such file exists, use it.
+
+      The loaded module has access to the following variables:
+
+      - exports - an empty object, module should use it to export its
+        functionalities; this is the value returned from the require()
+        function,
+      - module - a module object, contains the exports object described above;
+        can be used to i.e. change the type of exports or store module-specific
+        data,
+      - __filename - absolute path to the module file,
+      - __dirname - absolute path to the directory containing the module file.
+
+      Each module is loaded only once, any subsequent call to require() which
+      would use the same module will return a cached value instead.
+
+      If two modules form a cycle (try to load each other using the require()
+      function), one of them is going to receive an unfinished copy of the
+      other ones exports object.
+
+      Here is a sample module called test.js which stores some data in the
+      module object and exports a function callme():
+        module.counter = 0;
+
+        exports.callme = function() {
+          const plural = ++module.counter > 1 ? 's' : '';
+          println(`I was called ${module.counter} time${plural}.`);
+        };
+
+      If placed in the current working directory, it can be used in shell as
+      follows:
+        mysql-js> var test = require('./test');
+        mysql-js> test.callme();
+        I was called 1 time.
+        mysql-js> test.callme();
+        I was called 2 times.
+        mysql-js> test.callme();
+        I was called 3 times.
+
+EXCEPTIONS
+      TypeError in the following scenarios:
+
+      - if module_name_or_path is not a string.
+
+      Error in the following scenarios:
+
+      - if module_name_or_path is empty,
+      - if module_name_or_path contains a backslash character,
+      - if module_name_or_path is an absolute path,
+      - if local module could not be found,
+      - if local module could not be loaded.
+
 //@<OUT> Help on X DevAPI Category
 The X DevAPI allows working with MySQL databases through a modern and fluent
 API. It specially simplifies using MySQL as a Document Store.
@@ -341,6 +508,10 @@ No help items found matching 'unknown'
 //@<OUT> Help on topic with several matches
 Found several entries matching session
 
+The following topics were found at the AdminAPI category:
+
+- dba.session
+
 The following topics were found at the X DevAPI category:
 
 - mysqlx.Collection.session
@@ -351,7 +522,7 @@ The following topics were found at the X DevAPI category:
 
 For help on a specific topic use: \? <topic>
 
-e.g.: \? mysqlx.Collection.session
+e.g.: \? dba.session
 
 //@<OUT> Help for sandbox operations
 Found several entries matching *sandbox*
@@ -416,12 +587,12 @@ command line, by following a specific syntax.
 
 SYNTAX:
 
-  mysqlsh [options] -- <object> <method> [arguments]
+  mysqlsh [options] -- <object> <operation> [arguments]
 
 WHERE:
 
 - object - a string identifying shell object to be called.
-- method - a string identifying method to be called on the object.
+- operation - a string identifying method to be called on the object.
 - arguments - arguments passed to the called method.
 
 DETAILS:
@@ -434,7 +605,7 @@ The following objects can be called using this format:
 - shell.options - shell.options object.
 - util - util global object
 
-The method name can be given in the following naming styles:
+The operation name can be given in the following naming styles:
 
 - Camel case: (e.g. createCluster, checkForServerUpgrade)
 - Kebab case: (e.g. create-cluster, check-for-server-upgrade)
@@ -474,18 +645,6 @@ specified either using camelCaseNaming or kebab-case-naming, examples:
 If the value is not provided the option name will be handled as a boolean
 option with a default value of TRUE.
 
-Grouping Named Arguments
-
-It is possible to create a group of named arguments by enclosing them between
-curly braces. This group would be handled as a positional argument at the
-position where the opening curly brace was found, it will be passed to the
-function as a dictionary containing all the named arguments defined on the
-group.
-
-Named arguments that are not placed inside curly braces (independently of the
-position on the command line), are packed a single dictionary, passed as a last
-parameter on the method call.
-
 Following are some examples of command line calls as well as how they are
 mapped to the API method call.
 
@@ -506,8 +665,88 @@ $ mysqlsh -- util checkForServerUpgrade root@localhost --outputFormat=JSON
       mysql-js> util.checkForServerUpgrade("root@localhost",{outputFormat:
       "JSON"})
 
-$ mysqlsh -- util check-for-server-upgrade { --user=root --host=localhost }
+$ mysqlsh -- util check-for-server-upgrade --user=root --host=localhost
 --password=
       mysql-js> util.checkForServerUpgrade({user:"root", host:"localhost"},
       {password:""})
 
+//@<OUT> Help using wildcard prefix prints single element found
+NAME
+      createClusterSet - Creates a MySQL InnoDB ClusterSet from an existing
+                         standalone InnoDB Cluster.
+
+SYNTAX
+      <Cluster>.createClusterSet(domainName[, options])
+
+WHERE
+      domainName: An identifier for the ClusterSet's logical dataset.
+      options: Dictionary with additional parameters described below.
+
+RETURNS
+      The created ClusterSet object.
+
+DESCRIPTION
+      Creates a ClusterSet object from an existing cluster, with the given data
+      domain name.
+
+      Several checks and validations are performed to ensure that the target
+      Cluster complies with the requirements for ClusterSets and if so, the
+      Metadata schema will be updated to create the new ClusterSet and the
+      target Cluster becomes the PRIMARY cluster of the ClusterSet.
+
+      InnoDB ClusterSet
+
+      A ClusterSet is composed of a single PRIMARY InnoDB Cluster that can have
+      one or more replica InnoDB Clusters that replicate from the PRIMARY using
+      asynchronous replication.
+
+      ClusterSets allow InnoDB Cluster deployments to achieve fault-tolerance
+      at a whole Data Center / region or geographic location, by creating
+      REPLICA clusters in different locations (Data Centers), ensuring Disaster
+      Recovery is possible.
+
+      If the PRIMARY InnoDB Cluster becomes completely unavailable, it's
+      possible to promote a REPLICA of that cluster to take over its duties
+      with minimal downtime or data loss.
+
+      All Cluster operations are available at each individual member (cluster)
+      of the ClusterSet. The AdminAPI ensures all updates are performed at the
+      PRIMARY and controls the command availability depending on the individual
+      status of each Cluster.
+
+      Please note that InnoDB ClusterSets don't have the same consistency and
+      data loss guarantees as InnoDB Clusters. To read more about ClusterSets,
+      see \? ClusterSet or refer to the MySQL manual.
+
+      Pre-requisites
+
+      The following is a non-exhaustive list of requirements to create a
+      ClusterSet:
+
+      - The target cluster must not already be part of a ClusterSet.
+      - MySQL 8.0.27 or newer.
+      - The target cluster's Metadata schema version is 2.1.0 or newer.
+      - Unmanaged replication channels are not allowed.
+
+      Options
+
+      The options dictionary can contain the following values:
+
+      - dryRun: boolean if true, all validations and steps for creating a
+        ClusterSet are executed, but no changes are actually made. An exception
+        will be thrown when finished.
+      - clusterSetReplicationSslMode: SSL mode for the ClusterSet replication
+        channels.
+
+      The clusterSetReplicationSslMode option supports the following values:
+
+      - REQUIRED: if used, SSL (encryption) will be enabled for the ClusterSet
+        replication channels.
+      - DISABLED: if used, SSL (encryption) will be disabled for the ClusterSet
+        replication channels.
+      - AUTO: if used, SSL (encryption) will be enabled if supported by the
+        instance, otherwise disabled.
+
+      If clusterSetReplicationSslMode is not specified AUTO will be used by
+      default.
+
