@@ -19,6 +19,10 @@ FUNCTIONS
       connect(connectionData[, password])
             Establishes the shell global session.
 
+      connectToPrimary([connectionData][, password])
+            Establishes the shell global session, connecting to a primary of an
+            InnoDB cluster or ReplicaSet.
+
       createExtensionObject()
             Creates an extension object, it can be used to extend shell
             functionality.
@@ -32,6 +36,9 @@ FUNCTIONS
       disablePager()
             Disables pager for the current scripting mode.
 
+      disconnect()
+            Disconnects the global session.
+
       dumpRows(result, format)
             Formats and dumps the given resultset object to the console.
 
@@ -55,6 +62,9 @@ FUNCTIONS
       log(level, message)
             Logs an entry to the shell's log file.
 
+      openSession(connectionData[, password])
+            Establishes and returns session.
+
       parseUri(uri)
             Utility function to parse a URI string.
 
@@ -111,6 +121,9 @@ DESCRIPTION
         allowed values: "always", "prompt" or "never"
       - dba.gtidWaitTimeout: timeout value in seconds to wait for GTIDs to be
         synchronized
+      - dba.logSql: 0..2, log SQL statements executed by AdminAPI operations: 0
+        - logging disabled; 1 - log statements other than SELECT and SHOW; 2 -
+        log all statements.
       - defaultCompress: Enable compression in client/server protocol by
         default in global shell sessions.
       - defaultMode: shell mode to use when shell is started, allowed values:
@@ -121,14 +134,22 @@ DESCRIPTION
       - history.maxSize: number of entries to keep in command history
       - history.sql.ignorePattern: colon separated list of glob patterns to
         filter out of the command history in SQL mode
+      - history.sql.syslog: true to log filtered interactive commands to the
+        system log, filtering of commands depends on the value of
+        history.sql.ignorePattern
       - interactive: read-only, boolean value that indicates if the shell is
         running in interactive mode
       - logLevel: current log level
-      - resultFormat: controls the type of output produced for SQL results.
+      - mysqlPluginDir: Directory for client-side authentication plugins
+      - oci.configFile: Path to OCI (Oracle Cloud Infrastructure) configuration
+        file
+      - oci.profile: Specify which section in oci.configFile will be used as
+        profile settings
       - pager: string which specifies the external command which is going to be
         used to display the paged output
       - passwordsFromStdin: boolean value that indicates if the shell should
         read passwords from stdin instead of the tty
+      - resultFormat: controls the type of output produced for SQL results.
       - sandboxDir: default path where the new sandbox instances for InnoDB
         cluster will be deployed
       - showColumnTypeInfo: display column type information in SQL mode. Please
@@ -359,18 +380,23 @@ DESCRIPTION
       The following options are valid for use either in a URI or in a
       dictionary:
 
-      - ssl-mode: the SSL mode to be used in the connection.
-      - ssl-ca: the path to the X509 certificate authority in PEM format.
-      - ssl-capath: the path to the directory that contains the X509
-        certificates authorities in PEM format.
-      - ssl-cert: The path to the X509 certificate in PEM format.
-      - ssl-key: The path to the X509 key in PEM format.
+      - ssl-mode: The SSL mode to be used in the connection.
+      - ssl-ca: The path to the X509 certificate authority file in PEM format.
+      - ssl-capath: The path to the directory that contains the X509
+        certificate authority files in PEM format.
+      - ssl-cert: The path to the SSL public key certificate file in PEM
+        format.
+      - ssl-key: The path to the SSL private key file in PEM format.
       - ssl-crl: The path to file that contains certificate revocation lists.
       - ssl-crlpath: The path of directory that contains certificate revocation
         list files.
-      - ssl-cipher: SSL Cipher to use.
-      - tls-version: List of protocols permitted for secure connections
-      - auth-method: Authentication method
+      - ssl-cipher: The list of permissible encryption ciphers for connections
+        that use TLS protocols up through TLSv1.2.
+      - tls-version: List of protocols permitted for secure connections. TLS
+        versions TLSv1 and TLSv1.1 are now deprecated and will be removed in a
+        future release of MySQL Shell. Use TLSv1.2 or TLSv1.3.
+      - tls-ciphers: List of TLS v1.3 ciphers to use.
+      - auth-method: Authentication method.
       - get-server-public-key: Request public key from the server required for
         RSA key pair-based password exchange. Use when connecting to MySQL 8.0
         servers with classic MySQL sessions with SSL mode DISABLED.
@@ -381,10 +407,15 @@ DESCRIPTION
       - connect-timeout: The connection timeout in milliseconds. If not
         provided a default timeout of 10 seconds will be used. Specifying a
         value of 0 disables the connection timeout.
-      - compression: Enable/disable compression in client/server protocol,
-        valid values: "true", "false", "1", and "0".
+      - compression: Enable compression in client/server protocol.
+      - compression-algorithms: Use compression algorithm in server/client
+        protocol.
+      - compression-level: Use this compression level in the client/server
+        protocol.
       - connection-attributes: List of connection attributes to be registered
         at the PERFORMANCE_SCHEMA connection attributes tables.
+      - local-infile: Enable/disable LOAD DATA LOCAL INFILE.
+      - net-buffer-length: The buffer size for TCP/IP and socket communication.
 
       When these options are defined in a URI, their values must be URL
       encoded.
@@ -413,6 +444,121 @@ DESCRIPTION
 
       For additional information on connection data use \? connection.
 
+//@<OUT> Help on connectToPrimary
+NAME
+      connectToPrimary - Establishes the shell global session, connecting to a
+                         primary of an InnoDB cluster or ReplicaSet.
+
+SYNTAX
+      shell.connectToPrimary([connectionData][, password])
+
+WHERE
+      connectionData: The connection data to be used to establish the session.
+      password: The password to be used when establishing the session.
+
+RETURNS
+      The established session.
+
+DESCRIPTION
+      Ensures that the target server is a member of an InnoDB cluster or
+      ReplicaSet and if it is not a PRIMARY, finds the PRIMARY and connects to
+      it. Sets the global session object to the established session and returns
+      that object.
+
+      If connectionData is not given, this function uses the global shell
+      session, if there is none, an exception is raised.
+
+      The password may be included on the connectionData, the optional
+      parameter should be used only if the connectionData does not contain it
+      already. If both are specified the password parameter will override the
+      password defined on the connectionData.
+
+      The connection data may be specified in the following formats:
+
+      - A URI string
+      - A dictionary with the connection options
+
+      A basic URI string has the following format:
+
+      [scheme://][user[:password]@]<host[:port]|socket>[/schema][?option=value&option=value...]
+
+      Connection Options
+
+      The following options are valid for use either in a URI or in a
+      dictionary:
+
+      - ssl-mode: The SSL mode to be used in the connection.
+      - ssl-ca: The path to the X509 certificate authority file in PEM format.
+      - ssl-capath: The path to the directory that contains the X509
+        certificate authority files in PEM format.
+      - ssl-cert: The path to the SSL public key certificate file in PEM
+        format.
+      - ssl-key: The path to the SSL private key file in PEM format.
+      - ssl-crl: The path to file that contains certificate revocation lists.
+      - ssl-crlpath: The path of directory that contains certificate revocation
+        list files.
+      - ssl-cipher: The list of permissible encryption ciphers for connections
+        that use TLS protocols up through TLSv1.2.
+      - tls-version: List of protocols permitted for secure connections. TLS
+        versions TLSv1 and TLSv1.1 are now deprecated and will be removed in a
+        future release of MySQL Shell. Use TLSv1.2 or TLSv1.3.
+      - tls-ciphers: List of TLS v1.3 ciphers to use.
+      - auth-method: Authentication method.
+      - get-server-public-key: Request public key from the server required for
+        RSA key pair-based password exchange. Use when connecting to MySQL 8.0
+        servers with classic MySQL sessions with SSL mode DISABLED.
+      - server-public-key-path: The path name to a file containing a
+        client-side copy of the public key required by the server for RSA key
+        pair-based password exchange. Use when connecting to MySQL 8.0 servers
+        with classic MySQL sessions with SSL mode DISABLED.
+      - connect-timeout: The connection timeout in milliseconds. If not
+        provided a default timeout of 10 seconds will be used. Specifying a
+        value of 0 disables the connection timeout.
+      - compression: Enable compression in client/server protocol.
+      - compression-algorithms: Use compression algorithm in server/client
+        protocol.
+      - compression-level: Use this compression level in the client/server
+        protocol.
+      - connection-attributes: List of connection attributes to be registered
+        at the PERFORMANCE_SCHEMA connection attributes tables.
+      - local-infile: Enable/disable LOAD DATA LOCAL INFILE.
+      - net-buffer-length: The buffer size for TCP/IP and socket communication.
+
+      When these options are defined in a URI, their values must be URL
+      encoded.
+
+      The following options are also valid when a dictionary is used:
+
+      Base Connection Options
+
+      - scheme: the protocol to be used on the connection.
+      - user: the MySQL user name to be used on the connection.
+      - dbUser: alias for user.
+      - password: the password to be used on the connection.
+      - dbPassword: same as password.
+      - host: the hostname or IP address to be used on the connection.
+      - port: the port to be used in a TCP connection.
+      - socket: the socket file name to be used on a connection through unix
+        sockets.
+      - schema: the schema to be selected once the connection is done.
+
+      ATTENTION: The dbUser and dbPassword options are will be removed in a
+                 future release.
+
+      The connection options are case insensitive and can only be defined once.
+
+      If an option is defined more than once, an error will be generated.
+
+      For additional information on connection data use \? connection.
+
+EXCEPTIONS
+      RuntimeError in the following scenarios:
+
+      - If connectionData was not given and there is no global shell session.
+      - If there is no primary member of an InnoDB cluster or ReplicaSet.
+      - If the target server is not a member of an InnoDB cluster or
+        ReplicaSet.
+
 //@<OUT> Help on createExtensionObject
 NAME
       createExtensionObject - Creates an extension object, it can be used to
@@ -539,7 +685,7 @@ SYNTAX
       shell.listCredentialHelpers()
 
 RETURNS
-       A list of string with names of available credential helpers.
+      A list of string with names of available credential helpers.
 
 DESCRIPTION
       The special values "default" and "<disabled>" are not on the list.
@@ -556,7 +702,7 @@ SYNTAX
       shell.listCredentials()
 
 RETURNS
-       A list of URLs stored by the configured credential helper.
+      A list of URLs stored by the configured credential helper.
 
 EXCEPTIONS
       Throws RuntimeError in the following scenarios:
@@ -590,7 +736,7 @@ WHERE
       uri: a URI string.
 
 RETURNS
-       A dictionary containing all the elements contained on the given URI
+      A dictionary containing all the elements contained on the given URI
       string.
 
 DESCRIPTION
@@ -616,7 +762,7 @@ WHERE
       options: Dictionary with options that change the function behavior.
 
 RETURNS
-       A string value containing the input from the user.
+      A string value containing the input from the user.
 
 DESCRIPTION
       This function allows creating scripts that require interaction with the
@@ -635,6 +781,13 @@ DESCRIPTION
 
       - password: the user input will not be echoed on the screen.
 
+//@<OUT> Help on shell.disconnect
+NAME
+      disconnect - Disconnects the global session.
+
+SYNTAX
+      shell.disconnect()
+
 //@<OUT> Help on reconnect
 NAME
       reconnect - Reconnect the global session.
@@ -729,6 +882,8 @@ DESCRIPTION
         and * separated by a '-' specifying a range of arguments without an
         upper bound. If this is not provided, the report does not accept any
         additional arguments.
+      - examples - A list of dictionaries describing the example usage of the
+        report.
 
       The optional options list must hold dictionaries with the following keys:
 
@@ -760,6 +915,18 @@ DESCRIPTION
       - values (list of strings, optional) - list of allowed values. Only
         'string' options may have this key. If this key is not specified, this
         option accepts any values.
+      - empty (Boolean, optional) - whether this option accepts empty strings.
+        Only 'string' options may have this key. If this key is not specified,
+        defaults to false.
+
+      The optional examples list must hold dictionaries with the following
+      keys:
+
+      - description (string, required) - Description text of the example.
+      - args (list of strings, optional) - List of the arguments used in the
+        example.
+      - options (dictionary of strings, optional) - Options used in the
+        example.
 
       The type of the report determines the expected result of a report
       invocation:
@@ -886,18 +1053,22 @@ Connection Options
 
 The following options are valid for use either in a URI or in a dictionary:
 
-- ssl-mode: the SSL mode to be used in the connection.
-- ssl-ca: the path to the X509 certificate authority in PEM format.
-- ssl-capath: the path to the directory that contains the X509 certificates
-  authorities in PEM format.
-- ssl-cert: The path to the X509 certificate in PEM format.
-- ssl-key: The path to the X509 key in PEM format.
+- ssl-mode: The SSL mode to be used in the connection.
+- ssl-ca: The path to the X509 certificate authority file in PEM format.
+- ssl-capath: The path to the directory that contains the X509 certificate
+  authority files in PEM format.
+- ssl-cert: The path to the SSL public key certificate file in PEM format.
+- ssl-key: The path to the SSL private key file in PEM format.
 - ssl-crl: The path to file that contains certificate revocation lists.
 - ssl-crlpath: The path of directory that contains certificate revocation list
   files.
-- ssl-cipher: SSL Cipher to use.
-- tls-version: List of protocols permitted for secure connections
-- auth-method: Authentication method
+- ssl-cipher: The list of permissible encryption ciphers for connections that
+  use TLS protocols up through TLSv1.2.
+- tls-version: List of protocols permitted for secure connections. TLS versions
+  TLSv1 and TLSv1.1 are now deprecated and will be removed in a future release
+  of MySQL Shell. Use TLSv1.2 or TLSv1.3.
+- tls-ciphers: List of TLS v1.3 ciphers to use.
+- auth-method: Authentication method.
 - get-server-public-key: Request public key from the server required for RSA
   key pair-based password exchange. Use when connecting to MySQL 8.0 servers
   with classic MySQL sessions with SSL mode DISABLED.
@@ -908,10 +1079,13 @@ The following options are valid for use either in a URI or in a dictionary:
 - connect-timeout: The connection timeout in milliseconds. If not provided a
   default timeout of 10 seconds will be used. Specifying a value of 0 disables
   the connection timeout.
-- compression: Enable/disable compression in client/server protocol, valid
-  values: "true", "false", "1", and "0".
+- compression: Enable compression in client/server protocol.
+- compression-algorithms: Use compression algorithm in server/client protocol.
+- compression-level: Use this compression level in the client/server protocol.
 - connection-attributes: List of connection attributes to be registered at the
   PERFORMANCE_SCHEMA connection attributes tables.
+- local-infile: Enable/disable LOAD DATA LOCAL INFILE.
+- net-buffer-length: The buffer size for TCP/IP and socket communication.
 
 When these options are defined in a URI, their values must be URL encoded.
 
@@ -1028,6 +1202,58 @@ The tls-version option accepts the following values:
 - TLSv1.1
 - TLSv1.2 (Supported only on commercial packages)
 
+Authentication method
+
+In case of classic session, this is the name of the authentication plugin to
+use, i.e. caching_sha2_password.
+
+In case of X protocol session, it should be one of:
+
+- AUTO,
+- FROM_CAPABILITIES,
+- FALLBACK,
+- MYSQL41,
+- PLAIN,
+- SHA256_MEMORY.
+
+Connection Compression
+
+Connection compression is governed by following connection options:
+"compression", "compression-algorithms", and "compression-level".
+
+"compression" accepts following values:
+
+- REQUIRED: connection will only be made when compression negotiation is
+  succesful.
+- PREFFERED: (default for X protocol connections) shell will attempt to
+  establish connection with compression enabled, but if compression negotiation
+  fails, connection will be established without compression.
+- DISABLED: (defalut for classic protocol connections) connection will be
+  established without compression.
+
+For convenience "compression" also accepts Boolean: 'True', 'False', '1', and
+'0' values which map to REQUIRED and DISABLED respectively.
+
+"compression-algorithms" expects comma separated list of algorithms. Supported
+algorithms include:
+
+- zstd
+- zlib
+- lz4 (X protocol only)
+- uncompressed - special value, which if it appears in the list, causes
+  connection to succeed even if compression negotiation fails.
+
+If "compression" connection option is not defined, its value will be deduced
+from "compression-algorithms" value when it is provided.
+
+"compression-level" expects an integer value. Valid range depends on the
+compression algorithm and server configuration, but generally following is
+expected:
+
+- zstd: 1-22 (default 3)
+- zlib: 1-9 (default 3), supported only by X protocol
+- lz4: 0-16 (default 2), supported only by X protocol.
+
 Connection Attributes
 
 Connection attributes are key-value pairs to be sent to the server at connect
@@ -1092,18 +1318,22 @@ Connection Options
 
 The following options are valid for use either in a URI or in a dictionary:
 
-- ssl-mode: the SSL mode to be used in the connection.
-- ssl-ca: the path to the X509 certificate authority in PEM format.
-- ssl-capath: the path to the directory that contains the X509 certificates
-  authorities in PEM format.
-- ssl-cert: The path to the X509 certificate in PEM format.
-- ssl-key: The path to the X509 key in PEM format.
+- ssl-mode: The SSL mode to be used in the connection.
+- ssl-ca: The path to the X509 certificate authority file in PEM format.
+- ssl-capath: The path to the directory that contains the X509 certificate
+  authority files in PEM format.
+- ssl-cert: The path to the SSL public key certificate file in PEM format.
+- ssl-key: The path to the SSL private key file in PEM format.
 - ssl-crl: The path to file that contains certificate revocation lists.
 - ssl-crlpath: The path of directory that contains certificate revocation list
   files.
-- ssl-cipher: SSL Cipher to use.
-- tls-version: List of protocols permitted for secure connections
-- auth-method: Authentication method
+- ssl-cipher: The list of permissible encryption ciphers for connections that
+  use TLS protocols up through TLSv1.2.
+- tls-version: List of protocols permitted for secure connections. TLS versions
+  TLSv1 and TLSv1.1 are now deprecated and will be removed in a future release
+  of MySQL Shell. Use TLSv1.2 or TLSv1.3.
+- tls-ciphers: List of TLS v1.3 ciphers to use.
+- auth-method: Authentication method.
 - get-server-public-key: Request public key from the server required for RSA
   key pair-based password exchange. Use when connecting to MySQL 8.0 servers
   with classic MySQL sessions with SSL mode DISABLED.
@@ -1114,10 +1344,13 @@ The following options are valid for use either in a URI or in a dictionary:
 - connect-timeout: The connection timeout in milliseconds. If not provided a
   default timeout of 10 seconds will be used. Specifying a value of 0 disables
   the connection timeout.
-- compression: Enable/disable compression in client/server protocol, valid
-  values: "true", "false", "1", and "0".
+- compression: Enable compression in client/server protocol.
+- compression-algorithms: Use compression algorithm in server/client protocol.
+- compression-level: Use this compression level in the client/server protocol.
 - connection-attributes: List of connection attributes to be registered at the
   PERFORMANCE_SCHEMA connection attributes tables.
+- local-infile: Enable/disable LOAD DATA LOCAL INFILE.
+- net-buffer-length: The buffer size for TCP/IP and socket communication.
 
 When these options are defined in a URI, their values must be URL encoded.
 
@@ -1234,6 +1467,58 @@ The tls-version option accepts the following values:
 - TLSv1.1
 - TLSv1.2 (Supported only on commercial packages)
 
+Authentication method
+
+In case of classic session, this is the name of the authentication plugin to
+use, i.e. caching_sha2_password.
+
+In case of X protocol session, it should be one of:
+
+- AUTO,
+- FROM_CAPABILITIES,
+- FALLBACK,
+- MYSQL41,
+- PLAIN,
+- SHA256_MEMORY.
+
+Connection Compression
+
+Connection compression is governed by following connection options:
+"compression", "compression-algorithms", and "compression-level".
+
+"compression" accepts following values:
+
+- REQUIRED: connection will only be made when compression negotiation is
+  succesful.
+- PREFFERED: (default for X protocol connections) shell will attempt to
+  establish connection with compression enabled, but if compression negotiation
+  fails, connection will be established without compression.
+- DISABLED: (defalut for classic protocol connections) connection will be
+  established without compression.
+
+For convenience "compression" also accepts Boolean: 'True', 'False', '1', and
+'0' values which map to REQUIRED and DISABLED respectively.
+
+"compression-algorithms" expects comma separated list of algorithms. Supported
+algorithms include:
+
+- zstd
+- zlib
+- lz4 (X protocol only)
+- uncompressed - special value, which if it appears in the list, causes
+  connection to succeed even if compression negotiation fails.
+
+If "compression" connection option is not defined, its value will be deduced
+from "compression-algorithms" value when it is provided.
+
+"compression-level" expects an integer value. Valid range depends on the
+compression algorithm and server configuration, but generally following is
+expected:
+
+- zstd: 1-22 (default 3)
+- zlib: 1-9 (default 3), supported only by X protocol
+- lz4: 0-16 (default 2), supported only by X protocol.
+
 Connection Attributes
 
 Connection attributes are key-value pairs to be sent to the server at connect
@@ -1336,41 +1621,6 @@ Note that the connection-attribute values are expected to be strings, if other
 data type is used in the dictionary, the string representation of the used data
 will be stored on the database.
 
-//@<OUT> Help on plugins
-The MySQL Shell allows extending its base functionality through the creation of
-plugins.
-
-An plugin is a folder containing the code that provides the functionality to be
-made available on the MySQL Shell.
-
-User defined plugins should be located at plugins folder at the following
-paths:
-
-- Windows: %AppData%\MySQL\mysqlsh\plugins
-- Others: ~/.mysqlsh/plugins
-
-A plugin must contain an init file which is the entry point to load the
-extension:
-
-- init.js for plugins written in JavaScript.
-- init.py for plugins written in Python.
-
-On startup, the shell traverses the folders inside of the *plugins* folder
-searching for the plugin init file. The init file will be loaded on the
-corresponding context (JavaScript or Python).
-
-Use Cases
-
-The main use cases for MySQL Shell plugins include:
-
-- Definition of shell reports to be used with the \show and \watch Shell
-  Commands.
-- Definition of new Global Objects with user defined functionality.
-
-For additional information on shell reports execute: \? reports
-
-For additional information on extension objects execute: \? extension objects
-
 //@<OUT> Help on extension objects
 The MySQL Shell allows for an extension of its base functionality by using
 special objects called Extension Objects.
@@ -1487,7 +1737,7 @@ WHERE
       options: a dictionary with the connection options.
 
 RETURNS
-       A URI string
+      A URI string
 
 DESCRIPTION
       This function assembles a MySQL connection string which can be used in
@@ -1506,7 +1756,7 @@ WHERE
               json/array, json/pretty. Default is table.
 
 RETURNS
-       The number of printed rows
+      The number of printed rows
 
 DESCRIPTION
       This function shows a resultset object returned by a DB Session query in
