@@ -14,22 +14,29 @@ validate_members(cluster, [
     'dissolve',
     'disconnect',
     'force_quorum_using_partition_of',
+    'get_cluster_set',
     'get_name',
     'help',
+    'list_routers',
     'name',
     'rejoin_instance',
+    'remove_router_metadata',
     'remove_instance',
     'rescan',
+    'reset_recovery_accounts_password',
     'options',
+    'setup_admin_account',
+    'setup_router_account',
     'status',
     'set_primary_instance',
     'set_option',
     'set_instance_option',
     'switch_to_multi_primary_mode',
-    'switch_to_single_primary_mode'
+    'switch_to_single_primary_mode',
+    'create_cluster_set'
     ])
 
 cluster.disconnect()
 session.close()
 
-testutil.destroy_sandbox(__mysql_sandbox_port1);
+testutil.destroy_sandbox(__mysql_sandbox_port1)
