@@ -1,14 +1,44 @@
 #@<OUT> dba
 NAME
-      dba - Global variable for InnoDB cluster management.
+      dba - InnoDB cluster and replicaset management functions.
 
 DESCRIPTION
-      The global variable dba is used to access the AdminAPI functionality and
-      perform DBA operations. It is used for managing MySQL InnoDB clusters.
+      Entry point for AdminAPI functions, including InnoDB clusters and replica
+      sets.
+
+      InnoDB clusters
+
+      The dba.configure_instance() function can be used to configure a MySQL
+      instance with the settings required to use it in an InnoDB cluster.
+
+      InnoDB clusters can be created with the dba.create_cluster() function.
+
+      Once created, InnoDB cluster management objects can be obtained with the
+      dba.get_cluster() function.
+
+      InnoDB ReplicaSets
+
+      The dba.configure_replica_set_instance() function can be used to
+      configure a MySQL instance with the settings required to use it in a
+      replicaset.
+
+      ReplicaSets can be created with the dba.create_replica_set() function.
+
+      Once created, replicaset management objects can be obtained with the
+      dba.get_replica_set() function.
+
+      Sandboxes
+
+      Utility functions are provided to create sandbox MySQL instances, which
+      can be used to create test clusters and replicasets.
 
 PROPERTIES
+      session
+            The session the dba object will use by default.
+
       verbose
-            Enables verbose mode on the dba operations.
+            Controls debug message verbosity for sandbox related dba
+            operations.
 
 FUNCTIONS
       check_instance_configuration(instance[, options])
@@ -22,9 +52,16 @@ FUNCTIONS
             Validates and configures a local instance for MySQL InnoDB Cluster
             usage.
 
+      configure_replica_set_instance([instance][, options])
+            Validates and configures an instance for use in an InnoDB
+            ReplicaSet.
+
       create_cluster(name[, options])
             Creates a MySQL InnoDB cluster.
 
+      create_replica_set(name[, options])
+            Creates a MySQL InnoDB ReplicaSet.
+
       delete_sandbox_instance(port[, options])
             Deletes an existing MySQL Server instance on localhost.
 
@@ -37,6 +74,12 @@ FUNCTIONS
       get_cluster([name][, options])
             Retrieves a cluster from the Metadata Store.
 
+      get_cluster_set()
+            Returns an object representing a ClusterSet.
+
+      get_replica_set()
+            Returns an object representing a ReplicaSet.
+
       help([member])
             Provides help about this object and it's members
 
@@ -52,9 +95,16 @@ FUNCTIONS
       stop_sandbox_instance(port[, options])
             Stops a running MySQL Server instance on localhost.
 
-      For more help on a specific function use: dba.help('<functionName>')
+      upgrade_metadata([options])
+            Upgrades (or restores) the metadata to the version supported by the
+            Shell.
 
-      e.g. dba.help('deploy_sandbox_instance')
+      SEE ALSO
+
+      - For general information about the AdminAPI use: \? AdminAPI
+      - For help on a specific function use: \? dba.<functionName>
+
+      e.g. \? dba.deploy_sandbox_instance
 
 #@<OUT> dba.check_instance_configuration
 NAME
@@ -69,7 +119,7 @@ WHERE
       options: Data for the operation.
 
 RETURNS
-       A descriptive text of the operation result.
+      A descriptive text of the operation result.
 
 DESCRIPTION
       This function reviews the instance configuration to identify if it is
@@ -81,8 +131,6 @@ DESCRIPTION
 
       For additional information on connection data use \? connection.
 
-      Only TCP/IP connections are allowed for this function.
-
       The options dictionary may contain the following options:
 
       - mycnfPath: Optional path to the MySQL configuration file for the
@@ -92,9 +140,10 @@ DESCRIPTION
         verified for the expected option values, in addition to the global
         MySQL system variables.
       - password: The password to get connected to the instance.
-      - interactive: boolean value used to disable the wizards in the command
-        execution, i.e. prompts are not provided to the user and confirmation
-        prompts are not shown.
+      - interactive: boolean value used to disable/enable the wizards in the
+        command execution, i.e. prompts and confirmations will be provided or
+        not according to the value set. The default value is equal to MySQL
+        Shell wizard mode.
 
       The connection password may be contained on the instance definition,
       however, it can be overwritten if it is specified on the options.
@@ -117,21 +166,6 @@ DESCRIPTION
       - Update the server variable.
       - Restart the server.
 
-EXCEPTIONS
-      ArgumentError in the following scenarios:
-
-      - If the instance parameter is empty.
-      - If the instance definition is invalid.
-      - If the instance definition is a connection dictionary but empty.
-
-      RuntimeError in the following scenarios:
-
-      - If the instance accounts are invalid.
-      - If the instance is offline.
-      - If the instance is already part of a Replication Group.
-      - If the instance is already part of an InnoDB Cluster.
-      - If the given the instance cannot be used for Group Replication.
-
 #@<OUT> dba.configure_instance
 NAME
       configure_instance - Validates and configures an instance for MySQL
@@ -145,34 +179,35 @@ WHERE
       options: Additional options for the operation.
 
 RETURNS
-       A descriptive text of the operation result.
+      A descriptive text of the operation result.
 
 DESCRIPTION
-      This function auto-configures the instance for InnoDB Cluster usage.If
+      This function auto-configures the instance for InnoDB Cluster usage. If
       the target instance already belongs to an InnoDB Cluster it errors out.
 
       The instance definition is the connection data for the instance.
 
       For additional information on connection data use \? connection.
 
-      Only TCP/IP connections are allowed for this function.
-
       The options dictionary may contain the following options:
 
       - mycnfPath: The path to the MySQL configuration file of the instance.
       - outputMycnfPath: Alternative output path to write the MySQL
         configuration file of the instance.
       - password: The password to be used on the connection.
-      - clusterAdmin: The name of the InnoDB cluster administrator account.
-      - clusterAdminPassword: The password for the InnoDB cluster administrator
+      - clusterAdmin: The name of the "cluster administrator" account.
+      - clusterAdminPassword: The password for the "cluster administrator"
         account.
       - clearReadOnly: boolean value used to confirm that super_read_only must
         be disabled.
-      - interactive: boolean value used to disable the wizards in the command
-        execution, i.e. prompts are not provided to the user and confirmation
-        prompts are not shown.
       - restart: boolean value used to indicate that a remote restart of the
         target instance should be performed to finalize the operation.
+      - interactive: boolean value used to disable/enable the wizards in the
+        command execution, i.e. prompts and confirmations will be provided or
+        not according to the value set. The default value is equal to MySQL
+        Shell wizard mode.
+      - applierWorkerThreads: Number of threads used for applying replicated
+        transactions. The default value is 4.
 
       If the outputMycnfPath option is used, only that file is updated and
       mycnfPath is treated as read-only.
@@ -199,31 +234,6 @@ DESCRIPTION
         variable.
       - Required Value: the required value for the configuration variable.
 
-EXCEPTIONS
-      ArgumentError in the following scenarios:
-
-      - If 'interactive' is disabled and the instance parameter is empty.
-      - If the instance definition is invalid.
-      - If the instance definition is a connection dictionary but empty.
-      - If the instance definition is a connection dictionary but any option is
-        invalid.
-      - If 'interactive' mode is disabled and the instance definition is
-        missing the password.
-      - If 'interactive' mode is enabled and the provided password is empty.
-      - If the clusterAdminPassword is provided and clusterAdmin is not
-        provided.
-      - If the clusterAdminPassword is provided but the clusterAdmin account
-        already exists.
-
-      RuntimeError in the following scenarios:
-
-      - If the configuration file path is required but not provided or wrong.
-      - If the instance accounts are invalid.
-      - If the instance is offline.
-      - If the instance is already part of a Replication Group.
-      - If the instance is already part of an InnoDB Cluster.
-      - If the given instance cannot be used for Group Replication.
-
 #@<OUT> dba.configure_local_instance
 NAME
       configure_local_instance - Validates and configures a local instance for
@@ -237,33 +247,34 @@ WHERE
       options: Additional options for the operation.
 
 RETURNS
-       Nothing
+      Nothing
 
 DESCRIPTION
       This function reviews the instance configuration to identify if it is
-      valid for usage in group replication and cluster. An exception is thrown
-      if not.
+      valid for usage in an InnoDB cluster, making configuration changes if
+      necessary.
 
       The instance definition is the connection data for the instance.
 
       For additional information on connection data use \? connection.
 
-      Only TCP/IP connections are allowed for this function.
-
       The options dictionary may contain the following options:
 
       - mycnfPath: The path to the MySQL configuration file of the instance.
       - outputMycnfPath: Alternative output path to write the MySQL
         configuration file of the instance.
       - password: The password to be used on the connection.
-      - clusterAdmin: The name of the InnoDB cluster administrator account.
-      - clusterAdminPassword: The password for the InnoDB cluster administrator
+      - clusterAdmin: The name of the "cluster administrator" account.
+      - clusterAdminPassword: The password for the "cluster administrator"
         account.
       - clearReadOnly: boolean value used to confirm that super_read_only must
         be disabled.
-      - interactive: boolean value used to disable the wizards in the command
-        execution, i.e. prompts are not provided to the user and confirmation
-        prompts are not shown.
+      - restart: boolean value used to indicate that a remote restart of the
+        target instance should be performed to finalize the operation.
+      - interactive: boolean value used to disable/enable the wizards in the
+        command execution, i.e. prompts and confirmations will be provided or
+        not according to the value set. The default value is equal to MySQL
+        Shell wizard mode.
 
       If the outputMycnfPath option is used, only that file is updated and
       mycnfPath is treated as read-only.
@@ -290,28 +301,69 @@ DESCRIPTION
         variable.
       - Required Value: the required value for the configuration variable.
 
-EXCEPTIONS
-      ArgumentError in the following scenarios:
+#@<OUT> dba.configure_replica_set_instance
+NAME
+      configure_replica_set_instance - Validates and configures an instance for
+                                       use in an InnoDB ReplicaSet.
 
-      - If the instance parameter is empty.
-      - If the instance definition is invalid.
-      - If the instance definition is a connection dictionary but empty.
-      - If the instance definition is a connection dictionary but any option is
-        invalid.
-      - If the instance definition is missing the password.
-      - If the provided password is empty.
-      - If the clusterAdminPassword is provided and clusterAdmin is not
-        provided.
-      - If the clusterAdminPassword is provided but the clusterAdmin account
-        already exists.
-      - If the configuration file path is required but not provided or wrong.
+SYNTAX
+      dba.configure_replica_set_instance([instance][, options])
 
-      RuntimeError in the following scenarios:
+WHERE
+      instance: An instance definition. By default, the active shell session is
+                used.
+      options: Additional options for the operation.
 
-      - If the instance accounts are invalid.
-      - If the instance is offline.
-      - If the instance is already part of a Replication Group.
-      - If the given instance cannot be used for Group Replication.
+RETURNS
+      Nothing
+
+DESCRIPTION
+      This function will verify and automatically configure the target instance
+      for use in an InnoDB ReplicaSet.
+
+      The function can optionally create a "cluster administrator" account, if
+      the "clusterAdmin" and "clusterAdminPassword" options are given. The
+      account is created with the minimal set of privileges required to manage
+      InnoDB clusters or ReplicaSets. The "cluster administrator" account must
+      have matching username and password across all instances of the same
+      cluster or replicaset.
+
+      Options
+
+      The instance definition is the connection data for the instance.
+
+      For additional information on connection data use \? connection.
+
+      The options dictionary may contain the following options:
+
+      - password: The password to be used on the connection.
+      - clusterAdmin: The name of a "cluster administrator" user to be created.
+        The supported format is the standard MySQL account name format.
+      - clusterAdminPassword: The password for the "cluster administrator"
+        account.
+      - interactive: boolean value used to disable/enable the wizards in the
+        command execution, i.e. prompts and confirmations will be provided or
+        not according to the value set. The default value is equal to MySQL
+        Shell wizard mode.
+      - restart: boolean value used to indicate that a remote restart of the
+        target instance should be performed to finalize the operation.
+      - applierWorkerThreads: Number of threads used for applying replicated
+        transactions. The default value is 4.
+
+      The connection password may be contained on the instance definition,
+      however, it can be overwritten if it is specified on the options.
+
+      This function reviews the instance configuration to identify if it is
+      valid for usage in replicasets. An exception is thrown if not.
+
+      If the instance was not valid for InnoDB ReplicaSet and interaction is
+      enabled, before configuring the instance a prompt to confirm the changes
+      is presented and a table with the following information:
+
+      - Variable: the invalid configuration variable.
+      - Current Value: the current value for the invalid configuration
+        variable.
+      - Required Value: the required value for the configuration variable.
 
 #@<OUT> dba.create_cluster
 NAME
@@ -321,22 +373,18 @@ SYNTAX
       dba.create_cluster(name[, options])
 
 WHERE
-      name: The name of the cluster object to be created.
-      options: Dictionary with options that modify the behavior of this
-               function.
+      name: An identifier for the cluster to be created.
+      options: Dictionary with additional parameters described below.
 
 RETURNS
-       The created cluster object.
+      The created cluster object.
 
 DESCRIPTION
-      Creates a MySQL InnoDB cluster taking as seed instance the active global
-      session.
+      Creates a MySQL InnoDB cluster taking as seed instance the server the
+      shell is currently connected to.
 
       The options dictionary can contain the following values:
 
-      - interactive: boolean value used to disable the wizards in the command
-        execution, i.e. prompts are not provided to the user and confirmation
-        prompts are not shown.
       - disableClone: boolean value used to disable the clone usage on the
         cluster.
       - gtidSetIsComplete: boolean value which indicates whether the GTID set
@@ -344,11 +392,19 @@ DESCRIPTION
         is false.
       - multiPrimary: boolean value used to define an InnoDB cluster with
         multiple writable instances.
-      - force: boolean, confirms that the multiPrimary option must be applied.
+      - force: boolean, confirms that the multiPrimary option must be applied
+        and/or the operation must proceed even if unmanaged replication
+        channels were detected.
+      - interactive: boolean value used to disable/enable the wizards in the
+        command execution, i.e. prompts and confirmations will be provided or
+        not according to the value set. The default value is equal to MySQL
+        Shell wizard mode.
       - adoptFromGR: boolean value used to create the InnoDB cluster based on
         existing replication group.
       - memberSslMode: SSL mode used to configure the members of the cluster.
       - ipWhitelist: The list of hosts allowed to connect to the instance for
+        group replication. Deprecated.
+      - ipAllowlist: The list of hosts allowed to connect to the instance for
         group replication.
       - groupName: string value with the Group Replication group name UUID to
         be used instead of the automatically generated one.
@@ -357,10 +413,15 @@ DESCRIPTION
       - groupSeeds: string value with a comma-separated list of the Group
         Replication peer addresses to be used instead of the automatically
         generated one.
+      - manualStartOnBoot: boolean (default false). If false, Group Replication
+        in cluster instances will automatically start and rejoin when MySQL
+        starts, otherwise it must be started manually.
       - exitStateAction: string value indicating the group replication exit
         state action.
       - memberWeight: integer value with a percentage weight for automatic
         primary election on failover.
+      - consistency: string value indicating the consistency guarantees that
+        the cluster provides.
       - failoverConsistency: string value indicating the consistency guarantees
         that the cluster provides.
       - expelTimeout: integer value to define the time period in seconds that
@@ -419,16 +480,22 @@ DESCRIPTION
 
       - REQUIRED: if used, SSL (encryption) will be enabled for the instances
         to communicate with other members of the cluster
+      - VERIFY_CA: Like REQUIRED, but additionally verify the server TLS
+        certificate against the configured Certificate Authority (CA)
+        certificates.
+      - VERIFY_IDENTITY: Like VERIFY_CA, but additionally verify that the
+        server certificate matches the host to which the connection is
+        attempted.
       - DISABLED: if used, SSL (encryption) will be disabled
       - AUTO: if used, SSL (encryption) will be enabled if supported by the
         instance, otherwise disabled
 
       If memberSslMode is not specified AUTO will be used by default.
 
-      The ipWhitelist format is a comma separated list of IP addresses or
+      The ipAllowlist format is a comma separated list of IP addresses or
       subnet CIDR notation, for example: 192.168.1.0/24,10.0.0.1. By default
       the value is set to AUTOMATIC, allowing addresses from the instance
-      private network to be automatically set for the whitelist.
+      private network to be automatically set for the allowlist.
 
       The groupName, localAddress, and groupSeeds are advanced options and
       their usage is discouraged since incorrect values can lead to Group
@@ -457,9 +524,13 @@ DESCRIPTION
       The exitStateAction option supports the following values:
 
       - ABORT_SERVER: if used, the instance shuts itself down if it leaves the
-        cluster unintentionally.
+        cluster unintentionally or exhausts its auto-rejoin attempts.
       - READ_ONLY: if used, the instance switches itself to super-read-only
-        mode if it leaves the cluster unintentionally.
+        mode if it leaves the cluster unintentionally or exhausts its
+        auto-rejoin attempts.
+      - OFFLINE_MODE: if used, the instance switches itself to offline mode if
+        it leaves the cluster unintentionally or exhausts its auto-rejoin
+        attempts. Requires MySQL 8.0.18 or newer.
 
       If exitStateAction is not specified READ_ONLY will be used by default.
 
@@ -474,12 +545,16 @@ DESCRIPTION
       If consistency is not specified, EVENTUAL will be used by default.
 
       The value for exitStateAction is used to configure how Group Replication
-      behaves when a server instance leaves the group unintentionally, for
-      example after encountering an applier error. When set to ABORT_SERVER,
-      the instance shuts itself down, and when set to READ_ONLY the server
-      switches itself to super-read-only mode. The exitStateAction option
-      accepts case-insensitive string values, being the accepted values:
-      ABORT_SERVER (or 1) and READ_ONLY (or 0).
+      behaves when a server instance leaves the group unintentionally (for
+      example after encountering an applier error) or exhausts its auto-rejoin
+      attempts. When set to ABORT_SERVER, the instance shuts itself down. When
+      set to READ_ONLY the server switches itself to super-read-only mode. When
+      set to OFFLINE_MODE it switches itself to offline mode. In this mode,
+      connected client users are disconnected on their next request and
+      connections are no longer accepted, with the exception of client users
+      that have the CONNECTION_ADMIN or SUPER privilege. The exitStateAction
+      option accepts case-insensitive string values, being the accepted values:
+      OFFLINE_MODE (or 2), ABORT_SERVER (or 1) and READ_ONLY (or 0).
 
       The default value is READ_ONLY.
 
@@ -560,31 +635,101 @@ DESCRIPTION
       ATTENTION: The failoverConsistency option will be removed in a future
                  release. Please use the consistency option instead.
 
-EXCEPTIONS
-      MetadataError in the following scenarios:
+      ATTENTION: The ipWhitelist option will be removed in a future release.
+                 Please use the ipAllowlist option instead.
 
-      - If the Metadata is inaccessible.
-      - If the Metadata update operation failed.
+#@<OUT> dba.create_replica_set
+NAME
+      create_replica_set - Creates a MySQL InnoDB ReplicaSet.
 
-      ArgumentError in the following scenarios:
+SYNTAX
+      dba.create_replica_set(name[, options])
 
-      - If the Cluster name is empty.
-      - If the Cluster name is not valid.
-      - If the options contain an invalid attribute.
-      - If adoptFromGR is true and the memberSslMode option is used.
-      - If the value for the memberSslMode option is not one of the allowed.
-      - If adoptFromGR is true and the multiPrimary option is used.
-      - If the value for the ipWhitelist, groupName, localAddress, groupSeeds,
-        exitStateAction or consistency options is empty.
-      - If the value for the expelTimeout is not in the range: [0, 3600]
+WHERE
+      name: An identifier for the replicaset to be created.
+      options: Dictionary with additional parameters described below.
 
-      RuntimeError in the following scenarios:
+RETURNS
+      The created replicaset object.
 
-      - If the value for the groupName, localAddress, groupSeeds,
-        exitStateAction, memberWeight, consistency, expelTimeout or
-        autoRejoinTries options is not valid for Group Replication.
-      - If the current connection cannot be used for Group Replication.
-      - If disableClone is not supported on the target instance.
+DESCRIPTION
+      This function will create a managed replicaset using MySQL master/slave
+      replication, as opposed to Group Replication. The MySQL instance the
+      shell is connected to will be the initial PRIMARY of the replica set.
+
+      The function will perform several checks to ensure the instance state and
+      configuration are compatible with a managed replicaset and if so, a
+      metadata schema will be initialized there.
+
+      New replica instances can be added through the add_instance() function of
+      the returned replicaset object. Status of the instances and their
+      replication channels can be inspected with status().
+
+      InnoDB ReplicaSets
+
+      A replicaset allows managing a GTID-based MySQL replication setup, in a
+      single PRIMARY/multiple SECONDARY topology.
+
+      A replicaset has several limitations compared to a InnoDB cluster and
+      thus, it is recommended that InnoDB clusters be preferred unless not
+      possible. Generally, a ReplicaSet on its own does not provide High
+      Availability. Among its limitations are:
+
+      - No automatic failover
+      - No protection against inconsistencies or partial data loss in a crash
+
+      Pre-Requisites
+
+      The following is a non-exhaustive list of requirements for managed
+      replicasets. The dba.configure_instance() command can be used to make
+      necessary configuration changes automatically.
+
+      - MySQL 8.0 or newer required
+      - Statement Based Replication (SBR) is unsupported, only Row Based
+        Replication
+      - GTIDs required
+      - Replication filters are not allowed
+      - All instances in the replicaset must be managed
+      - Unmanaged replication channels are not allowed in any instance
+
+      Adopting an Existing Topology
+
+      Existing asynchronous setups can be managed by calling this function with
+      the adoptFromAR option. The topology will be automatically scanned and
+      validated, starting from the instance the shell is connected to, and all
+      instances that are part of the topology will be automatically added to
+      the replicaset.
+
+      The only changes made by this function to an adopted replicaset are the
+      creation of the metadata schema. Existing replication channels will not
+      be changed during adoption, although they will be changed during PRIMARY
+      switch operations.
+
+      However, it is only possible to manage setups that use supported
+      configurations and topology. Configuration of all instances will be
+      checked during adoption, to ensure they are compatible. All replication
+      channels must be active and their transaction sets as verified through
+      GTID sets must be consistent. The data set of all instances are expected
+      to be identical, but is not verified.
+
+      Options
+
+      The options dictionary can contain the following values:
+
+      - adoptFromAR: boolean value used to create the replicaset based on an
+        existing asynchronous replication setup.
+      - instanceLabel: string a name to identify the target instance. Defaults
+        to hostname:port
+      - dryRun: boolean if true, all validations and steps for creating a
+        replica set are executed, but no changes are actually made. An
+        exception will be thrown when finished.
+      - gtidSetIsComplete: boolean value which indicates whether the GTID set
+        of the seed instance corresponds to all transactions executed. Default
+        is false.
+      - interactive: boolean value used to disable/enable the wizards in the
+        command execution, i.e. prompts and confirmations will be provided or
+        not according to the value set. The default value is equal to MySQL
+        Shell wizard mode.
 
 #@<OUT> dba.delete_sandbox_instance
 NAME
@@ -600,7 +745,7 @@ WHERE
                executed.
 
 RETURNS
-       Nothing.
+      Nothing.
 
 DESCRIPTION
       This function will delete an existing MySQL Server instance on the local
@@ -616,12 +761,6 @@ DESCRIPTION
 
       If the instance is not located on the used path an error will occur.
 
-EXCEPTIONS
-      ArgumentError in the following scenarios:
-
-      - If the options contain an invalid attribute.
-      - If the port value is < 1024 or > 65535.
-
 #@<OUT> dba.deploy_sandbox_instance
 NAME
       deploy_sandbox_instance - Creates a new MySQL Server instance on
@@ -635,7 +774,7 @@ WHERE
       options: Dictionary with options affecting the new deployed instance.
 
 RETURNS
-       Nothing.
+      Nothing.
 
 DESCRIPTION
       This function will deploy a new MySQL Server instance, the result may be
@@ -646,9 +785,11 @@ DESCRIPTION
       - sandboxDir: path where the new instance will be deployed.
       - password: password for the MySQL root user on the new instance.
       - allowRootFrom: create remote root account, restricted to the given
-        address pattern (eg %).
+        address pattern (default: %).
       - ignoreSslError: Ignore errors when adding SSL support for the new
         instance, by default: true.
+      - mysqldOptions: List of MySQL configuration options to write to the
+        my.cnf file, as option=value strings.
 
       If the portx option is not specified, it will be automatically calculated
       as 10 times the value of the provided MySQL port.
@@ -667,17 +808,6 @@ DESCRIPTION
       ignoreSslError option to false to ensure the new instance is deployed
       with SSL support.
 
-EXCEPTIONS
-      ArgumentError in the following scenarios:
-
-      - If the options contain an invalid attribute.
-      - If the root password is missing on the options.
-      - If the port value is < 1024 or > 65535.
-
-      RuntimeError in the following scenarios:
-
-      - If SSL support can be provided and ignoreSslError: false.
-
 #@<OUT> dba.drop_metadata_schema
 NAME
       drop_metadata_schema - Drops the Metadata Schema.
@@ -689,7 +819,7 @@ WHERE
       options: Dictionary containing an option to confirm the drop operation.
 
 RETURNS
-       Nothing.
+      Nothing.
 
 DESCRIPTION
       The options dictionary may contain the following options:
@@ -698,15 +828,6 @@ DESCRIPTION
       - clearReadOnly: boolean value used to confirm that super_read_only must
         be disabled
 
-EXCEPTIONS
-      MetadataError in the following scenarios:
-
-      - If the Metadata is inaccessible.
-
-      RuntimeError in the following scenarios:
-
-      - If the current connection cannot be used for Group Replication.
-
 #@<OUT> dba.get_cluster
 NAME
       get_cluster - Retrieves a cluster from the Metadata Store.
@@ -719,7 +840,7 @@ WHERE
       options: Dictionary with additional options.
 
 RETURNS
-       The cluster object identified by the given name or the default cluster.
+      The cluster object identified by the given name or the default cluster.
 
 DESCRIPTION
       If name is not specified or is null, the default cluster will be
@@ -732,21 +853,44 @@ DESCRIPTION
       defaults to true and indicates the shell to automatically connect to the
       primary member of the cluster.
 
-EXCEPTIONS
-      MetadataError in the following scenarios:
+#@<OUT> dba.get_replica_set
+NAME
+      get_replica_set - Returns an object representing a ReplicaSet.
 
-      - If the Metadata is inaccessible.
-      - If the Metadata update operation failed.
+SYNTAX
+      dba.get_replica_set()
 
-      ArgumentError in the following scenarios:
+DESCRIPTION
+      The returned object is identical to the one returned by
+      create_replica_set() and can be used to manage the replicaset.
 
-      - If the Cluster name is empty.
-      - If the Cluster name is invalid.
-      - If the Cluster does not exist.
+      The function will work regardless of whether the target instance is a
+      PRIMARY or a SECONDARY, but its copy of the metadata is expected to be
+      up-to-date. This function will also work if the PRIMARY is unreachable or
+      unavailable, although replicaset change operations will not be possible,
+      except for force_primary_instance().
 
-      RuntimeError in the following scenarios:
+#@<OUT> dba.get_cluster_set
+NAME
+      get_cluster_set - Returns an object representing a ClusterSet.
 
-      - If the current connection cannot be used for Group Replication.
+SYNTAX
+      dba.get_cluster_set()
+
+RETURNS
+      The ClusterSet object to which the current session belongs to.
+
+DESCRIPTION
+      The returned object is identical to the one returned by
+      create_cluster_set() and can be used to manage the ClusterSet.
+
+      The function will work regardless of whether the active session is
+      established to an instance that belongs to a PRIMARY or a REPLICA
+      Cluster, but its copy of the metadata is expected to be up-to-date.
+
+      This function will also work if the PRIMARY Cluster is unreachable or
+      unavailable, although ClusterSet change operations will not be possible,
+      except for failover with force_primary_cluster().
 
 #@<OUT> dba.help
 NAME
@@ -771,7 +915,7 @@ WHERE
       options: Dictionary with options affecting the result.
 
 RETURNS
-       Nothing.
+      Nothing.
 
 DESCRIPTION
       This function will kill the process of a running MySQL Server instance on
@@ -787,12 +931,6 @@ DESCRIPTION
 
       If the instance is not located on the used path an error will occur.
 
-EXCEPTIONS
-      ArgumentError in the following scenarios:
-
-      - If the options contain an invalid attribute.
-      - If the port value is < 1024 or > 65535.
-
 #@<OUT> dba.reboot_cluster_from_complete_outage
 NAME
       reboot_cluster_from_complete_outage - Brings a cluster back ONLINE when
@@ -807,25 +945,35 @@ WHERE
                function.
 
 RETURNS
-       The rebooted cluster object.
+      The rebooted cluster object.
 
 DESCRIPTION
       The options dictionary can contain the next values:
 
+      - user: The user used for the instances sessions required operations.
       - password: The password used for the instances sessions required
         operations.
       - removeInstances: The list of instances to be removed from the cluster.
-      - rejoinInstances: The list of instances to be rejoined on the cluster.
+      - rejoinInstances: The list of instances to be rejoined to the cluster.
       - clearReadOnly: boolean value used to confirm that super_read_only must
         be disabled
 
       ATTENTION: The clearReadOnly option will be removed in a future release.
 
+      ATTENTION: The user option will be removed in a future release.
+
+      ATTENTION: The password option will be removed in a future release.
+
       This function reboots a cluster from complete outage. It picks the
       instance the MySQL Shell is connected to as new seed instance and
       recovers the cluster. Optionally it also updates the cluster
       configuration based on user provided options.
 
+      NOTE: When used with a metadata version lower than the one supported by
+            this version of the Shell, the removeInstances option is NOT
+            allowed, as in such scenario, no changes to the metadata are
+            allowed.
+
       On success, the restored cluster object is returned by the function.
 
       The current session must be connected to a former instance of the
@@ -833,21 +981,8 @@ DESCRIPTION
 
       If name is not specified, the default cluster will be returned.
 
-EXCEPTIONS
-      MetadataError in the following scenarios:
-
-      - If the Metadata is inaccessible.
-
-      ArgumentError in the following scenarios:
-
-      - If the Cluster name is empty.
-      - If the Cluster name is not valid.
-      - If the options contain an invalid attribute.
-
-      RuntimeError in the following scenarios:
-
-      - If the Cluster does not exist on the Metadata.
-      - If some instance of the Cluster belongs to a Replication Group.
+      NOTE: The user and password options are no longer used, the connection
+            data is taken from the active shell session.
 
 #@<OUT> dba.start_sandbox_instance
 NAME
@@ -862,7 +997,7 @@ WHERE
       options: Dictionary with options affecting the result.
 
 RETURNS
-       Nothing.
+      Nothing.
 
 DESCRIPTION
       This function will start an existing MySQL Server instance on the local
@@ -878,12 +1013,6 @@ DESCRIPTION
 
       If the instance is not located on the used path an error will occur.
 
-EXCEPTIONS
-      ArgumentError in the following scenarios:
-
-      - If the options contain an invalid attribute.
-      - If the port value is < 1024 or > 65535.
-
 #@<OUT> dba.stop_sandbox_instance
 NAME
       stop_sandbox_instance - Stops a running MySQL Server instance on
@@ -897,7 +1026,7 @@ WHERE
       options: Dictionary with options affecting the result.
 
 RETURNS
-       Nothing.
+      Nothing.
 
 DESCRIPTION
       This function will gracefully stop a running MySQL Server instance on the
@@ -914,16 +1043,10 @@ DESCRIPTION
 
       If the instance is not located on the used path an error will occur.
 
-EXCEPTIONS
-      ArgumentError in the following scenarios:
-
-      - If the options contain an invalid attribute.
-      - If the root password is missing on the options.
-      - If the port value is < 1024 or > 65535.
-
 #@<OUT> dba.verbose
 NAME
-      verbose - Enables verbose mode on the dba operations.
+      verbose - Controls debug message verbosity for sandbox related dba
+                operations.
 
 SYNTAX
       dba.verbose
@@ -937,3 +1060,57 @@ DESCRIPTION
       - >1: enables mysqlprovision debug verbosity
       - Boolean: equivalent to assign either 0 or 1
 
+#@<OUT> Upgrade Metadata
+NAME
+      upgrade_metadata - Upgrades (or restores) the metadata to the version
+                         supported by the Shell.
+
+SYNTAX
+      dba.upgrade_metadata([options])
+
+WHERE
+      options: Dictionary with option for the operation.
+
+DESCRIPTION
+      This function will compare the version of the installed metadata schema
+      with the version of the metadata schema supported by this Shell. If the
+      installed metadata version is lower, an upgrade process will be started.
+
+      The options dictionary accepts the following attributes:
+
+      - dryRun: boolean value used to enable a dry run of the upgrade process.
+      - interactive: boolean value used to disable/enable the wizards in the
+        command execution, i.e. prompts and confirmations will be provided or
+        not according to the value set. The default value is equal to MySQL
+        Shell wizard mode.
+
+      If dryRun is used, the function will determine whether a metadata upgrade
+      or restore is required and inform the user without actually executing the
+      operation.
+
+      The interactive option can be used to explicitly enable or disable the
+      interactive prompts that help the user through te upgrade process. The
+      default value is equal to MySQL Shell wizard mode.
+
+      The Upgrade Process
+
+      When upgrading the metadata schema of clusters deployed by Shell versions
+      before 8.0.19, a rolling upgrade of existing MySQL Router instances is
+      required. This process allows minimizing disruption to applications
+      during the upgrade.
+
+      The rolling upgrade process must be performed in the following order:
+
+      1. Execute dba.upgrade_metadata() using the latest Shell version. The
+         upgrade function will stop if outdated Router instances are detected,
+         at which point you can stop the upgrade to resume later.
+      2. Upgrade MySQL Router to the latest version (same version number as the
+         Shell)
+      3. Continue or re-execute dba.upgrade_metadata()
+
+      Failed Upgrades
+
+      If the installed metadata is not available because a previous call to
+      this function ended unexpectedly, this function will restore the metadata
+      to the state it was before the failed upgrade operation.
+
