@@ -1,6 +1,5 @@
 // Assumptions: ensure_schema_does_not_exist is available
 // Assumes __uripwd is defined as <user>:<pwd>@<host>:<plugin_port>
-// validateMemer and validateNotMember are defined on the setup script
 testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
 testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
 testutil.deploySandbox(__mysql_sandbox_port3, "root", {report_host: hostname});
@@ -8,39 +7,41 @@ testutil.deploySandbox(__mysql_sandbox_port3, "root", {report_host: hostname});
 shell.connect(__sandbox_uri1);
 var singleSession = session;
 
-if (__have_ssl)
-  var devCluster = dba.createCluster('devCluster', {memberSslMode:'REQUIRED', gtidSetIsComplete: true});
-else
-  var devCluster = dba.createCluster('devCluster', {memberSslMode:'DISABLED', gtidSetIsComplete: true});
+var devCluster = dba.createCluster('devCluster', {memberSslMode:'REQUIRED', gtidSetIsComplete: true});
 
 devCluster.disconnect();
 
-//@ Cluster: validating members
+//@<> Cluster: validating members
 var Cluster = dba.getCluster('devCluster');
 
-var members = dir(Cluster);
-
-print("Cluster Members:", members.length);
-
-validateMember(members, 'name');
-validateMember(members, 'getName');
-validateMember(members, 'addInstance');
-validateMember(members, 'removeInstance');
-validateMember(members, 'rejoinInstance');
-validateMember(members, 'checkInstanceState');
-validateMember(members, 'describe');
-validateMember(members, 'status');
-validateMember(members, 'help');
-validateMember(members, 'dissolve');
-validateMember(members, 'disconnect');
-validateMember(members, 'rescan');
-validateMember(members, 'forceQuorumUsingPartitionOf');
-validateMember(members, 'switchToSinglePrimaryMode');
-validateMember(members, 'switchToMultiPrimaryMode');
-validateMember(members, 'setPrimaryInstance');
-validateMember(members, 'options');
-validateMember(members, 'setOption');
-validateMember(members, 'setInstanceOption');
+validateMembers(Cluster, [
+  'name',
+  'getClusterSet',
+  'getName',
+  'addInstance',
+  'removeInstance',
+  'rejoinInstance',
+  'checkInstanceState',
+  'describe',
+  'status',
+  'help',
+  'dissolve',
+  'disconnect',
+  'rescan',
+  'listRouters',
+  'removeRouterMetadata',
+  'resetRecoveryAccountsPassword',
+  'forceQuorumUsingPartitionOf',
+  'switchToSinglePrimaryMode',
+  'switchToMultiPrimaryMode',
+  'setPrimaryInstance',
+  'options',
+  'setOption',
+  'setInstanceOption',
+  'setupAdminAccount',
+  'setupRouterAccount',
+  'createClusterSet'
+])
 
 //@ Cluster: addInstance errors
 Cluster.addInstance();
@@ -49,9 +50,7 @@ Cluster.addInstance(5, 5);
 Cluster.addInstance('', 5);
 Cluster.addInstance("");
 Cluster.addInstance({user:"sample", weird:1},{});
-Cluster.addInstance({user:"sample"});
 Cluster.addInstance({host: "localhost", schema: 'abs', user:"sample", "auth-method":56, memberSslMode: "foo", ipWhitelist: " "});
-Cluster.addInstance({port: __mysql_sandbox_port1});
 Cluster.addInstance({dbUser: "root", host: "localhost", port:__mysql_sandbox_port2}, "root");
 Cluster.addInstance({dbUser: "root", host: "localhost", port:__mysql_sandbox_port2}, {memberSslMode: "foo", password: "root"});
 Cluster.addInstance({dbUser: "root", host: "localhost", port:__mysql_sandbox_port2}, {memberSslMode: "", password: "root"});
@@ -78,17 +77,20 @@ testutil.waitMemberState(__mysql_sandbox_port3, "ONLINE");
 //@<OUT> Cluster: describe cluster with instance
 Cluster.describe();
 
-//@<OUT> Cluster: status cluster with instance
-Cluster.status();
+//@<> Cluster: status cluster with instance
+var status = Cluster.status();
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port1}`]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port3}`]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]["2nd"]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]["2nd"]["address"], `${hostname}:${__mysql_sandbox_port2}`);
 
 //@ Cluster: removeInstance errors
 Cluster.removeInstance();
 Cluster.removeInstance(1,2,3);
 Cluster.removeInstance(1);
 Cluster.removeInstance({host: "localhost", schema: 'abs', user:"sample", fakeOption:56});
-Cluster.removeInstance("localhost:3306");
-Cluster.removeInstance("localhost");
-Cluster.removeInstance({host: "localhost"});
+// try to remove instance that is not in the cluster using the classic port
+Cluster.removeInstance({user: __user, host: __host, port: __mysql_port, password: shell.parseUri(__uripwd).password});
 
 //@ Cluster: removeInstance read only
 Cluster.removeInstance({host: "localhost", port:__mysql_sandbox_port2});
@@ -96,8 +98,12 @@ Cluster.removeInstance({host: "localhost", port:__mysql_sandbox_port2});
 //@<OUT> Cluster: describe removed read only member
 Cluster.describe();
 
-//@<OUT> Cluster: status removed read only member
-Cluster.status();
+//@<> Cluster: status removed read only member
+var status = Cluster.status();
+EXPECT_EQ(2, Object.keys(status["defaultReplicaSet"]["topology"]).length)
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port1}`]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port3}`]["status"], "ONLINE");
+
 
 //@ Cluster: addInstance read only back
 Cluster.addInstance(__sandbox_uri2);
@@ -107,8 +113,11 @@ testutil.waitMemberState(__mysql_sandbox_port2, "ONLINE");
 //@<OUT> Cluster: describe after adding read only instance back
 Cluster.describe();
 
-//@<OUT> Cluster: status after adding read only instance back
-Cluster.status();
+//@<> Cluster: status after adding read only instance back
+var status = Cluster.status();
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port1}`]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port2}`]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port3}`]["status"], "ONLINE");
 
 // Make sure uri2 is selected as the new master
 Cluster.removeInstance(uri3);
@@ -139,8 +148,13 @@ testutil.waitMemberState(__mysql_sandbox_port3, "ONLINE");
 //@<OUT> Cluster: describe on new master
 Cluster.describe();
 
-//@<OUT> Cluster: status on new master
-Cluster.status();
+//@<> Cluster: status on new master
+var status = Cluster.status();
+EXPECT_EQ(2, Object.keys(status["defaultReplicaSet"]["topology"]).length)
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port2}`]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port2}`]["mode"], "R/W");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]['3rd_sandbox']["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]["3rd_sandbox"]["address"], `${hostname}:${__mysql_sandbox_port3}`);
 
 //@ Cluster: addInstance adding old master as read only
 Cluster.addInstance(__sandbox_uri1, {'label': '1st_sandbox'});
@@ -149,8 +163,14 @@ testutil.waitMemberState(__mysql_sandbox_port1, "ONLINE");
 //@<OUT> Cluster: describe on new master with slave
 Cluster.describe();
 
-//@<OUT> Cluster: status on new master with slave
-Cluster.status();
+//@<> Cluster: status on new master with slave
+var status = Cluster.status();
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]['1st_sandbox']["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]["1st_sandbox"]["address"], `${hostname}:${__mysql_sandbox_port1}`);
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port2}`]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port2}`]["mode"], "R/W");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]['3rd_sandbox']["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]["3rd_sandbox"]["address"], `${hostname}:${__mysql_sandbox_port3}`);
 
 // Rejoin tests
 
@@ -177,10 +197,7 @@ Cluster.rejoinInstance({dbUser: "root", host: "localhost", port:__mysql_sandbox_
 Cluster.rejoinInstance({dbUser: "root", host: "localhost", port:__mysql_sandbox_port3}, {ipWhitelist: " ", password: "root"});
 
 //@#: Dba: rejoin instance 3 ok {VER(<8.0.11)}
-if (__have_ssl)
-  Cluster.rejoinInstance({dbUser: "root", host: "localhost", port:__mysql_sandbox_port3}, {memberSslMode: "AUTO", "password": "root"});
-else
-  Cluster.rejoinInstance({dbUser: "root", host: "localhost", port:__mysql_sandbox_port3, "password":"root"});
+Cluster.rejoinInstance({dbUser: "root", host: "localhost", port:__mysql_sandbox_port3}, {memberSslMode: "AUTO", "password": "root"});
 
 testutil.waitMemberState(__mysql_sandbox_port3, "ONLINE");
 
@@ -189,8 +206,14 @@ testutil.waitMemberState(__mysql_sandbox_port3, "ONLINE");
 
 // Verify if the cluster is OK
 
-//@<OUT> Cluster: status for rejoin: success
-Cluster.status();
+//@<> Cluster: status for rejoin: success
+var status = Cluster.status();
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]['1st_sandbox']["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]["1st_sandbox"]["address"], `${hostname}:${__mysql_sandbox_port1}`);
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port2}`]["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"][`${hostname}:${__mysql_sandbox_port2}`]["mode"], "R/W");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]['3rd_sandbox']["status"], "ONLINE");
+EXPECT_EQ(status["defaultReplicaSet"]["topology"]["3rd_sandbox"]["address"], `${hostname}:${__mysql_sandbox_port3}`);
 
 //@ Cluster: dissolve errors
 Cluster.dissolve(1);
