@@ -1,29 +1,4 @@
 
-//@ Schema: validating members
-|Member Count: 18|
-
-|name: OK|
-|schema: OK|
-|session: OK|
-|existsInDatabase: OK|
-|getName: OK|
-|getSchema: OK|
-|getSession: OK|
-|getTable: OK|
-|getTables: OK|
-|getCollection: OK|
-|getCollections: OK|
-|createCollection: OK|
-|getCollectionAsTable: OK|
-|help: OK|
-|dropCollection: OK|
-|dropView: OK|
-|dropTable: OK|
-
-|table1: OK|
-|view1: OK|
-|collection1: OK|
-
 //@ Testing schema name retrieving
 |getName(): js_shell_test|
 |name: js_shell_test|
@@ -35,8 +10,8 @@
 |session: <Session:|
 
 //@ Testing schema schema retrieving
-|getSchema(): null|
-|schema: null|
+|getSchema(): <Schema:js_shell_test>|
+|schema: <Schema:js_shell_test>|
 
 //@ Testing tables, views and collection retrieval
 |getTables(): <Table:table1>|
