@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -110,6 +110,8 @@ class Shell_cmdline_options : public tests::Shell_base_test {
       return options->ssl_options.get_cipher();
     else if (option == "tls-version")
       return options->ssl_options.get_tls_version();
+    else if (option == "tls-ciphersuites")
+      return options->ssl_options.get_tls_ciphersuites();
     else if (option == "uri")
       return options->uri;
     else if (option == "result_format")
@@ -151,7 +153,9 @@ class Shell_cmdline_options : public tests::Shell_base_test {
     else if (option == "showColumnTypeInfo")
       return AS__STRING(options->show_column_type_info);
     else if (option == "compress")
-      return AS__STRING(options->compress);
+      return options->compress;
+    else if (option == "mysqlPluginDir")
+      return options->mysql_plugin_dir;
 
     return "";
   }
@@ -686,6 +690,7 @@ TEST_F(Shell_cmdline_options, default_values) {
   EXPECT_TRUE(!options.ssl_options.has_crlpath());
   EXPECT_TRUE(!options.ssl_options.has_cipher());
   EXPECT_TRUE(!options.ssl_options.has_tls_version());
+  EXPECT_FALSE(options.ssl_options.has_tls_ciphersuites());
   EXPECT_FALSE(options.trace_protocol);
   EXPECT_TRUE(options.uri.empty());
   EXPECT_TRUE(options.user.empty());
@@ -695,8 +700,10 @@ TEST_F(Shell_cmdline_options, default_values) {
   EXPECT_TRUE(options.m_connect_timeout.empty());
   EXPECT_EQ(Shell_options::Quiet_start::NOT_SET, options.quiet_start);
   EXPECT_FALSE(options.show_column_type_info);
-  EXPECT_FALSE(options.compress);
+  EXPECT_TRUE(options.compress.empty());
   EXPECT_FALSE(options.default_compress);
+  EXPECT_TRUE(options.compress_algorithms.empty());
+  EXPECT_TRUE(options.compress_level.is_null());
 }
 
 TEST_F(Shell_cmdline_options, app) {
@@ -721,8 +728,8 @@ TEST_F(Shell_cmdline_options, app) {
 #endif
   test_option_with_value("connect-timeout", "", "1000", "", IS_CONNECTION_DATA,
                          !IS_NULLABLE);
-  test_option_with_no_value("-C", "compress", "1");
-  test_option_with_no_value("--compress", "compress", "1");
+  test_option_with_no_value("-C", "compress", "REQUIRED");
+  test_option_with_no_value("--compress", "compress", "REQUIRED");
   test_option_with_no_value("-p", "prompt_password", "1");
 
   test_option_equal_value("dbpassword", "mypwd", IS_CONNECTION_DATA,
@@ -746,6 +753,8 @@ TEST_F(Shell_cmdline_options, app) {
                          !IS_NULLABLE);
   test_option_with_value("ssl-key", "", "some_value", "", IS_CONNECTION_DATA,
                          !IS_NULLABLE);
+  test_option_with_value("tls-ciphersuites", "", "some_value", "",
+                         IS_CONNECTION_DATA, !IS_NULLABLE);
 
   test_option_with_value("execute", "e", "show databases;", "",
                          !IS_CONNECTION_DATA, !IS_NULLABLE,
@@ -825,6 +834,9 @@ TEST_F(Shell_cmdline_options, app) {
 
   test_option_with_value("file", "f", "/some/file", "", !IS_CONNECTION_DATA,
                          !IS_NULLABLE, "run_file");
+
+  test_option_with_value("mysql-plugin-dir", "", "/some/path", "",
+                         !IS_CONNECTION_DATA, IS_NULLABLE, "mysqlPluginDir");
 }
 
 TEST_F(Shell_cmdline_options, test_session_type_conflicts) {
@@ -1254,6 +1266,18 @@ TEST_F(Shell_cmdline_options, conflicting_port_and_socket) {
   test_conflicting_options("--uri --socket", 3, argv2, error2);
 }
 
+TEST_F(Shell_cmdline_options, conflicts_compression) {
+  char uri[] = {"--uri=mysqlx://root:password@localhost"};
+  char *argv0[] = {const_cast<char *>("ut"), const_cast<char *>("--compress"),
+                   const_cast<char *>("--compression-algorithms=uncompressed"),
+                   uri, NULL};
+  Shell_options so(4, argv0);
+  EXPECT_EQ(0, so.get().exit_code);
+  EXPECT_THROW_LIKE(so.get().connection_options(), std::invalid_argument,
+                    "Conflicting connection options: compression=REQUIRED, "
+                    "compression-algorithms=uncompressed");
+}
+
 TEST_F(Shell_cmdline_options, test_uri_with_password) {
   // Redirect cerr.
   std::streambuf *backup = std::cerr.rdbuf();
@@ -1431,24 +1455,6 @@ TEST_F(Shell_cmdline_options, invalid_connect_timeout) {
   }
 }
 
-#ifdef WITH_OCI
-TEST_F(Shell_cmdline_options, conflicts_oci) {
-  auto error =
-      "Conflicting options: --oci can not be used unless initial mode is "
-      "Python.\n";
-  std::vector<std::string> options = {"--js", "--javascript", "--sql", "--sqlc",
-                                      "--sqlc"};
-
-  for (const auto &option : options) {
-    char *argv0[] = {const_cast<char *>("ut"),
-                     const_cast<char *>(option.c_str()),
-                     const_cast<char *>("--oci"), NULL};
-
-    test_conflicting_options(option + " --oci", 3, argv0, error);
-  }
-}
-#endif
-
 TEST_F(Shell_cmdline_options, conflicts_execute_and_file) {
   static constexpr auto error =
       "Conflicting options: --execute and --file cannot be used at the same "
