@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -20,6 +20,7 @@
    51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
 #ifdef HAVE_PYTHON
+#include "mysqlshdk/include/scripting/python_context.h"
 #include "mysqlshdk/include/scripting/python_utils.h"
 #endif
 
@@ -34,8 +35,8 @@
 #include <clocale>
 #include <fstream>
 #include <iostream>
+#include <set>
 
-#include "my_dbug.h"
 #include "mysqlshdk/libs/db/replay/setup.h"
 #include "mysqlshdk/libs/textui/textui.h"
 #include "mysqlshdk/libs/utils/debug.h"
@@ -55,8 +56,7 @@ using Version = mysqlshdk::utils::Version;
 
 // Begin test configuration block
 
-// TODO(.) remove Interrupt_ from the filter, delete the deprecated Python tests
-const char *k_default_test_filter = "*:-Interrupt_mysql.*";
+const char *k_default_test_filter = "*";
 
 // Default execution mode for replayable tests
 mysqlshdk::db::replay::Mode g_test_recording_mode =
@@ -69,23 +69,24 @@ int g_test_trace_scripts = 0;
 int g_test_default_verbosity = 0;
 bool g_test_fail_early = false;
 int g_test_color_output = 0;
+bool g_bp = false;
+std::set<int> g_break;
 
 // Default trace set (MySQL version) to be used for replay mode
 mysqlshdk::utils::Version g_target_server_version = Version("8.0.16");
+// Highest supported TLS version by MySQL Server
+mysqlshdk::utils::Version g_highest_server_tls_version = Version();
+// Highest common (server<->client) TLS supported version
 mysqlshdk::utils::Version g_highest_tls_version = Version();
 
 // End test configuration block
 
-#ifdef _WIN32
-#define putenv _putenv
-#endif
-
 extern "C" {
 const char *g_test_home = nullptr;
 }
 const char *g_mysqlsh_path;
-char *g_mppath = nullptr;
-bool g_profile_test_scripts = false;
+const char *g_mysqld_path_variables;
+int g_profile_test_scripts = 0;
 
 std::vector<std::pair<std::string, std::string>> g_skipped_tests;
 std::vector<std::pair<std::string, std::string>> g_skipped_chunks;
@@ -119,17 +120,19 @@ static void detect_mysql_environment(int port, const char *pwd) {
   // if connect succeeds or error is a server error, then there's a server
   if (mysql_real_connect(mysql, "127.0.0.1", "root", pwd, NULL, port, NULL,
                          0)) {
-    const char *query = "show variables like '%socket'";
-    if (mysql_real_query(mysql, query, strlen(query)) == 0) {
-      MYSQL_RES *res = mysql_store_result(mysql);
-      while (MYSQL_ROW row = mysql_fetch_row(res)) {
-        auto lengths = mysql_fetch_lengths(res);
-        if (strcmp(row[0], "socket") == 0 && row[1])
-          socket = std::string(row[1], lengths[1]);
-        if (strcmp(row[0], "mysqlx_socket") == 0 && row[1])
-          xsocket = std::string(row[1], lengths[1]);
+    {
+      const char *query = "show variables like '%socket'";
+      if (mysql_real_query(mysql, query, strlen(query)) == 0) {
+        MYSQL_RES *res = mysql_store_result(mysql);
+        while (MYSQL_ROW row = mysql_fetch_row(res)) {
+          auto lengths = mysql_fetch_lengths(res);
+          if (strcmp(row[0], "socket") == 0 && row[1])
+            socket = std::string(row[1], lengths[1]);
+          if (strcmp(row[0], "mysqlx_socket") == 0 && row[1])
+            xsocket = std::string(row[1], lengths[1]);
+        }
+        mysql_free_result(res);
       }
-      mysql_free_result(res);
     }
 
     {
@@ -159,31 +162,44 @@ static void detect_mysql_environment(int port, const char *pwd) {
     {
       const char *query =
           "select @@version, @@have_ssl = 'YES', @@have_openssl = 'YES', "
-          "@@mysqlx_port, @@server_id";
+          "@@server_id";
       if (mysql_real_query(mysql, query, strlen(query)) == 0) {
         MYSQL_RES *res = mysql_store_result(mysql);
         if (MYSQL_ROW row = mysql_fetch_row(res)) {
           g_target_server_version = mysqlshdk::utils::Version(row[0]);
           if (row[1] && strcmp(row[1], "1") == 0) have_ssl = true;
           if (row[2] && strcmp(row[2], "1") == 0) have_openssl = true;
-          if (row[3]) xport = atoi(row[3]);
-          server_id = atoi(row[4]);
+          server_id = atoi(row[3]);
         }
         mysql_free_result(res);
       }
     }
 
+    {
+      const char *query = "select @@mysqlx_port";
+      if (mysql_real_query(mysql, query, strlen(query)) == 0) {
+        MYSQL_RES *res = mysql_store_result(mysql);
+        if (MYSQL_ROW row = mysql_fetch_row(res)) {
+          if (row[0]) xport = atoi(row[0]);
+        }
+        mysql_free_result(res);
+      }
+    }
+
+    // highest server tls version
     {
       char const *const query = "SELECT @@tls_version";
       if (mysql_real_query(mysql, query, strlen(query)) == 0) {
         MYSQL_RES *res = mysql_store_result(mysql);
         if (MYSQL_ROW row = mysql_fetch_row(res)) {
           auto tls_versions = shcore::str_split(row[0], ",");
+          // we assume that highest tls version string is last
           for (auto i = tls_versions.crbegin(); i != tls_versions.crend();
                i++) {
             if (shcore::str_beginswith(tls_versions.back(), "TLSv")) {
               std::string ver((*i).begin() + 4, (*i).end());
-              g_highest_tls_version = Version(ver);
+              g_highest_server_tls_version = g_highest_tls_version =
+                  Version(ver);
               break;
             }
           }
@@ -191,6 +207,27 @@ static void detect_mysql_environment(int port, const char *pwd) {
         mysql_free_result(res);
       }
     }
+
+    // highest client tls version. Might be empty if user do not use TLS
+    // connection.
+    // Update common (server<->user) highest supported TLS version.
+    {
+      char const *const query = "show status like 'Ssl_version'";
+      if (mysql_real_query(mysql, query, strlen(query)) == 0) {
+        MYSQL_RES *res = mysql_store_result(mysql);
+        if (MYSQL_ROW row = mysql_fetch_row(res)) {
+          const std::string tls_version = row[1];
+          if (shcore::str_beginswith(tls_version, "TLSv")) {
+            std::string ver(tls_version.begin() + 4, tls_version.end());
+            auto client_tls_version = Version(ver);
+            g_highest_tls_version =
+                std::min(client_tls_version, g_highest_server_tls_version);
+          }
+        }
+        mysql_free_result(res);
+      }
+    }
+
   } else {
     std::cerr << "Cannot connect to MySQL server at " << port << ": "
               << mysql_error(mysql) << "\n";
@@ -198,7 +235,7 @@ static void detect_mysql_environment(int port, const char *pwd) {
   }
   mysql_close(mysql);
 
-  if (!xport) {
+  if (!xport && g_target_server_version >= mysqlshdk::utils::Version(5, 7, 0)) {
     std::cerr << "Could not query mysqlx_port. X plugin not installed?\n";
     exit(1);
   }
@@ -222,9 +259,10 @@ static void detect_mysql_environment(int port, const char *pwd) {
   std::cout << "hostname=" << hostname << ", ip=" << hostname_ip << "\n";
   std::cout << "report_host=" << report_host << "\n";
   std::cout << "server_id=" << server_id << ", ssl=" << have_ssl
-            << ", openssl=" << have_openssl
-            << ", highest_tls_version=" << g_highest_tls_version.get_full()
-            << "\n";
+            << ", openssl=" << have_openssl << ", highest_server_tls_version="
+            << g_highest_server_tls_version.get_full()
+            << ", highest_common_tls_version="
+            << g_highest_tls_version.get_full() << "\n";
 
   std::cout << "Classic protocol:\n";
   std::cout << "  port=" << port << '\n';
@@ -239,9 +277,7 @@ static void detect_mysql_environment(int port, const char *pwd) {
                                               : "\n");
 
   {
-    static char path[1024];
-    snprintf(path, sizeof(path), "MYSQL_SOCKET=%s", socket_absolute.c_str());
-    if (putenv(path) != 0) {
+    if (!shcore::setenv("MYSQL_SOCKET", socket_absolute)) {
       std::cerr << "MYSQL_SOCKET putenv failed to set it\n";
       exit(1);
     }
@@ -250,23 +286,17 @@ static void detect_mysql_environment(int port, const char *pwd) {
   {
     // This environment variable makes libmysqlclient override the default
     // compiled-in socket path with the actual path in use
-    static char path[1024];
-    snprintf(path, sizeof(path), "MYSQL_UNIX_PORT=%s", socket_absolute.c_str());
-    putenv(path);
+    shcore::setenv("MYSQL_UNIX_PORT", socket_absolute);
   }
 
   {
-    static char path[1024];
-    snprintf(path, sizeof(path), "MYSQLX_SOCKET=%s", xsocket_absolute.c_str());
-    if (putenv(path) != 0) {
+    if (!shcore::setenv("MYSQLX_SOCKET", xsocket_absolute)) {
       std::cerr << "MYSQLX_SOCKET putenv failed to set it\n";
       exit(1);
     }
   }
   {
-    static char path[1024];
-    snprintf(path, sizeof(path), "MYSQLX_PORT=%i", xport);
-    if (putenv(path) != 0) {
+    if (!shcore::setenv("MYSQLX_PORT", std::to_string(xport))) {
       std::cerr << "MYSQLX_SOCKET putenv failed to set it\n";
       exit(1);
     }
@@ -283,26 +313,118 @@ static void detect_mysql_environment(int port, const char *pwd) {
   // to an address that can be reached from outside.
   // OTOH MYSQL_REAL_HOSTNAME will always have @@hostname
   if (!getenv("MYSQL_HOSTNAME")) {
-    static char my_hostname[1024];
-    snprintf(my_hostname, sizeof(my_hostname), "MYSQL_HOSTNAME=%s",
-             hostname.c_str());
-    if (putenv(my_hostname) != 0) {
+    if (!shcore::setenv("MYSQL_HOSTNAME", hostname)) {
       std::cerr << "MYSQL_HOSTNAME could not be set\n";
       exit(1);
     }
   }
 
   {
-    static char my_hostname[1024];
-    snprintf(my_hostname, sizeof(my_hostname), "MYSQL_REAL_HOSTNAME=%s",
-             hostname.c_str());
-    if (putenv(my_hostname) != 0) {
+    if (!shcore::setenv("MYSQL_REAL_HOSTNAME", hostname)) {
       std::cerr << "MYSQL_REAL_HOSTNAME could not be set\n";
       exit(1);
     }
   }
 }
 
+static std::string verify_target_server(
+    const std::string &variable,
+    const mysqlshdk::utils::Version &expected_version,
+    std::vector<std::string> *path_variables) {
+  const char *path = getenv(variable.c_str());
+
+  if (!path) {
+    return shcore::str_format(
+        "The environment variable %s must be defined with the path to the "
+        "MySQL Server %s binary",
+        variable.c_str(), expected_version.get_base().c_str());
+  } else {
+    mysqlshdk::utils::Version actual_version(
+        tests::Testutils::get_mysqld_version(path));
+    // if patch version is not used, accept any patch version as long as major
+    // and minor versions match
+    if (expected_version == actual_version ||
+        (expected_version.get_patch() == 0 &&
+         expected_version.get_major() == actual_version.get_major() &&
+         expected_version.get_minor() == actual_version.get_minor())) {
+      path_variables->push_back(variable);
+    } else {
+      return shcore::str_format(
+          "The environment variable %s must be defined with the path to the "
+          "MySQL Server %s binary, it points to a MySQL Server %s",
+          variable.c_str(), expected_version.get_base().c_str(),
+          actual_version.get_base().c_str());
+    }
+  }
+  return "";
+}
+
+/*
+ * Verifies the target servers to be used on the test session.
+ * returns the base server version.
+ */
+static std::string verify_target_servers(const std::string &target,
+                                         std::string *path_variables) {
+  std::vector<std::string> path_var_list;
+  std::string new_target;
+  if (target.empty()) {
+    new_target = g_target_server_version.get_base();
+  } else {
+    auto versions = shcore::str_split(target, "+");
+
+    // The base server version is the first one
+    new_target = versions[0];
+
+    // The rest must have their paths defined in MYSQLD<M><m>[r]_PATH env
+    // variables
+    auto count = versions.size();
+    std::vector<std::string> errors;
+    // if just 2 versions are given, a MYSQLDMm_PATH env var should be required
+    // with the specific server version (only if the major version differs from
+    // the major version of the base server)
+    if (count == 2) {
+      mysqlshdk::utils::Version secondary(versions[1]);
+
+      if (g_target_server_version.get_major() != secondary.get_major() ||
+          g_target_server_version.get_minor() != secondary.get_minor()) {
+        std::string variable = shcore::str_format(
+            "MYSQLD%d%d_PATH", secondary.get_major(), secondary.get_minor());
+
+        std::string error =
+            verify_target_server(variable, secondary, &path_var_list);
+        if (!error.empty()) errors.push_back(error);
+      }
+    }
+
+    // If no target variables yet and server count is more than 1 then specific
+    // MYSQLDMmp_PATH variables should be defined
+    if (count > 1 && path_var_list.empty() && errors.empty()) {
+      for (size_t index = 1; index < versions.size(); index++) {
+        mysqlshdk::utils::Version secondary(versions[index]);
+
+        std::string variable =
+            shcore::str_format("MYSQLD%d%d%d_PATH", secondary.get_major(),
+                               secondary.get_minor(), secondary.get_patch());
+
+        std::string error =
+            verify_target_server(variable, secondary, &path_var_list);
+        if (!error.empty()) errors.push_back(error);
+      }
+    }
+
+    if (!errors.empty()) {
+      std::cerr << "Please correct the following errors regarding the target "
+                   "servers for the test execution:\n"
+                << shcore::str_join(errors, "\n").c_str() << std::endl;
+      exit(1);
+    } else {
+      *path_variables = shcore::str_join(path_var_list, ",");
+    }
+  }
+
+  return new_target;
+}
+
 static bool delete_sandbox(int port) {
   MYSQL *mysql;
   mysql = mysql_init(nullptr);
@@ -342,12 +464,11 @@ static bool delete_sandbox(int port) {
   return true;
 }
 
-static void check_zombie_sandboxes(
-    int sports[tests::k_max_default_sandbox_ports]) {
+static void check_zombie_sandboxes(int sports[tests::sandbox::k_num_ports]) {
   bool have_zombies = false;
   std::string ports;
 
-  for (int i = 0; i < tests::k_max_default_sandbox_ports; i++) {
+  for (int i = 0; i < tests::sandbox::k_num_ports; ++i) {
     have_zombies |= !delete_sandbox(sports[i]);
     if (!ports.empty()) ports.append(", ");
     ports.append(std::to_string(sports[i]));
@@ -359,7 +480,7 @@ static void check_zombie_sandboxes(
     std::cout << "If they're left from a previous run, terminate them first\n";
     std::cout << "Or setenv TEST_SKIP_ZOMBIE_CHECK to skip this check\n";
     std::cout << "Or setenv MYSQL_SANDBOX_PORT1.."
-              << tests::k_max_default_sandbox_ports + 1
+              << tests::sandbox::k_num_ports + 1
               << " to pick different ports for "
                  "test sandboxes\n";
     exit(1);
@@ -374,10 +495,6 @@ static void catch_segv(int sig) {
 }
 #endif
 
-#ifdef _WIN32
-#define unsetenv(var) _putenv(var "=")
-#endif
-
 #ifdef __APPLE__
 static std::string get_test_keychain() {
   static constexpr auto k_keychain = "mysqlsh-test-keychain";
@@ -398,8 +515,7 @@ static void setup_test_keychain() {
             << system(("security set-keychain-settings " + keychain).c_str())
             << std::endl;
 
-  static std::string env = "MYSQLSH_CREDENTIAL_STORE_KEYCHAIN=" + keychain;
-  putenv(&env[0]);
+  shcore::setenv("MYSQLSH_CREDENTIAL_STORE_KEYCHAIN", keychain);
 }
 
 static void remove_test_keychain() {
@@ -418,7 +534,7 @@ class Fail_logger : public EmptyTestEventListener {
   }
 
   // Called before a test starts.
-  void OnTestStart(const ::testing::TestInfo &test_info) override {
+  void OnTestStart(const ::testing::TestInfo & /* test_info */) override {
     m_failed = false;
   }
 
@@ -465,9 +581,48 @@ static void setup_test_skipper() {
   listeners.Append(new testing::Fail_logger());
 }
 
+std::shared_ptr<shcore::Logger> setup_logger() {
+  // Set HOME to same as TMPDIR
+  {
+    if (!shcore::setenv("HOME", getenv("TMPDIR"))) {
+      std::cerr << "HOME could not be overriden\n";
+      exit(1);
+    }
+  }
+
+  if (!getenv("MYSQLSH_USER_CONFIG_HOME")) {
+    // Override the configuration home for tests, to not mess with custom data
+    if (!shcore::setenv("MYSQLSH_USER_CONFIG_HOME",
+                        shcore::get_binary_folder())) {
+      std::cerr << "MYSQLSH_USER_CONFIG_HOME could not be set with putenv\n";
+    }
+  }
+
+  // Setup logger with default configs
+  std::string log_path =
+      shcore::path::join_path(shcore::get_user_config_path(), "mysqlsh.log");
+  if (shcore::path_exists(log_path)) {
+    std::cerr << "Deleting old " << log_path << " file\n";
+    shcore::delete_file(log_path);
+  }
+
+  shcore::Logger::LOG_LEVEL log_level = shcore::Logger::LOG_LEVEL::LOG_INFO;
+  const char *rut_log_level = getenv("RUT_LOG_LEVEL");
+  if (rut_log_level) {
+    try {
+      log_level = shcore::Logger::parse_log_level(rut_log_level);
+    } catch (const std::invalid_argument &err) {
+      std::cerr << err.what() << std::endl;
+      std::cerr << "Using default log level: info" << std::endl;
+    }
+  }
+
+  return shcore::Logger::create_instance(log_path.c_str(), false, log_level);
+}
+
 void setup_test_environment() {
   if (!getenv("MYSQL_PORT")) {
-    if (putenv(const_cast<char *>("MYSQL_PORT=3306")) != 0) {
+    if (!shcore::setenv("MYSQL_PORT", "3306")) {
       std::cerr << "MYSQL_PORT was not set and putenv failed to set it\n";
       exit(1);
     }
@@ -476,14 +631,13 @@ void setup_test_environment() {
   detect_mysql_environment(atoi(getenv("MYSQL_PORT")), "");
 
   if (!getenv("MYSQL_REMOTE_HOST")) {
-    static char hostname[1024] = "MYSQL_REMOTE_HOST=";
-    if (gethostname(hostname + strlen(hostname),
-                    sizeof(hostname) - strlen(hostname)) != 0) {
+    char hostname[1024] = {0};
+    if (gethostname(hostname, sizeof(hostname)) != 0) {
       std::cerr << "gethostname() returned error: " << strerror(errno) << "\n";
       std::cerr << "Set MYSQL_REMOTE_HOST\n";
       // exit(1); this option is not used for now, so no need to fail
     }
-    if (putenv(hostname) != 0) {
+    if (!shcore::setenv("MYSQL_REMOTE_HOST", hostname)) {
       std::cerr
           << "MYSQL_REMOTE_HOST was not set and putenv failed to set it\n";
       // exit(1);
@@ -491,7 +645,7 @@ void setup_test_environment() {
   }
 
   if (!getenv("MYSQL_REMOTE_PORT")) {
-    if (putenv(const_cast<char *>("MYSQL_REMOTE_PORT=3306")) != 0) {
+    if (!shcore::setenv("MYSQL_REMOTE_PORT", "3306")) {
       std::cerr
           << "MYSQL_REMOTE_PORT was not set and putenv failed to set it\n";
       // exit(1);
@@ -504,11 +658,10 @@ void setup_test_environment() {
   const char *tmpdir = getenv("TMPDIR");
   if (tmpdir == nullptr || strlen(tmpdir) == 0) {
     tmpdir = getenv("TEMP");  // TEMP usually used on Windows.
-    static std::string temp("TMPDIR=");
+    std::string temp;
     if (tmpdir == nullptr || strlen(tmpdir) == 0) {
       // Use the binary folder as default for the TMPDIR.
-      std::string bin_folder = shcore::get_binary_folder();
-      temp.append(bin_folder);
+      temp = shcore::get_binary_folder();
       std::cout << std::endl
                 << "WARNING: TMPDIR environment variable is "
                    "empty or not defined. It will be set with the binary "
@@ -516,68 +669,29 @@ void setup_test_environment() {
                 << temp << std::endl
                 << std::endl;
     } else {
-      temp.append(tmpdir);
+      temp = tmpdir;
     }
-    if (putenv(&temp[0]) != 0) {
+    if (!shcore::setenv("TMPDIR", temp)) {
       std::cerr << "TMPDIR was not set and putenv failed to set it\n";
       exit(1);
     }
   }
 
-  // Set HOME to same as TMPDIR
-  {
-    static std::string home("HOME=");
-    home.append(getenv("TMPDIR"));
-    if (putenv(&home[0]) != 0) {
-      std::cerr << "HOME could not be overriden\n";
-      exit(1);
-    }
-  }
-
-  if (!getenv("MYSQLSH_USER_CONFIG_HOME")) {
-    // Override the configuration home for tests, to not mess with custom data
-    auto user_config = shcore::get_binary_folder();
-    user_config = "MYSQLSH_USER_CONFIG_HOME=" + user_config;
-    if (putenv(const_cast<char *>(user_config.c_str())) != 0) {
-      std::cerr << "MYSQLSH_USER_CONFIG_HOME could not be set with putenv\n";
-    }
-  }
-
-  // Setup logger with default configs
-  std::string log_path =
-      shcore::path::join_path(shcore::get_user_config_path(), "mysqlsh.log");
-  if (shcore::path_exists(log_path)) {
-    std::cerr << "Deleting old " << log_path << " file\n";
-    shcore::delete_file(log_path);
-  }
-  shcore::Logger::setup_instance(log_path.c_str(), false);
-
-  int sports[tests::k_max_default_sandbox_ports];  // NOLINT
-  for (int i = 0; i < tests::k_max_default_sandbox_ports; i++) {
-    const char *sandbox_port =
-        getenv(shcore::str_format("MYSQL_SANDBOX_PORT%i", i + 1).c_str());
-    if (sandbox_port) {
-      sports[i] = atoi(sandbox_port);
-    } else {
-      sports[i] = std::stoi(getenv("MYSQL_PORT")) + (i + 1) * 10;
-    }
-  }
+  tests::sandbox::initialize();
 
   // Check for leftover sandbox servers
   if (!getenv("TEST_SKIP_ZOMBIE_CHECK")) {
-    check_zombie_sandboxes(sports);
+    check_zombie_sandboxes(tests::sandbox::k_ports);
   }
-  tests::Shell_test_env::setup_env(sports);
+  tests::Shell_test_env::setup_env();
 
   tests::Testutils::validate_boilerplate(getenv("TMPDIR"));
 
-  putenv(const_cast<char *>("MYSQLSH_CREDENTIAL_STORE_HELPER=<disabled>"));
+  shcore::setenv("MYSQLSH_CREDENTIAL_STORE_HELPER", "<disabled>");
 
   if (!getenv("MYSQL_TEST_LOGIN_FILE")) {
-    static std::string env =
-        "MYSQL_TEST_LOGIN_FILE=" +
-        shcore::path::join_path(getenv("TMPDIR"), ".mylogin.cnf");
-    putenv(&env[0]);
+    shcore::setenv("MYSQL_TEST_LOGIN_FILE",
+                   shcore::path::join_path(getenv("TMPDIR"), ".mylogin.cnf"));
   }
 
 #ifdef __APPLE__
@@ -585,10 +699,18 @@ void setup_test_environment() {
 #endif  // __APPLE__
 
   // disable PAGER so it doesn't break the tests
-  unsetenv("PAGER");
+  shcore::unsetenv("PAGER");
+
+  // disable \edit-related variables
+  shcore::unsetenv("EDITOR");
+  shcore::unsetenv("VISUAL");
 }
 
 int main(int argc, char **argv) {
+  // Has to be called once in main so internal static variable is properly set
+  // with the main thread id.
+  mysqlshdk::utils::in_main_thread();
+
   mysqlshdk::utils::init_stacktrace();
 #ifdef WIN32
   UINT origcp = GetConsoleCP();
@@ -605,7 +727,12 @@ int main(int argc, char **argv) {
   });
 #else
   auto locale = std::setlocale(LC_ALL, "en_US.UTF-8");
-  if (!locale) log_error("Failed to set locale to en_US.UTF-8");
+  if (!locale) {
+    // logger is not yet initialized here
+    std::cerr << "Failed to set LC_ALL locale to en_US.UTF-8: "
+              << strerror(errno) << '\n';
+  }
+  shcore::setenv("LC_ALL", "en_US.UTF-8");
 #endif
 
 #if defined(WIN32)
@@ -664,15 +791,15 @@ int main(int argc, char **argv) {
     std::cout << "  TO: " << rlp.rlim_cur << std::endl;
   }
 #endif
+  mysqlsh::Scoped_interrupt interrupt_handler(
+      shcore::Interrupts::create(nullptr));
 
-  // init the ^C handler, so it knows what's the main thread
-  shcore::Interrupts::init(nullptr);
   // Disable colors for tests
   mysqlshdk::textui::set_color_capability(mysqlshdk::textui::No_color);
 
   // Reset these environment vars to start with a clean environment
-  putenv(const_cast<char *>("MYSQLSH_RECORDER_PREFIX="));
-  putenv(const_cast<char *>("MYSQLSH_RECORDER_MODE="));
+  shcore::unsetenv("MYSQLSH_RECORDER_PREFIX");
+  shcore::unsetenv("MYSQLSH_RECORDER_MODE");
 
   bool listing_tests = false;
   bool show_all_skipped = false;
@@ -688,8 +815,12 @@ int main(int argc, char **argv) {
       listing_tests = true;
     } else if (shcore::str_beginswith(argv[index], "--gtest_filter")) {
       got_filter = true;
-    } else if (shcore::str_caseeq(argv[index], "--direct")) {
+    } else if (shcore::str_beginswith(argv[index], "--direct")) {
       g_test_recording_mode = mysqlshdk::db::replay::Mode::Direct;
+      char *p = strchr(argv[index], '=');
+      if (p) {
+        target = p + 1;
+      }
     } else if (shcore::str_beginswith(argv[index], "--record")) {
       g_test_recording_mode = mysqlshdk::db::replay::Mode::Record;
       char *p = strchr(argv[index], '=');
@@ -711,14 +842,17 @@ int main(int argc, char **argv) {
         exit(1);
       }
       tracedir = p + 1;
-    } else if (shcore::str_caseeq(argv[index], "--generate-validation-file")) {
+    } else if (shcore::str_caseeq(argv[index], "--generate-validation-file") ||
+               strcmp(argv[index], "-g") == 0) {
       g_generate_validation_file = true;
     } else if (shcore::str_beginswith(argv[index], "--debug=")) {
       DBUG_SET_INITIAL(argv[index] + strlen("--debug="));
-    } else if (strcmp(argv[index], "--trace-no-stop") == 0) {
+    } else if (strcmp(argv[index], "--trace-no-stop") == 0 ||
+               strcmp(argv[index], "-T") == 0) {
       // continue executing script until the end on failure
       g_test_trace_scripts = 1;
-    } else if (strcmp(argv[index], "--trace") == 0) {
+    } else if (strcmp(argv[index], "--trace") == 0 ||
+               strcmp(argv[index], "-t") == 0) {
       // stop executing script on failure
       g_test_trace_scripts = 2;
       g_test_fail_early = true;
@@ -741,8 +875,13 @@ int main(int argc, char **argv) {
       tdb_step = true;
       g_test_trace_scripts = 1;
       g_test_fail_early = true;
-    } else if (strcmp(argv[index], "--profile-scripts") == 0) {
-      g_profile_test_scripts = true;
+    } else if (shcore::str_beginswith(argv[index], "--break=")) {
+      g_break.insert(std::stod(strchr(argv[index], '=') + 1));
+    } else if (strcmp(argv[index], "--profile-scripts") == 0 ||
+               strcmp(argv[index], "-p") == 0) {
+      g_profile_test_scripts = 1;
+    } else if (strcmp(argv[index], "-P") == 0) {
+      g_profile_test_scripts = 2;
     } else if (strcmp(argv[index], "--show-skipped") == 0) {
       show_all_skipped = true;
     } else if (strcmp(argv[index], "--only-failures") == 0) {
@@ -756,6 +895,8 @@ int main(int argc, char **argv) {
     }
   }
 
+  std::string mysqld_path_variables;
+  mysqlsh::Scoped_logger sclogger(setup_logger());
   if (!listing_tests) {
     setup_test_environment();
 
@@ -764,6 +905,12 @@ int main(int argc, char **argv) {
       enable_tdb(tdb_step);
     }
 
+    // The call to the verify_target_servers is needed so the path to the
+    // different mysqld servers gets in place for every execution mode
+    if (!target.empty())
+      target = verify_target_servers(target, &mysqld_path_variables);
+    g_mysqld_path_variables = mysqld_path_variables.c_str();
+
     if (g_test_recording_mode != mysqlshdk::db::replay::Mode::Direct) {
       if (target.empty()) target = g_target_server_version.get_base();
 
@@ -789,13 +936,6 @@ int main(int argc, char **argv) {
     ::testing::GTEST_FLAG(filter) = new_filter.c_str();
   }
 
-  // This will consider the MYSQLSH_HOME environment variable if set,
-  // otherwise it assumes parent dir of the current executable
-  std::string mppath = shcore::get_mysqlx_home_path();
-  mppath =
-      shcore::path::join_path(mppath, "share", "mysqlsh", "mysqlprovision.zip");
-  g_mppath = strdup(mppath.c_str());
-
   std::string mysqlsh_path;
   // mysqlshrec is supposed to be in the same dir as run_unit_tests
   mysqlsh_path =
@@ -812,7 +952,6 @@ int main(int argc, char **argv) {
     std::cout << "Testing: Shell Package." << std::endl;
   std::cout << "Shell Binary: " << g_mysqlsh_path << std::endl;
   std::cout << "Shell Home: " << shcore::get_mysqlx_home_path() << std::endl;
-  std::cout << "MySQL Provision: " << g_mppath << std::endl;
   std::cout << "Test Data Home: " << g_test_home << std::endl;
   if (!listing_tests) {
     std::cout << "Effective Hostname (external address of this host): "
@@ -822,11 +961,11 @@ int main(int argc, char **argv) {
     if (mysqlshdk::utils::Net::is_loopback(getenv("MYSQL_REAL_HOSTNAME"))) {
       std::cout << "Note: " << getenv("MYSQL_REAL_HOSTNAME")
                 << " resolves to a loopback\n";
-      if (strcmp(getenv("MYSQL_HOSTNAME"), getenv("MYSQL_REAL_HOSTNAME")) ==
-          0) {
+      if (!mysqlshdk::utils::Net::is_externally_addressable(
+              getenv("MYSQL_HOSTNAME"))) {
         std::cout
-            << "Set the MYSQL_HOSTNAME to an externally addressable "
-               "hostname or IP when executing AdminAPI tests in this host.\n";
+            << "Set the MYSQL_HOSTNAME to an externally addressable hostname "
+               "or IP when executing AdminAPI tests in this host.\n";
       }
     } else {
       if (strcmp(getenv("MYSQL_HOSTNAME"), getenv("MYSQL_REAL_HOSTNAME")) !=
@@ -920,5 +1059,8 @@ int main(int argc, char **argv) {
   remove_test_keychain();
 #endif  // __APPLE__
 
+#ifdef HAVE_PYTHON
+  shcore::Python_init_singleton::destroy_python();
+#endif
   return ret_val;
 }
