@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -19,6 +19,7 @@
    along with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
+#include <chrono>
 #include <fstream>
 #include <list>
 #include <map>
@@ -33,6 +34,7 @@
 #include <vector>
 #include "gtest_clean.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
+#include "mysqlshdk/shellcore/shell_console.h"
 #include "scripting/common.h"
 #include "scripting/lang_base.h"
 #include "shellcore/base_session.h"
@@ -124,15 +126,20 @@ class Shell_test_output_handler {
   shcore::Interpreter_delegate deleg;
   std::string std_err;
   std::string std_out;
+  std::string internal_std_err;
+  std::string internal_std_out;
   std::stringstream full_output;
   std::mutex stdout_mutex;
+  shcore::Array_t dba_sql_log = nullptr;
   static std::vector<std::string> log;
 
   void set_log_level(shcore::Logger::LOG_LEVEL log_level) {
-    _logger->set_log_level(log_level);
+    m_logger->set_log_level(log_level);
   }
 
-  shcore::Logger::LOG_LEVEL get_log_level() { return _logger->get_log_level(); }
+  shcore::Logger::LOG_LEVEL get_log_level() {
+    return m_logger->get_log_level();
+  }
 
   void validate_stdout_content(const std::string &content, bool expected);
   void validate_stderr_content(const std::string &content, bool expected);
@@ -144,7 +151,7 @@ class Shell_test_output_handler {
   void debug_print(const std::string &line);
   void debug_print_header(const std::string &line);
   void flush_debug_log();
-  void whipe_debug_log() { full_output.clear(); }
+  void wipe_debug_log() { full_output.clear(); }
 
   bool debug;
 
@@ -160,7 +167,7 @@ class Shell_test_output_handler {
   void set_errors_to_stderr(bool value) { m_errors_on_stderr = value; }
 
  protected:
-  static shcore::Logger *_logger;
+  std::shared_ptr<shcore::Logger> m_logger;
 
   static void log_hook(const shcore::Logger::Log_entry &entry, void *);
   bool m_internal;
@@ -168,35 +175,35 @@ class Shell_test_output_handler {
   bool m_errors_on_stderr;
 };
 
-#define MY_EXPECT_STDOUT_CONTAINS(x, ...)                           \
-  do {                                                              \
-    SCOPED_TRACE("...in stdout check\n");                           \
-    output_handler.validate_stdout_content(x, true, ##__VA_ARGS__); \
+#define MY_EXPECT_STDOUT_CONTAINS(x)                 \
+  do {                                               \
+    SCOPED_TRACE("...in stdout check\n");            \
+    output_handler.validate_stdout_content(x, true); \
   } while (0)
-#define MY_EXPECT_STDERR_CONTAINS(x, ...)                           \
-  do {                                                              \
-    SCOPED_TRACE("...in stderr check\n");                           \
-    output_handler.validate_stderr_content(x, true, ##__VA_ARGS__); \
+#define MY_EXPECT_STDERR_CONTAINS(x)                 \
+  do {                                               \
+    SCOPED_TRACE("...in stderr check\n");            \
+    output_handler.validate_stderr_content(x, true); \
   } while (0)
-#define MY_EXPECT_LOG_CONTAINS(x, ...)                           \
-  do {                                                           \
-    SCOPED_TRACE("...in log check\n");                           \
-    output_handler.validate_log_content(x, true, ##__VA_ARGS__); \
+#define MY_EXPECT_LOG_CONTAINS(x)                 \
+  do {                                            \
+    SCOPED_TRACE("...in log check\n");            \
+    output_handler.validate_log_content(x, true); \
   } while (0)
-#define MY_EXPECT_STDOUT_NOT_CONTAINS(x, ...)                        \
-  do {                                                               \
-    SCOPED_TRACE("...in stdout check\n");                            \
-    output_handler.validate_stdout_content(x, false, ##__VA_ARGS__); \
+#define MY_EXPECT_STDOUT_NOT_CONTAINS(x)              \
+  do {                                                \
+    SCOPED_TRACE("...in stdout check\n");             \
+    output_handler.validate_stdout_content(x, false); \
   } while (0)
-#define MY_EXPECT_STDERR_NOT_CONTAINS(x, ...)                        \
-  do {                                                               \
-    SCOPED_TRACE("...in stderr check\n");                            \
-    output_handler.validate_stderr_content(x, false, ##__VA_ARGS__); \
+#define MY_EXPECT_STDERR_NOT_CONTAINS(x)              \
+  do {                                                \
+    SCOPED_TRACE("...in stderr check\n");             \
+    output_handler.validate_stderr_content(x, false); \
   } while (0)
-#define MY_EXPECT_LOG_NOT_CONTAINS(x, ...)                        \
-  do {                                                            \
-    SCOPED_TRACE("...in log check\n");                            \
-    output_handler.validate_log_content(x, false, ##__VA_ARGS__); \
+#define MY_EXPECT_LOG_NOT_CONTAINS(x)              \
+  do {                                             \
+    SCOPED_TRACE("...in log check\n");             \
+    output_handler.validate_log_content(x, false); \
   } while (0)
 
 /**
@@ -233,17 +240,18 @@ class Shell_core_test_wrapper : public tests::Shell_base_test {
   // options First set the options on _options
   void reset_options(int argc = 0, const char **argv = nullptr,
                      bool remove_config = true) {
-    extern char *g_mppath;
     extern int g_test_default_verbosity;
     std::string options_file = get_options_file_name();
     if (remove_config) std::remove(options_file.c_str());
     _opts.reset(new mysqlsh::Shell_options(argc, const_cast<char **>(argv),
                                            options_file));
     _options = const_cast<mysqlsh::Shell_options::Storage *>(&_opts->get());
-    _options->gadgets_path = g_mppath;
     if (!argv) _options->verbose_level = g_test_default_verbosity;
     _options->db_name_cache = false;
 
+    // don't load any plugins except in plugin specific tests
+    _options->plugins_path = "";
+
     // Allows derived classes configuring specific options
     set_options();
   }
@@ -265,6 +273,10 @@ class Shell_core_test_wrapper : public tests::Shell_base_test {
     _interactive_shell.reset();
     _interactive_shell.reset(
         new mysqlsh::Command_line_shell(options, std::move(delegate)));
+
+    std::dynamic_pointer_cast<mysqlsh::Shell_console>(
+        mysqlsh::current_console())
+        ->set_use_colors(false);
   }
 
   virtual void reset_shell() {
@@ -276,7 +288,6 @@ class Shell_core_test_wrapper : public tests::Shell_base_test {
                   std::unique_ptr<shcore::Interpreter_delegate>{
                       new shcore::Interpreter_delegate(output_handler.deleg)});
 
-    _interactive_shell->finish_init();
     set_defaults();
     enable_testutil();
   }
@@ -290,10 +301,6 @@ class Shell_core_test_wrapper : public tests::Shell_base_test {
   std::shared_ptr<mysqlsh::Shell_options> _opts;
   mysqlsh::Shell_options::Storage *_options;
 
-  // set to true in a subclass if reset_shell() should not be called
-  // during SetUp()
-  bool _delay_reset_shell = false;
-
   virtual void debug_print(const std::string &s) {
     output_handler.debug_print(s);
   }
@@ -317,9 +324,13 @@ class Shell_core_test_wrapper : public tests::Shell_base_test {
     return _opts;
   }
 
-  unsigned int m_start_time = 0;
+  std::chrono::time_point<std::chrono::steady_clock> m_start_time =
+      std::chrono::steady_clock::now();
+
+  std::list<std::pair<int, std::string>> m_slowest_lines;
 
   friend class mysqlsh::Test_debugger;
+  friend class tests::Testutils;
 };
 
 /**
