@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
 
 #include "gtest_clean.h"
 #include "unittest/test_utils/mocks/gmock_clean.h"
+#include "unittest/test_utils/shell_test_env.h"
 #include "utils/utils_general.h"
 
 #include <mysql_version.h>
@@ -96,12 +97,12 @@ TEST(utils_general, split_account) {
       {"root@%", "root", "%"},
       {"test_user", "test_user", ""},
       {"123@bar", "123", "bar"},
-      {"@@@", "@@", ""},
-      {"@foo@@localhost", "@foo@", "localhost"},
-      {"@\"''\"foo@localhost", "@\"''\"foo", "localhost"},
+      {"'@@'@", "@@", ""},
+      {"'@foo@'@localhost", "@foo@", "localhost"},
+      {"`@\"\'\'\"foo`@localhost", "@\"''\"foo", "localhost"},
       {"skip-grants user@skip-grants host", "skip-grants user",
        "skip-grants host"},
-      {"foo@@'stuf'", "foo@", "stuf"},
+      {"'foo@'@'stuf'", "foo@", "stuf"},
       {"foo bar@localhost", "foo bar", "localhost"}};
   for (auto &t : good_cases_no_auto_quote) {
     a.clear();
@@ -116,7 +117,9 @@ TEST(utils_general, split_account) {
       {"ic@192.168.%", "ic", "192.168.%"},  // Regression test for BUG#25528695
       {"foo@192.168.1.%", "foo", "192.168.1.%"},
       {"foo@192.58.197.0/255.255.255.0", "foo", "192.58.197.0/255.255.255.0"},
-      {"root@foo-bar.com", "root", "foo-bar.com"}};
+      {"root@foo-bar.com", "root", "foo-bar.com"},
+      {"root@aa1-bbbb-ccc33.corp.example.com", "root",
+       "aa1-bbbb-ccc33.corp.example.com"}};
   for (auto &t : good_cases_auto_quote) {
     a.clear();
     b.clear();
@@ -129,6 +132,10 @@ TEST(utils_general, split_account) {
       "'foo",
       "'foo'bar",
       "'foo'@@bar",
+      "f@'foo@'@bar",
+      "foo@bar@baz",
+      "'foo'@bar@baz",
+      "'foo@b'ar@baz",
       "'foo@bar",
       "''foo",
       "\"foo'@bar",
@@ -152,6 +159,12 @@ TEST(utils_general, split_account) {
       "foo@ .::1lol\\t\\n\\'\"&$%",
       "`foo@bar",
       "foo@`bar",
+      "@@@",
+      "@@",
+      "@foo@@localhost",
+      "foo@@'stuf'",
+      "@\"\'\'\"foo@localhost",
+      "",
   };
   for (auto &t : bad_cases) {
     SCOPED_TRACE(t);
@@ -179,6 +192,81 @@ TEST(utils_general, make_account) {
   }
 }
 
+TEST(utils_general, split_schema_and_table) {
+  std::string schema;
+  std::string table;
+
+  EXPECT_THROW_LIKE(split_schema_and_table("", &schema, &table),
+                    std::runtime_error,
+                    "Invalid object name, table name cannot be empty.");
+  EXPECT_THROW_LIKE(split_schema_and_table("`schema`x`table`", &schema, &table),
+                    std::runtime_error,
+                    "Invalid object name, expected '.', but got: 'x'.");
+  EXPECT_THROW_LIKE(
+      split_schema_and_table("`schema`.`table`.", &schema, &table),
+      std::runtime_error,
+      "Invalid object name, expected end of name, but got: '.'.");
+  EXPECT_THROW_LIKE(split_schema_and_table("`schema`.", &schema, &table),
+                    std::runtime_error,
+                    "Invalid object name, table name cannot be empty.");
+
+  schema = table = "";
+  EXPECT_NO_THROW(split_schema_and_table("table", &schema, &table));
+  EXPECT_EQ("", schema);
+  EXPECT_EQ("table", table);
+
+  schema = table = "";
+  EXPECT_NO_THROW(split_schema_and_table("`table`", &schema, &table));
+  EXPECT_EQ("", schema);
+  EXPECT_EQ("table", table);
+
+  schema = table = "";
+  EXPECT_NO_THROW(split_schema_and_table("schema.table", &schema, &table));
+  EXPECT_EQ("schema", schema);
+  EXPECT_EQ("table", table);
+
+  schema = table = "";
+  EXPECT_NO_THROW(split_schema_and_table("`schema`.table", &schema, &table));
+  EXPECT_EQ("schema", schema);
+  EXPECT_EQ("table", table);
+
+  schema = table = "";
+  EXPECT_NO_THROW(split_schema_and_table("schema.`table`", &schema, &table));
+  EXPECT_EQ("schema", schema);
+  EXPECT_EQ("table", table);
+
+  schema = table = "";
+  EXPECT_NO_THROW(split_schema_and_table("`schema`.`table`", &schema, &table));
+  EXPECT_EQ("schema", schema);
+  EXPECT_EQ("table", table);
+
+  schema = table = "";
+  EXPECT_NO_THROW(split_schema_and_table("`schema.table`", &schema, &table));
+  EXPECT_EQ("", schema);
+  EXPECT_EQ("schema.table", table);
+}
+
+TEST(utils_general, unquote_identifier) {
+  EXPECT_THROW_LIKE(unquote_identifier(""), std::runtime_error,
+                    "Object name cannot be empty.");
+
+  EXPECT_THROW_LIKE(unquote_identifier("``"), std::runtime_error,
+                    "Object name cannot be empty.");
+
+  EXPECT_THROW_LIKE(unquote_identifier("schema.table"), std::runtime_error,
+                    "Invalid object name, expected end of name, but got: '.'.");
+
+  EXPECT_THROW_LIKE(unquote_identifier("`schema`.table"), std::runtime_error,
+                    "Invalid object name, expected end of name, but got: '.'.");
+
+  EXPECT_THROW_LIKE(unquote_identifier("`schema`table"), std::runtime_error,
+                    "Invalid object name, expected end of name, but got: 't'.");
+
+  EXPECT_EQ("table", unquote_identifier("table"));
+  EXPECT_EQ("table", unquote_identifier("`table`"));
+  EXPECT_EQ("schema.table", unquote_identifier("`schema.table`"));
+}
+
 TEST(utils_general, split_string_chars) {
   std::vector<std::string> strs =
       split_string_chars("aaaa-vvvvv>bbbbb", "->", true);
@@ -344,8 +432,14 @@ TEST(utils_general, lexical_cast) {
   EXPECT_EQ(12345, lexical_cast<int>("12345"));
   EXPECT_EQ(12345, lexical_cast<unsigned>("12345"));
   EXPECT_TRUE(lexical_cast<bool>("true"));
+  EXPECT_TRUE(lexical_cast<bool>("TRUE"));
+  EXPECT_TRUE(lexical_cast<bool>("True"));
+  EXPECT_TRUE(lexical_cast<bool>("truE"));
   EXPECT_TRUE(lexical_cast<bool>("1"));
   EXPECT_FALSE(lexical_cast<bool>("false"));
+  EXPECT_FALSE(lexical_cast<bool>("FALSE"));
+  EXPECT_FALSE(lexical_cast<bool>("False"));
+  EXPECT_FALSE(lexical_cast<bool>("false"));
   EXPECT_FALSE(lexical_cast<bool>(0));
   EXPECT_EQ(123.123, lexical_cast<double>("123.123"));
   EXPECT_EQ(std::string("12345"), lexical_cast<std::string>(12345));
@@ -353,10 +447,72 @@ TEST(utils_general, lexical_cast) {
 
   EXPECT_THROW(lexical_cast<bool>(12345), std::invalid_argument);
   EXPECT_THROW(lexical_cast<bool>("tru"), std::invalid_argument);
+  EXPECT_THROW(lexical_cast<bool>("TRU"), std::invalid_argument);
+  EXPECT_THROW(lexical_cast<bool>("falze"), std::invalid_argument);
+  EXPECT_THROW(lexical_cast<bool>("FALS"), std::invalid_argument);
   EXPECT_THROW(lexical_cast<int>(123.45), std::invalid_argument);
   EXPECT_THROW(lexical_cast<double>("12345f"), std::invalid_argument);
   EXPECT_THROW(lexical_cast<unsigned>("-12345"), std::invalid_argument);
   EXPECT_THROW(lexical_cast<unsigned>(-12345), std::invalid_argument);
 }
 
+TEST(utils_general, environment_variables) {
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", "VaLu3"));
+  EXPECT_STREQ("VaLu3", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::unsetenv("TEST_ENV_VAR"));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", std::string("another value")));
+  EXPECT_STREQ("another value", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::unsetenv("TEST_ENV_VAR"));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(
+      shcore::setenv(std::string("TEST_ENV_VAR"), std::string("third value")));
+  EXPECT_STREQ("third value", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::unsetenv(std::string("TEST_ENV_VAR")));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(shcore::setenv(std::string("TEST_ENV_VAR=fourth")));
+  EXPECT_STREQ("fourth", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::unsetenv("TEST_ENV_VAR"));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR=fifth"));
+  EXPECT_STREQ("fifth", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::unsetenv(std::string("TEST_ENV_VAR")));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", "sixth"));
+  EXPECT_STREQ("sixth", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", "seventh"));
+  EXPECT_STREQ("seventh", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR=eight"));
+  EXPECT_STREQ("eight", getenv("TEST_ENV_VAR"));
+  EXPECT_FALSE(shcore::setenv("TEST_ENV_VAR"));
+  EXPECT_STREQ("eight", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::unsetenv("TEST_ENV_VAR"));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", "10"));
+  EXPECT_STREQ("10", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", nullptr));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", "11"));
+  EXPECT_STREQ("11", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", ""));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", "12"));
+  EXPECT_STREQ("12", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", std::string("")));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR", "13"));
+  EXPECT_STREQ("13", getenv("TEST_ENV_VAR"));
+  EXPECT_TRUE(shcore::setenv("TEST_ENV_VAR="));
+  EXPECT_EQ(nullptr, getenv("TEST_ENV_VAR"));
+}
+
 }  // namespace shcore
